@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -15,11 +15,13 @@
 
 #include <QtCore/QMap>
 #include <QtCore/QObject>
+#include <QtScript/QScriptValue>
+#include <QtScript/QScriptEngine>
 #include <QtCore/QStringList>
 
 namespace GB2 {
 
-class DBXRefInfo {
+class GB2_COREAPI_EXPORT DBXRefInfo {
 public:
     DBXRefInfo(){};
     DBXRefInfo(const QString& _name, const QString& _url, 
@@ -30,19 +32,31 @@ public:
     QString url;
     QString fileUrl;
     QString comment;
+
+    static void setupToEngine(QScriptEngine *engine);
+private:    
+    static QScriptValue toScriptValue(QScriptEngine *engine, DBXRefInfo const &in);
+    static void fromScriptValue(const QScriptValue &object, DBXRefInfo &out);
 };
 
+
 class GB2_COREAPI_EXPORT DBXRefRegistry : public QObject {
     Q_OBJECT
 public:
     DBXRefRegistry(QObject* p = NULL);
 	
-    DBXRefInfo getRefByKey(const QString& dbxrefKey)const {return refsByKey.value(dbxrefKey);}
+    Q_INVOKABLE DBXRefInfo getRefByKey(const QString& dbxrefKey)const {return refsByKey.value(dbxrefKey);}
+
+    static void setupToEngine(QScriptEngine *engine);
 
 private:
+    static QScriptValue toScriptValue(QScriptEngine *engine, DBXRefRegistry* const &in);
+    static void fromScriptValue(const QScriptValue &object, DBXRefRegistry* &out);
+
 	QMap<QString,DBXRefInfo> refsByKey;
 };
-
 }//namespace
+Q_DECLARE_METATYPE(GB2::DBXRefRegistry*)
+Q_DECLARE_METATYPE(GB2::DBXRefInfo)
 
 #endif
