@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -23,6 +23,9 @@ const DocumentFormatId BaseDocumentFormats::RAW_DNA_SEQUENCE("raw");
 const DocumentFormatId BaseDocumentFormats::CLUSTAL_ALN("clustal");
 const DocumentFormatId BaseDocumentFormats::STOCKHOLM("stockholm");
 const DocumentFormatId BaseDocumentFormats::NEWICK("newick");
+const DocumentFormatId BaseDocumentFormats::INDEX("index");
+const DocumentFormatId BaseDocumentFormats::FASTQ("fastq");
+const DocumentFormatId BaseDocumentFormats::PLAIN_ASN("mmdb");
 
 
 }//namespace
