@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -14,6 +14,7 @@
 #include "PluginModel.h"
 #include "ServiceTypes.h"
 #include "SelectionModel.h"
+#include "DocumentModel.h"
 
 class QMenu;
 class QAction;
@@ -26,8 +27,8 @@ class GObjectSelection;
 class GB2_COREAPI_EXPORT ProjectView : public Service {
 	Q_OBJECT
 public:
-    ProjectView(Plugin* p, const QString& sname, const QString& sdesc) 
-		: Service(p, Service_ProjectView, sname, sdesc, QList<ServiceType>()<<Service_Project)  
+    ProjectView(const QString& sname, const QString& sdesc) 
+		: Service(Service_ProjectView, sname, sdesc, QList<ServiceType>()<<Service_Project)  
 	{
     }
 
@@ -39,6 +40,8 @@ public:
 	
 	virtual QAction* getAddExistingDocumentAction() const = 0;
 
+    virtual void highlightItem(Document*) = 0;
+
 signals:
 	void si_onDocTreePopupMenuRequested(QMenu& m);
 };
