@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -28,6 +28,22 @@ public:
 
     bool openLastProjectAtStartup() const;
     void setOpenLastProjectAtStartup(bool v);
+
+    //a style selected by user
+    QString getVisualStyle() const;
+    void setVisualStyle(const QString& newStyle);
+    
+    // specify dir for downloaded files
+    QString getDownloadDirPath() const;
+    void setDownloadDirPath(const QString& newPath) const;
+
+    // recently downloaded file names
+    QStringList getRecentlyDownloadedFileNames() const;
+    void setRecentlyDownloadedFileNames(const QStringList& fileNames) const;
+
+    bool enableCollectingStatistics() const;
+    void setEnableCollectingStatistics(bool b);
+
 };
 
 }//namespace
