@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -12,6 +12,7 @@
 #define _GB2_DNA_SEQUENCE_H_
 
 #include <core_api/core_api.h>
+#include "DNAInfo.h"
 
 #include <QtCore/QByteArray>
 
@@ -21,12 +22,15 @@ class DNAAlphabet;
 
 class GB2_COREAPI_EXPORT DNASequence {
 public:
-    DNASequence(const QByteArray& s = QByteArray(), DNAAlphabet* a = NULL);
+    DNASequence(const QString& name, const QByteArray& s = QByteArray(), DNAAlphabet* a = NULL);
+    DNASequence(const QByteArray& s = QByteArray(), DNAAlphabet* a = NULL) : seq(s), alphabet(a) {}
     ~DNASequence();
 
+    QVariantMap     info;
     QByteArray      seq;
     DNAAlphabet*    alphabet;
 
+    QString getName() const {return DNAInfo::getName(info);}
     bool isNull() const {return !alphabet && seq.length() == 0;}
     int length() const {return seq.length();}
     const char* constData() const {return seq.constData();}
