@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -16,7 +16,7 @@
 #include <core_api/IOAdapter.h>
 #include <core_api/DNAAlphabet.h>
 #include <core_api/GObjectReference.h>
-
+#include <datatype/DNAInfo.h>
 #include <gobjects/DNASequenceObject.h>
 #include <gobjects/GObjectTypes.h>
 #include <gobjects/GObjectRelationRoles.h>
@@ -59,10 +59,6 @@ bool ABIFormat::isDataFormatSupported(const char* data, int size) const {
 	return TextUtils::contains(TextUtils::BINARY, data, size);
 }
 
-void ABIFormat::storeDocument(Document*, TaskStateInfo&, IOAdapterFactory*, const QString&) {
-	assert(0);
-}
-
 bool ABIFormat::isObjectOpSupported(const Document* d , DocumentFormat::DocObjectOp op, GObjectType t) const {
     Q_UNUSED(d); Q_UNUSED(op);
     return (t == GObjectTypes::DNA_SEQUENCE || t == GObjectTypes::DNA_CHROMATOGRAM); 
@@ -81,6 +77,9 @@ bool ABIFormat::checkConstraints(const DocumentFormatConstraints& c) const {
 
     if (c.mustSupportWrite) {
         return false;
+    }
+    if( c.supportsStreamingRead ) {
+        return false;
     }
 	return true;
 }
@@ -90,7 +89,7 @@ Document* ABIFormat::loadExistingDocument(IOAdapterFactory* iof, const QString&
     std::auto_ptr<IOAdapter> io(iof->createIOAdapter());
     bool ok = io->open(url, IOAdapterMode_Read);
     if (!ok) {
-        ti.error = Translations::errorOpeningFileRead(url);
+        ti.setError(Translations::errorOpeningFileRead(url));
         return NULL;
     }
     QByteArray readBuff;
@@ -99,7 +98,7 @@ Document* ABIFormat::loadExistingDocument(IOAdapterFactory* iof, const QString&
     while ((len=io->readBlock(block.data(),BUFF_SIZE)) > 0) {
         readBuff.append(QByteArray(block.data(), len));
         if (readBuff.size()>CHECK_MB) {
-            ti.error = Translations::errorFileTooLarge(url); 
+            ti.setError(Translations::errorFileTooLarge(url)); 
             break;
         }
     }
@@ -114,7 +113,7 @@ Document* ABIFormat::loadExistingDocument(IOAdapterFactory* iof, const QString&
     sf.size = readBuff.size();
     Document* doc = parseABI(&sf, iof, url, fs);
     if (doc == NULL && !ti.hasErrors()) {
-        ti.error = tr("Not a valid ABIF file: ").arg(url);
+        ti.setError(tr("Not a valid ABIF file: %1").arg(url));
     }
     return doc;
 }
@@ -187,7 +186,7 @@ Document* ABIFormat::loadExistingDocument(IOAdapterFactory* iof, const QString&
 * Returns -1 for failure, 0 for success.
 */
 static int getABIIndexOffset(SeekableBuf* fp, uint *indexO) {
-    uint magic;
+    uint magic = 0;
 
     /*
     * Initialise header_fudge.
@@ -294,8 +293,8 @@ int getABIString(SeekableBuf *fp, int indexO, uint label, uint count, char *stri
     if (!off)
         return -1;
 
-    if (off = getABIIndexEntryLW(fp, indexO, label, count, 4, &len)) {
-        uchar len2;
+    if ((off = getABIIndexEntryLW(fp, indexO, label, count, 4, &len))) {
+        uchar len2 = 0;
 
         if (!len)
             return 0;
@@ -654,11 +653,11 @@ skip_bases:
             char *commstrp = commstr;
             char *p;
             do {
-                if (p = strchr(commstrp, '\n')){
+                if ((p = strchr(commstrp, '\n'))){
                     *p++ = 0;
                 }
-                sequenceComment.append(QString("Comment: %1\n").arg(commstrp));
-            } while(commstrp = p);
+                sequenceComment.append(QString("ABI Comment: %1\n").arg(commstrp));
+            } while((commstrp = p));
         }
 
 
@@ -835,14 +834,18 @@ skip_bases:
     }
 
     /* SUCCESS */
-    cd.hasQV = false;
+    cd.hasQV = true;
     cd.seqLength = sequence.size();
-    assert(sequence.size() == numBases);
+    assert(sequence.size() == int(numBases));
     cd.traceLength = numPoints;
-    assert(cd.A.size() == numPoints + 1);
+    assert(cd.A.size() == int(numPoints + 1));
+
+    DNASequence dna(sequence);
+    dna.info.insert(DNAInfo::COMMENT, sequenceComment.split("\n"));
+    dna.info.insert(DNAInfo::ID, sequenceName);
 
     QList<GObject*> objects;
-    DNASequenceObject* seqObj = DocumentFormatUtils::addSequenceObject(objects, sequenceName, sequence);
+    DNASequenceObject* seqObj = DocumentFormatUtils::addSequenceObject(objects, sequenceName, dna);
     DNAChromatogramObject* chromObj = new DNAChromatogramObject(cd, "Chromatogram");
     objects.append(chromObj);
     objects.append(new TextObject(sequenceComment, "Info"));
