@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -11,6 +11,7 @@
 #include "EMBLGenbankAbstractDocument.h"
 
 #include "GenbankLocationParser.h"
+#include "GenbankFeatures.h"
 #include "DocumentFormatUtils.h"
 
 #include <gobjects/DNASequenceObject.h>
@@ -20,15 +21,14 @@
 #include <core_api/IOAdapter.h>
 #include <core_api/Task.h>
 #include <core_api/DNAAlphabet.h>
-
+#include <datatype/DNAInfo.h>
 #include <util_text/TextUtils.h>
+#include <util_gui/GUIUtils.h>
 #include <memory>
 
 namespace GB2 {
 
 /* TRANSLATOR GB2::EMBLGenbankAbstractDocument */    
-/* TRANSLATOR GB2::IOAdapter */    
-
 //TODO: local8bit or ascii??
 
 EMBLGenbankAbstractDocument::EMBLGenbankAbstractDocument(const DocumentFormatId& _id, const QString& _formatName, int mls, QObject* p) 
@@ -52,7 +52,12 @@ bool EMBLGenbankAbstractDocument::checkConstraints(const DocumentFormatConstrain
 	if (c.checkRawData) {
 		return isDataFormatSupported(c.rawData.constData(), c.rawData.size());
 	}
-    return true;
+    
+    bool ret = true;
+    if( c.supportsStreamingRead ) {
+        ret = ret && true;
+    }
+    return ret;
 }
 
 
@@ -62,14 +67,14 @@ bool EMBLGenbankAbstractDocument::checkConstraints(const DocumentFormatConstrain
 Document* EMBLGenbankAbstractDocument::loadExistingDocument(IOAdapterFactory* iof, const QString& url, TaskStateInfo& ti, const QVariantMap& _fs) {
 	std::auto_ptr<IOAdapter> io(iof->createIOAdapter());
 	if (!io->open(url, IOAdapterMode_Read)) {
-		ti.error = IOAdapter::tr("error_opening_url_for_read '%1'").arg(url);
+        ti.setError(Translations::errorOpeningFileRead(url));
 		return NULL;
 	}
 
     QVariantMap fs = _fs;
     QList<GObject*> objects;
     QString writeLockReason;
-    load(url, io.get(), objects, fs, ti, writeLockReason);
+    load(url, io.get(), objects, fs, ti, writeLockReason, false );
 	io->close();
 
     if (ti.hasErrors() || ti.cancelFlag) {
@@ -81,12 +86,31 @@ Document* EMBLGenbankAbstractDocument::loadExistingDocument(IOAdapterFactory* io
 	return doc;
 }
 
+Document* EMBLGenbankAbstractDocument::loadExistingDocument( IOAdapter* io, TaskStateInfo& ti, const QVariantMap& fSettings ) {
+    if( NULL == io || !io->isOpen() ) {
+        ti.setError(Translations::badArgument("IO adapter"));
+        return NULL;
+    }
+    QVariantMap fs = fSettings;
+    QList<GObject*> objects;
+    QString writeLockReason;
+    QString url = io->getUrl();
+    
+    load( url, io, objects, fs, ti, writeLockReason, true );
+    if (ti.hasErrors() || ti.cancelFlag) {
+        return NULL;
+    }
+    
+    DocumentFormatUtils::updateFormatSettings( objects, fs );
+    Document* doc = new Document( this, io->getFactory(), url, objects, fs, writeLockReason );
+    return doc;
+}
 
-const QString EMBLGenbankAbstractDocument::UNIMARK("UNIMARK");
-const QString EMBLGenbankAbstractDocument::UNKNOWN_OBJ_NAME("_object");
+const QString EMBLGenbankAbstractDocument::UGENE_MARK("UNIMARK");
+const QString EMBLGenbankAbstractDocument::DEFAULT_OBJ_NAME("unnamed");
 
 
-void EMBLGenbankAbstractDocument::load(const QString& docUrl, IOAdapter* io, QList<GObject*>& objects, QVariantMap& fs, TaskStateInfo& si, QString& writeLockReason) {
+void EMBLGenbankAbstractDocument::load(const QString& docUrl, IOAdapter* io, QList<GObject*>& objects, QVariantMap& fs, TaskStateInfo& si, QString& writeLockReason, bool onlyOne ) {
     writeLockReason.clear();
 
     //get settings
@@ -104,12 +128,15 @@ void EMBLGenbankAbstractDocument::load(const QString& docUrl, IOAdapter* io, QLi
 
     sequence.reserve(predictedSize + (merge ? gapSize : 0));
     
-	QStringList prevNames;
-	
+	QSet<QString> usedNames;
 	bool toolMark = false;
     int sequenceStart = 0;
     GObjectReference sequenceRef(GObjectReference(docUrl, "", GObjectTypes::DNA_SEQUENCE));
 
+    QByteArray readBuffer(ParserState::READ_BUFF_SIZE, '\0');
+    ParserState st(getFormatId() == BaseDocumentFormats::PLAIN_GENBANK ? 12 : 5, io, NULL, si);
+    st.buff = readBuffer.data();
+
 	for (int i=0; !si.hasErrors() && !si.cancelFlag; i++) {
         if (!merge) {
             sequence.clear();
@@ -118,61 +145,73 @@ void EMBLGenbankAbstractDocument::load(const QString& docUrl, IOAdapter* io, QLi
             sequenceStart = sequence.size();
         }
 
-		//reading meta info
-        si.stateDesc = tr("reading_header");
-		std::auto_ptr<EMBLGenbankAbstractHeader> h(readHeader(io, si));
-		if (si.hasErrors() || h.get() == NULL || si.cancelFlag) {
+        EMBLGenbankDataEntry data;
+        st.entry = &data;
+        si.setStateDesc(tr("Reading entry header"));
+		if (!readEntry(sequence, &st)) {
 			break;
 		}
-        int headerSeqLen = h->idLine->seqLen;
+        // tolerate blank lines between records
+        char ch;
+        bool b;
+        while ((b = st.io->getChar(&ch)) && (ch == '\n' || ch == '\r')){}
+        if (b) {
+            st.io->skip(-1);
+        }
 
-		toolMark = h->tags.contains(UNIMARK);
-		
-		// reading annotations
-        si.stateDesc = tr("reading_annotations_for_%1").arg(h->idLine->name);
-        QList<SharedAnnotationData> features = readAnnotations(io, si, sequence.size());
-		if (si.hasErrors() || si.cancelFlag) {
-			break;
-		}
+		toolMark = data.tags.contains(UGENE_MARK);
         AnnotationTableObject* annotationsObject  = NULL;
-		if (!features.isEmpty()) {
-            QString annotationName = genObjectName(prevNames, h.get(), "FT", i+1);
+
+		if (data.hasAnnotationObjectFlag) {
+            QString annotationName = genObjectName(usedNames, data.name, data.tags, i+1, GObjectTypes::ANNOTATION_TABLE);
             if (merge && mergedAnnotations == NULL) {
                 mergedAnnotations = new AnnotationTableObject(annotationName);
             }
             annotationsObject = merge ? mergedAnnotations : new AnnotationTableObject(annotationName);
 
-            QString groupName;//TODO:
-			foreach(SharedAnnotationData d, features) {
-                annotationsObject->addAnnotation(new Annotation(d), groupName);
+            QStringList groupNames;
+            foreach(SharedAnnotationData d, data.features) {
+                groupNames.clear();
+                d->removeAllQualifiers(GBFeatureUtils::QUALIFIER_GROUP, groupNames);
+                if (groupNames.isEmpty()) {
+                    annotationsObject->addAnnotation(new Annotation(d));
+                } else {
+                    Annotation* a = new Annotation(d);
+                    foreach(const QString& gName, groupNames) {
+                        annotationsObject->getRootGroup()->getSubgroup(gName, true)->addAnnotation(a);
+                    }
+                }
 			}
 
             if (!merge) {
                 objects.append(annotationsObject);
             }
-		}
+        } else {
+            assert(data.features.isEmpty());
+        }
         
-        //reading sequence
-        si.stateDesc = tr("reading_sequence_for_%1").arg(h->idLine->name);
-        sequence.reserve(sequence.size() + headerSeqLen);
-        readSequence(sequence, io, si);
 		if (!si.hasErrors() && !si.cancelFlag) {
             int sequenceLen = sequence.size() - sequenceStart;
-            QString sequenceName = genObjectName(prevNames, h.get(), "SEQ", i+1);
+            QString sequenceName = genObjectName(usedNames, data.name, data.tags, i+1, GObjectTypes::DNA_SEQUENCE);
             if (merge && sequenceLen == 0 && annotationsObject!=NULL) {
-                si.error = tr("error_merging_no_sequence_for_annotation_object");
+                si.setError(tr("Merge error: found annotations without sequence"));
                 break;
             } else if (merge) {
                 contigs.append(sequenceName);
                 mergedMapping.append(LRegion(sequenceStart, sequenceLen));
             } else if (sequenceLen!=0){
-                DNASequenceObject* seqObj = DocumentFormatUtils::addSequenceObject(objects, sequenceName, sequence);
+                DNASequence seq(sequence);
+                seq.info = data.tags;
+                DNASequenceObject* seqObj = DocumentFormatUtils::addSequenceObject(objects, sequenceName, seq);
                 if (annotationsObject!=NULL) {
                     sequenceRef.objName = seqObj->getGObjectName();
                     annotationsObject->addObjectRelation(GObjectRelation(sequenceRef, GObjectRelationRole::SEQUENCE));
                 }
             }
         }
+        if( onlyOne ) {
+            break;
+        }
     }
     
     if (!si.hasErrors() && !si.cancelFlag && merge && !contigs.isEmpty()) {
@@ -187,8 +226,14 @@ void EMBLGenbankAbstractDocument::load(const QString& docUrl, IOAdapter* io, QLi
         }
     }
 
+    if (!si.hasErrors() && !si.cancelFlag && objects.isEmpty()) {
+        si.setError(GB2::Document::tr("Document is empty."));
+    }
+
     if (!toolMark) {
 		writeLockReason = DocumentFormat::CREATED_NOT_BY_UGENE;
+    } else if (merge && contigs.size() > 1) {
+        writeLockReason = DocumentFormat::MERGED_SEQ_LOCK;
     }
 }
 
@@ -266,23 +311,42 @@ int EMBLGenbankAbstractDocument::readMultilineQualifier(IOAdapter* io, char* cbu
 }
 
 
-QString EMBLGenbankAbstractDocument::genObjectName(QStringList& prevNames, const EMBLGenbankAbstractHeader* h, const QString& suffix, int n) {
-    QString name = h->tags.value("ACCESSION");
-    if (name.isEmpty()) {
-        name = h->idLine->name;
+QString EMBLGenbankAbstractDocument::genObjectName(QSet<QString>& usedNames, const QString& seqName, const QVariantMap& tags, int n, const GObjectType& t) {
+    //try to check UGENE_MARK first
+    QString name;
+    QStringList unimark = tags.value(UGENE_MARK).toStringList();
+    if (!unimark.isEmpty()) {
+        int pos = usedNames.size();
+        if (pos < unimark.size()) {
+            name = unimark.at(pos);
+        }
     }
     if (name.isEmpty()) {
-        name = h->tags.value(UNIMARK);
-    }
-    if (name.isEmpty()) {
-        name = EMBLGenbankAbstractDocument::UNKNOWN_OBJ_NAME;
+        name = seqName;
+        if (name.isEmpty()) {
+            name = DNAInfo::getPrimaryAccession(tags);
+            int spaceIdx = name.indexOf(' ');
+            if (spaceIdx > 0) {
+                name = name.left(spaceIdx-1);
+            }
+            if (name.isEmpty()) {
+                name = EMBLGenbankAbstractDocument::DEFAULT_OBJ_NAME;
+            }
+        }
+        if(t == GObjectTypes::ANNOTATION_TABLE) {
+            name+=" features";
+        } else if (t == GObjectTypes::DNA_SEQUENCE) {
+            name+=" sequence";
+        }
     }
+    //now variate name if there are duplicates
     int n2 = 1;
-    while (name.isEmpty() || prevNames.contains(name)) {
-        name = name + "_" + suffix + "_" + QString::number(n) + (n2 == 1 ? QString("") : ("_"+QString::number(n2)));
+    QString res = name;
+    while (usedNames.contains(res)) {
+        res =  name + " " + QString::number(n) + (n2 == 1 ? QString("") : ("."+QString::number(n2)));
     }
-    prevNames.append(name);
-    return name;
+    usedNames.insert(res);
+    return res;
 }
 
 
@@ -298,32 +362,28 @@ static void checkQuotes(const char* str, int len, bool& outerQuotes, bool& doubl
     }
 }
 
-
-const QByteArray EMBLGenbankAbstractDocument::AMINO_STRAND_QNAME("uni_amino");
-const QByteArray EMBLGenbankAbstractDocument::AMINO_STRAND_QVAL_YES("yes");
-const QByteArray EMBLGenbankAbstractDocument::AMINO_STRAND_QVAL_NO("no");
-
 SharedAnnotationData EMBLGenbankAbstractDocument::readAnnotation(IOAdapter* io, char* cbuff, int len, 
                                                                  int READ_BUFF_SIZE, TaskStateInfo& si, int offset) 
 {
     AnnotationData* a = new AnnotationData();
     SharedAnnotationData f(a);
-    a->name = QString::fromAscii(cbuff+5, 15).trimmed();
-    if (a->name.isEmpty()) {
-        si.error = EMBLGenbankAbstractDocument::tr("annotation_name_is_empty");
+    QString key = QString::fromAscii(cbuff+5, 15).trimmed();
+    if (key.isEmpty()) {
+        si.setError(EMBLGenbankAbstractDocument::tr("annotation_name_is_empty"));
         return SharedAnnotationData();
     }
-
+    a->name = key;
+    
     //qualifier starts on offset 22;
     int qlen = len + readMultilineQualifier(io, cbuff+len, READ_BUFF_SIZE - len, true);
     if (qlen < 21) {
-        si.error = EMBLGenbankAbstractDocument::tr("annotation_location_is_empty");
+        si.setError(EMBLGenbankAbstractDocument::tr("annotation_location_is_empty"));
         return SharedAnnotationData();
     }
 
     Genbank::LocationParser::parseLocation(cbuff+21, qlen-21, a->complement, a->location);
     if (a->location.empty()) {
-        si.error = EMBLGenbankAbstractDocument::tr("annotation_location_is_empty");
+        si.setError(EMBLGenbankAbstractDocument::tr("annotation_location_is_empty"));
         return SharedAnnotationData();
     } else if (a->location.size() > 1) {
         qSort(a->location);
@@ -333,7 +393,11 @@ SharedAnnotationData EMBLGenbankAbstractDocument::readAnnotation(IOAdapter* io,
     }
 
     const QBitArray& LINE_BREAKS = TextUtils::LINE_BREAKS;
-
+    const QByteArray& aminoQ = GBFeatureUtils::QUALIFIER_AMINO_STRAND;
+    const QByteArray& aminoQYes = GBFeatureUtils::QUALIFIER_AMINO_STRAND_YES;
+    const QByteArray& aminoQNo = GBFeatureUtils::QUALIFIER_AMINO_STRAND_NO;
+    const QByteArray& nameQ = GBFeatureUtils::QUALIFIER_NAME;
+    
     //here we have valid key and location;
     //reading qualifiers
     bool lineOk = true;
@@ -343,7 +407,7 @@ SharedAnnotationData EMBLGenbankAbstractDocument::readAnnotation(IOAdapter* io,
             break;
         }
         if (!lineOk) {
-            si.error = EMBLGenbankAbstractDocument::tr("line_is_too_long_or_unexpected_oef");
+            si.setError(EMBLGenbankAbstractDocument::tr("line_is_too_long_or_unexpected_oef"));
             break;
         }
         for (; QN_COL < len && LINE_BREAKS[(uchar)cbuff[len-1]]; len--){}; //remove line breaks
@@ -367,12 +431,14 @@ SharedAnnotationData EMBLGenbankAbstractDocument::readAnnotation(IOAdapter* io,
             qval++;
             qvalLen-=2;
         }
-        if (qnameLen == AMINO_STRAND_QNAME.length() && TextUtils::equals(qname, AMINO_STRAND_QNAME.constData(), qnameLen)) {
-            a->aminoStrand = qvalLen == AMINO_STRAND_QVAL_YES.length() && TextUtils::equals(qval, AMINO_STRAND_QVAL_YES.constData(), qvalLen) 
-                ? TriState_Yes : TriState_No;
+        if (qnameLen == aminoQ.length() && TextUtils::equals(qname, aminoQ.constData(), qnameLen)) {
+            a->aminoStrand = qvalLen == aminoQYes.length() && TextUtils::equals(qval, aminoQYes.constData(), qvalLen) ? TriState_Yes
+                         :  (qvalLen == aminoQNo.length()  && TextUtils::equals(qval, aminoQNo.constData(), qvalLen) ? TriState_No : TriState_Unknown);
+        } else if (qnameLen == nameQ.length() && TextUtils::equals(qname, nameQ.constData(), qnameLen)) {
+            a->name = QString::fromLocal8Bit(qval, qvalLen);
         } else {
-            QString nameQStr = QString::fromAscii(qname, qnameLen);
-            QString valQStr = QString::fromAscii(qval, qvalLen);
+            QString nameQStr = QString::fromLocal8Bit(qname, qnameLen);
+            QString valQStr = QString::fromLocal8Bit(qval, qvalLen);
             if (containsDoubleQuotes) {
                 valQStr = valQStr.replace("\"\"", "\"");
             }
@@ -382,35 +448,33 @@ SharedAnnotationData EMBLGenbankAbstractDocument::readAnnotation(IOAdapter* io,
     return f;
 }
 
-void EMBLGenbankAbstractDocument::readSequence(QByteArray& res, IOAdapter* io, TaskStateInfo& si) {
+bool EMBLGenbankAbstractDocument::readSequence(QByteArray& res, ParserState* st) {
+    // FIXME use ParserState instead
+    IOAdapter* io = st->io;
+    TaskStateInfo& si = st->si;
+    si.setStateDesc(tr("Reading sequence %1").arg(st->entry->name));
+    int headerSeqLen = st->entry->seqLen;
+    res.reserve(res.size() + headerSeqLen);
+
     static int READ_BUFF_SIZE = 4096;
     QByteArray readBuffer(READ_BUFF_SIZE, '\0');
     char* buff  = readBuffer.data();
 
     //reading sequence		
-    const QBitArray& LINE_BREAKS = TextUtils::LINE_BREAKS;
-
-    int len = io->readUntil(buff, READ_BUFF_SIZE, TextUtils::LINE_BREAKS, IOAdapter::Term_Include);
-    if (len < 6 || !TextUtils::equals(buff, sequenceStartPrefix.constData(), sequenceStartPrefix.size())) {
-        return;
-    }
     QBuffer writer(&res);
     writer.open( QIODevice::WriteOnly | QIODevice::Append );
     bool ok = true;
+    int len;
     int dataOffset = 0;
     bool numIsPrefix = getFormatId() == BaseDocumentFormats::PLAIN_GENBANK;
-    while (ok && (len = io->readUntil(buff, READ_BUFF_SIZE, LINE_BREAKS, IOAdapter::Term_Include)) > 0) {
+    while (ok && (len = io->readLine(buff, READ_BUFF_SIZE)) > 0) {
         if (si.cancelFlag) {
             res.clear();
             break;
         }
         
-        while(len > 0 && LINE_BREAKS[(uchar)buff[len-1]])  { //remove line breaks from sequence
-            len--;
-        }
-
         if (len <= 0)  {
-            si.error = tr("error_reading_sequence_abnormal_term");
+            si.setError(tr("error_reading_sequence_abnormal_term"));
             break;
         }
 
@@ -437,7 +501,7 @@ void EMBLGenbankAbstractDocument::readSequence(QByteArray& res, IOAdapter* io, T
         }
         
         if (dataOffset == len) {
-            si.error=tr("error_reading_sequence_invalid_format");    
+            si.setError(tr("error_reading_sequence_invalid_format"));    
             break;
         }
         
@@ -452,14 +516,83 @@ void EMBLGenbankAbstractDocument::readSequence(QByteArray& res, IOAdapter* io, T
             }
         }
         if (!ok) {
-            si.error = tr("erorr_reading_sequence_enable_to_add_more_data");
+            si.setError(tr("erorr_reading_sequence_enable_to_add_more_data"));
             break;
         }
 
 		si.progress = io->getProgress();
     }
+    if (!si.hasErrors() && !si.cancelFlag && buff[0] != '/') {
+        si.setError(tr("Sequence is truncated"));
+    }
     writer.close();
+    return true; //FIXME
 }
 
+void EMBLGenbankAbstractDocument::readAnnotations(ParserState* st, int offset) {
+    st->si.setStateDesc(tr("Reading annotations %1").arg(st->entry->name));
+    st->entry->hasAnnotationObjectFlag = true;
+    do {
+        if (st->hasKey("XX") && getFormatId() == BaseDocumentFormats::PLAIN_EMBL) {
+            continue;
+        }
+        int fplen = fPrefix.length();
+        if (st->len >= 6 && TextUtils::equals(fPrefix.data(), st->buff, fplen)) {
+            while (fplen < 5) {
+                if (st->buff[fplen++] != ' ') {
+                    st->si.setError(tr("Invalid format of feature table"));
+                    break;
+                }
+            }
+        } else {
+            // end of feature table
+            break;
+        }
+        //parsing feature;
+        SharedAnnotationData f = readAnnotation(st->io, st->buff, st->len, ParserState::READ_BUFF_SIZE, st->si, offset);
+        st->entry->features.push_back(f);
+    } while (st->readNextLine());
+}
+
+bool ParserState::hasKey( const char* key, int slen ) const {
+    assert(slen <= valOffset);
+    if (slen <= len && TextUtils::equals(key, buff, slen)) {
+        while (slen < qMin(valOffset, len)) {
+            if (buff[slen++] != ' ') {
+                return false;
+            }
+        }
+        return true;
+    }
+    return false;
+}
+
+QString ParserState::value() const {
+    assert (len >= valOffset);
+    return len > valOffset ? QString::fromLocal8Bit(buff + valOffset, len - valOffset) : QString();
+}
+
+bool ParserState::readNextLine(bool emptyOK)
+{
+    if (si.cancelFlag || si.hasErrors()) {
+        len = 0;
+        return false;
+    }
+    bool ok = false;
+    len = io->readLine(buff, READ_BUFF_SIZE, &ok);
+    si.progress = io->getProgress();
+
+    if (!ok && len == READ_BUFF_SIZE) {
+        si.setError(GB2::EMBLGenbankAbstractDocument::tr("Line is too long."));
+    } else if (len == -1) {
+        si.setError(GB2::EMBLGenbankAbstractDocument::tr("IO error."));
+    }
+    return (len > 0 || (emptyOK && ok));
+}
+
+QString ParserState::key() const {   
+    //assert(len > 0);
+    return QString::fromLocal8Bit(buff, qMin(valOffset - 1, len));
+}
 
 }//namespace
