@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -20,15 +20,15 @@ class GB2_COREAPI_EXPORT EMBLPlainTextFormat : public EMBLGenbankAbstractDocumen
 public:
     EMBLPlainTextFormat(QObject* p);
 
-    virtual void storeDocument(Document* d, TaskStateInfo& ts, IOAdapterFactory* io = NULL, const QString& newDocURL = QString::null);
     virtual bool isDataFormatSupported(const char* data, int size) const;
     virtual bool checkConstraints(const DocumentFormatConstraints& c) const;
 
 protected:
 
-    virtual EMBLGenbankAbstractIDLine*   readIdLine(const QString& line, TaskStateInfo& si);
-    virtual EMBLGenbankAbstractHeader*	 readHeader(IOAdapter* io, TaskStateInfo& si);
-    virtual QList<SharedAnnotationData>	 readAnnotations(IOAdapter* io, TaskStateInfo& si, int offset);
+    bool readIdLine(ParserState*);
+    bool readEntry(QByteArray&, ParserState*);
+    //void readAnnotations(ParserState*, int offset);
+    QMap<QString, QString> tagMap;
 };
 
 
