@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -21,15 +21,16 @@ public:
 	GenbankPlainTextFormat(QObject* p);
 
     virtual void storeDocument(Document* d, TaskStateInfo& ts, IOAdapterFactory* io = NULL, const QString& newDocURL = QString::null);
-
+    virtual void storeDocument( Document* d, TaskStateInfo& ts, IOAdapter* io );
+    
     virtual bool isDataFormatSupported(const char* data, int size) const;
 
 protected:
     virtual void save(IOAdapter* io, Document* doc, TaskStateInfo& si);
 
-    virtual EMBLGenbankAbstractIDLine*   readIdLine(const QString& line, TaskStateInfo& si);
-    virtual EMBLGenbankAbstractHeader*	 readHeader(IOAdapter* io, TaskStateInfo& si);
-    virtual QList<SharedAnnotationData>	 readAnnotations(IOAdapter* io, TaskStateInfo& si, int offset);
+    bool readIdLine(ParserState*);
+    bool readEntry(QByteArray&, ParserState*);
+    //void readAnnotations(ParserState*, int offset);
 };
 
 
