@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -13,7 +13,9 @@
 namespace GB2 {
 
 GObject* PhyTreeObject::clone() const {
-	return new PhyTreeObject(tree, getGObjectName(), getGHintsMap());
+	PhyTreeObject* cln = new PhyTreeObject(tree, getGObjectName(), getGHintsMap());
+    cln->setIndexInfo(getIndexInfo());
+    return cln;
 }
 
 
