@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -18,7 +18,9 @@ void TextObject::setText(const QString& newText) {
 }
 
 GObject* TextObject::clone() const {
-	return new TextObject(text, getGObjectName(), getGHintsMap());
+	TextObject* cln = new TextObject(text, getGObjectName(), getGHintsMap());
+    cln->setIndexInfo(getIndexInfo());
+    return cln;
 }
 
 
