@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -26,6 +26,9 @@
 #include <util_gui/CreateAnnotationWidgetController.h>
 
 #include <util_ov_annotated_dna/ADVSequenceObjectContext.h>
+#include <util_ov_annotated_dna/ADVSingleSequenceWidget.h>
+#include <util_ov_annotated_dna/AnnotatedDNAView.h>
+
 #include <util_tasks/CreateAnnotationTask.h>
 #include <util_text/TextUtils.h>
 
@@ -40,6 +43,7 @@ namespace GB2 {
 /* TRANSLATOR GB2::FindDialog */ 
 
 class FRListItem : public QListWidgetItem {
+
 public:
     FRListItem(const FindAlgorithmResult& r);
     FindAlgorithmResult res;
@@ -63,7 +67,7 @@ bool FindDialog::runDialog(ADVSequenceObjectContext* ctx) {
     return true;
 }
 
-FindDialog::FindDialog(ADVSequenceObjectContext* context) {
+FindDialog::FindDialog(ADVSequenceObjectContext* context): QDialog(context->getAnnotatedDNAView()->getWidget()) {
     setupUi(this);
 
     ctx = context;
@@ -210,12 +214,14 @@ void FindDialog::sl_onSaveAnnotations() {
     CreateAnnotationModel m;
     m.sequenceObjectRef = ctx->getSequenceObject();
     m.hideLocation = true;
-    CreateAnnotationDialog d(&m);
+    m.sequenceLen = ctx->getSequenceObject()->getSequenceLen();
+    CreateAnnotationDialog d(this, m);
     int rc = d.exec();
     if (rc != QDialog::Accepted) {
         return;
     }
-    assert(m.annotationObject!=NULL);
+    AnnotationTableObject* aobj = m.getAnnotationObject();
+    assert(aobj!=NULL);
     const QString& name = m.data->name;
     QList<SharedAnnotationData> list;
     for (int i=0, n = lbResult->count(); i<n; ++i) {
@@ -223,7 +229,7 @@ void FindDialog::sl_onSaveAnnotations() {
         list.append(item->res.toAnnotation(name));
     }
 
-    CreateAnnotationsTask* t = new CreateAnnotationsTask(m.annotationObject, m.groupName, list);
+    CreateAnnotationsTask* t = new CreateAnnotationsTask(aobj, m.groupName, list);
     AppContext::getTaskScheduler()->registerTopLevelTask(t);
 }
 
@@ -529,6 +535,14 @@ void FindDialog::sl_onResultActivated(QListWidgetItem* i) {
     sel->clear();
     sel->addRegion(item->res.region);
     sbCurrentPos->setValue(item->res.region.startPos+1);
+    ADVSingleSequenceWidget* sw = qobject_cast<ADVSingleSequenceWidget*>(ctx->getAnnotatedDNAView()->getSequenceWidgetInFocus());
+    if (sw!=NULL && sw->getSequenceContext() == ctx)  {
+        if (item->res.complement) {
+            sw->centerPosition(item->res.region.endPos());
+        } else {
+            sw->centerPosition(item->res.region.startPos);
+        }
+    }
     //TODO: add complement info to selection!!
 }
 
