@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -28,11 +28,16 @@ class GObject;
 class GB2_COREAPI_EXPORT ProjectTreeItemSelectorDialog: public QDialog, public Ui::ProjectTreeItemSelectorDialogBase {
     Q_OBJECT
 public:
-    static QList<Document*> selectDocuments(const ProjectTreeControllerModeSettings& s);
-    static QList<GObject*> selectObjects(const ProjectTreeControllerModeSettings& s);
+    static QList<Document*> selectDocuments(const ProjectTreeControllerModeSettings& s, QWidget* p);
+    static QList<GObject*> selectObjects(const ProjectTreeControllerModeSettings& s, QWidget* p);
+
+    bool acceptByDoubleClick;
+
+private slots:
+    void sl_objectClicked(GObject* obj);
 
 private:
-    ProjectTreeItemSelectorDialog(const ProjectTreeControllerModeSettings& s);
+    ProjectTreeItemSelectorDialog(QWidget* p, const ProjectTreeControllerModeSettings& s);
     ~ProjectTreeItemSelectorDialog();
     ProjectTreeController* controller;
     
