@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -13,27 +13,68 @@
 #include "AnnotatedDNAView.h"
 
 #include <gobjects/DNASequenceObject.h>
+#include <gobjects/AnnotationTableObject.h>
+
 #include <selection/DNASequenceSelection.h>
+#include <selection/AnnotationSelection.h>
 
 #include <util_gui/RangeSelector.h>
 #include <util_gui/PositionSelector.h>
 #include <util_gui/GUIUtils.h>
-
+#include <util_gui/DialogUtils.h>
 
 #include "ADVConstants.h"
 #include "DetView.h"
 #include "PanView.h"
+#include "CreateRulerDialogController.h"
 
 #include <QtGui/QApplication>
 #include <QtGui/QToolButton>
 #include <QtGui/QWidgetAction>
 #include <QtGui/QDialog>
+#include <QtGui/QFileDialog>
 
 namespace GB2 {
 #define ADV_HEADER_HEIGHT 24
+#define IMAGE_DIR   "image"
 
 ADVSingleSequenceWidget::ADVSingleSequenceWidget(ADVSequenceObjectContext* seqCtx, AnnotatedDNAView* ctx) : ADVSequenceWidget(ctx) {
     seqContexts.append(seqCtx);
+
+    toggleViewAction = new QAction(this);
+    connect(toggleViewAction, SIGNAL(triggered()), SLOT(sl_toggleView()));
+
+    togglePanViewAction = new QAction(this);
+    connect(togglePanViewAction, SIGNAL(triggered()), SLOT(sl_togglePanView()));
+
+    toggleDetViewAction = new QAction(this);
+    connect(toggleDetViewAction, SIGNAL(triggered()), SLOT(sl_toggleDetView()));
+
+    connect(seqCtx->getAnnotatedDNAView()->getAnnotationsSelection(), 
+        SIGNAL(si_selectionChanged(AnnotationSelection* , const QList<Annotation*>&, const QList<Annotation*>&)),
+        SLOT(sl_onAnnotationSelectionChanged(AnnotationSelection* , const QList<Annotation*>&, const QList<Annotation*>&)));
+
+    selectRangeAction1 = new QAction(QIcon(":/core/images/select_region.png"), tr("Select sequence region.."), this);
+    selectRangeAction1->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_A));
+    selectRangeAction1->setObjectName("select_range_action");
+    selectRangeAction1->setShortcutContext(Qt::WidgetShortcut);
+    connect(selectRangeAction1, SIGNAL(triggered()), SLOT(sl_onSelectRange()));
+
+    selectRangeAction2 = new QAction(QIcon(":/core/images/select_region.png"), tr("Sequence region.."), this);
+    connect(selectRangeAction2, SIGNAL(triggered()), SLOT(sl_onSelectRange()));
+
+    selectInAnnotationRageAction= new QAction(tr("Sequence between selected annotations"), this);
+    connect(selectInAnnotationRageAction, SIGNAL(triggered()), SLOT(sl_onSelectInRange()));
+    
+    selectOutAnnotationRangeAction= new QAction(tr("Sequence around selected annotations"), this);
+    connect(selectOutAnnotationRangeAction, SIGNAL(triggered()), SLOT(sl_onSelectOutRange()));
+
+    zoomToRangeAction = new QAction(QIcon(":/core/images/zoom_reg.png"), tr("range_zoom"), this);
+    connect(zoomToRangeAction, SIGNAL(triggered()), SLOT(sl_zoomToRange()));
+
+    createNewRulerAction = new QAction(tr("Create new ruler.."), this);
+    connect(createNewRulerAction, SIGNAL(triggered()), SLOT(sl_createCustomRuler()));
+
     linesLayout = new QVBoxLayout();
     linesLayout->setMargin(0);
     linesLayout->setSpacing(0);
@@ -43,36 +84,17 @@ ADVSingleSequenceWidget::ADVSingleSequenceWidget(ADVSequenceObjectContext* seqCt
     linesLayout->addWidget(headerWidget);
 
 
-    toggleViewAction = new QAction(tr("show_view"), this);
-    toggleViewAction->setCheckable(true);
-    toggleViewAction->setChecked(true);
-    connect(toggleViewAction, SIGNAL(triggered(bool)), SLOT(sl_toggleView(bool)));
-
-    togglePanViewAction = new QAction(tr("show_panview"), this);
-    togglePanViewAction->setCheckable(true);
-    togglePanViewAction->setChecked(true);
-    connect(togglePanViewAction, SIGNAL(triggered(bool)), SLOT(sl_togglePanView(bool)));
-
-    toggleDetViewAction = new QAction(tr("show_detview"), this);
-    toggleDetViewAction->setCheckable(true);
-    toggleDetViewAction->setChecked(true);
-    connect(toggleDetViewAction, SIGNAL(triggered(bool)), SLOT(sl_toggleDetView(bool)));
-
-    selectRangeAction = new QAction(QIcon(":/core/images/select_region.png"), tr("range_selection"), this);
-    connect(selectRangeAction, SIGNAL(triggered()), SLOT(sl_onSelectRange()));
-
-    zoomToRangeAction = new QAction(QIcon(":/core/images/zoom_reg.png"), tr("range_zoom"), this);
-    connect(zoomToRangeAction, SIGNAL(triggered()), SLOT(sl_zoomToRange()));
-
     init();
 }
 
 void ADVSingleSequenceWidget::init() {
     ADVSequenceObjectContext* seqCtx = getSequenceContext();
     detView = new DetView(this, seqCtx);
+    detView->setObjectName("det_view");
     addSequenceView(detView);
 
     panView = new PanView(this, seqCtx);
+    panView->setObjectName("pan_view");
     connect(panView, SIGNAL(si_centerPosition(int)), SLOT(sl_onLocalCenteringRequest(int)));
 
     addSequenceView(panView);
@@ -91,7 +113,7 @@ void ADVSingleSequenceWidget::init() {
         hBar->addWidget(tt);
         tbMenues.append(ttMenu);
     }
-    hBar->addAction(selectRangeAction);
+    hBar->addAction(selectRangeAction1);
     hBar->addSeparator();
 
     QAction* showComplementAction = detView->getShowComplementAction();
@@ -104,11 +126,18 @@ void ADVSingleSequenceWidget::init() {
         hBar->addSeparator();
     }
 
+    QAction* shotScreenAction = new QAction(QIcon(":/core/images/cam2.png"), tr("Capture screen"), this);
+    shotScreenAction->setObjectName("capture_screen");
+    connect(shotScreenAction, SIGNAL(triggered()), this, SLOT(sl_saveScreenshot()));
+    hBar->addAction(shotScreenAction);
+
     hBar->addAction(panView->getZoomInAction());
     hBar->addAction(panView->getZoomOutAction());
-    hBar->addAction(panView->getZoomToSelectionAction());
+    //hBar->addAction(panView->getZoomToSelectionAction());
     hBar->addAction(zoomToRangeAction);
     hBar->addAction(panView->getZoomToSequenceAction());
+
+    updateSelectionActions();
 }
 
 
@@ -118,40 +147,34 @@ ADVSingleSequenceWidget::~ADVSingleSequenceWidget() {
     }
 }
 
-void ADVSingleSequenceWidget::setViewCollapsed(bool v) {
-    toggleViewAction->disconnect(this);
-    toggleViewAction->setChecked(!v);
-    connect(toggleViewAction, SIGNAL(triggered(bool)), SLOT(sl_toggleView(bool)));
+bool ADVSingleSequenceWidget::isPanViewCollapsed() const {return panView->isHidden();}
+bool ADVSingleSequenceWidget::isDetViewCollapsed() const {return detView->isHidden();}
 
+bool ADVSingleSequenceWidget::isViewCollapsed() const {
+    bool collapsed = true;
     foreach(GSequenceLineView* seqView, lineViews) {
-        bool visible = !isViewCollapsed();
-        if (seqView == panView) {
-            visible = visible && !isPanViewCollapsed();
-        } else if (seqView == detView) {
-            visible = visible && !isDetViewCollapsed();
+        if (seqView->isVisible()) {
+            collapsed = false;
+            break;
         }
-        seqView->setVisible(visible);
+    }
+    return collapsed;
+}
+
+void ADVSingleSequenceWidget::setViewCollapsed(bool v) {
+    foreach(GSequenceLineView* seqView, lineViews) {
+        seqView->setHidden(v);
     }
     updateMinMaxHeight();
 }
 
 void ADVSingleSequenceWidget::setPanViewCollapsed(bool v) {
-    togglePanViewAction->disconnect(this);
-    togglePanViewAction->setChecked(!v);
-    connect(togglePanViewAction, SIGNAL(triggered(bool)), SLOT(sl_togglePanView(bool)));
-
-    bool visible = !isViewCollapsed() && !isPanViewCollapsed();
-    panView->setVisible(visible);
+    panView->setHidden(v);
     updateMinMaxHeight();
 }
 
 void ADVSingleSequenceWidget::setDetViewCollapsed(bool v) {
-    toggleDetViewAction->disconnect(this);
-    toggleDetViewAction->setChecked(!v);
-    connect(toggleDetViewAction, SIGNAL(triggered(bool)), SLOT(sl_toggleDetView(bool)));
-
-    bool visible = !isViewCollapsed() && !isDetViewCollapsed();
-    detView->setVisible(visible);
+    detView->setHidden(v);
     updateMinMaxHeight();
 }
 
@@ -178,6 +201,23 @@ void ADVSingleSequenceWidget::removeSequenceView(GSequenceLineView* v, bool dele
     updateMinMaxHeight();
 }
 
+LRegion ADVSingleSequenceWidget::getVisibleRange() const {
+    return panView->getVisibleRange();
+}
+
+void ADVSingleSequenceWidget::setVisibleRange(const LRegion& r) {
+    return panView->setVisibleRange(r);
+}
+
+int ADVSingleSequenceWidget::getNumBasesVisible() const {
+    return panView->getVisibleRange().len;
+}
+
+void ADVSingleSequenceWidget::setNumBasesVisible(int n) {
+    panView->setNumBasesVisible(n);
+}
+
+
 void ADVSingleSequenceWidget::sl_onViewDestroyed(QObject* o) {
     GSequenceLineView* v = (GSequenceLineView*)o;
     bool r = lineViews.removeOne(v);
@@ -281,13 +321,57 @@ GSequenceLineView* ADVSingleSequenceWidget::getDetGSLView() const {
 
 void ADVSingleSequenceWidget::buildPopupMenu(QMenu& m) {
     m.insertAction(GUIUtils::findActionAfter(m.actions(), ADV_GOTO_ACTION), getAnnotatedDNAView()->getCreateAnnotationAction());
-    m.insertAction(GUIUtils::findActionAfter(m.actions(), ADV_GOTO_ACTION), selectRangeAction);
+    m.insertAction(GUIUtils::findActionAfter(m.actions(), ADV_GOTO_ACTION), selectRangeAction1);
+
+    addSelectMenu(m);
+
+    if (panView->isVisible()) {
+        addRulersMenu(m);
+    }
+
     ADVSequenceWidget::buildPopupMenu(m);
     foreach(GSequenceLineView* v, lineViews) {
         v->buildPopupMenu(m);
     }
 }
 
+void ADVSingleSequenceWidget::addSelectMenu(QMenu& m) {
+    QMenu* selectMenu = new QMenu(tr("Select"),  &m);
+    
+    selectMenu->addAction(selectRangeAction2);
+    selectMenu->addAction(selectInAnnotationRageAction);
+    selectMenu->addAction(selectOutAnnotationRangeAction);
+
+    QAction* aBefore = GUIUtils::findActionAfter(m.actions(), ADV_MENU_COPY);
+    m.insertMenu(aBefore, selectMenu);
+}
+
+void ADVSingleSequenceWidget::addRulersMenu(QMenu& m) {
+    qDeleteAll(rulerActions.qlist);
+    rulerActions.qlist.clear();
+
+    QMenu* rulersM = new QMenu(tr("Rulers.."),  &m);
+    rulersM->setIcon(QIcon(":core/images/ruler.png"));
+    
+    rulersM->addAction(createNewRulerAction);
+    rulersM->addSeparator();
+    rulersM->addAction(panView->getToggleMainRulerAction());
+    rulersM->addAction(panView->getToggleCustomRulersAction());
+    rulersM->addSeparator();
+
+    foreach(const RulerInfo& ri, panView->getCustomRulers()) {
+        QAction* rulerAction = new QAction(tr("Remove '%1'").arg(ri.name), this);
+        rulerAction->setData(ri.name);
+        connect(rulerAction, SIGNAL(triggered()), SLOT(sl_removeCustomRuler()));
+        rulerActions.qlist.append(rulerAction);
+    }
+    rulersM->addActions(rulerActions.qlist);
+
+    QAction* aBefore = GUIUtils::findActionAfter(m.actions(), ADV_MENU_SECTION2_SEP);
+    m.insertMenu(aBefore, rulersM);
+    m.insertSeparator(aBefore)->setObjectName("SECOND_SEP");
+}
+
 bool ADVSingleSequenceWidget::isWidgetOnlyObject(GObject* o) const {
     foreach(GSequenceLineView* v, lineViews) {
         bool ok = v->isWidgetOnlyObject(o);
@@ -338,19 +422,60 @@ void ADVSingleSequenceWidget::sl_onSelectRange() {
     dlg.setModal(true);
     dlg.setWindowTitle(tr("selection_range_title"));
     ADVSequenceObjectContext* ctx = getSequenceContext();
-    RangeSelector* rs = new RangeSelector(&dlg, 1, ctx->getSequenceLen(), true);
+    RangeSelector rs(&dlg, 1, ctx->getSequenceLen(), true);
     int rc = dlg.exec();
     if (rc == QDialog::Accepted) {
-        LRegion r(rs->getStart() - 1, rs->getEnd() - rs->getStart() + 1);
+        LRegion r(rs.getStart() - 1, rs.getEnd() - rs.getStart() + 1);
         ctx->getSequenceSelection()->clear();
         getSequenceSelection()->addRegion(r);
         if (!detView->getVisibleRange().intersects(r)) {
             detView->setCenterPos(r.startPos);
         }
     }
-    delete rs;
 }
 
+QList<LRegion> ADVSingleSequenceWidget::getSelectedAnnotationRegions(int max) {
+    ADVSequenceObjectContext* seqCtx = getSequenceContext();
+    const QList<AnnotationSelectionData> selection = seqCtx->getAnnotatedDNAView()->getAnnotationsSelection()->getSelection();
+    const QSet<AnnotationTableObject*> myAnns = seqCtx->getAnnotationObjects();
+
+    QList<LRegion> res;
+    foreach(const AnnotationSelectionData& sd, selection) {
+        AnnotationTableObject* aObj = sd.annotation->getGObject();
+        if (myAnns.contains(aObj)) {
+            res << sd.getSelectedRegions();
+            if (max > 0 && res.size() >= max) {
+                break;
+            }
+        }
+    }
+    return res;
+}
+
+void ADVSingleSequenceWidget::sl_onSelectInRange() {
+    QList<LRegion> selRegs = getSelectedAnnotationRegions(3);
+    assert(selRegs.size() == 2);
+    
+    const LRegion& r1 = selRegs.at(0);
+    const LRegion& r2 = selRegs.at(1);
+    assert(!r1.intersects(r2));
+    
+    LRegion r;
+    r.startPos = qMin(r1.endPos(), r2.endPos());
+    r.len = qMax(r1.startPos, r2.startPos) - r.startPos;
+    
+    getSequenceContext()->getSequenceSelection()->clear();
+    getSequenceContext()->getSequenceSelection()->addRegion(r);
+}
+
+void ADVSingleSequenceWidget::sl_onSelectOutRange() {
+    QList<LRegion> selRegs = getSelectedAnnotationRegions(0);
+    assert(!selRegs.isEmpty());
+    LRegion r = LRegion::containingRegion(selRegs);
+
+    getSequenceContext()->getSequenceSelection()->clear();
+    getSequenceContext()->getSequenceSelection()->addRegion(r);
+}
 
 void ADVSingleSequenceWidget::sl_zoomToRange() {
     QDialog dlg;
@@ -367,9 +492,16 @@ void ADVSingleSequenceWidget::sl_zoomToRange() {
     delete rs;
 }
 
-#define SPLITTER_STATE_MAP_NAME  "ADVSI_MAP"
-#define PAN_REG_NAME    "PAN_REG"
-#define DET_POS_NAME    "DET_POS"
+#define SPLITTER_STATE_MAP_NAME "ADVSI_MAP"
+#define PAN_REG_NAME            "PAN_REG"
+#define DET_POS_NAME            "DET_POS"
+#define PAN_VISIBLE             "PAN_VISIBLE"
+#define DET_VISIBLE             "DET_VISIBLE"
+#define MAIN_RULER_VISIBLE      "MAINR_VISIBLE"
+#define CUSTOM_RULERS_VISIBLE   "CUSTOMR_VISIBLE"
+#define CUSTOM_R_NAMES          "CUSTOMR_NAMES"
+#define CUSTOM_R_COLORS         "CUSTOMR_COLORS"
+#define CUSTOM_R_OFFSETS        "CUSTOMR_OFFSETS"
 
 void ADVSingleSequenceWidget::updateState(const QVariantMap& m) {
     QVariantMap map = m.value(SPLITTER_STATE_MAP_NAME).toMap();
@@ -384,6 +516,31 @@ void ADVSingleSequenceWidget::updateState(const QVariantMap& m) {
     if (!panReg.isEmpty() && seqRange.contains(panReg)) {
         panView->setVisibleRange(panReg);
     }
+    
+    bool panIsVisible = myData.value(PAN_VISIBLE, true).toBool();
+    setPanViewCollapsed(!panIsVisible);
+
+    bool detIsVisible = myData.value(DET_VISIBLE, true).toBool();
+    setDetViewCollapsed(!detIsVisible);
+
+    bool mainRulerVisible = myData.value(MAIN_RULER_VISIBLE, true).toBool();
+    panView->getToggleMainRulerAction()->setChecked(mainRulerVisible);
+
+    bool customRulersVisible = myData.value(CUSTOM_RULERS_VISIBLE, true).toBool();
+    panView->getToggleCustomRulersAction()->setChecked(customRulersVisible);
+
+    QStringList rnames = myData[CUSTOM_R_NAMES].toStringList();
+    QList<QVariant> rcolors = myData[CUSTOM_R_COLORS].toList();
+    QList<QVariant> roffsets = myData[CUSTOM_R_OFFSETS].toList();
+    if (rnames.count() == rcolors.count() && rnames.count() == roffsets.count()) {
+        panView->removeAllCustomRulers();
+        for (int i=0; i< rnames.count(); i++) {
+            QString name = rnames[i];
+            int offset = roffsets[i].toInt();
+            QColor color = rcolors[i].value<QColor>();
+            panView->addCustomRuler(RulerInfo(name, offset, color));
+        }
+    }
 }
 
 void ADVSingleSequenceWidget::saveState(QVariantMap& m) {
@@ -392,10 +549,56 @@ void ADVSingleSequenceWidget::saveState(QVariantMap& m) {
     QVariantMap myData;
     myData[PAN_REG_NAME] = QVariant::fromValue<LRegion>(panView->getVisibleRange());
     myData[DET_POS_NAME] = QVariant::fromValue<int>(detView->getVisibleRange().startPos);
+    myData[PAN_VISIBLE] = !isPanViewCollapsed();
+    myData[DET_VISIBLE] = !isDetViewCollapsed();
+    myData[MAIN_RULER_VISIBLE] = panView->getToggleMainRulerAction()->isChecked();
+    myData[CUSTOM_RULERS_VISIBLE] = panView->getToggleCustomRulersAction()->isChecked();
+    
+    QStringList rnames;
+    QList<QVariant> roffsets;
+    QList<QVariant> rcolors;
+    foreach(const RulerInfo& ri, panView->getCustomRulers()) {
+        rnames.append(ri.name);
+        roffsets.append(ri.offset);
+        rcolors.append(ri.color);
+    }
+    myData[CUSTOM_R_NAMES] = rnames;
+    myData[CUSTOM_R_OFFSETS] = roffsets;
+    myData[CUSTOM_R_COLORS] = rcolors;
+    
     map[getActiveSequenceContext()->getSequenceObject()->getGObjectName()] = myData;
     m[SPLITTER_STATE_MAP_NAME] = map;
 }
 
+// QT 4.5.0 bug workaround
+void ADVSingleSequenceWidget::sl_closeView()
+{
+    closeView();
+}
+void ADVSingleSequenceWidget::sl_saveScreenshot()
+{
+    LastOpenDirHelper lod(IMAGE_DIR);
+    QRect* screenRect = new QRect(linesLayout->itemAt(1)->geometry().topLeft().x(),
+                                  linesLayout->itemAt(1)->geometry().topLeft().y(),
+                                  this->geometry().bottomRight().x(),
+                                  this->geometry().bottomRight().y());
+    QPixmap curPixMap = QPixmap::grabWidget(this, *screenRect);
+    QStringList formats = (QStringList() << "png" << "jpg" << "tiff");
+    QString initialPath = lod.dir + "/untitled";//QDir::currentPath() + tr("/untitled");
+    QString fileName = QFileDialog::getSaveFileName(this, tr("Save As"),
+        initialPath,
+        QString("%1 Files (*.%2);;%3 Files (*.%4);;%5 Files (*.%6);;All Files (*)")
+        .arg(formats.at(0).toUpper())
+        .arg(formats.at(0))
+        .arg(formats.at(1).toUpper())
+        .arg(formats.at(1))
+        .arg(formats.at(2).toUpper())
+        .arg(formats.at(2)));
+    lod.url = fileName;
+    if (!fileName.isEmpty()) {
+        curPixMap.save(fileName);
+    };
+}
 
 void ADVSingleSequenceWidget::closeView() {
     DNASequenceObject* dnaObj = getSequenceObject();
@@ -403,6 +606,47 @@ void ADVSingleSequenceWidget::closeView() {
     v->removeObject(dnaObj);
 }
 
+void ADVSingleSequenceWidget::sl_createCustomRuler() {
+    QSet<QString> namesToFilter;
+    foreach(const RulerInfo& ri, panView->getCustomRulers()) {
+        namesToFilter.insert(ri.name);
+    }
+    CreateRulerDialogController d(namesToFilter, getSequenceObject()->getSequenceRange(), panView->getVisibleRange().center());
+    int rc = d.exec();
+    if (rc != QDialog::Accepted) {
+        return;
+    }
+    RulerInfo ri(d.name, d.offset, d.color);
+    panView->addCustomRuler(ri);
+}
+
+void ADVSingleSequenceWidget::sl_removeCustomRuler() {
+    QString rulerName = qobject_cast<QAction*>(sender())->data().toString();
+    panView->removeCustomRuler(rulerName);
+}
+
+void ADVSingleSequenceWidget::sl_onAnnotationSelectionChanged(AnnotationSelection* s, const QList<Annotation*>& added, const QList<Annotation*>& removed) {
+    Q_UNUSED(s); Q_UNUSED(added); Q_UNUSED(removed);
+    updateSelectionActions();
+}
+
+void ADVSingleSequenceWidget::updateSelectionActions() {
+    QList<LRegion> selRegs = getSelectedAnnotationRegions(3);
+
+    selectInAnnotationRageAction->setEnabled(selRegs.size() == 2 && !selRegs[0].intersects(selRegs[1]));
+    selectOutAnnotationRangeAction->setEnabled(!selRegs.isEmpty());
+}
+
+void ADVSingleSequenceWidget::addStateActions( QMenu& m )
+{
+    toggleViewAction->setText(isViewCollapsed() ? tr("Show all") : tr("Hide all"));
+    togglePanViewAction->setText(isPanViewCollapsed() ? tr("Show overview") : tr("Hide overview"));
+    toggleDetViewAction->setText(isDetViewCollapsed() ? tr("Show details") : tr("Hide details"));
+    m.addAction(toggleViewAction);
+    m.addAction(toggleDetViewAction);
+    m.addAction(togglePanViewAction);
+}
+
 //////////////////////////////////////////////////////////////////////////
 // header
 
@@ -443,16 +687,15 @@ ADVSingleSequenceHeaderWidget::ADVSingleSequenceHeaderWidget(ADVSingleSequenceWi
     pixLabel->setToolTip(objName);
     pixLabel->installEventFilter(this);
     
-    int labelWidth = 200;
-    QString labelStr = objName;
+    int labelWidth = 50;
     QFontMetrics fm(f);
-    while(fm.width(labelStr) > labelWidth && labelStr.length() > 3) {
-        labelStr = labelStr.left(labelStr.length()-3) + "..";
-    }
-    nameLabel = new QLabel(labelStr, this);
-    nameLabel->setFixedWidth(qMin(labelWidth, fm.width(labelStr)));
+    nameLabel = new QLabel(objName, this);
+    nameLabel->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
+    nameLabel->setMinimumWidth(labelWidth);
+    nameLabel->setMaximumWidth(fm.width(objName));
     nameLabel->setFont(f);
     nameLabel->setToolTip(objName);
+    nameLabel->setTextInteractionFlags(Qt::TextSelectableByMouse);
 
     toolBar = new HBar(this);
     toolBar->layout()->setSpacing(0);
@@ -518,6 +761,7 @@ void ADVSingleSequenceHeaderWidget::updateActiveState() {
     bool focused = ctx->getAnnotatedDNAView()->getSequenceWidgetInFocus() == ctx;
     nameLabel->setEnabled(focused);
     pixLabel->setEnabled(focused);
+    ctx->getSelectRangeAction()->setShortcutContext(focused ? Qt::WindowShortcut : Qt::WidgetShortcut);
     //toolBar->setEnabled(focused); TODO: click on disabled buttons does not switch focus!
 }
 
@@ -525,9 +769,7 @@ void ADVSingleSequenceHeaderWidget::sl_showStateMenu() {
     QPointer<QToolButton> widgetStateMenuButtonPtr(widgetStateMenuButton);
 
     QMenu m;
-    m.addAction(ctx->getToggleViewAction());
-    m.addAction(ctx->getTogglePanViewAction());
-    m.addAction(ctx->getToggleDetViewAction());
+    ctx->addStateActions(m);
     m.addAction(closeViewAction);
     m.exec(QCursor::pos());
 
@@ -538,11 +780,22 @@ void ADVSingleSequenceHeaderWidget::sl_showStateMenu() {
 
 
 void ADVSingleSequenceHeaderWidget::sl_closeView() {
-    ctx->closeView();
+#ifdef Q_OS_LINUX
+    if(QString("4.5.0") == qVersion())
+    {
+        QTimer::singleShot(0,ctx,SLOT(sl_closeView()));
+    }
+    else
+    {
+#endif // Q_OS_LINUX
+        ctx->closeView();
+#ifdef Q_OS_LINUX
+    }
+#endif // Q_OS_LINUX
 }
 
 void ADVSingleSequenceHeaderWidget::mouseDoubleClickEvent(QMouseEvent *e) {
-    ctx->getToggleViewAction()->trigger();
+    ctx->toggleViewAction->trigger();
     QWidget::mouseDoubleClickEvent(e);
 }
 
