@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -12,6 +12,7 @@
 #include "AnnotatedDNAViewState.h"
 #include "AnnotatedDNAViewFactory.h"
 #include "AnnotatedDNAViewTasks.h"
+#include "ADVUtils.h"
 
 #include "AnnotationsTreeView.h"
 #include "ADVAnnotationCreation.h"
@@ -21,12 +22,16 @@
 #include "ADVSequenceObjectContext.h"
 #include "ADVSingleSequenceWidget.h"
 #include "ADVSplitWidget.h"
+#include "ADVSyncViewManager.h"
 
 #include <core_api/AppContext.h>
 #include <core_api/Log.h>
 #include <core_api/ProjectModel.h>
 #include <core_api/Task.h>
 #include <core_api/DocumentModel.h>
+#include <core_api/DocumentFormats.h>
+#include <core_api/Timer.h>
+#include <core_api/IOAdapter.h>
 
 #include <gobjects/DNASequenceObject.h>
 #include <gobjects/AnnotationTableObject.h>
@@ -40,11 +45,15 @@
 #include <selection/DNASequenceSelection.h>
 
 #include <util_find_dialog/FindDialog.h>//BUG:423: move to plugins!?
+#include <util_sec_struct_predict/SecStructPredictUtils.h>
 
 #include <util_gui/AnnotationSettingsDialogController.h>
 #include <util_gui/GUIUtils.h>
 #include <util_gui/CreateObjectRelationDialogController.h>
 #include <util_gui/PositionSelector.h>
+#include <util_gui/DialogUtils.h>
+#include <util_gui/AddPartToSequenceDialogController.h>
+#include <util_gui/RemovePartFromSequenceDialogController.h>
 
 #include <QtGui/QVBoxLayout>
 #include <QtGui/QScrollArea>
@@ -72,13 +81,15 @@ AnnotatedDNAView::AnnotatedDNAView(const QString& viewName, const QList<DNASeque
     clipb = NULL;
     aliSupport = NULL;
     
+    mainSplitter = NULL;
+    scrollArea = NULL;
+    posSelector = NULL;
+    posSelectorWidgetAction = NULL;
+    annotationsView = NULL;
+    focusedWidget = NULL;
+
     createAnnotationAction = (new ADVAnnotationCreation(this))->getCreateAnnotationAction();
     
-    findDialogAction = new QAction(QIcon(":core/images/find_dialog.png"), tr("find"), this);
-    findDialogAction->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_F));
-    findDialogAction->setShortcutContext(Qt::WindowShortcut);
-    connect(findDialogAction, SIGNAL(triggered()), SLOT(sl_onFindDialog()));
-
     annotationSettingsAction  = new QAction(QIcon(":core/images/annotation_settings.png"), tr("annotation_settings"), this);
     connect(annotationSettingsAction, SIGNAL(triggered()), SLOT(sl_onAnnotationSettings()));
 
@@ -91,22 +102,33 @@ AnnotatedDNAView::AnnotatedDNAView(const QString& viewName, const QList<DNASeque
     toggleHLAction = new QAction("", this);
     connect(toggleHLAction, SIGNAL(triggered()), SLOT(sl_toggleHL()));
 
-    mainSplitter = NULL;
-    scrollArea = NULL;
-    posSelector = NULL;
-    posSelectorWidgetAction = NULL;
-    annotationsView = NULL;
-    focusedWidget = NULL;
+    syncViewManager = new ADVSyncViewManager(this);
     
     foreach(DNASequenceObject* dnaObj, dnaObjects) {
         addObject(dnaObj);
     }
+
+    findDialogAction = new ADVGlobalAction(this, QIcon(":core/images/find_dialog.png"), tr("find"), 10);
+    findDialogAction->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_F));
+    findDialogAction->setShortcutContext(Qt::WindowShortcut);
+    connect(findDialogAction, SIGNAL(triggered()), SLOT(sl_onFindDialog()));
+
+    addSequencePart = new QAction(tr("Add part of sequence"), this);
+    connect(addSequencePart, SIGNAL(triggered()), this, SLOT(sl_addSequencePart()));
+
+    removeSequencePart = new QAction(tr("Remove selected part of sequence"), this);
+    connect(removeSequencePart, SIGNAL(triggered()), this, SLOT(sl_removeSequencePart()));
+
+    SecStructPredictViewAction::createAction(this);
+
 }
 
 QWidget* AnnotatedDNAView::createWidget() {
+    GTIMER(c1,t1,"AnnotatedDNAView::createWidget");
     assert(scrollArea == NULL);
     
     mainSplitter = new QSplitter(Qt::Vertical);
+    mainSplitter->setObjectName("annotated_DNA_splitter");
     //mainSplitter->setOpaqueResize(false);
     mainSplitter->setMaximumHeight(200);
     connect(mainSplitter, SIGNAL(splitterMoved(int, int)), SLOT(sl_splitterMoved(int, int)));
@@ -115,27 +137,31 @@ QWidget* AnnotatedDNAView::createWidget() {
     connect(mainSplitter, SIGNAL(customContextMenuRequested(const QPoint &)), SLOT(sl_onContextMenuRequested(const QPoint &)));
 
     scrollArea = new QScrollArea();
+    scrollArea->setObjectName("annotated_DNA_scrollarea");
     scrollArea->setWidgetResizable(true);
     mainSplitter->addWidget(scrollArea);
     mainSplitter->setStretchFactor(mainSplitter->count()-1, 5);
     
-    
     scrolledWidget = new QWidget(scrollArea);
     scrolledWidgetLayout = new QVBoxLayout();
     scrolledWidgetLayout->setContentsMargins(0, 0, 0, 0);
     scrolledWidgetLayout->setSpacing(0);
     scrolledWidget->setBackgroundRole(QPalette::Light);
 
-    foreach(ADVSequenceObjectContext* seqCtx, seqContexts) {
+    for (int i = seqContexts.size(); --i>=0;) { //use reverse mode -> so sequence widget for a lower index will be on top
+        ADVSequenceObjectContext* seqCtx = seqContexts[i];
         ADVSingleSequenceWidget* block = new ADVSingleSequenceWidget(seqCtx, this);
+        block->setObjectName("ADV_single_sequence_widget_"+QString::number(i));
         addSequenceWidget(block);
     }
-    
+
     annotationsView = new AnnotationsTreeView(this);
+    annotationsView->setObjectName("annotations_tree_view");
     mainSplitter->addWidget(annotationsView);
     mainSplitter->setStretchFactor(mainSplitter->count()-1, 1);
 
     scrolledWidget->setLayout(scrolledWidgetLayout);
+    scrolledWidget->setObjectName("scrolled_widget_layout");
     
     //TODO: scroll area does not restore focus for last active child widget after Alt-Tab...
     scrollArea->setWidget(scrolledWidget);
@@ -143,7 +169,7 @@ QWidget* AnnotatedDNAView::createWidget() {
     
     clipb = new ADVClipboard(this);
     aliSupport = new ADVAlignmentSupport(this);
-
+    
     mainSplitter->installEventFilter(this);
     mainSplitter->setAcceptDrops(true);
     
@@ -201,9 +227,7 @@ void AnnotatedDNAView::updateScrollAreaHeight() {
 }
 
 AnnotatedDNAView::~AnnotatedDNAView() {
-    if (posSelector!=NULL) {
-        delete posSelector;
-    }
+    delete posSelector;
 }
 
 bool AnnotatedDNAView::eventFilter(QObject* o, QEvent* e) {
@@ -276,6 +300,19 @@ bool AnnotatedDNAView::onObjectRemoved(GObject* o) {
     return seqContexts.isEmpty();
 }
 
+void AnnotatedDNAView::addADVAction(ADVGlobalAction* a1) {
+    for (int i=0; i<advActions.size(); i++) {
+        ADVGlobalAction* a2 = advActions[i];
+        int p1 = a1->getPosition();
+        int p2 = a2->getPosition();
+        if (p1 < p2 || (p1 == p2 && a1->text() < a2->text())) {
+            advActions.insert(i, a1);
+            return;
+        }
+    }
+    advActions.append(a1);
+}
+
 void AnnotatedDNAView::buildStaticToolbar(QToolBar* tb) {
     tb->addAction(createAnnotationAction);
     tb->addAction(annotationSettingsAction);
@@ -297,9 +334,21 @@ void AnnotatedDNAView::buildStaticToolbar(QToolBar* tb) {
     }  else {
         tb->addAction(posSelectorWidgetAction);
     }
-
     
+    tb->addSeparator();
+    syncViewManager->updateToolbar1(tb);
+    tb->addSeparator();
+
+    foreach(ADVGlobalAction* a, advActions) {
+        if (a->getFlags().testFlag(ADVGlobalActionFlag_AddToToolbar)) {
+            tb->addAction(a);
+        }
+    }
+
     GObjectView::buildStaticToolbar(tb);
+
+    tb->addSeparator();
+    syncViewManager->updateToolbar2(tb);
 }
 
 void AnnotatedDNAView::buildStaticMenu(QMenu* m) {
@@ -310,6 +359,7 @@ void AnnotatedDNAView::buildStaticMenu(QMenu* m) {
     addAnalyseMenu(m);
     addExportMenu(m);
     addRemoveMenu(m);
+    addEditMenu(m);
     m->addSeparator();
 
  
@@ -334,10 +384,13 @@ ADVSequenceWidget* AnnotatedDNAView::findSequenceWidgetByPos(const QPoint& globa
 void AnnotatedDNAView::addAnalyseMenu(QMenu* m) {
     QMenu* am = m->addMenu(tr("analyse_menu"));
     am->menuAction()->setObjectName(ADV_MENU_ANALYSE);
-    am->addAction(findDialogAction);
+    foreach(ADVGlobalAction* a, advActions) {
+        if (a->getFlags().testFlag(ADVGlobalActionFlag_AddToAnalyseMenu)) {
+            am->addAction(a);
+        }
+    }
 }
 
-
 void AnnotatedDNAView::addAddMenu(QMenu* m) {
     QMenu* am = m->addMenu(tr("add_menu"));
     am->menuAction()->setObjectName(ADV_MENU_ADD);
@@ -354,6 +407,21 @@ void AnnotatedDNAView::addRemoveMenu(QMenu* m) {
     rm->menuAction()->setObjectName(ADV_MENU_REMOVE);
 }
 
+void AnnotatedDNAView::addEditMenu(QMenu* m) {
+    ADVSequenceObjectContext* seqCtx = getSequenceInFocus();
+    DNASequenceObject *seqObj = seqCtx->getSequenceObject();
+    Document *curDoc = seqObj->getDocument();
+    QMenu* rm = m->addMenu(tr("edit_menu"));
+    
+    if(curDoc->findGObjectByType(GObjectTypes::DNA_SEQUENCE).isEmpty() || curDoc->isStateLocked()){
+        rm->setDisabled(true);
+    }else{
+        rm->setEnabled(true);
+    };
+    rm->menuAction()->setObjectName(ADV_MENU_EDIT);
+    rm->addAction(addSequencePart);
+    rm->addAction(removeSequencePart);
+}
 
 Task* AnnotatedDNAView::updateViewTask(const QString& stateName, const QVariantMap& stateData) {
     return new UpdateAnnotatedDNAViewTask(this, stateName, stateData);
@@ -431,6 +499,7 @@ void AnnotatedDNAView::addSequenceWidget(ADVSequenceWidget* v) {
     v->setVisible(true);
     v->installEventFilter(this);
     updateScrollAreaHeight();
+    updateMultiViewActions();
     emit si_sequenceWidgetAdded(v);
 }
 
@@ -459,6 +528,7 @@ void AnnotatedDNAView::removeSequenceWidget(ADVSequenceWidget* v) {
     foreach(ADVSequenceObjectContext* c, contexts) {
         c->removeSequenceWidget(v);
     }
+    updateMultiViewActions();
     emit si_sequenceWidgetRemoved(v);
     scrolledWidgetLayout->removeWidget(v);
     delete v;
@@ -466,32 +536,36 @@ void AnnotatedDNAView::removeSequenceWidget(ADVSequenceWidget* v) {
     updateScrollAreaHeight();
 }
 
+void AnnotatedDNAView::updateMultiViewActions() {
+}
 
 void AnnotatedDNAView::sl_onContextMenuRequested(const QPoint & scrollAreaPos) {
     QMenu m;
 
     QPoint globalPos = scrollArea->mapToGlobal(scrollAreaPos);
     m.addAction(posSelectorAction); 
-    m.addSeparator();
+    m.addSeparator()->setObjectName("FIRST_SEP");
     clipb->addCopyMenu(&m);
-    m.addSeparator()->setObjectName(ADV_MENU_SECTION1_SEP);;
+    m.addSeparator()->setObjectName(ADV_MENU_SECTION1_SEP);
     addAddMenu(&m);
     addAnalyseMenu(&m);
     addExportMenu(&m);
     addRemoveMenu(&m);
+    addEditMenu(&m);
     m.addSeparator()->setObjectName(ADV_MENU_SECTION2_SEP);
     
     if (annotationSelection->getSelection().size() == 1) {
         Annotation* a = annotationSelection->getSelection().first().annotation;
         toggleHLAction->setText(tr("Toggle '%1' highlight").arg(a->getAnnotationName()));
 
-        AnnotationSettingsRegistry* asr = AppContext::getAnnotationsSettingsRegistry();
-        const AnnotationSettings* asettings = asr->getSettings(a->getAnnotationName());
+        AnnotationSettings* asettings = AppContext::getAnnotationsSettingsRegistry()->getAnnotationSettings(a);
         QIcon icon = GUIUtils::createSquareIcon(asettings->color, 10);
         toggleHLAction->setIcon(icon);
-
+        
+        toggleHLAction->setObjectName("toggle_HL_action");
         m.addAction(toggleHLAction);
     }
+    annotationSettingsAction->setObjectName("annotation_settings_action");
     m.addAction(annotationSettingsAction);
 
     if (focusedWidget!=NULL) {
@@ -520,13 +594,16 @@ void AnnotatedDNAView::sl_toggleHL() {
         return;
     }
     QString name = annotationSelection->getSelection().first().annotation->getAnnotationName();
-    AnnotationSettings* as = new AnnotationSettings(AppContext::getAnnotationsSettingsRegistry()->getSettings(name));
+    AnnotationSettings* as = AppContext::getAnnotationsSettingsRegistry()->getAnnotationSettings(name);
     as->visible = !as->visible;
-    QList<AnnotationSettings*> l; l<<as;
-    AppContext::getAnnotationsSettingsRegistry()->changeSettings(l);
+    AppContext::getAnnotationsSettingsRegistry()->changeSettings(QList<AnnotationSettings*>()<<as, true);
 }
 
 QString AnnotatedDNAView::tryAddObject(GObject* o) {
+    if (o->getGObjectType() == GObjectTypes::UNLOADED) {
+        AppContext::getTaskScheduler()->registerTopLevelTask(new AddToViewTask(this, o));
+        return "";
+    }
     QList<ADVSequenceObjectContext*> rCtx;
     if (o->getGObjectType() == GObjectTypes::ANNOTATION_TABLE) {
         rCtx = findRelatedSequenceContexts(o);
@@ -587,7 +664,7 @@ QString AnnotatedDNAView::addObject(GObject* o) {
 }
 
 QList<ADVSequenceObjectContext*> AnnotatedDNAView::findRelatedSequenceContexts(GObject* obj) const {
-    QSet<GObject*> relatedObjects = GObjectUtils::selectRelations(obj, GObjectTypes::DNA_SEQUENCE, GObjectRelationRole::SEQUENCE, objects);
+    QSet<GObject*> relatedObjects = GObjectUtils::selectRelations(obj, GObjectTypes::DNA_SEQUENCE, GObjectRelationRole::SEQUENCE, objects, UOF_LoadedOnly);
     QList<ADVSequenceObjectContext*> res;
     foreach(GObject* seqObj, relatedObjects) {
         DNASequenceObject* dnaObj = qobject_cast<DNASequenceObject*>(seqObj);
@@ -665,8 +742,10 @@ ADVSequenceObjectContext* AnnotatedDNAView::getSequenceContext(const GObjectRefe
 }
 
 void AnnotatedDNAView::addRelatedAnnotations(ADVSequenceObjectContext* seqCtx) {
+    QList<GObject*> allLoadedAnnotations = GObjectUtils::findAllObjects(UOF_LoadedOnly, GObjectTypes::ANNOTATION_TABLE);
     QList<GObject*> annotations = GObjectUtils::findObjectsRelatedToObjectByRole(seqCtx->getSequenceObject(), 
-        GObjectTypes::ANNOTATION_TABLE, GObjectRelationRole::SEQUENCE, GObjectUtils::findAllObjects(GObjectTypes::ANNOTATION_TABLE));
+                                    GObjectTypes::ANNOTATION_TABLE, GObjectRelationRole::SEQUENCE, 
+                                    allLoadedAnnotations, UOF_LoadedOnly);
 
     foreach(GObject* ao, annotations) {
         if (objects.contains(ao)) {
@@ -755,4 +834,52 @@ void AnnotatedDNAView::updateState(const AnnotatedDNAViewState& s) {
     annotationsView->updateState(s.stateData);
 }
 
+void AnnotatedDNAView::sl_addSequencePart(){
+    ADVSequenceObjectContext* seqCtx = getSequenceInFocus();
+    DNASequenceObject *seqObj = seqCtx->getSequenceObject();
+    
+    LRegion source(0, seqObj->getSequenceLen());
+
+    DNAAlphabet *alph = seqObj->getAlphabet();
+    AddPartToSequenceDialogController dialog(source, alph, getSequenceWidgetInFocus());
+    int result = dialog.exec();
+    Task *t;
+    if(result == QDialog::Accepted){
+        if(dialog.modifyCurrentDocument()){
+            t = new AddPartToSequenceTask(dialog.getDocumentFormatId(), seqObj, dialog.getPosToInsert(), dialog.getSequenceToInsert(), dialog.getStrategy(), (seqObj->getDocument())->getURL());
+        }else{
+            t = new AddPartToSequenceTask(dialog.getDocumentFormatId(), seqObj, dialog.getPosToInsert(), dialog.getSequenceToInsert(), dialog.getStrategy(), dialog.getNewDocumentPath(), dialog.mergeAnnotations());
+        }
+        AppContext::getTaskScheduler()->registerTopLevelTask(t);
+    }
+}
+
+void AnnotatedDNAView::sl_removeSequencePart(){
+    ADVSequenceObjectContext* seqCtx = getSequenceInFocus();
+    DNASequenceObject *seqObj = seqCtx->getSequenceObject();
+    Project *p = AppContext::getProject();
+    QList<Document*> docs = p->getDocuments();
+
+    Document *curDoc = seqObj->getDocument();
+    QString filter = DialogUtils::prepareDocumentsFileFilter(curDoc->getDocumentFormat()->getFormatId(), false);
+    LRegion source(0, seqObj->getSequenceLen());
+
+    LRegion selection = source;
+    if( seqCtx->getSequenceSelection()->getSelectedRegions().size() > 0){
+        selection = seqCtx->getSequenceSelection()->getSelectedRegions().first();
+    }
+
+    RemovePartFromSequenceDialogController dialog(selection, source, getSequenceWidgetInFocus());
+    int result = dialog.exec();
+    Task *t;
+    if(result == QDialog::Accepted){
+        if(dialog.modifyCurrentDocument()){
+            t = new RemovePartFromSequenceTask(dialog.getDocumentFormatId(), seqObj, dialog.getRegionToDelete(), dialog.getStrategy(), seqObj->getDocument()->getURL());
+        }else{
+            t = new RemovePartFromSequenceTask(dialog.getDocumentFormatId(), seqObj, dialog.getRegionToDelete(), dialog.getStrategy(), dialog.getNewDocumentPath(), dialog.mergeAnnotations());
+        }
+        AppContext::getTaskScheduler()->registerTopLevelTask(t);
+    }
+}
+
 }//namespace
