@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -12,6 +12,7 @@
 #define _GB2_ANNOTATED_DNA_VIEW_TASKS_H_
 
 #include <util_tasks/ObjectViewTasks.h>
+#include <core_api/GObjectReference.h>
 
 namespace GB2 {
 
@@ -21,15 +22,13 @@ class AnnotatedDNAView;
 class OpenAnnotatedDNAViewTask : public ObjectViewTask {
     Q_OBJECT
 public:
-	OpenAnnotatedDNAViewTask(DNASequenceObject* obj);
-    //opens a single view for all sequence object in the list of sequence objects related to the objects in the list
-    OpenAnnotatedDNAViewTask(QList<GObject*> objects);
-	OpenAnnotatedDNAViewTask(Document* doc);
-
+	//opens a single view for all sequence object in the list of sequence objects related to the objects in the list
+    OpenAnnotatedDNAViewTask(const QList<GObject*>& objects);
+	
 	virtual void open();
 
 private:
-	QList< QPointer<DNASequenceObject> > dnaObjects;
+	QList<GObjectReference> sequenceObjectRefs;
 };
 
 class OpenSavedAnnotatedDNAViewTask : public ObjectViewTask {
