@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -30,6 +30,7 @@
 #include <QtGui/QPainter>
 #include <QtGui/QFontMetrics>
 #include <QtGui/QMenu>
+#include <QtGui/QApplication>
 
 
 namespace GB2 {
@@ -58,9 +59,12 @@ DetView::DetView(QWidget* p, ADVSequenceObjectContext* ctx)
     assert(ctx->getSequenceObject()!=NULL);
     featureFlags&=!GSLV_FF_SupportsCustomRange;
     renderArea = new DetViewRenderArea(this);
+    renderArea->setObjectName("render_area");
 
     connect(ctx, SIGNAL(si_aminoTranslationChanged()), SLOT(sl_onAminoTTChanged()));
 
+    connect(ctx->getSequenceGObject(), SIGNAL(si_sequenceChanged()), this, SLOT(sl_sequenceChanged()));
+
 	pack();
 }
 
@@ -71,7 +75,7 @@ void DetView::updateSize() {
 
 void DetView::resizeEvent(QResizeEvent *e) { 
     int w = width();
-    int charWidth = ((DetViewRenderArea*)renderArea)->getCharWidth();
+    int charWidth = renderArea->getCharWidth();
     int visibleSymbolsCount = w / charWidth;
     
     if (visibleSymbolsCount > seqLen) {
@@ -141,28 +145,54 @@ void DetView::setShowTranslation(bool t) {
     updateSize();
 }
 
+void DetView::mouseReleaseEvent(QMouseEvent* me) {
+    //click with 'alt' shift selects single base in GSingleSeqWidget;
+    //here we adjust this behavior -> if click was done in translation line -> select 3 bases
+    Qt::KeyboardModifiers km = QApplication::keyboardModifiers();
+    bool singleBaseSelectionMode = km.testFlag(Qt::AltModifier);
+    if (me->button() == Qt::LeftButton && singleBaseSelectionMode) {
+        QPoint areaPoint = toRenderAreaPoint(me->pos());
+        if (((DetViewRenderArea*)renderArea)->isOnTranslationsLine(areaPoint.y())) {
+            int pos = renderArea->coordToPos(areaPoint.x());
+            if (pos == lastPressPos) {
+                LRegion rgn(pos-1, 3);
+                if (rgn.startPos >=0 && rgn.endPos() <= seqLen) {
+                    setSelection(rgn);
+                    lastPressPos=-1;
+                }
+            }
+        }
+    }
+    GSequenceLineViewAnnotated::mouseReleaseEvent(me);
+}
+
+void DetView::sl_sequenceChanged(){
+    seqLen = ctx->getSequenceLen();
+    int w = width();
+    int charWidth = renderArea->getCharWidth();
+    int visibleSymbolsCount = w / charWidth;
+    if(seqLen > visibleSymbolsCount){
+        if((visibleRange.len != visibleSymbolsCount) || (visibleRange.endPos() > seqLen)){
+            visibleRange.len = visibleSymbolsCount;
+            if(visibleRange.endPos() > visibleSymbolsCount){
+                visibleRange.startPos = seqLen - visibleSymbolsCount;
+            }
+            onVisibleRangeChanged(true);
+        }
+    }else{
+        visibleRange.len = seqLen;
+        onVisibleRangeChanged(true);
+    }
+    GSequenceLineView::sl_sequenceChanged();
+}
+
 
 //////////////////////////////////////////////////////////////////////////
 /// render
 DetViewRenderArea::DetViewRenderArea(DetView* v) : GSequenceLineViewAnnotatedRenderArea(v, true) {
-	font.setFamily("Courier New");
-	font.setPointSize(12);
-
-	rulerFont.setFamily("Arial");
-	rulerFont.setPointSize(8);
-
-	updateFontMetrics();
     updateSize();
 }
 
-void DetViewRenderArea::updateFontMetrics() {
-	QFontMetrics fm(font);
-	yCharOffset = 4;
-	lineHeight = fm.boundingRect('W').height() + 2 * yCharOffset;
-	xCharOffset = 1;
-	charWidth = fm.boundingRect('W').width() + 2 * xCharOffset;
-}
-
 void DetViewRenderArea::updateLines() {
 	numLines = -1;
 	rulerLine = -1;
@@ -225,6 +255,22 @@ LRegion DetViewRenderArea::getAnnotationYRange(Annotation* a, const LRegion& r,
 	return LRegion(y, lineHeight);
 }
 
+bool DetViewRenderArea::isOnTranslationsLine(int y) const {
+    if (firstDirectTransLine != -1) {
+        LRegion dtr(getLineY(firstDirectTransLine), 3*lineHeight);
+        if (dtr.contains(y)) {
+            return true;
+        }
+    }
+    if (firstComplTransLine !=-1) {
+        LRegion ctr(getLineY(firstComplTransLine), 3*lineHeight);
+        if (ctr.contains(y)) {
+            return true;
+        }
+    }
+    return false;
+}
+
 void DetViewRenderArea::drawAll(QPaintDevice* pd) {
     GSLV_UpdateFlags uf = view->getUpdateFlags();
     bool completeRedraw = uf.testFlag(GSLV_UF_NeedCompleteRedraw)  || uf.testFlag(GSLV_UF_ViewResized)  ||
@@ -267,7 +313,7 @@ void DetViewRenderArea::drawAll(QPaintDevice* pd) {
 
 
 void DetViewRenderArea::drawDirect(QPainter& p) {
-	p.setFont(font);
+	p.setFont(sequenceFont);
     p.setPen(Qt::black);
 
 	const LRegion visibleRange = view->getVisibleRange();
@@ -285,7 +331,7 @@ void DetViewRenderArea::drawDirect(QPainter& p) {
 }
 
 void DetViewRenderArea::drawComplement(QPainter& p) {
-	p.setFont(font);
+	p.setFont(sequenceFont);
     p.setPen(Qt::black);
 
 	DetView* detView = getDetView();
@@ -309,12 +355,12 @@ void DetViewRenderArea::drawComplement(QPainter& p) {
 static QByteArray translate(DNATranslation* t, const char* seq, int seqLen) {
 	QByteArray res(seqLen / 3, 0);
 	int n = t->translate(seq, seqLen, res.data(), res.length());
-	assert(n == res.length());
+        assert(n == res.length()); Q_UNUSED(n);
 	return res;
 }
 
 void DetViewRenderArea::drawTranslations(QPainter& p) {
-	p.setFont(font);
+	p.setFont(sequenceFont);
 
 	if (firstDirectTransLine < 0 && firstComplTransLine < 0) {
 		return;
@@ -332,9 +378,9 @@ void DetViewRenderArea::drawTranslations(QPainter& p) {
 
     QColor startC(0,0x99,0);
     QColor stopC(0x99,0,0);
-    QFont fontB = font;
+    QFont fontB = sequenceFont;
     fontB.setBold(true);
-	QFont fontI = font;
+	QFont fontI = sequenceFont;
 	fontI.setItalic(true);
 
 	{//direct translations
@@ -367,7 +413,7 @@ void DetViewRenderArea::drawTranslations(QPainter& p) {
                     p.setFont(fontB);
                 } else {
                     p.setPen(Qt::black);
-                    p.setFont(font);
+                    p.setFont(sequenceFont);
                 }
                 p.drawText(x, y, QString(amin));
 			}
@@ -412,14 +458,14 @@ void DetViewRenderArea::drawTranslations(QPainter& p) {
                     p.setFont(fontB);
                 } else {
                     p.setPen(Qt::black);
-                    p.setFont(font);
+                    p.setFont(sequenceFont);
                 }
                 p.drawText(x, y, QString(amin));
 			}
 		}
 	}
 	p.setPen(Qt::black);
-	p.setFont(font);
+	p.setFont(sequenceFont);
 }
 
 void DetViewRenderArea::drawSequenceSelection(QPainter& p) {
@@ -429,9 +475,8 @@ void DetViewRenderArea::drawSequenceSelection(QPainter& p) {
 		return;
 	}
 	
-	QPen pen1(Qt::SolidLine);
-	pen1.setWidth(1);
-	p.setPen(pen1);
+    QPen pen1(Qt::black, 1, Qt::DashLine);
+    p.setPen(pen1);
 
 	foreach(const LRegion& r, sel->getSelectedRegions()) {
 		highlight(p, r, baseLine);
@@ -489,7 +534,7 @@ void DetViewRenderArea::highlight(QPainter& p, const LRegion& r, int line) {
 
 int DetViewRenderArea::coordToPos(int x) const {
 	LRegion visibleRange = view->getVisibleRange();
-	int pos = visibleRange.startPos + qRound(x / (float)charWidth);
+        int pos = visibleRange.startPos + int(x / (float)charWidth + 0.5f);
 	if (pos > visibleRange.endPos()) {
 		pos = visibleRange.endPos();
 	}
