@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -12,19 +12,24 @@
 #include "ADVSequenceObjectContext.h"
 
 #include <core_api/AppContext.h>
+#include <core_api/Timer.h>
 #include <gobjects/AnnotationTableObject.h>
 #include <gobjects/AnnotationSettings.h>
 #include <selection/DNASequenceSelection.h>
 #include <util_gui/GUIUtils.h>
 #include <datatype/AnnotationData.h>
+#include <document_format/GenbankFeatures.h>
 
 #include <QtGui/QMenu>
 #include <QtGui/QToolTip>
+#include <QtGui/QPainterPath>
 
 namespace GB2 {
 
 #define MIN_ANNOTATION_WIDTH 3
+#define MIN_ANNOTATION_TEXT_WIDTH 5
 #define MIN_SELECTED_ANNOTATION_WIDTH 4
+#define MIN_WIDTH_TO_DRAW_EXTRA_FEATURES 10
 
 GSequenceLineViewAnnotated::GSequenceLineViewAnnotated(QWidget* p, ADVSequenceObjectContext* ctx) 
 : GSequenceLineView(p, ctx)
@@ -48,6 +53,7 @@ GSequenceLineViewAnnotated::GSequenceLineViewAnnotated(QWidget* p, ADVSequenceOb
 void GSequenceLineViewAnnotated::connectAnnotationObject(AnnotationTableObject* ao) {
 	connect(ao, SIGNAL(si_onAnnotationsAdded(const QList<Annotation*>&)), SLOT(sl_onAnnotationsAdded(const QList<Annotation*>&)));
 	connect(ao, SIGNAL(si_onAnnotationsRemoved(const QList<Annotation*>&)), SLOT(sl_onAnnotationsRemoved(const QList<Annotation*>&)));
+    connect(ao, SIGNAL(si_onAnnotationModified(const AnnotationModification&)), SLOT(sl_onAnnotationsModified(const AnnotationModification&)));
 }
 
 void GSequenceLineViewAnnotated::sl_onAnnotationSettingsChanged(const QStringList&) {
@@ -67,6 +73,7 @@ void GSequenceLineViewAnnotated::sl_onAnnotationObjectRemoved(AnnotationTableObj
 
 
 void GSequenceLineViewAnnotated::sl_onAnnotationsAdded(const QList<Annotation*>& l) {
+    GTIMER(c2,t2,"GSequenceLineViewAnnotated::sl_onAnnotationsAdded");
 	registerAnnotations(l);
     addUpdateFlags(GSLV_UF_AnnotationsChanged);
 	update();
@@ -151,7 +158,7 @@ QList<AnnotationSelectionData> GSequenceLineViewAnnotated::selectAnnotationByCoo
                         ok = p.x() <= x2 && p.x() >= x1;
                     }
                     if (ok) {
-                        const AnnotationSettings* as = asr->getSettings(a->getAnnotationName());
+                        AnnotationSettings* as = asr->getAnnotationSettings(a);
                         if (as->visible) {
                             LRegion ry = ra->getAnnotationYRange(a, l, as);
                             if (ry.contains(p.y())) {
@@ -170,8 +177,8 @@ void GSequenceLineViewAnnotated::mousePressEvent(QMouseEvent * me) {
     setFocus();
     QPoint p = toRenderAreaPoint(me->pos());
     if (renderArea->rect().contains(p) && me->button()== Qt::LeftButton) {
-        bool expandAnnotationSelectionToSequence = (me->modifiers() & Qt::ALT) != 0;
-        if (!(me->modifiers() & Qt::SHIFT)) {
+        bool expandAnnotationSelectionToSequence = me->modifiers().testFlag(Qt::ControlModifier);
+        if (!me->modifiers().testFlag(Qt::ShiftModifier)) {
             ctx->getAnnotationsSelection()->clear();
         }
         QList<AnnotationSelectionData> selected = selectAnnotationByCoord(p);
@@ -185,7 +192,7 @@ void GSequenceLineViewAnnotated::mousePressEvent(QMouseEvent * me) {
                     const LRegion& r = location[qMax(0, as.locationIdx)];
                     QString text = as.annotation->getAnnotationName()
                                    + QString(" [%1, %2]").arg(QString::number(r.startPos+1)).arg(QString::number(r.endPos()));
-                    const AnnotationSettings* asettings = asr->getSettings(as.annotation->getAnnotationName());
+                    AnnotationSettings* asettings = asr->getAnnotationSettings(as.annotation->getAnnotationName());
                     QIcon icon = GUIUtils::createSquareIcon(asettings->color, 10);
                     popup.addAction(icon, text);
                 }
@@ -197,16 +204,31 @@ void GSequenceLineViewAnnotated::mousePressEvent(QMouseEvent * me) {
                     asd = &selected[idx];
                 }
             }
-            if (asd!=NULL) {
-                //add to annotation selection
-                ctx->getAnnotationsSelection()->addToSelection(asd->annotation, asd->locationIdx);
-
+            if (asd!=NULL) { //add to annotation selection
+                AnnotationSelection* asel = ctx->getAnnotationsSelection();
+                if (asel->contains(asd->annotation, asd->locationIdx)) {
+                    ctx->getAnnotationsSelection()->removeFromSelection(asd->annotation, asd->locationIdx);
+                } else {
+                    ctx->getAnnotationsSelection()->addToSelection(asd->annotation, asd->locationIdx);
+                }
                 //select region
                 if (expandAnnotationSelectionToSequence) {
-                    assert(asd->locationIdx >=0 && asd->locationIdx <= asd->annotation->getLocation().size());
-                    const LRegion& expandRegion = asd->annotation->getLocation().at(asd->locationIdx);
                     ctx->getSequenceSelection()->clear();
-                    ctx->getSequenceSelection()->addRegion(expandRegion);
+                    LRegion regionToSelect;
+                    foreach(const AnnotationSelectionData& asd, asel->getSelection()) {
+                        AnnotationTableObject* aobj = asd.annotation->getGObject();
+                        if (!ctx->getAnnotationObjects().contains(aobj)) {
+                            continue;
+                        }
+                        LRegion aregion;
+                        if (asd.locationIdx == -1) {
+                            aregion = LRegion::containingRegion(asd.annotation->getLocation());
+                        } else {
+                            aregion = asd.annotation->getLocation().at(asd.locationIdx);
+                        }
+                        regionToSelect = regionToSelect.len == 0 ? aregion : LRegion::containingRegion(regionToSelect, aregion);
+                    }
+                    ctx->getSequenceSelection()->addRegion(regionToSelect);
                 }
             }
         }
@@ -226,11 +248,16 @@ GSequenceLineViewAnnotatedRenderArea::GSequenceLineViewAnnotatedRenderArea(GSequ
 	afmNormal = new QFontMetrics(*afNormal);
 	afmSmall = new QFontMetrics(*afSmall);
 	
-	afNormalCharHeight = afmNormal->width('w');
-	afNormalCharWidth = afmNormal->height();
+	afNormalCharWidth  = afmNormal->width('w');
+	afSmallCharWidth   = afmSmall->width('w');
 
-	afSmallCharHeight = afmSmall->width('w');
-	afSmallCharWidth = afmSmall->height();
+    QLinearGradient gradient(0, 0, 0, 1); //vertical
+    gradient.setCoordinateMode(QGradient::ObjectBoundingMode);
+    gradient.setColorAt(0.00, QColor(255, 255, 255, 120));
+    gradient.setColorAt(0.50, QColor(  0,   0,   0,   0));
+    gradient.setColorAt(0.70, QColor(  0,   0,   0,   0));
+    gradient.setColorAt(1.00, QColor(  0,   0,   0,  70));
+    gradientMaskBrush = QBrush(gradient); 	
 }
 
 GSequenceLineViewAnnotatedRenderArea::~GSequenceLineViewAnnotatedRenderArea() {
@@ -241,71 +268,177 @@ GSequenceLineViewAnnotatedRenderArea::~GSequenceLineViewAnnotatedRenderArea() {
 }
 
 void GSequenceLineViewAnnotatedRenderArea::drawAnnotations(QPainter& p) {
-    drawAnnotations(p, DrawAnnotationPass_DrawFill); 
-    drawAnnotations(p, DrawAnnotationPass_DrawBorder); 
-}
-
-void GSequenceLineViewAnnotatedRenderArea::drawAnnotation(QPainter& p, DrawAnnotationPass pass, Annotation* a, const QPen& borderPen, bool selected) {
-    const GSequenceLineViewAnnotated::DrawSettings& drawSettings = getGSequenceLineViewAnnotated()->getDrawSettings();
-    const LRegion& vr = view->getVisibleRange();
-    AnnotationSettingsRegistry* asr = AppContext::getAnnotationsSettingsRegistry();
-
-    const QString& aName = a->getAnnotationName();
-    const AnnotationSettings* as = asr->getSettings(aName);
-
-    if (!as->visible && (pass == DrawAnnotationPass_DrawFill || !selected)) {
-        return;
-    }
-
-    foreach(const LRegion& r, a->getLocation()) {
-        LRegion y = getAnnotationYRange(a, r, as);
-        if (y.startPos <  0){
-            continue;
-        }
-        LRegion visibleLocation = r.intersect(vr);
-        if (visibleLocation.isEmpty()) {
-            continue;
-        }
-        float x1f = posToCoordF(visibleLocation.startPos);
-        float x2f = posToCoordF(visibleLocation.endPos());
-        assert(x2f>=x1f);
-
-        int rw = qMax(selected ? MIN_SELECTED_ANNOTATION_WIDTH : MIN_ANNOTATION_WIDTH, qRound(x2f-x1f));
-        int x1 = qRound(x1f);
-        if (pass == DrawAnnotationPass_DrawFill) {
-            QColor fillColor = as->color;
-            p.fillRect(x1, y.startPos, rw, y.len, fillColor);
-        } else {
-            assert(pass == DrawAnnotationPass_DrawBorder);
-            p.setPen(borderPen);
-            if (rw > MIN_ANNOTATION_WIDTH) {
-                QRect annotationRect(x1, y.startPos, rw, y.len);
-                p.drawRect(annotationRect);
-                if (drawSettings.drawAnnotationArrows) {
-                    drawAnnotationArrow(p, annotationRect, a->isOnComplementStrand());
-                }
-
-                if (drawSettings.drawAnnotationNames) {
-                    drawBoundedText(p, annotationRect, aName);
-                }
-            }
-            drawAnnotationConnections(p, a, as);
-        } 
-    } 
-}
-
-
-void GSequenceLineViewAnnotatedRenderArea::drawAnnotations(QPainter& p, DrawAnnotationPass pass) {
+    GTIMER(c2,t2,"GSequenceLineViewAnnotatedRenderArea::drawAnnotations");
     ADVSequenceObjectContext* ctx = view->getSequenceContext();
 
     QPen pen1(Qt::SolidLine);
     pen1.setWidth(1);
 
     foreach(AnnotationTableObject* ao, ctx->getAnnotationObjects()) {
-		foreach(Annotation* a, ao->getAnnotations()) {
-            drawAnnotation(p, pass, a, pen1);
+        foreach(Annotation* a, ao->getAnnotations()) {
+            AnnotationSettingsRegistry* asr = AppContext::getAnnotationsSettingsRegistry();
+            AnnotationSettings* as = asr->getAnnotationSettings(a->getAnnotationName());
+            drawAnnotation(p, DrawAnnotationPass_DrawFill, a, pen1, false, as);
+            drawAnnotation(p, DrawAnnotationPass_DrawBorder, a, pen1, false, as);
         }
-	}
+    }
+}
+
+
+static QString prepareAnnotationText(const Annotation* a, const AnnotationSettings* as) {
+    if (as->nameQuals.isEmpty()) {
+        return a->getAnnotationName();
+    }
+    
+    static QVector<Qualifier> qs; //warn: main thread model only
+
+    foreach(const QString& qn, as->nameQuals) {
+        qs.clear();
+        a->findQualifiers(qn, qs);
+        if (!qs.isEmpty()) {
+            QString res = qs[0].getQualifierValue();
+            return res;
+        }
+    }
+    return a->getAnnotationName();
+}
+
+#define FEATURE_ARROW_HLEN 3
+#define FEATURE_ARROW_VLEN 3
+void static addArrowPath(QPainterPath& path, const QRect& rect, bool leftArrow) {
+    if (rect.width() <= FEATURE_ARROW_HLEN || rect.height() <= 0) {
+        return;
+    }
+    int x = leftArrow ? rect.left() : rect.right();
+    int dx = leftArrow ? -FEATURE_ARROW_HLEN : FEATURE_ARROW_HLEN;
+    
+    QPolygon arr;
+    arr << QPoint(x - dx, rect.top()    - FEATURE_ARROW_VLEN);
+    arr << QPoint(x + dx, rect.top()    + rect.height() / 2);
+    arr << QPoint(x - dx, rect.bottom() + FEATURE_ARROW_VLEN);
+    arr << QPoint(x - dx, rect.top()    - FEATURE_ARROW_VLEN);
+    QPainterPath arrowPath;
+    arrowPath.addPolygon(arr);
+
+    QPainterPath dRectPath;
+    dRectPath.addRect(leftArrow ? x : x - (FEATURE_ARROW_HLEN - 1), rect.top(), FEATURE_ARROW_HLEN, rect.height());
+
+    path = path.subtracted(dRectPath);
+    path = path.united(arrowPath);
+}
+
+#define FEATURE_TRIM_HLEN 5
+void static addTrimPath(QPainterPath& path, const QRect& rect, bool leftTrim) {
+    //TODO: normalize line line by 4
+    if (rect.width() <= FEATURE_TRIM_HLEN || rect.height() <= 0) {
+        return;
+    }
+    int x = leftTrim ? rect.left() - 1: rect.right() + 1;
+    int dx = leftTrim ? -FEATURE_TRIM_HLEN : FEATURE_TRIM_HLEN;
+    float dy = rect.height() / 4.0f;
+
+    QPolygon arr;
+    arr << QPoint(x     , rect.top()-1);
+    arr << QPoint(x - dx, rect.top()-1);
+    arr << QPoint(x     , qRound(rect.top() + dy));
+    arr << QPoint(x - dx, qRound(rect.top() + dy * 2));
+    arr << QPoint(x     , qRound(rect.top() + dy * 3));
+    arr << QPoint(x - dx, rect.bottom()+1);
+    arr << QPoint(x     , rect.bottom()+1);
+    arr << QPoint(x     , rect.top());
+    QPainterPath trimPath;
+    trimPath.addPolygon(arr);
+
+    path = path.subtracted(trimPath);
+}
+
+
+void GSequenceLineViewAnnotatedRenderArea::drawAnnotation(QPainter& p, DrawAnnotationPass pass, Annotation* a, const QPen& borderPen, bool selected, 
+                                                          const AnnotationSettings* as, LRegion predefinedy) {
+    if (as == NULL) {
+        AnnotationSettingsRegistry* asr = AppContext::getAnnotationsSettingsRegistry();
+        as = asr->getAnnotationSettings(a->getAnnotationName());
+    }
+    if (!as->visible && (pass == DrawAnnotationPass_DrawFill || !selected)) {
+        return;
+    }
+    const GSequenceLineViewAnnotated::DrawSettings& drawSettings = getGSequenceLineViewAnnotated()->getDrawSettings();
+    const LRegion& vr = view->getVisibleRange();
+
+    const QList<LRegion>& location = a->getLocation();
+    bool simple = location.size() == 1;
+    foreach(const LRegion& r, location) {
+        if (!r.intersects(vr) || predefinedy.startPos < 0) {
+            continue;
+        }
+        LRegion visibleLocation = r.intersect(vr);
+        LRegion y = predefinedy.isEmpty() ? getAnnotationYRange(a, r, as) : predefinedy;
+        if (y.startPos <  0){
+            continue;
+        }
+        float x1f = posToCoordF(visibleLocation.startPos);
+        float x2f = posToCoordF(visibleLocation.endPos());
+        assert(x2f >= x1f);
+
+        int rw = qMax(selected ? MIN_SELECTED_ANNOTATION_WIDTH : MIN_ANNOTATION_WIDTH, qRound(x2f - x1f));
+        int x1 = qRound(x1f);
+        
+        QRect annotationRect(x1, y.startPos, rw, y.len);
+        QPainterPath rectPath;
+        rectPath.addRect(x1, y.startPos, rw, y.len);
+        bool leftTrim  = visibleLocation.startPos != r.startPos;
+        bool rightTrim = visibleLocation.endPos()!= r.endPos();
+        if (drawSettings.drawAnnotationArrows && (!leftTrim && !rightTrim)) {
+            addArrowPath(rectPath, annotationRect, a->isOnComplementStrand());
+        } /* else  {
+            if (leftTrim) {
+                addTrimPath(rectPath, annotationRect, true);
+            }
+            if (rightTrim) {
+                addTrimPath(rectPath, annotationRect, false);
+            }
+        } */
+        if (pass == DrawAnnotationPass_DrawFill) {
+            rectPath.setFillRule(Qt::WindingFill);
+            p.fillPath(rectPath, as->color);
+            p.fillPath(rectPath, gradientMaskBrush);
+        } else {
+            assert(pass == DrawAnnotationPass_DrawBorder);
+            p.setPen(borderPen);
+            if (rw > MIN_ANNOTATION_WIDTH) {
+                p.drawPath(rectPath);
+                if (drawSettings.drawAnnotationNames &&  annotationRect.width() >= MIN_ANNOTATION_TEXT_WIDTH) {
+                    QString aText = prepareAnnotationText(a, as);
+                    drawBoundedText(p, annotationRect, aText);
+                }
+                if (simple && annotationRect.width() > MIN_WIDTH_TO_DRAW_EXTRA_FEATURES) {
+                    if (drawSettings.drawCutSites) {
+                        QString cutStr = a->findFirstQualifierValue(GBFeatureUtils::QUALIFIER_CUT);
+                        bool hasD = false, hasC = false;
+                        int cutD = 0, cutC = 0;
+                        if (!cutStr.isEmpty()) {
+                            int complSplit = cutStr.indexOf('/');
+                            if (complSplit != -1) {
+                                cutD = cutStr.left(complSplit).toInt(&hasD);
+                                cutC = cutStr.mid(qMin(cutStr.length(), complSplit+1)).toInt(&hasC);
+                            } else {
+                                cutD = cutStr.toInt(&hasD);
+                                cutC = cutD;
+                                hasC = hasD;
+                            }
+                        }
+                        if (hasD) {
+                            drawCutSite(p, annotationRect, r.startPos + cutD, true);
+                        }
+                        if (hasC) {
+                            drawCutSite(p, annotationRect, r.endPos() - cutC, false);
+                        }
+                    }
+                }
+            }
+            drawAnnotationConnections(p, a, as);
+        } 
+    } 
 }
 
 void GSequenceLineViewAnnotatedRenderArea::drawAnnotationsSelection(QPainter& p) {
@@ -335,44 +468,47 @@ void GSequenceLineViewAnnotatedRenderArea::drawBoundedText(QPainter& p, const QR
 	}
 	QFont *font = afNormal;
 	QFontMetrics* fm = afmNormal;
-	int ch = afNormalCharHeight;
 	if (fm->width(text) > r.width()) {
 		font = afSmall;
 		fm = afmSmall;
-		ch = afSmallCharHeight;
 	}
+    p.setFont(*font);
+
 	int len = text.length();
-	int textWidth = fm->width(text, len);
-	while (textWidth > r.width()) {
-		len--;
-		textWidth = fm->width(text, len);
-	}
-	if (len == 0) {
+	int textWidth = 0, prefixLen = 0;
+    do {
+        int cw = fm->width(text[prefixLen]);
+        if (textWidth + cw > r.width()) {
+            break;
+        }
+        textWidth+=cw;
+    } while (++prefixLen < len);
+    
+    if (prefixLen == 0) {
 		return;
 	}
-
-	p.setFont(*font);
-
-	int dx = (r.width() - textWidth) / 2;
-	int dy = (r.height() - ch) / 2;
-	int x = r.left() + dx;
-	int y = r.bottom() - dy;
-	p.drawText(x, y, text.left(len));
+    p.drawText(r, Qt::TextSingleLine | Qt::AlignCenter, text.left(prefixLen));
 }
 
-#define FEATURE_ARROW_WIDTH 4
-#define MIN_FEATURE_RECT_WIDTH 2
-void GSequenceLineViewAnnotatedRenderArea::drawAnnotationArrow(QPainter& p, const QRect& rect, bool leftArrow) {
-	if (rect.width() <= MIN_FEATURE_RECT_WIDTH) {
-		return;
-	}
-	int y = rect.top();
-	int x = leftArrow ? rect.left() - 1 : rect.right() + 1;
-	int dx = leftArrow ? -FEATURE_ARROW_WIDTH : FEATURE_ARROW_WIDTH;
-	p.drawLine(x, y, x+dx, y + (rect.height()+1)/2);
-	p.drawLine(x, y + rect.height(), x+dx, y + (rect.height()+1)/2);
+
+#define CUT_SITE_HALF_WIDTH  4
+#define CUT_SITE_HALF_HEIGHT 2
+void GSequenceLineViewAnnotatedRenderArea::drawCutSite(QPainter& p, const QRect& rect, int pos, bool direct) {
+    int xCenter = posToCoord(pos, true);
+    if (xCenter < rect.left() || xCenter > rect.right()) {
+        return; // drawing cut sites out of the annotation region is not supported
+    }
+    int xLeft = xCenter - CUT_SITE_HALF_WIDTH;
+    int xRight= xCenter + CUT_SITE_HALF_WIDTH;
+    int yFlat = direct ? rect.top() - CUT_SITE_HALF_HEIGHT : rect.bottom() + CUT_SITE_HALF_HEIGHT;
+    int yPeak = direct ? rect.top() + CUT_SITE_HALF_HEIGHT : rect.bottom() - CUT_SITE_HALF_HEIGHT;
+    
+    p.drawLine(xLeft, yFlat, xRight, yFlat);
+    p.drawLine(xLeft, yFlat, xCenter, yPeak);
+    p.drawLine(xRight, yFlat, xCenter, yPeak);
 }
 
+
 #define MAX_VIRTUAL_RANGE 10000
 
 void GSequenceLineViewAnnotatedRenderArea::drawAnnotationConnections(QPainter& p, Annotation* a, const AnnotationSettings* as) {
@@ -386,9 +522,9 @@ void GSequenceLineViewAnnotatedRenderArea::drawAnnotationConnections(QPainter& p
 	int dx2 = 0;
 	if (drawSettings.drawAnnotationArrows) {
 		if (a->isOnComplementStrand()) {
-			dx2 = - FEATURE_ARROW_WIDTH;
+			dx2 = - FEATURE_ARROW_HLEN;
 		} else {
-			dx1 = FEATURE_ARROW_WIDTH;
+			dx1 = FEATURE_ARROW_HLEN;
 		}
 	}
 	foreach(const LRegion& r, a->getLocation()) {
@@ -421,7 +557,6 @@ void GSequenceLineViewAnnotatedRenderArea::drawAnnotationConnections(QPainter& p
 	}
 }
 
-
 void GSequenceLineViewAnnotated::ensureVisible(Annotation* a, int locationIdx) {
 	const QList<LRegion>& location = a->getLocation();
 	assert(locationIdx < location.size());
@@ -467,15 +602,16 @@ QString GSequenceLineViewAnnotated::createToolTip(QHelpEvent* e) {
 	foreach(AnnotationSelectionData ad, la) {
 		if (++rows > ROWS_LIMIT) break;
 		QString aname = ad.annotation->getAnnotationName();
-		QColor acl = AppContext::getAnnotationsSettingsRegistry()->getSettings(aname)->color;
+		QColor acl = AppContext::getAnnotationsSettingsRegistry()->getAnnotationSettings(aname)->color;
 		tip += "<tr><td bgcolor="+acl.name()+" bordercolor=black width=15></td><td><big>"+aname+"</big></td></tr>";
-		if (ad.annotation->getQualifiers().size() !=0) {
+		//if (ad.annotation->getQualifiers().size() !=0) 
+        {
 			if (skipDetails) {
 				tip += "<tr><td/><td>...</td>";
 				rows++;
 			} else {
 				tip += "<tr><td/><td>"; 
-				tip += ad.annotation->getQualifiersTip(ROWS_LIMIT - rows);
+				tip += ad.annotation->getQualifiersTip(ROWS_LIMIT - rows, getSequenceObject(), ctx->getComplementTT(), ctx->getAminoTT());
 				tip += "</td></tr>";
 				rows += ad.annotation->getQualifiers().size();
 			}
@@ -488,6 +624,13 @@ QString GSequenceLineViewAnnotated::createToolTip(QHelpEvent* e) {
 	return tip;
 }
 
+void GSequenceLineViewAnnotated::sl_onAnnotationsModified( const AnnotationModification& md )
+{
+    if (md.type == AnnotationModification_LocationChanged || md.type == AnnotationModification_NameChanged ) {
+        addUpdateFlags(GSLV_UF_AnnotationsChanged);
+        update();
+    }
+}
 bool GSequenceLineViewAnnotatedRenderArea::isAnnotationSelectionInVisibleRange() const {
     const QSet<AnnotationTableObject*>& aos = view->getSequenceContext()->getAnnotationObjects();
     AnnotationSelection* as = view->getSequenceContext()->getAnnotationsSelection();
