@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -16,6 +16,7 @@
 #include <core_api/DNAAlphabet.h>
 #include <core_api/SelectionModel.h>
 #include <core_api/Log.h>
+#include <core_api/Timer.h>
 #include <core_api/AppContext.h>
 
 #include <gobjects/DNASequenceObject.h>
@@ -29,7 +30,6 @@
 
 #include <QtGui/QTextEdit>
 #include <QtGui/QGridLayout>
-#include <QtGui/QMenu>
 #include <QtGui/QPainter>
 #include <QtGui/QFontMetrics>
 #include <QtGui/QDialog>
@@ -60,14 +60,29 @@ PanView::PanView(QWidget* p, ADVSequenceObjectContext* ctx) : GSequenceLineViewA
     zoomToSequenceAction = new QAction(QIcon(":/core/images/zoom_whole.png"), tr("zseq_action"), this);
     connect(zoomToSequenceAction, SIGNAL(triggered()), SLOT(sl_zoomToSequence()));
 
+    toggleMainRulerAction = new QAction(tr("Show main ruler"), this);
+    toggleMainRulerAction->setCheckable(true);
+    toggleMainRulerAction->setChecked(getRenderArea()->showMainRuler);
+    connect(toggleMainRulerAction, SIGNAL(triggered(bool)), SLOT(sl_toggleMainRulerVisibility(bool)));
+
+    toggleCustomRulersAction = new QAction(tr("Show custom rulers"), this);
+    toggleCustomRulersAction->setCheckable(true);
+    toggleCustomRulersAction->setChecked(getRenderArea()->showCustomRulers);
+    toggleCustomRulersAction->setEnabled(!getRenderArea()->customRulers.isEmpty());
+    connect(toggleCustomRulersAction, SIGNAL(triggered(bool)), SLOT(sl_toggleCustomRulersVisibility(bool)));
+
     drawSettings.drawAnnotationArrows = true;
     drawSettings.drawAnnotationNames = true;
+    drawSettings.drawCutSites = false;
+    syncOffset = 0;
 
     //can't move to the GSequenceLineViewAnnotated -> virtual calls does not work in  constructor
     foreach(AnnotationTableObject* obj, ctx->getAnnotationObjects()) {
         registerAnnotations(obj->getAnnotations());
     }
 
+    connect(ctx->getSequenceGObject(), SIGNAL(si_sequenceChanged()), this, SLOT(sl_sequenceChanged()));
+
     updateActions();
     updateRowBar();
 
@@ -92,9 +107,10 @@ PanView::~PanView() {
 }
 
 void PanView::registerAnnotations(const QList<Annotation*>& l) {
+    GTIMER(c1,t1,"PanView::registerAnnotations");
     AnnotationSettingsRegistry* asr = AppContext::getAnnotationsSettingsRegistry();
     foreach(Annotation* a, l) {
-        const AnnotationSettings* as = asr->getSettings(a->getAnnotationName());
+        AnnotationSettings* as = asr->getAnnotationSettings(a);
         if (as->visible) {
             rowsManager->addAnnotation(a, a->getAnnotationName());
         }
@@ -105,7 +121,7 @@ void PanView::registerAnnotations(const QList<Annotation*>& l) {
 void PanView::unregisterAnnotations(const QList<Annotation*>& l) {
     AnnotationSettingsRegistry* asr = AppContext::getAnnotationsSettingsRegistry();
     foreach(Annotation* a, l) {
-        const AnnotationSettings* as = asr->getSettings(a->getAnnotationName());
+        AnnotationSettings* as = asr->getAnnotationSettings(a);
         if (as->visible) {
             rowsManager->removeAnnotation(a);
         }
@@ -114,21 +130,25 @@ void PanView::unregisterAnnotations(const QList<Annotation*>& l) {
 }
 
 void PanView::updateRows() {
-    rowsManager->resort();
-
-    PanViewRenderArea* ra = (PanViewRenderArea*)renderArea;
-    bool heightChanged = ra->updateNumVisibleRows();
-    if (heightChanged) {
-        setFixedHeight(renderArea->height() + scrollBar->height());
-    }
+    updateRAHeight();
 
     int maxSteps = calculateNumRowBarSteps();
     if (qAbs(rowBar->maximum() - rowBar->minimum())!=maxSteps) {
         updateRowBar();
     }
 }
+
+void PanView::updateRAHeight() {
+    PanViewRenderArea* ra = getRenderArea();
+
+    bool heightChanged = ra->updateNumVisibleRows();
+    if (heightChanged) {
+        setFixedHeight(renderArea->height() + scrollBar->height());
+    }
+}
+
 int  PanView::calculateNumRowBarSteps() const {
-    PanViewRenderArea* ra = (PanViewRenderArea*)renderArea;
+    PanViewRenderArea* ra = getRenderArea();
     int visibleRows = ra->getNumVisibleRows();
     int numRows = rowsManager->getNumRows();
     int res = qMax(0, numRows - visibleRows);
@@ -138,20 +158,23 @@ int  PanView::calculateNumRowBarSteps() const {
 void PanView::updateRowBar() {
     rowBar->disconnect(this);
 
-    PanViewRenderArea* ra = (PanViewRenderArea*)renderArea;
+    PanViewRenderArea* ra = getRenderArea();
     int visibleRows = ra->getNumVisibleRows();
     int maxSteps = calculateNumRowBarSteps();
     rowBar->setMinimum(-maxSteps); //inverted appearance
     rowBar->setMaximum(0);
     rowBar->setSingleStep(1);
     rowBar->setPageStep(visibleRows - 1);
-    rowBar->setSliderPosition(-ra->getRowLinesOffset());
+    int rowsOffset = qMin(maxSteps, ra->getRowLinesOffset());
+    ra->setRowLinesOffset(rowsOffset);
+    rowBar->setSliderPosition(-rowsOffset);
     rowBar->setEnabled(maxSteps > 0);
+    
     connect(rowBar, SIGNAL(valueChanged(int)), SLOT(sl_onRowBarMoved(int)));
 }
 
 void PanView::sl_onRowBarMoved(int v) {
-    PanViewRenderArea* ra = (PanViewRenderArea*)renderArea;
+    PanViewRenderArea* ra = getRenderArea();
     ra->setRowLinesOffset(-v); // '-' because of inverted appearance
     addUpdateFlags(GSLV_UF_NeedCompleteRedraw);    
     update();
@@ -160,7 +183,7 @@ void PanView::sl_onRowBarMoved(int v) {
 void PanView::sl_onAnnotationSettingsChanged(const QStringList& changedSettings) {
     AnnotationSettingsRegistry* asr = AppContext::getAnnotationsSettingsRegistry();
     foreach (const QString& name, changedSettings) {
-        const AnnotationSettings* as =asr->getSettings(name);
+        AnnotationSettings* as =asr->getAnnotationSettings(name);
         bool hasRow = rowsManager->contains(name);
         if (as->visible == hasRow) {
             continue;
@@ -213,6 +236,14 @@ void PanView::updateActions() {
 
 void PanView::sl_zoomInAction() {
     assert(visibleRange.len >= minNuclsPerScreen);
+    DNASequenceSelection* sel = getSequenceContext()->getSequenceSelection();
+    if (!sel->isEmpty()) {
+        const LRegion& selRange = sel->getSelectedRegions().first();
+        if (selRange.len >= minNuclsPerScreen && visibleRange.contains(selRange) && visibleRange != selRange) {
+            sl_zoomToSelection();
+            return;
+        }
+    }
     LRegion newVisibleRange = visibleRange;
     newVisibleRange.len = qMax((visibleRange.len + 1) / 2, minNuclsPerScreen);
     if (newVisibleRange.len!=visibleRange.len) {
@@ -264,8 +295,7 @@ void PanView::sl_zoomToSequence() {
 
 void PanView::setVisibleRange(const LRegion& newRange, bool signal) {
     if (newRange.len < minNuclsPerScreen) {
-        //todo: log this
-        return;//ignored
+        minNuclsPerScreen = newRange.len;
     }
     GSequenceLineView::setVisibleRange(newRange, signal);
 }
@@ -273,10 +303,10 @@ void PanView::setVisibleRange(const LRegion& newRange, bool signal) {
 
 void PanView::ensureVisible(Annotation* a, int locationIdx) {
     AnnotationSettingsRegistry* asr = AppContext::getAnnotationsSettingsRegistry();
-    const AnnotationSettings* as = asr->getSettings(a->getAnnotationName());
+    AnnotationSettings* as = asr->getAnnotationSettings(a);
     if (as->visible) {
-        int row = rowsManager->getAnnotationRow(a);
-        PanViewRenderArea* pr = (PanViewRenderArea*)renderArea;
+        int row = rowsManager->getAnnotationRowIdx(a);
+        PanViewRenderArea* pr = getRenderArea();
         if (!pr->isRowVisible(row)) {
             centerRow(row);
         }
@@ -285,7 +315,7 @@ void PanView::ensureVisible(Annotation* a, int locationIdx) {
 }
 
 void PanView::centerRow(int row) {
-    PanViewRenderArea* pr = (PanViewRenderArea*)renderArea;
+    PanViewRenderArea* pr = getRenderArea();
     int targetFirstRowLine = qMax(0, row - pr->getNumVisibleRows() / 2);
     int rowOnTheFirstLine = pr->getRowLinesOffset();
     if (targetFirstRowLine == rowOnTheFirstLine) {
@@ -302,21 +332,116 @@ void PanView::sl_onRangeChangeRequest(int start, int end) {
     setVisibleRange(LRegion(start-1, end));
 }
 
+void PanView::setNumBasesVisible(int n) {
+    int nBases = qBound(minNuclsPerScreen, n, seqLen);
+    int center = visibleRange.startPos + visibleRange.len / 2;
+    int newStart = qMax(0, center - nBases / 2);
+    assert(newStart + nBases <= seqLen);
+    setVisibleRange(LRegion(newStart, nBases));
+}
+
+PanViewRenderArea* PanView::getRenderArea() const {
+    return (PanViewRenderArea*)renderArea;
+}
+
+QList<RulerInfo> PanView::getCustomRulers() const {
+    return getRenderArea()->customRulers;
+}
+
+void PanView::addCustomRuler(const RulerInfo& r) {
+    PanViewRenderArea* ra = getRenderArea();
+    ra->customRulers.append(r);
+    if (ra->showCustomRulers) {
+        updateRAHeight();
+
+        addUpdateFlags(GSLV_UF_NeedCompleteRedraw);
+        update();
+    }
+    toggleCustomRulersAction->setEnabled(true);
+}
+
+void PanView::removeCustomRuler(const QString& name) {
+    PanViewRenderArea* ra = getRenderArea();
+    for (int i=0, n = ra->customRulers.count(); i < n; i++) {
+        if (ra->customRulers[i].name == name) {
+            ra->customRulers.removeAt(i);
+            break;
+        }
+    }
+    toggleCustomRulersAction->setEnabled(!ra->customRulers.isEmpty());
+    if (ra->showCustomRulers) {
+        updateRAHeight();
+
+        addUpdateFlags(GSLV_UF_NeedCompleteRedraw);
+        update();
+    }
+}
+
+void PanView::removeAllCustomRulers() {
+    toggleCustomRulersAction->setEnabled(false);
+    if (!getRenderArea()->customRulers.isEmpty()) {
+        getRenderArea()->customRulers.clear();
+        updateRAHeight();
+
+        addUpdateFlags(GSLV_UF_NeedCompleteRedraw);
+        update();
+    }
+}
+
+void PanView::sl_toggleMainRulerVisibility(bool visible) {
+    getRenderArea()->showMainRuler = visible;
+    updateRAHeight();
+
+    addUpdateFlags(GSLV_UF_NeedCompleteRedraw);
+    update();
+}
+
+void PanView::sl_toggleCustomRulersVisibility(bool visible) {
+    getRenderArea()->showCustomRulers = visible;
+    updateRAHeight();
+
+    addUpdateFlags(GSLV_UF_NeedCompleteRedraw);
+    update();
+}
+
+
+void PanView::setSyncOffset(int o) {
+    if (o == syncOffset) {
+        return;
+    }
+    syncOffset = o;
+    addUpdateFlags(GSLV_UF_NeedCompleteRedraw);
+    update();
+}
+
+void PanView::sl_sequenceChanged(){
+    seqLen = ctx->getSequenceLen();
+    LRegion curSource(0, ctx->getSequenceLen()), newRange(0,0);
+    if(!curSource.contains(visibleRange)){
+        if (curSource.len > visibleRange.len){
+            newRange.startPos = visibleRange.startPos - (visibleRange.endPos() - curSource.endPos());
+            newRange.len = visibleRange.len;
+        }else{
+            newRange = curSource;
+        }
+        setVisibleRange(newRange);
+    }
+    GSequenceLineView::sl_sequenceChanged();
+}
 //////////////////////////////////////////////////////////////////////////
 /// render
 PanViewRenderArea::PanViewRenderArea(PanView* d) : GSequenceLineViewAnnotatedRenderArea(d, false), panView(d) {
+    showMainRuler = true;
+    showCustomRulers = true;
+    numLines = 0;
+
     rowLinesOffset = 0;
-
-    rulerFont.setFamily("Arial");
-    rulerFont.setPointSize(8);
-
-    lineHeight = afmNormal->height() + 2;
-
     updateNumVisibleRows();
 }
 
 
 void PanViewRenderArea::drawAll(QPaintDevice* pd) {
+    GTIMER(c2,t2,"PanViewRenderArea::drawAll");
     GSLV_UpdateFlags uf = view->getUpdateFlags();
     bool completeRedraw = uf.testFlag(GSLV_UF_NeedCompleteRedraw) || uf.testFlag(GSLV_UF_ViewResized) || 
                           uf.testFlag(GSLV_UF_VisibleRangeChanged) || uf.testFlag(GSLV_UF_AnnotationsChanged);
@@ -327,6 +452,7 @@ void PanViewRenderArea::drawAll(QPaintDevice* pd) {
         pCached.setPen(Qt::black);
 
         drawRuler(pCached);
+        drawCustomRulers(pCached);
 
         drawAnnotations(pCached);
 
@@ -337,6 +463,7 @@ void PanViewRenderArea::drawAll(QPaintDevice* pd) {
 
     drawFrame(p); //frame is always redrawn
 
+    drawSequence(p);
     drawAnnotationsSelection(p);
     drawSequenceSelection(p);
 
@@ -346,6 +473,9 @@ void PanViewRenderArea::drawAll(QPaintDevice* pd) {
 }
 #define RULER_NOTCH_SIZE 2
 void PanViewRenderArea::drawRuler(QPainter& p) {
+    if (!showMainRuler) {
+        return;
+    }
     const LRegion& visibleRange = view->getVisibleRange();
     float halfChar = getCurrentScale() / 2;
     int firstCharCenter = qRound(posToCoordF(visibleRange.startPos) + halfChar);
@@ -363,55 +493,158 @@ void PanViewRenderArea::drawRuler(QPainter& p) {
 }
 
 
-LRegion PanViewRenderArea::getAnnotationYRange(Annotation* a, const LRegion& r, const AnnotationSettings* as) const {
-    Q_UNUSED(r);
+#define LINE_TEXT_OFFSET 10
+
+void PanViewRenderArea::drawCustomRulers(QPainter& p) {
+    if (!showCustomRulers || customRulers.isEmpty()) {
+        return;
+    }
+    const LRegion& visibleRange = view->getVisibleRange();
+    float pixelsPerChar = getCurrentScale();
+    float halfChar =  pixelsPerChar / 2;
+    int firstCharCenter = qRound(posToCoordF(visibleRange.startPos) + halfChar);
+    int lastCharCenter = qRound(posToCoordF(visibleRange.endPos()-1) + halfChar);
+    
+    QFont crf = rulerFont;
+    crf.setBold(true);
+    
+    QFontMetrics fm(crf);
+    int w = width();
+
+    GraphUtils::RulerConfig c;
+    c.notchSize = RULER_NOTCH_SIZE;
+    int maxRulerTextWidth = 0;
+    foreach(const RulerInfo& ri, customRulers) {
+        int w = fm.width(ri.name);
+        maxRulerTextWidth = qMax(maxRulerTextWidth, w);
+    }
+    for (int i=0, n = customRulers.count();  i<n; i++) {
+        const RulerInfo& ri = customRulers[i];
+        p.setPen(ri.color);
+        p.setFont(crf);
+        
+        int y = getLineY(getCustomRulerLine(i)) + c.notchSize;
+        p.drawText(QRect(LINE_TEXT_OFFSET, y, maxRulerTextWidth, lineHeight), ri.name);
+    
+        int rulerStartOffset = maxRulerTextWidth + LINE_TEXT_OFFSET;
+        if (rulerStartOffset >= w)  {
+            continue;
+        }
+        int startPos = visibleRange.startPos + 1 - ri.offset;
+        int endPos = visibleRange.endPos() - ri.offset;
+        int x = firstCharCenter;
+
+        if (firstCharCenter <= rulerStartOffset) {
+            int deltaPixels = rulerStartOffset - firstCharCenter;
+            int nChars = qMax(1, qRound(deltaPixels / pixelsPerChar));
+            int deltaPixels2 = nChars * pixelsPerChar;
+            startPos+=nChars;
+            x+=deltaPixels2;
+        }
+        
+        int rulerWidth = lastCharCenter - x;
+        if (qRound(halfChar) == 0) {
+            assert(firstCharCenter == 0 && lastCharCenter == w);
+            rulerWidth--; // make the end of the ruler visible
+        }
+        GraphUtils::drawRuler(p, QPoint(x, y), rulerWidth, startPos, endPos, rulerFont, c);	
+    }
+}
+
+LRegion PanViewRenderArea::getAnnotationYRange(Annotation* a, const LRegion&, const AnnotationSettings* as) const {
     if (!as->visible) {
         return LRegion(-1, 0);
     }
-    int row = getPanView()->getRowsManager()->getAnnotationRow(a);
+    int row = getPanView()->getRowsManager()->getAnnotationRowIdx(a);
     int line = getRowLine(row);
     return LRegion(getLineY(line) + 2, lineHeight - 4);
 }
 
 void PanViewRenderArea::drawAnnotations(QPainter& p) {
+    GTIMER(c2,t2,"PanViewRenderArea::drawAnnotations");
     QPen dotty(Qt::lightGray, 1, Qt::DotLine);
     p.setPen(dotty);
     p.setFont(*afSmall);
+    int w = width();
 
     //draw row names
     PVRowsManager* rm = getPanView()->getRowsManager();
     int maxVisibleRows = getNumVisibleRows();
-    int dy = (lineHeight - afSmallCharHeight) / 2;
     for (int i = 0; i < maxVisibleRows; i++) {
         int row = i + rowLinesOffset;
         int rowLine = getRowLine(row);
         int lineY = getLineY(rowLine);
         p.drawLine(0, lineY, cachedView->width(), lineY);
 
-        bool emptyRow = row >= rm->getNumRows();
-        int numAnnotations = emptyRow ? 0 : rm->getNumAnnotationsInRow(row);
-        QString rowName = emptyRow ? PanView::tr("empty") : rm->getRowKey(row);
-        QString text = rowName;
-        if (!emptyRow) {
-            text+=" (" + QString::number(numAnnotations) + ")";
+        PVRowData* rData = rm->getRow(row);
+        QString text;
+        if (rData == NULL) {
+            text = GB2::PanView::tr("empty");
+        } else {
+            text = rData->key + " (" + QString::number(rData->annotations.size()) + ")";
+        }
+        int y = lineY + lineHeight;
+        QRect textRect(LINE_TEXT_OFFSET, lineY + 1, w, lineHeight - 2);
+        p.drawText(textRect, text);
+
+        if (rData != NULL) {
+            AnnotationSettingsRegistry* asr = AppContext::getAnnotationsSettingsRegistry();
+            AnnotationSettings* as = asr->getAnnotationSettings(rData->key);
+            if (as->visible) {
+                QPen pen1(Qt::SolidLine);
+                pen1.setWidth(1);
+                LRegion yr(lineY + 2, lineHeight - 4);
+                foreach(Annotation* a, rData->annotations) {
+                    drawAnnotation(p, DrawAnnotationPass_DrawFill, a, pen1, false, as, yr);
+                    drawAnnotation(p, DrawAnnotationPass_DrawBorder, a, pen1, false, as, yr);
+                }
+                //restore pen
+                p.setPen(dotty);
+                p.setFont(*afSmall);
+            }
         }
-        int y = lineY + lineHeight - dy;
-        p.drawText(10, y, text);
     }
 
     int firstRowLine = getRowLine(0);
     int lineY = getLineY(firstRowLine) + lineHeight;
     p.drawLine(0, lineY, cachedView->width(), lineY);
+}
+
+bool PanViewRenderArea::isSequenceCharsVisible() const {
+    return getCurrentScale() >= smallCharWidth;
+}
 
 
-    GSequenceLineViewAnnotatedRenderArea::drawAnnotations(p);
+void PanViewRenderArea::drawSequence(QPainter& p) {
+    if (!isSequenceCharsVisible()) {
+        return;
+    }
+    p.setPen(Qt::black);
+    float halfCharByScale = getCurrentScale() / 2;
+    float halfCharByFont = 0.0f;
+    if (getCurrentScale() >= charWidth) {
+        p.setFont(sequenceFont);
+        halfCharByFont = charWidth / 2.0f;
+    } else {
+        p.setFont(smallSequenceFont);
+        halfCharByFont = smallCharWidth / 2.0f;
+    }
+    const QByteArray& seq = view->getSequenceContext()->getSequenceData();
+    const LRegion& visibleRange = view->getVisibleRange();
+    int y = getLineY(getSelectionLine()) + lineHeight - yCharOffset;
+    for (int i = visibleRange.startPos; i < visibleRange.endPos(); i++) {
+        char c = seq[i];
+        int x = qRound(posToCoordF(i) + halfCharByScale - halfCharByFont);
+        p.drawText(x, y, QString(c));
+    }
+
 }
 
 #define ARROW_DY 5
 #define ARROW_DX 5
 void PanViewRenderArea::drawSequenceSelection(QPainter& p) {
     const QList<LRegion>& selection = panView->getSequenceContext()->getSequenceSelection()->getSelectedRegions();
-    bool showSequenceMode = false;
+    bool showSequenceMode = isSequenceCharsVisible();
     if (selection.isEmpty()) {
         return;
     }
@@ -431,7 +664,7 @@ void PanViewRenderArea::drawSequenceSelection(QPainter& p) {
         drawGraphics = ly > 0;
     }
     int halfNum = rfm.boundingRect('1').height() / 2;
-    int rty = ly + halfNum + 1;
+    int rty = ly + halfNum;
     QString rangePattern = " "+tr("[%1 bp]")+" ";
     foreach(const LRegion& r, selection) {
         if (!visibleRange.intersects(r)) {
@@ -532,20 +765,20 @@ int PanViewRenderArea::getRowLine(int i) const {
     return line;
 }
 
+#define MIN_VISIBLE_ROWS  1
+#define EXTRA_EMPTY_ROWS  0
+#define MAX_VISIBLE_ROWS 10
+
 void PanViewRenderArea::setRowLinesOffset(int r) {
     int maxRows = getPanView()->getRowsManager()->getNumRows();
     int visibleRows = getNumVisibleRows();
-    assert(r<=maxRows-visibleRows); Q_UNUSED(maxRows); Q_UNUSED(visibleRows);
+    assert(r <= maxRows - (visibleRows - MIN_VISIBLE_ROWS)) ; Q_UNUSED(maxRows); Q_UNUSED(visibleRows);
     if (r!=rowLinesOffset) {
         rowLinesOffset = r;
         update();
     }
 }
 
-#define MIN_VISIBLE_ROWS  1
-#define EXTRA_EMPTY_ROWS  0
-#define MAX_VISIBLE_ROWS 10
-
 bool PanViewRenderArea::updateNumVisibleRows() {
     int actualRows = getPanView()->getRowsManager()->getNumRows();
     int expectedRows = actualRows + EXTRA_EMPTY_ROWS;
