@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -10,10 +10,12 @@
 
 #include "MSAEditorSequenceArea.h"
 #include "MSAEditor.h"
-#include "MSAEditorNameList.h"
+#include "MSAColorScheme.h"
 
 #include <core_api/Log.h>
 #include <core_api/DNAAlphabet.h>
+#include <core_api/AppContext.h>
+#include <core_api/Settings.h>
 #include <datatype/MAlignment.h>
 #include <gobjects/MAlignmentObject.h>
 #include <util_gui/GUIUtils.h>
@@ -26,7 +28,7 @@
 #include <QtGui/QClipboard>
 #include <QtGui/QApplication>
 #include <QtGui/QDialog>
-
+#include <QtGui/QFontDialog>
 
 namespace GB2 {
 
@@ -34,37 +36,81 @@ namespace GB2 {
 
 static LogCategory log(ULOG_CAT_MSA);
 
-MSAEditorSequenceArea::MSAEditorSequenceArea(MSAEditorUI* _ui) : editor(_ui->editor), ui(_ui) {
-    colorScheme = new MSAEditorColorScheme(this, editor->getMSAObject()->getMAlignment().alphabet->isAmino());
+#define MIN_FONT_SIZE 6
+#define MAX_FONT_SIZE 24
+
+#define SETTINGS_ROOT QString("msaeditor/")
+
+#define SETTINGS_FONT_FAMILY    "font_family"
+#define SETTINGS_FONT_SIZE      "font_size"
+#define SETTINGS_FONT_ITALIC    "font_italic"
+#define SETTINGS_FONT_BOLD      "font_bold"
+#define SETTINGS_COLOR_NUCL     "color_nucl"
+#define SETTINGS_COLOR_AMINO    "color_amino"
+
+#define DEFAULT_FONT_FAMILY "Verdana" 
+#define DEFAULT_FONT_SIZE 10
+
+MSAEditorSequenceArea::MSAEditorSequenceArea(MSAEditorUI* _ui, QScrollBar* hb, QScrollBar* vb) 
+: editor(_ui->editor), ui(_ui), shBar(hb), svBar(vb) 
+{
+    setFocusPolicy(Qt::WheelFocus);
+
+    cachedView = new QPixmap();
+    completeRedraw = true;
 
     setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
     setMinimumSize(100, 100);
     startPos = 0;
-    seqFont.setFamily("Verdana");
-    seqFont.setPointSize(10);
-    QFontMetrics seqFM(seqFont);
-    seqCharWidth = seqFM.width('W') * 5 / 4;
+    startSeq = 0;
+    highlightSelection = false;
 
+    Settings* s = AppContext::getSettings();
+    seqFont.setFamily(s->getValue(SETTINGS_ROOT + SETTINGS_FONT_FAMILY, DEFAULT_FONT_FAMILY).toString());
+    seqFont.setPointSize(s->getValue(SETTINGS_ROOT + SETTINGS_FONT_SIZE, DEFAULT_FONT_SIZE).toInt());
+    seqFont.setItalic(s->getValue(SETTINGS_ROOT + SETTINGS_FONT_ITALIC, false).toBool());
+    seqFont.setBold(s->getValue(SETTINGS_ROOT + SETTINGS_FONT_BOLD, false).toBool());
+    updateSeqFontMetrics();
+    
     delSymAction = new QAction(tr("del_sym"), this);
     delSymAction->setShortcut(QKeySequence(Qt::Key_Delete));
+    delSymAction->setShortcutContext(Qt::WidgetShortcut);
     connect(delSymAction, SIGNAL(triggered()), SLOT(sl_delSym()));
     
     delColAction = new QAction(tr("del_col"), this);
     delColAction->setShortcut(QKeySequence(Qt::SHIFT| Qt::Key_Delete));
+    delColAction->setShortcutContext(Qt::WidgetShortcut);
     connect(delColAction, SIGNAL(triggered()), SLOT(sl_delCol()));
     
     insSymAction = new QAction(tr("ins_sym"), this);
     insSymAction->setShortcut(QKeySequence(Qt::Key_Space));
+    insSymAction->setShortcutContext(Qt::WidgetShortcut);
     connect(insSymAction, SIGNAL(triggered()), SLOT(sl_insSym()));
     
     insColAction = new QAction(tr("ins_col"), this);
     insColAction->setShortcut(QKeySequence(Qt::SHIFT| Qt::Key_Space));
+    insColAction->setShortcutContext(Qt::WidgetShortcut);
     connect(insColAction, SIGNAL(triggered()), SLOT(sl_insCol()));
     
     gotoAction = new QAction(QIcon(":core/images/goto.png"), tr("goto_pos"), this);
     gotoAction->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_G));
+    insColAction->setShortcutContext(Qt::WindowShortcut);
     connect(gotoAction, SIGNAL(triggered()), SLOT(sl_goto()));
 
+    takeScreenshotAction = new QAction(QIcon(":core/images/cam2.png"), tr("Capture scren"), this);
+    connect(takeScreenshotAction, SIGNAL(triggered()), ui, SLOT(sl_saveScreenshot()));
+
+    increaseFontAction = new QAction(QIcon(":core/images/zoom_in.png"), tr("Increase Font"), this);
+    connect(increaseFontAction, SIGNAL(triggered()), SLOT(sl_zoomIn()));
+    
+    decreseFontAction = new QAction(QIcon(":core/images/zoom_out.png"), tr("Decrease Font"), this);
+    connect(decreseFontAction, SIGNAL(triggered()), SLOT(sl_zoomOut()));
+    
+    changeFontAction = new QAction(QIcon(":core/images/font.png"), tr("Change Font"), this);
+    connect(changeFontAction, SIGNAL(triggered()), SLOT(sl_changeFont()));
+
+    resetFontAction = new QAction(QIcon(":core/images/font_default.png"), tr("Reset Font"), this);
+    connect(resetFontAction, SIGNAL(triggered()), SLOT(sl_resetFont()));
 
     removeGapColumnsAction = new QAction(QIcon(":core/images/msaed_remove_columns_with_gaps.png"), tr("Remove columns with gaps"), this);
     connect(removeGapColumnsAction, SIGNAL(triggered()), SLOT(sl_removeColumnsWithGaps()));
@@ -72,27 +118,74 @@ MSAEditorSequenceArea::MSAEditorSequenceArea(MSAEditorUI* _ui) : editor(_ui->edi
     removeAllGapsAction = new QAction(QIcon(":core/images/msaed_remove_all_gaps.png"), tr("Remove all gaps"), this);
     connect(removeAllGapsAction, SIGNAL(triggered()), SLOT(sl_removeAllGaps()));
     
-    
-    connect(ui->nameList, SIGNAL(itemSelectionChanged()), SLOT(sl_nameListItemSelectionChanged()));
+    connect(editor->getMSAObject(), SIGNAL(si_alignmentChanged(const MAlignment&, const MAlignmentModInfo&)), 
+        SLOT(sl_alignmentChanged(const MAlignment&, const MAlignmentModInfo&)));
 
-
-    //synchronize 2 vertical scrollbars
-    QScrollBar* nlBar = ui->nameList->verticalScrollBar();
-    connect(nlBar, SIGNAL(valueChanged(int)), ui->svBar, SLOT(setValue(int)));
-    connect(ui->svBar, SIGNAL(valueChanged(int)), nlBar, SLOT(setValue(int)));
-    connect(nlBar, SIGNAL(rangeChanged(int, int)),SLOT(sl_vScrollRangeChanged(int, int)));
-    connect(ui->svBar, SIGNAL(valueChanged(int)), SLOT(sl_onVScrollMoved(int)));
-    connect(editor->getMSAObject(), SIGNAL(si_alignmentModified()), SLOT(sl_alignmentModified()));
-    connect(editor->getMSAObject(), SIGNAL(si_sequenceListModified()), SLOT(sl_sequenceListModified()));
     connect(editor, SIGNAL(si_buildStaticMenu(GObjectView*, QMenu*)), SLOT(sl_buildStaticMenu(GObjectView*, QMenu*)));
     connect(editor, SIGNAL(si_buildStaticToolbar(GObjectView*, QToolBar*)), SLOT(sl_buildStaticToolbar(GObjectView*, QToolBar*)));
     connect(editor, SIGNAL(si_buildPopupMenu(GObjectView* , QMenu*)), SLOT(sl_buildContextMenu(GObjectView*, QMenu*)));
     connect(editor->getMSAObject(), SIGNAL(si_lockedStateChanged()), SLOT(sl_lockedStateChanged()));
 
-
+    prepareColorSchemeMenuActions();
     updateActions();
 }
 
+MSAEditorSequenceArea::~MSAEditorSequenceArea() {
+    delete cachedView;
+}
+
+void MSAEditorSequenceArea::prepareColorSchemeMenuActions() {
+    Settings* s = AppContext::getSettings();
+    MAlignmentObject* maObj = editor->getMSAObject();
+    DNAAlphabetType atype = maObj->getMAlignment().alphabet->getType();
+    MSAColorSchemeRegistry* csr = AppContext::getMSAColorSchemeRegistry();
+    QString csid = atype == DNAAlphabet_AMINO ? 
+            s->getValue(SETTINGS_ROOT + SETTINGS_COLOR_AMINO, MSAColorScheme::UGENE_AMINO).toString()
+          : s->getValue(SETTINGS_ROOT + SETTINGS_COLOR_NUCL, MSAColorScheme::UGENE_NUCL).toString();
+
+    MSAColorSchemeFactory* csf = csr->getMSAColorSchemeFactoryById(csid);
+    if (csf == NULL) {
+        csf = csr->getMSAColorSchemeFactoryById(atype == DNAAlphabet_AMINO ? MSAColorScheme::UGENE_AMINO : MSAColorScheme::UGENE_NUCL);
+    }
+    assert(csf!=NULL);
+    colorScheme = csf->create(this, maObj);
+
+    QList<MSAColorSchemeFactory*> factories= csr->getMSAColorSchemes(atype);
+    foreach(MSAColorSchemeFactory* f, factories) {
+        QAction* action = new QAction(f->getName(), this);
+        action->setCheckable(true);
+        action->setChecked(f == csf);
+        action->setData(f->getId());
+        connect(action, SIGNAL(triggered()), SLOT(sl_changeColorScheme()));
+        colorSchemeMenuActions.append(action);
+    }
+}
+
+void MSAEditorSequenceArea::sl_changeColorScheme() {
+    QAction* a = qobject_cast<QAction*>(sender());
+    QString id = a->data().toString();
+    MSAColorSchemeFactory* f = AppContext::getMSAColorSchemeRegistry()->getMSAColorSchemeFactoryById(id);
+    delete colorScheme;
+    colorScheme = f->create(this, ui->editor->getMSAObject());
+    foreach(QAction* action, colorSchemeMenuActions) {
+        action->setChecked(action == a);
+    }
+    if (f->getAlphabetType() == DNAAlphabet_AMINO) {
+        AppContext::getSettings()->setValue(SETTINGS_ROOT + SETTINGS_COLOR_AMINO, id);
+    } else {
+        AppContext::getSettings()->setValue(SETTINGS_ROOT + SETTINGS_COLOR_NUCL, id);
+    }
+
+    completeRedraw = true;
+    update();
+}
+
+void MSAEditorSequenceArea::updateSeqFontMetrics() {
+    QFontMetrics seqFM(seqFont);
+    seqCharWidth = seqFM.width('W') * 5 / 4;
+    seqCharHeight = seqFM.height() * 5 / 4;
+}
+
 void MSAEditorSequenceArea::updateActions() {
     bool readOnly = editor->getMSAObject()->isStateLocked();
     
@@ -102,46 +195,52 @@ void MSAEditorSequenceArea::updateActions() {
     insSymAction->setEnabled(!readOnly);
     removeGapColumnsAction->setEnabled(!readOnly);
     removeAllGapsAction->setEnabled(!readOnly);
-}
 
-LRegion MSAEditorSequenceArea::getSequenceYRange(int n, bool useVirsualCoords) const {
-#ifdef _DEBUG
-    int seqAreaY = geometry().y();
-    int nameListY = ui->nameList->geometry().y();
-    assert(seqAreaY == nameListY);
-    int numSeqs = editor->getNumSequences();
-    assert(n >=0 && n < numSeqs);
-#endif
-    return ui->nameList->getSequenceYRange(n, useVirsualCoords);
+    increaseFontAction->setEnabled(seqFont.pointSize() < MAX_FONT_SIZE);
+    decreseFontAction->setEnabled(seqFont.pointSize() > MIN_FONT_SIZE);
+    assert(checkState());
 }
 
 
 
 void MSAEditorSequenceArea::paintEvent(QPaintEvent *e) {
-    QPainter p(this);
-    p.fillRect(rect(), Qt::white);
-
-    drawSequences(p);
-
-    drawCursor(p);
-
+    drawAll();
     QWidget::paintEvent(e);
 }
 
-void MSAEditorSequenceArea::drawSequences(QPainter& p) {
-    p.setFont(seqFont);
+void MSAEditorSequenceArea::drawAll() {
+    QSize s = size();
+    if (cachedView->size() != s) {
+        assert(completeRedraw);
+        delete cachedView;
+        cachedView = new QPixmap(s);
+    }
+    if (completeRedraw) {
+        QPainter pCached(cachedView);
+        drawContent(pCached);
+        completeRedraw = false;
+    }
+    QPainter p(this);
+    p.drawPixmap(0, 0, *cachedView);
+    drawCursor(p);
+    drawFocus(p);
+}
 
+void MSAEditorSequenceArea::drawContent(QPainter& p) {
+    p.fillRect(cachedView->rect(), Qt::white);
+    p.setFont(seqFont);
+    
     //for every sequence in msa starting from first visible
     //draw it starting from startPos
-    int firstVisible = ui->nameList->getFirstVisibleSequence();
-    int lastVisible = ui->nameList->getLastVisibleSequence(true);
+    int firstVisibleSeq = getFirstVisibleSequence();
+    int lastVisibleSeq  = getLastVisibleSequence(true);
     int lastPos = getLastVisibleBase(true);
     int w = width();
     int h = height();
     const MAlignment& msa = editor->getMSAObject()->getMAlignment();
-    for (int i = firstVisible; i <= lastVisible; i++) {
-        const MAlignmentItem& item = msa.alignedSeqs[i];
-        LRegion baseYRange = getSequenceYRange(i, true);
+    for (int seq = firstVisibleSeq; seq <= lastVisibleSeq; seq++) {
+        const MAlignmentItem& item = msa.alignedSeqs[seq];
+        LRegion baseYRange = getSequenceYRange(seq, true);
 
         //draw horizontal grid
         //p.drawLine(0, r.startPos, width(), r.startPos); p.drawLine(0, r.endPos(), width(), r.endPos());
@@ -149,10 +248,10 @@ void MSAEditorSequenceArea::drawSequences(QPainter& p) {
         for (int pos = startPos; pos <= lastPos; pos++) {
             LRegion baseXRange = getBaseXRange(pos, true);
             QRect cr(baseXRange.startPos, baseYRange.startPos, baseXRange.len+1, baseYRange.len);
-            Q_ASSERT(cr.left() < w && cr.top() < h);
-            char c = item.sequence[pos];
-            if (c != MAlignment_GapChar) {
-                QColor color = colorScheme->getColor(c);
+            assert(cr.left() < w && cr.top() < h); Q_UNUSED(w); Q_UNUSED(h);
+            char c = msa.charAt(seq, pos);
+            QColor color = colorScheme->getColor(seq, pos);
+            if (color.isValid()) {
                 p.fillRect(cr, color);
             }
             p.drawText(cr, Qt::AlignCenter, QString(c));
@@ -161,29 +260,36 @@ void MSAEditorSequenceArea::drawSequences(QPainter& p) {
 }
 
 void MSAEditorSequenceArea::drawCursor(QPainter& p) {
-    if (!isPosVisible(cursorPos, true)) {
+    if (!isVisible(cursorPos, true)) {
         return;
     }
     LRegion xRange = getBaseXRange(cursorPos.x(), true);
     LRegion yRange = getSequenceYRange(cursorPos.y(), true);
 
-    QPen pen(hasFocus()? Qt::black : Qt::gray);
+    QPen pen(highlightSelection || hasFocus()? Qt::black : Qt::gray);
     pen.setStyle(Qt::DashLine);
+    pen.setWidth(highlightSelection ? 2 : 1);
     p.setPen(pen);
     p.drawRect(xRange.startPos, yRange.startPos, xRange.len, yRange.len);
 }
 
+void MSAEditorSequenceArea::drawFocus(QPainter& p) {
+    if (hasFocus()) {
+        p.setPen(QPen(Qt::black, 1, Qt::DotLine));
+        p.drawRect(0, 0, width()-1, height()-1);
+    }
+}
+
+
 bool MSAEditorSequenceArea::isPosInRange(int p) const {
     return p >= 0 && p < editor->getAlignmentLen();
 }
 
-bool MSAEditorSequenceArea::isPosInRange(const QPoint& p) const {
-    bool xInRange = isPosInRange(p.x());
-    int numSeqs = editor->getNumSequences();
-    bool res = xInRange && p.y() >= 0 && p.y() < numSeqs;
-    return res;
+bool MSAEditorSequenceArea::isSeqInRange(int s) const {
+    return s >= 0 && s < editor->getNumSequences();
 }
 
+
 bool MSAEditorSequenceArea::isPosVisible(int pos, bool countClipped) const {
     if (pos < getFirstVisibleBase() || pos > getLastVisibleBase(countClipped)) {
         return false;
@@ -191,114 +297,187 @@ bool MSAEditorSequenceArea::isPosVisible(int pos, bool countClipped) const {
     return true;
 }
 
-bool MSAEditorSequenceArea::isPosVisible(const QPoint& p, bool countClipped) const {
-    
-    //check that sequence is visible
-    bool yVisible = ui->nameList->isSequenceVisible(p.y(), countClipped);
-    if (!yVisible) {
+bool MSAEditorSequenceArea::isSeqVisible(int seq, bool countClipped) const {
+    if (seq < getFirstVisibleSequence() || seq > getLastVisibleSequence(countClipped)) {
         return false;
     }
-
-    //check that pos is sequence visible
-    return isPosVisible(p.x(), countClipped);
+    return true;
 }
 
+
 void MSAEditorSequenceArea::setFirstVisibleBase(int pos) {
     Q_ASSERT(isPosInRange(pos));
     if (pos == startPos) {
         return;
     }
+    QPoint prev(startPos, startSeq);
     int aliLen = editor->getAlignmentLen();
-    int effectiveFirst = qMin(aliLen - countSpaceForBases(false), pos);
-    startPos = effectiveFirst;
+    int effectiveFirst = qMin(aliLen - countWidthForBases(false), pos);
+    startPos = qMax(0, effectiveFirst);
 
     updateHScrollBar();
-    update();
 
-    emit si_startPosChanged(pos);
+    QPoint p(startPos, startSeq);
+    emit si_startChanged(p,  prev);
+
+    completeRedraw = true;
+    update();
 }
 
-void MSAEditorSequenceArea::resizeEvent(QResizeEvent *e) {
-
-    int aliLen = editor->getAlignmentLen();
-    int visibleSymbolsCount = countSpaceForBases(false);
-    if (visibleSymbolsCount > aliLen) {
-        setFirstVisibleBase(0);
-    } else if (startPos + visibleSymbolsCount > aliLen) {
-        setFirstVisibleBase(aliLen - visibleSymbolsCount);
+void MSAEditorSequenceArea::setFirstVisibleSequence(int seq) {
+    Q_ASSERT(isSeqInRange(seq));
+    if (seq == startSeq) {
+        return;
     }
+    QPoint prev(startPos, startSeq);
 
-    Q_ASSERT(startPos >= 0);
-    Q_ASSERT((aliLen >= startPos + visibleSymbolsCount) || aliLen < visibleSymbolsCount);
+    int nSeq = editor->getNumSequences();
+    int effectiveFirst = qMin(nSeq - countHeightForSequences(false), seq);
+    startSeq = qMax(0, effectiveFirst);
 
-    updateHScrollBar();
+    updateVScrollBar();
 
+    QPoint p(startPos, startSeq);
+    emit si_startChanged(p, prev);
+
+    completeRedraw = true;
+    update();
+}
+
+
+void MSAEditorSequenceArea::resizeEvent(QResizeEvent *e) {
+    completeRedraw = true;
+    validateRanges();
     QWidget::resizeEvent(e);
 }
 
+void MSAEditorSequenceArea::validateRanges() {
+    //check x dimension
+    int aliLen = editor->getAlignmentLen();
+    int visibleBaseCount = countWidthForBases(false);
+    if (visibleBaseCount > aliLen) {
+        setFirstVisibleBase(0);
+    } else if (startPos + visibleBaseCount > aliLen) {
+        setFirstVisibleBase(aliLen - visibleBaseCount);
+    }
+    assert(startPos >= 0);
+    assert((aliLen >= startPos + visibleBaseCount) || aliLen < visibleBaseCount);
+    updateHScrollBar();
+
+    //check y dimension
+    int nSeqs = editor->getNumSequences();
+    int visibleSequenceCount = countHeightForSequences(false);
+    if (visibleSequenceCount > nSeqs) {
+        setFirstVisibleSequence(0);
+    } else if (startSeq + visibleSequenceCount > nSeqs) {
+        setFirstVisibleSequence(nSeqs - visibleSequenceCount);
+    }
+    assert(startSeq >= 0);
+    assert((nSeqs >= startSeq + visibleSequenceCount) || nSeqs < visibleSequenceCount);
+    updateVScrollBar();
+
+}
+
 void MSAEditorSequenceArea::sl_onHScrollMoved(int pos) {
-    Q_ASSERT(pos >=0 && pos <= editor->getAlignmentLen() - getNumVisibleBases(false));
+    assert(pos >=0 && pos <= editor->getAlignmentLen() - getNumVisibleBases(false));    
     setFirstVisibleBase(pos);
 }
 
+void MSAEditorSequenceArea::sl_onVScrollMoved(int seq) {
+    assert(seq >=0 && seq <= editor->getNumSequences() - getNumVisibleSequences(false));    
+    setFirstVisibleSequence(seq);
+}
+
 void MSAEditorSequenceArea::updateHScrollBar() {
-    ui->shBar->disconnect(this);
+    shBar->disconnect(this);
 
     int numVisibleBases = getNumVisibleBases(false);
     int alignmentLen = editor->getAlignmentLen();
     Q_ASSERT(numVisibleBases <= alignmentLen);
 
-    ui->shBar->setMinimum(0);
-    ui->shBar->setMaximum(alignmentLen - numVisibleBases);
-    ui->shBar->setSliderPosition(getFirstVisibleBase());
+    shBar->setMinimum(0);
+    shBar->setMaximum(alignmentLen - numVisibleBases);
+    shBar->setSliderPosition(getFirstVisibleBase());
 
-    ui->shBar->setSingleStep(1);
-    ui->shBar->setPageStep(numVisibleBases);
+    shBar->setSingleStep(1);
+    shBar->setPageStep(numVisibleBases);
 
-    ui->shBar->setDisabled(numVisibleBases == alignmentLen);
+    shBar->setDisabled(numVisibleBases == alignmentLen);
 
-    connect(ui->shBar, SIGNAL(valueChanged(int)), SLOT(sl_onHScrollMoved(int)));
+    connect(shBar, SIGNAL(valueChanged(int)), SLOT(sl_onHScrollMoved(int)));
+}
+
+void MSAEditorSequenceArea::updateVScrollBar() {
+    svBar->disconnect(this);
+
+    int numVisibleSequences = getNumVisibleSequences(false);
+    int nSeqs = editor->getNumSequences();
+    Q_ASSERT(numVisibleSequences <= nSeqs);
+
+    svBar->setMinimum(0);
+    svBar->setMaximum(nSeqs - numVisibleSequences);
+    svBar->setSliderPosition(getFirstVisibleSequence());
+
+    svBar->setSingleStep(1);
+    svBar->setPageStep(numVisibleSequences);
+
+    svBar->setDisabled(numVisibleSequences == nSeqs);
+
+    connect(svBar, SIGNAL(valueChanged(int)), SLOT(sl_onVScrollMoved(int)));
 }
 
 
-int MSAEditorSequenceArea::countSpaceForBases(bool countClipped) const {
+int MSAEditorSequenceArea::countWidthForBases(bool countClipped) const {
     int seqAreaWidth = width();
     int nVisible = seqAreaWidth / seqCharWidth + (countClipped && (seqAreaWidth % seqCharWidth != 0) ? 1 : 0);
     return nVisible;
 }
 
+int MSAEditorSequenceArea::countHeightForSequences(bool countClipped) const {
+    int seqAreaHeight = height();
+    int nVisible = seqAreaHeight / seqCharHeight + (countClipped && (seqAreaHeight % seqCharHeight != 0) ? 1 : 0);
+    return nVisible;
+}
+
 int MSAEditorSequenceArea::getNumVisibleBases(bool countClipped) const {
-    int lastVisible =  getLastVisibleBase(countClipped);
-    assert(lastVisible >= startPos);
+    int lastVisible = getLastVisibleBase(countClipped);
+    assert(lastVisible >= startPos || (!countClipped && lastVisible + 1 == startPos /*1 symbol is visible & clipped*/));
     assert(lastVisible < editor->getAlignmentLen());
     int res = lastVisible - startPos + 1;
     return res;
 }
 
 int MSAEditorSequenceArea::getLastVisibleBase(bool countClipped) const {
-    int nVisible = countSpaceForBases(countClipped);
+    int nVisible = countWidthForBases(countClipped);
     int alignLen = editor->getAlignmentLen();
     int res = qBound(0, startPos + nVisible - 1, alignLen - 1);
     return res;
 }
 
-void MSAEditorSequenceArea::sl_onVScrollMoved(int pos) {
-    Q_UNUSED(pos);
-    update();
+int MSAEditorSequenceArea::getLastVisibleSequence(bool countClipped) const {
+    int nVisible = countHeightForSequences(countClipped);
+    int numSeqs = editor->getNumSequences();
+    int res = qBound(0, startSeq + nVisible - 1, numSeqs - 1);
+    return res;
 }
 
-void MSAEditorSequenceArea::sl_nameListItemSelectionChanged() {
-    update();
+int MSAEditorSequenceArea::getNumVisibleSequences(bool countClipped) const {
+    int lastVisible =  getLastVisibleSequence(countClipped);
+    assert(lastVisible >= startSeq);
+    assert(lastVisible < editor->getNumSequences());
+    int res = lastVisible - startSeq + 1;
+    return res;
 }
 
-void MSAEditorSequenceArea::sl_vScrollRangeChanged(int min, int max) {
-    ui->svBar->setMinimum(min);    
-    ui->svBar->setMaximum(max);    
-    ui->svBar->setSingleStep(ui->nameList->verticalScrollBar()->singleStep());    
-    ui->svBar->setPageStep(ui->nameList->verticalScrollBar()->pageStep());    
+int MSAEditorSequenceArea::getSequenceNumByY(int y) {
+    int seqOffs = y / seqCharHeight;
+    int seq = startSeq + seqOffs;
+    if (seq >= editor->getNumSequences()) {
+        return -1; 
+    }
+    return seq;
 }
 
-
 LRegion MSAEditorSequenceArea::getBaseXRange(int pos, bool useVirtualCoords) const {
     LRegion res(seqCharWidth * (pos - startPos), seqCharWidth);
     if (!useVirtualCoords) {
@@ -308,6 +487,17 @@ LRegion MSAEditorSequenceArea::getBaseXRange(int pos, bool useVirtualCoords) con
     return res;
 }
 
+LRegion MSAEditorSequenceArea::getSequenceYRange(int seq, bool useVirtualCoords) const {
+    LRegion res(seqCharHeight* (seq - startSeq), seqCharHeight);
+    if (!useVirtualCoords) {
+        int h = height();
+        res = res.intersect(LRegion(0, h));
+    }
+    return res;
+}
+
+
+
 void MSAEditorSequenceArea::mousePressEvent(QMouseEvent *e) {
     if (!hasFocus()) {
         setFocus();
@@ -325,6 +515,9 @@ void MSAEditorSequenceArea::mousePressEvent(QMouseEvent *e) {
 }
 
 void MSAEditorSequenceArea::keyPressEvent(QKeyEvent *e) {
+    if (!hasFocus()) {
+        return;
+    }
     int key = e->key();
     bool shift = e->modifiers().testFlag(Qt::ShiftModifier);
     bool ctrl = e->modifiers().testFlag(Qt::ControlModifier);
@@ -336,20 +529,20 @@ void MSAEditorSequenceArea::keyPressEvent(QKeyEvent *e) {
     //part of these keys are assigned to actions -> so them never passed to keyPressEvent (action handling has higher priority)
     switch(key) {
         case Qt::Key_Left:
-            moveCursorRelative(-1, 0);    
+            moveCursor(-1, 0);    
             break;
         case Qt::Key_Right:
-            moveCursorRelative(1, 0);    
+            moveCursor(1, 0);    
             break;
         case Qt::Key_Up:
-            moveCursorRelative(0, -1);    
+            moveCursor(0, -1);    
             break;
         case Qt::Key_Down:
-            moveCursorRelative(0, 1);    
+            moveCursor(0, 1);    
             break;
         case Qt::Key_Home:
             if (shift) { //scroll namelist
-                ui->nameList->setFirstVisibleSequence(0);
+                setFirstVisibleSequence(0);
                 setCursorPos(QPoint(cursorPos.x(), 0));
             } else { //scroll sequence
                 setFirstVisibleBase(0);
@@ -359,7 +552,7 @@ void MSAEditorSequenceArea::keyPressEvent(QKeyEvent *e) {
         case Qt::Key_End:
             if (shift) { //scroll namelist
                 int n = editor->getNumSequences() - 1;
-                ui->nameList->setFirstVisibleSequence(n);
+                setFirstVisibleSequence(n);
                 setCursorPos(QPoint(cursorPos.x(), n));
             } else { //scroll sequence
                 int n = editor->getAlignmentLen() - 1;
@@ -369,10 +562,10 @@ void MSAEditorSequenceArea::keyPressEvent(QKeyEvent *e) {
             break;
         case Qt::Key_PageUp:
             if (shift) { //scroll namelist
-                int nVis = ui->nameList->getNumVisibleSequences(false);
-                int fp = qMax(0, ui->nameList->getFirstVisibleSequence() - nVis);
+                int nVis = getNumVisibleSequences(false);
+                int fp = qMax(0, getFirstVisibleSequence() - nVis);
                 int cp = qMax(0, cursorPos.y() - nVis);
-                ui->nameList->setFirstVisibleSequence(fp);
+                setFirstVisibleSequence(fp);
                 setCursorPos(QPoint(cursorPos.x(), cp));
             } else { //scroll sequence
                 int nVis = getNumVisibleBases(false);
@@ -384,11 +577,11 @@ void MSAEditorSequenceArea::keyPressEvent(QKeyEvent *e) {
             break;
         case Qt::Key_PageDown:
             if (shift) { //scroll namelist
-                int nVis = ui->nameList->getNumVisibleSequences(false);
+                int nVis = getNumVisibleSequences(false);
                 int nSeq = editor->getNumSequences();
-                int fp = qMin(nSeq-1, ui->nameList->getFirstVisibleSequence() + nVis);
+                int fp = qMin(nSeq-1, getFirstVisibleSequence() + nVis);
                 int cp = qMin(nSeq-1, cursorPos.y() + nVis);
-                ui->nameList->setFirstVisibleSequence(fp);
+                setFirstVisibleSequence(fp);
                 setCursorPos(QPoint(cursorPos.x(), cp));
             } else { //scroll sequence
                 int nVis = getNumVisibleBases(false);
@@ -416,33 +609,48 @@ void MSAEditorSequenceArea::keyPressEvent(QKeyEvent *e) {
     QWidget::keyPressEvent(e);
 }
 
+void MSAEditorSequenceArea::focusInEvent(QFocusEvent* fe) {
+    QWidget::focusInEvent(fe);
+    update();
+}
 
-void MSAEditorSequenceArea::moveCursorRelative(int dx, int dy) {
+void MSAEditorSequenceArea::focusOutEvent(QFocusEvent* fe) {
+    QWidget::focusOutEvent(fe);
+    update();
+}
+
+
+void MSAEditorSequenceArea::moveCursor(int dx, int dy) {
     QPoint p = cursorPos + QPoint(dx, dy);
-    if (!isPosInRange(p)) {
+    if (!isInRange(p)) {
         return;
     }   
-    if (!isPosVisible(p, false)) {
-        if (isPosVisible(cursorPos, true)) {
+    if (!isVisible(p, false)) {
+        if (isVisible(cursorPos, true)) {
             if (dx != 0) { 
                 setFirstVisibleBase(startPos + dx);
             } 
             if (dy!=0) {
-                ui->nameList->setFirstVisibleSequence(ui->nameList->getFirstVisibleSequence()+dy);
+                setFirstVisibleSequence(getFirstVisibleSequence()+dy);
             }
         } else {
             setFirstVisibleBase(p.x());
-            ui->nameList->setFirstVisibleSequence(p.y());
+            setFirstVisibleSequence(p.y());
         }
     }
     setCursorPos(p);
 }
 
 
+int MSAEditorSequenceArea::coordToPos(int x) const {
+    int y = getSequenceYRange(getFirstVisibleSequence(), false).startPos;
+    return coordToPos(QPoint(x, y)).x();
+}
+
 QPoint MSAEditorSequenceArea::coordToPos(const QPoint& coord) const {
     QPoint res(-1, -1);
     //Y: row
-    for (int i=ui->nameList->getFirstVisibleSequence(), n = ui->nameList->getLastVisibleSequence(true); i<=n; i++) {
+    for (int i=getFirstVisibleSequence(), n = getLastVisibleSequence(true); i<=n; i++) {
         LRegion r = getSequenceYRange(i, false);
         if (r.contains(coord.y())) {
             res.setY(i);
@@ -461,71 +669,79 @@ QPoint MSAEditorSequenceArea::coordToPos(const QPoint& coord) const {
     return res;
 }
 
-
 void MSAEditorSequenceArea::setCursorPos(const QPoint& p) {
-    assert(isPosInRange(p));
+    assert(isInRange(p));
     if (p == cursorPos) {
         return;
     }
     
-    bool up = isPosVisible(cursorPos, true) || isPosVisible(p, true);
-    bool upNameList = up && cursorPos.y() != p.y();
+    bool up = isVisible(cursorPos, true) || isVisible(p, true);
     QPoint prev = cursorPos;
     cursorPos = p;
     
-    emit si_cursorPosChanged(cursorPos, prev);
+    emit si_cursorMoved(cursorPos, prev);
     
-    if (upNameList) {
-        QListWidgetItem* item = ui->nameList->item(cursorPos.y());
-        ui->nameList->setCurrentItem(item, QItemSelectionModel::ClearAndSelect);
-    }
     if (up) {
         update();
     }
+    highlightSelection = false;
     updateActions();
 }
 
 void MSAEditorSequenceArea::ins(const QPoint& p, bool columnMode) {
-    assert(isPosInRange(p));
-    if (editor->getMSAObject()->isStateLocked()) {
+    assert(isInRange(p));
+    MAlignmentObject* maObj = editor->getMSAObject();
+    if (maObj->isStateLocked()) {
         return;
     }
     if (columnMode) {
-        editor->getMSAObject()->insertGap(p.x(), 1);
+        maObj->insertGap(p.x(), 1);
     } else {
-        editor->getMSAObject()->insertGap(p.y(), p.x(), 1);
+        maObj->insertGap(p.y(), p.x(), 1);
     }
 }
 
 void MSAEditorSequenceArea::del(const QPoint& p, bool columnMode) {
-    assert(isPosInRange(p));
-    if (editor->getMSAObject()->isStateLocked()) {
+    assert(isInRange(p));
+    MAlignmentObject* maObj = editor->getMSAObject();
+    if (maObj->isStateLocked()) {
         return;
     }
     if (columnMode) {
-        editor->getMSAObject()->deleteGap(p.x(), 1);
+        maObj->deleteGap(p.x(), 1);
     } else {
-        editor->getMSAObject()->deleteGap(p.y(), p.x(), 1);
+        maObj->deleteGap(p.y(), p.x(), 1);
     }
 }
 
-void MSAEditorSequenceArea::sl_alignmentModified() {
+void MSAEditorSequenceArea::sl_alignmentChanged(const MAlignment&, const MAlignmentModInfo&) {
     int aliLen = editor->getAlignmentLen();
-    setFirstVisibleBase(qBound(0, startPos, aliLen-countSpaceForBases(false)));
-    setCursorPos(qMin(cursorPos.x(), aliLen-1));
+    int nSeq = editor->getNumSequences();
+    
+    //todo: set in one method!
+    setFirstVisibleBase(qBound(0, startPos, aliLen-countWidthForBases(false)));
+    setFirstVisibleSequence(qBound(0, startSeq, nSeq - countHeightForSequences(false)));
+
+    setCursorPos(qMin(cursorPos.x(), aliLen-1), qMin(cursorPos.y(), nSeq-1));
+
     updateHScrollBar();
+    updateVScrollBar();
+
+    completeRedraw = true;
     update();
 }
 
-void MSAEditorSequenceArea::sl_sequenceListModified() {
-    sl_alignmentModified();
-}
-
-void MSAEditorSequenceArea::sl_buildStaticToolbar(GObjectView* v, QToolBar* t) {
-    Q_UNUSED(v);
+void MSAEditorSequenceArea::sl_buildStaticToolbar(GObjectView*, QToolBar* t) {
     t->addAction(gotoAction);
     t->addAction(removeGapColumnsAction);
     t->addAction(removeAllGapsAction);
+    t->addAction(takeScreenshotAction);
+    t->addAction(increaseFontAction);
+    t->addAction(decreseFontAction);
+    t->addAction(changeFontAction);
+    t->addAction(resetFontAction);
+    t->addAction(ui->getUndoAction());
+    t->addAction(ui->getRedoAction());
 }
 
 void MSAEditorSequenceArea::sl_buildStaticMenu(GObjectView* v, QMenu* m) {
@@ -550,6 +766,13 @@ void MSAEditorSequenceArea::buildMenu(QMenu* m) {
         << removeGapColumnsAction << removeAllGapsAction;
 
     editMenu->insertActions(editMenu->isEmpty() ? NULL : editMenu->actions().first(), actions);
+
+    QMenu* colorsSchemeMenu = new QMenu(tr("Colors"), m);
+    colorsSchemeMenu->setIcon(QIcon(":core/images/color_wheel.png"));
+    foreach(QAction* a, colorSchemeMenuActions) {
+        colorsSchemeMenu->addAction(a);
+    }
+    m->insertMenu(GUIUtils::findAction(m->actions(), MSAE_MENU_EDIT), colorsSchemeMenu);
 }
 
 
@@ -590,12 +813,12 @@ void MSAEditorSequenceArea::sl_lockedStateChanged() {
 }
 
 void MSAEditorSequenceArea::centerPos(const QPoint& pos) {
-    assert(isPosInRange(pos));
+    assert(isInRange(pos));
     int newStartPos = qMax(0, pos.x() - getNumVisibleBases(false)/2);
     setFirstVisibleBase(newStartPos);
 
-    int newStartSeq = qMax(0, pos.y() - ui->nameList->getNumVisibleSequences(false)/2);
-    ui->nameList->setFirstVisibleSequence(newStartSeq);
+    int newStartSeq = qMax(0, pos.y() - getNumVisibleSequences(false)/2);
+    setFirstVisibleSequence(newStartSeq);
 }
 
 
@@ -607,9 +830,9 @@ void MSAEditorSequenceArea::centerPos(int pos) {
 void MSAEditorSequenceArea::wheelEvent (QWheelEvent * we) {
     bool toMin = we->delta() > 0;
     if (we->modifiers() == 0) {
-        ui->shBar->triggerAction(toMin ? QAbstractSlider::SliderSingleStepSub : QAbstractSlider::SliderSingleStepAdd);
+        shBar->triggerAction(toMin ? QAbstractSlider::SliderSingleStepSub : QAbstractSlider::SliderSingleStepAdd);
     }  else if (we->modifiers() & Qt::SHIFT) {
-        ui->nameList->verticalScrollBar()->triggerAction(toMin ? QAbstractSlider::SliderSingleStepSub : QAbstractSlider::SliderSingleStepAdd);
+        svBar->triggerAction(toMin ? QAbstractSlider::SliderSingleStepSub : QAbstractSlider::SliderSingleStepAdd);
     }
     QWidget::wheelEvent(we);
 }
@@ -671,56 +894,76 @@ void MSAEditorSequenceArea::sl_removeAllGaps() {
     if (changed) {
         ma.normalizeModel();
         msa->setMAlignment(ma);
+        setFirstVisibleBase(0);
+        setFirstVisibleSequence(0);
     }
 }
 
-//////////////////////////////////////////////////////////////////////////
-MSAEditorColorScheme::MSAEditorColorScheme(QObject* p, bool amino) : QObject(p) {
-    colorsPerChar.fill(Qt::white, 256);
-    for (int i = 0; i < 256; i++) {
-        colorsPerChar[i] = GUIUtils::genLightColor(QString((char)i));
+bool MSAEditorSequenceArea::checkState() const {
+#ifdef _DEBUG
+    int aliLen = editor->getMSAObject()->getMAlignment().getLength();
+    int nSeqs = editor->getMSAObject()->getMAlignment().getNumSequences();
+    
+    assert(startPos >=0 && startSeq >=0);
+    int lastPos = getLastVisibleBase(true);
+    int lastSeq = getLastVisibleSequence(true);
+    assert(lastPos < aliLen && lastSeq < nSeqs);
+    
+    int cx = cursorPos.x();
+    int cy = cursorPos.y();
+    assert(cx >= 0 && cy >= 0);
+    assert(cx < aliLen && cy < nSeqs);
+#endif
+    return true;
+}
+
+static void saveFont(const QFont& f) {
+    Settings* s = AppContext::getSettings();
+    s->setValue(SETTINGS_ROOT + SETTINGS_FONT_FAMILY, f.family());
+    s->setValue(SETTINGS_ROOT + SETTINGS_FONT_SIZE, f.pointSize());
+    s->setValue(SETTINGS_ROOT + SETTINGS_FONT_ITALIC, f.italic());
+    s->setValue(SETTINGS_ROOT + SETTINGS_FONT_BOLD, f.bold());
+}
+
+void MSAEditorSequenceArea::setFont(const QFont& f) {
+    seqFont = f;
+    seqFont.setPointSize(qBound(MIN_FONT_SIZE, f.pointSize(), MAX_FONT_SIZE));
+    completeRedraw = true;
+    updateSeqFontMetrics();
+    validateRanges();
+    updateActions();
+    emit si_scaleChanged();
+    update();
+    saveFont(seqFont);
+}
+
+void MSAEditorSequenceArea::sl_changeFont() {
+    bool ok = false;
+    QFont f = QFontDialog::getFont(&ok, seqFont, this, tr("Select font for alignment"));
+    if (!ok) {
+        return;
     }
+    setFont(f);
+}
 
-    //fine tune most used colors
-    colorsPerChar['-'] = Qt::white;
-    if (!amino) {
-        //DNA
-        colorsPerChar['A'] = colorsPerChar['a'] = QColor("#FCFF92"); // yellow
-        colorsPerChar['C'] = colorsPerChar['c'] = QColor("#70F970"); // green
-        colorsPerChar['T'] = colorsPerChar['t'] = QColor("#FF99B1"); // light red
-        colorsPerChar['G'] = colorsPerChar['g'] = QColor("#4EADE1"); // light blue
-        colorsPerChar['U'] = colorsPerChar['u'] = colorsPerChar['T'].lighter(120);
-        colorsPerChar['N'] = colorsPerChar['n'] = colorsPerChar['-'].darker(103);
-    } else {
-        //amino groups: "KRH", "GPST", "FWY", "ILM"
-        QColor krh("#FFEE00");
-        colorsPerChar['K'] = colorsPerChar['k'] = krh;
-        colorsPerChar['R'] = colorsPerChar['r'] = krh.darker(120);
-        colorsPerChar['H'] = colorsPerChar['h'] = krh.lighter(120);
-
-        QColor gpst("#FF5082");
-        colorsPerChar['G'] = colorsPerChar['g'] = gpst;
-        colorsPerChar['P'] = colorsPerChar['p'] = gpst.darker(120);
-        colorsPerChar['S'] = colorsPerChar['s'] = gpst.lighter(120);
-        colorsPerChar['T'] = colorsPerChar['t'] = gpst.lighter(150);
-
-        QColor fwy("#3DF490");
-        colorsPerChar['F'] = colorsPerChar['f'] = fwy;
-        colorsPerChar['W'] = colorsPerChar['w'] = fwy.darker(120);
-        colorsPerChar['Y'] = colorsPerChar['y'] = fwy.lighter(120);
-
-        QColor ilm("#00ABED");
-        colorsPerChar['I'] = colorsPerChar['i'] = ilm;
-        colorsPerChar['L'] = colorsPerChar['l'] = ilm.darker(120);
-        colorsPerChar['M'] = colorsPerChar['m'] = ilm.lighter(120);
-
-        //fix some color overlaps:
-        //e looks like q by default
-        colorsPerChar['E'] = colorsPerChar['e'] = QColor("#C0BDBB"); //gray
-        colorsPerChar['X'] = colorsPerChar['x'] = colorsPerChar['-'].darker(103);
+void MSAEditorSequenceArea::sl_zoomIn() {
+    int pSize = seqFont.pointSize();
+    if (pSize < MAX_FONT_SIZE) {
+        seqFont.setPointSize(pSize+1);
+        setFont(seqFont);
     }
 }
 
+void MSAEditorSequenceArea::sl_zoomOut() {
+    int pSize = seqFont.pointSize();
+    if (pSize > MIN_FONT_SIZE) {
+        seqFont.setPointSize(pSize-1);
+        setFont(seqFont);
+    }
+}
 
-
+void MSAEditorSequenceArea::sl_resetFont() {
+    QFont f(DEFAULT_FONT_FAMILY, DEFAULT_FONT_SIZE);
+    setFont(f);
+}
 }//namespace
