@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -11,23 +11,27 @@
 #ifndef _GB2_MSA_EDITOR_TASKS_H_
 #define _GB2_MSA_EDITOR_TASKS_H_
 
+#include <core_api/GObjectReference.h>
 #include <util_tasks/ObjectViewTasks.h>
 
 namespace GB2 {
 
 class MAlignmentObject;
+class UnloadedObject;
 class MSAEditor;
 
 class OpenMSAEditorTask : public ObjectViewTask {
     Q_OBJECT
 public:
 	OpenMSAEditorTask(MAlignmentObject* obj);
+    OpenMSAEditorTask(UnloadedObject* obj);
 	OpenMSAEditorTask(Document* doc);
 
 	virtual void open();
 
 private:
 	QPointer<MAlignmentObject>  msaObject;
+    GObjectReference            unloadedReference;
 };
 
 class OpenSavedMSAEditorTask : public ObjectViewTask {
