@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -17,7 +17,7 @@ namespace GB2
 {
 
 AddDocumentTask::AddDocumentTask( Document * _d ) :
-Task( tr("add_document_task"), TaskFlags(TaskFlag_DeleteWhenFinished) ), d(_d)
+Task( tr("add_document_task"), TaskFlag_NoRun), d(_d)
 {
 
 }
@@ -30,7 +30,7 @@ Task::ReportResult AddDocumentTask::report()
     } else if(d) {
         p->addDocument( d );
     } else {
-        stateInfo.error += tr("document_removed");
+        stateInfo.setError(stateInfo.getError() + tr("document_removed"));
     }
     return ReportResult_Finished;
 }
