@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -19,17 +19,17 @@ void TaskSignalMapper::sl_taskStateChanged() {
         if (t) {
             switch (t->getState()) {
             case Task::State_Prepared:
-                emit si_taskPrepared();
+                emit si_taskPrepared(t);
                 break;
             case Task::State_Running:
-                emit si_taskRunning();
+                emit si_taskRunning(t);
                 break;
             case Task::State_Finished:
-                emit si_taskFinished();
+                emit si_taskFinished(t);
                 if (t->hasErrors() || t->hasSubtasksWithErrors()) {
-                    emit si_taskFailed();
+                    emit si_taskFailed(t);
                 } else {
-                    emit si_taskSucceeded();
+                    emit si_taskSucceeded(t);
                 }
                 break;
             case Task::State_New:
