@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -35,4 +35,14 @@ Configuration::~Configuration()
 {
     qDeleteAll(params.values());
 }
+
+void Configuration::setParameters(const QVariantMap& cfg)
+{
+    QMapIterator<QString,QVariant> it(cfg);
+    while (it.hasNext())
+    {
+        it.next();
+        setParameter(it.key(), it.value());
+    }
+}
 }//namespace
