@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -24,10 +24,12 @@ public:
     ActorPrototype* unregisterProto(const QString& id);
     const QMap<Descriptor, QList<ActorPrototype*> >& getProtos() const {return groups;}
     ActorPrototype* getProto(const QString& id) const;
+    void addAliases(const QMap<QString,QString>& m) {aliases.unite(m);}
 signals:
     void si_registryModified();
 private:
      QMap<Descriptor, QList<ActorPrototype*> > groups;
+     QMap<QString,QString> aliases;
 };
 
 }//namespace Workflow
