@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -32,6 +32,9 @@ public:
     static const Descriptor MA_SLOT();
     static const Descriptor FEATURE_TABLE_SLOT();
 
+    static const QString FILE_MODE_ATTR_ID;
+    static const Descriptor FILE_MODE_ATTR();
+
     static void init();
 };
 
