@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -8,9 +8,11 @@
 *     for details.
 *****************************************************************/
 
+#include "BioStruct3DColorScheme.h"
 #include "BioStruct3DGLRender.h"
 #include "BallAndStickGLRenderer.h"
 #include "TubeGLRenderer.h"
+#include "VanDerWaalsGLRenderer.h"
 #include "WormsGLRenderer.h"
 
 namespace GB2 { 
@@ -23,6 +25,7 @@ QMap<QString,BioStruct3DGLRendererFactory*> BioStruct3DGLRendererFactory::create
     QMap<QString,BioStruct3DGLRendererFactory*> map;
     REGISTER_FACTORY(BallAndStickGLRenderer);
     REGISTER_FACTORY(TubeGLRenderer);
+    REGISTER_FACTORY(VanDerWaalsGLRenderer);
     REGISTER_FACTORY(WormsGLRenderer);
     return map;
 }
@@ -34,9 +37,21 @@ const QString BioStruct3DGLRendererFactory::defaultFactoryName()
 
 
 
-void BioStruct3DGLRenderer::updateColorScheme( const BioStruct3DColorScheme* s )
+void BioStruct3DGLRenderer::updateColorScheme()
 {
-    Q_ASSERT(s != NULL);
-    colorScheme = s;
+    
 }
+
+BioStruct3DGLRenderer::BioStruct3DGLRenderer( const BioStruct3D& _bioStruct, const BioStruct3DColorScheme* _s )  :  
+    bioStruct(_bioStruct), colorScheme(_s), glWidget(_s->getBioStruct3DGlWidget())
+{
+
+}
+
+void BioStruct3DGLRenderer::setColorScheme( const BioStruct3DColorScheme* s )
+{
+     colorScheme = s;
+     updateColorScheme();
+}
+
 } //namespace
