@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -21,11 +21,14 @@
 #include <gobjects/AnnotationSettings.h>
 #include <gobjects/AnnotationTableObject.h>
 #include <gobjects/DNASequenceObject.h>
+#include <gobjects/BioStruct3DObject.h>
 #include <gobjects/GObjectUtils.h>
 #include <gobjects/GObjectRelationRoles.h>
 #include <core_api/Log.h>
 #include <core_api/AppContext.h>
 #include <core_api/DocumentModel.h>
+#include <core_api/Counter.h>
+#include <molecular_geometry/MolecularSurfaceFactoryRegistry.h>
 #include <selection/DNASequenceSelection.h>
 #include <selection/AnnotationSelection.h>
 #include <time.h>
@@ -33,67 +36,55 @@
 #include "BioStruct3DGLWidget.h"
 #include "BioStruct3DGLRender.h"
 #include "BioStruct3DColorScheme.h"
+#include "GLFrameManager.h"
 #include "ExportImageDialog.h"
+#include "SettingsDialog.h"
+#include "MolecularSurfaceRenderer.h"
+#include "gl2ps/gl2ps.h"
 
+//disable "unsafe functions" deprecation warnings on MS VS
+#ifdef Q_OS_WIN
+#pragma warning(disable : 4996)
+#endif
 
 namespace GB2 { 
 
-const float BioStruct3DGLWidget::DEFAULT_ZOOM = 45.0f;
 static LogCategory log(ULOG_CAT_PLUGIN_BIOSTRUCT_3D);
+static LogCategory logPerf(ULOG_CAT_PERFORMANCE);
 int BioStruct3DGLWidget::widgetCount = 0;
 
 
-BioStruct3DGLWidget::BioStruct3DGLWidget(const BioStruct3D&  struc, const Document* doc, const AnnotatedDNAView* view, QWidget *parent /* = 0*/)
-    : QGLWidget(parent), biostruc(struc), biostrucDoc(doc), dnaView(view), currentModelID(struc.pdbId), spinAngle(0), displayMenu(NULL)
+BioStruct3DGLWidget::BioStruct3DGLWidget(const BioStruct3DObject* obj, const AnnotatedDNAView* view, GLFrameManager* manager, QWidget *parent /* = 0*/)
+    : QGLWidget(parent), biostruc(obj->getBioStruct3D()), biostrucDoc(obj->getDocument()),  
+    dnaView(view), frameManager(manager), molSurface(NULL), currentModelID(obj->getBioStruct3D().pdbId), spinAngle(0), displayMenu(NULL)
 {
-    
-    setObjectName(struc.pdbId + "-" + QString("%1").arg(++widgetCount));
+    GCOUNTER( cvar, tvar, "BioStruct3DGLWidget" );
+    setObjectName(currentModelID + "-" + QString("%1").arg(++widgetCount));
     //TODO: ? setFormat(QGLFormat(QGL::DoubleBuffer | QGL::DepthBuffer));
-    yAxis.set(0,1,0);
     colorSchemeFactoryMap = BioStruct3DColorSchemeFactory::createFactories();
     rendererFactoryMap = BioStruct3DGLRendererFactory::createFactories();
+    surfaceRendererFactoryMap = MolecularSurfaceRendererFactory::createFactories();
+    createActiveModelIndexList();
     createActions();
     createMenus();
     connectExternalSignals();
-
-    currentColorSchemeName =  BioStruct3DColorSchemeFactory::defaultFactoryName();
-    currentGLRendererName =  BioStruct3DGLRendererFactory::defaultFactoryName();
     backgroundColor = Qt::black;
-
+    selectionColor = Qt::yellow;
     chainIdCache.clear();
     
-    // Load settings
-    QList<QAction*>::iterator iter;
-    QList<QAction*> schemeActions = colorSchemeActions->actions();
-    for (iter = schemeActions.begin(); iter != schemeActions.end(); ++iter) {
-        if ((*iter)->text() == currentColorSchemeName) {
-            (*iter)->setChecked(true);
-            break;
-        }
-    }
-    Q_ASSERT(iter != schemeActions.end());
-    colorScheme.reset(colorSchemeFactoryMap.value(currentColorSchemeName)->createInstance(this));
+    // Load renderers and color schemes
+    loadColorSchemes();
+    loadGLRenderers();
     
-    QList<QAction*> renderActions = rendererActions->actions();
-    for (iter = renderActions.begin(); iter != renderActions.end(); ++iter) {
-        if ((*iter)->text() == currentGLRendererName) {
-            (*iter)->setChecked(true);
-            break;
-        }
-    }
-    Q_ASSERT(iter != renderActions.end());
-    const BioStruct3DColorScheme* scheme = colorScheme.get();  
-    renderer.reset(rendererFactoryMap.value(currentGLRendererName)->createInstance(struc,scheme));
-	
-    // Init rotation matrix as identity
-    rotMatrix.loadIdentity();
     // Set view settings
     float scaleFactor = 2.5;
     cameraDistance = scaleFactor * biostruc.getMaxDistFromCenter();
     cameraClipNear = (cameraDistance - biostruc.getMaxDistFromCenter()) * 0.66f;
     cameraClipFar = (cameraDistance + biostruc.getMaxDistFromCenter()) * 1.2f; 
-    zoomFactor = DEFAULT_ZOOM;
-    saveDefaultsSettings();
+        
+    glFrame.reset( new GLFrame(this, cameraClipNear, cameraClipFar) );
+    frameManager->addGLFrame(glFrame.get());
+    saveDefaultSettings();
 }
 
 BioStruct3DGLWidget::~BioStruct3DGLWidget()
@@ -104,12 +95,18 @@ BioStruct3DGLWidget::~BioStruct3DGLWidget()
          BioStruct3DColorSchemeFactory* f = colorSchemeFactoryMap.take(key);
          delete f;
     }
+    
     foreach(QString key, rendererFactoryMap.keys()) {
         BioStruct3DGLRendererFactory* f = rendererFactoryMap.take(key);
         delete f;
     }
     
-   log.trace("Biostruct3DGLWdiget "+objectName()+" deleted");
+    foreach(QString key, surfaceRendererFactoryMap.keys()) {
+        MolecularSurfaceRendererFactory* f = surfaceRendererFactoryMap.take(key);
+        delete f;
+    }
+
+    log.trace("Biostruct3DGLWdiget "+objectName()+" deleted");
 }
 
 
@@ -129,22 +126,17 @@ void BioStruct3DGLWidget::initializeGL()
     glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
     glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
     glLightfv(GL_LIGHT0, GL_POSITION, lightPostion);
-    //glEnable(GL_NORMALIZE);
+    glEnable(GL_NORMALIZE);
     glEnable(GL_LIGHTING);
     glEnable(GL_LIGHT0);
     glEnable(GL_DEPTH_TEST);
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     
 }
 
 void BioStruct3DGLWidget::resizeGL(int width, int height)
 {
-    glViewport(0, 0, width, height);
-    glMatrixMode(GL_PROJECTION);
-    glLoadIdentity();
-    // Set view settings
-    GLfloat aspect = GLfloat(width) / height;
-    gluPerspective(zoomFactor, aspect, cameraClipNear, cameraClipFar);
-
+   glFrame->updateViewPort(width, height);
 }
 
 void BioStruct3DGLWidget::paintGL()
@@ -165,37 +157,50 @@ void BioStruct3DGLWidget::mouseMoveEvent(QMouseEvent *event)
     Vector3D rotCenter = biostruc.getCenter();
 	
 	if (event->buttons() & Qt::LeftButton) {
-
 		Vector3D curPos = getTrackballMapping(event->x(), event->y());
-		
 		Vector3D delta = curPos - lastPos;
 		if (delta.x || delta.y || delta.z) {
 			rotAngle = 90.0f*delta.length();
 			rotAxis =  vector_cross(lastPos,curPos); 
-			glMatrixMode(GL_MODELVIEW);
-			glLoadIdentity();
-            glRotatef(rotAngle, rotAxis.x, rotAxis.y, rotAxis.z );
-            glMultMatrixf(rotMatrix.getData());
-            glGetFloatv( GL_MODELVIEW_MATRIX, rotMatrix.getData());
+            bool syncLock = isSyncModeOn();
+            QList<GLFrame*> frames = frameManager->getActiveGLFrameList(glFrame.get(), syncLock);
+            foreach( GLFrame* frame, frames) {
+                frame->makeCurrent();
+                frame->rotateCamera(rotAxis, rotAngle);
+                frame->updateGL();
+            } 
         }
-        
-        lastPos = curPos;
-		updateGL();
+		lastPos = curPos;
 	}
-    
-	
-
+ 
 }
 
+
 void BioStruct3DGLWidget::draw()
 {
     Vector3D rotCenter = biostruc.getCenter();
     glMatrixMode(GL_MODELVIEW);
 	glLoadIdentity(); 
     gluLookAt(0.0, 0.0, cameraDistance, 0,  0, 0, 0.0, 1.0, 0.0);
-    glMultMatrixf(rotMatrix.getData());
+    glMultMatrixf(glFrame->getRotationMatrix() );
     glTranslatef(-rotCenter.x ,-rotCenter.y, -rotCenter.z);
-    renderer->drawBioStruct3D();
+    clock_t t1 =  clock();
+    biostructRenderer->drawBioStruct3D();
+    if(NULL != molSurface.get())
+    {
+        glEnable(GL_BLEND);
+        glEnable(GL_BLEND);
+        glEnable(GL_CULL_FACE);
+        glCullFace(GL_FRONT);
+        surfaceRenderer->drawSurface(*molSurface);
+        glCullFace(GL_BACK);
+        surfaceRenderer->drawSurface(*molSurface);
+        glDisable(GL_CULL_FACE);
+        glDisable(GL_BLEND);
+    }
+    clock_t t2 = clock();
+    logPerf.trace("BioStruct3D structure rendering time " + QString::number((float)( t2 - t1)/ CLOCKS_PER_SEC)+ " s");
+
     
 }
 
@@ -218,7 +223,7 @@ Vector3D BioStruct3DGLWidget::getTrackballMapping(int x, int y)
 
 void BioStruct3DGLWidget::wheelEvent ( QWheelEvent * event )
 {
-	float numDegrees =  event->delta() / 8;
+    float numDegrees =  event->delta() / 8;
     zoom(numDegrees / 10);
 }
 
@@ -244,13 +249,35 @@ void BioStruct3DGLWidget::createActions()
         action = new QAction(key, colorSchemeActions);
         action->setCheckable(true);
     }   
+    
+    molSurfaceRenderActions = new QActionGroup(this);
+    connect(molSurfaceRenderActions, SIGNAL(triggered(QAction *)), this, SLOT(sl_selectSurfaceRenderer(QAction*)));
+    foreach(QString key, surfaceRendererFactoryMap.keys()) {
+        action = new QAction(key, molSurfaceRenderActions);
+        action->setCheckable(true);
+        if (key == ConvexMapRenderer::ID) {
+            action->setChecked(true);
+        }
+    }   
 
+    molSurfaceTypeActions = new QActionGroup(this);
+    foreach(QString key, AppContext::getMolecularSurfaceFactoryRegistry()->getSurfNameList())
+    {
+        action = new QAction(key, molSurfaceTypeActions);
+        connect(action, SIGNAL(triggered()), this, SLOT(sl_showSurface()));
+        action->setCheckable(true);
+    }
+    action = new QAction(tr("Off"), molSurfaceTypeActions);
+    connect(action, SIGNAL(triggered()), this, SLOT(sl_hideSurface()));
+    action->setCheckable(true);
+    action->setChecked(true);
+    
     spinAction = new QAction(tr("Spin"), this);
     spinAction->setCheckable(true);
     connect(spinAction, SIGNAL(triggered()), this, SLOT(sl_acitvateSpin()));
-
-    setBackgroundColorAction = new QAction(QIcon(":core/images/color_wheel.png"), tr("Set background color"), this);
-    connect(setBackgroundColorAction, SIGNAL(triggered()), this, SLOT(sl_setBackgroundColor()));
+    
+    settingsAction = new QAction(tr("Settings"), this);
+    connect(settingsAction, SIGNAL(triggered()), this, SLOT(sl_settings()));
 
     closeAction = new QAction(tr("close_action"), this);
     connect(closeAction, SIGNAL(triggered()), this, SLOT(sl_closeWidget()));
@@ -258,39 +285,77 @@ void BioStruct3DGLWidget::createActions()
     exportImageAction = new QAction(tr("Export image"), this);
     connect(exportImageAction, SIGNAL(triggered()), this, SLOT(sl_exportImage()));
 
+    if (multipleModels) {
+        setNextModelAction = new QAction(tr("Next model"), this);
+        connect(setNextModelAction, SIGNAL(triggered()), this, SLOT(sl_setNextModelAcitve()));
 
-   	
+        setPrevModelAction = new QAction(tr("Prev model"), this);
+        connect(setPrevModelAction, SIGNAL(triggered()), this, SLOT(sl_setPrevModelAcitve()));
+
+        setAllModelsActiveAction = new QAction(tr("All models"), this);
+        setAllModelsActiveAction->setCheckable(true);
+        connect(setAllModelsActiveAction, SIGNAL(triggered()), this, SLOT(sl_setAllModelsActive()));
+        setAllModelsActiveAction->setChecked(true);
+    }
 }
 
 void BioStruct3DGLWidget::createMenus()
 {
+    // Renderer selection   
     selectRendererMenu = new QMenu(tr("Render style"));
     selectRendererMenu->addActions(rendererActions->actions());
     
+    // Color scheme selection
     selectColorSchemeMenu = new QMenu(tr("Color Scheme"));
 	selectColorSchemeMenu->addActions(colorSchemeActions->actions());
+    
+    // Molecular surface
+    QMenu* surfaceMenu = new QMenu(tr("Molecular Surface Render Style"));
+    surfaceMenu->addActions(molSurfaceRenderActions->actions());
 
+    QMenu *surfaceTypeMenu = new QMenu(tr("Molecular Surface"));
+    surfaceTypeMenu->addActions(molSurfaceTypeActions->actions());
+        
+    // Models selection 
+    if (multipleModels) {
+        modelsMenu = new QMenu(tr("Models"));
+        modelsMenu->addAction(setNextModelAction);
+        modelsMenu->addAction(setPrevModelAction);
+        modelsMenu->addAction(setAllModelsActiveAction);
+    } else {
+        modelsMenu = NULL;
+    }
+
+    // Display (context) menu
+    displayMenu = new QMenu(this);
+    displayMenu->addMenu(selectRendererMenu);
+    displayMenu->addMenu(selectColorSchemeMenu);
+    if (modelsMenu) {
+        displayMenu->addMenu(modelsMenu);
+    }
+    displayMenu->addMenu(surfaceMenu);
+    displayMenu->addMenu(surfaceTypeMenu);
+    displayMenu->addAction(spinAction);
+    displayMenu->addAction(settingsAction);
+    displayMenu->addAction(exportImageAction);
 
 }
 
 void BioStruct3DGLWidget::connectExternalSignals()
 {
-	AnnotationSettingsRegistry* asr = AppContext::getAnnotationsSettingsRegistry();
-	connect(asr, SIGNAL(si_annotationSettingsChanged(const QStringList& )), this, SLOT(sl_updateRenderSettings(const QStringList& )) );
+    AnnotationSettingsRegistry* asr = AppContext::getAnnotationsSettingsRegistry();
+    connect(asr, SIGNAL(si_annotationSettingsChanged(const QStringList& )), this, SLOT(sl_updateRenderSettings(const QStringList& )) );
 
     const QList<ADVSequenceObjectContext*> seqContexts = getSequenceContexts();
     foreach (ADVSequenceObjectContext* ctx, seqContexts) {
-    connect(ctx->getSequenceSelection(), 
+        connect(ctx->getSequenceSelection(), 
             SIGNAL(si_selectionChanged(LRegionsSelection*, const QList<LRegion>&, const QList<LRegion>&)), 
             SLOT(sl_onSequenceSelectionChanged(LRegionsSelection*, const QList<LRegion>& , const QList<LRegion>&)));
-    
-    connect(dnaView->getAnnotationsSelection(), 
-        SIGNAL(si_selectionChanged(AnnotationSelection* , const QList<Annotation*>&, const QList<Annotation*>&)),
-        SLOT(sl_onAnnotationSelectionChanged(AnnotationSelection* , const QList<Annotation*>&, const QList<Annotation*>&)));
-    
-    
-    }
 
+        connect(dnaView->getAnnotationsSelection(), 
+            SIGNAL(si_selectionChanged(AnnotationSelection* , const QList<Annotation*>&, const QList<Annotation*>&)),
+            SLOT(sl_onAnnotationSelectionChanged(AnnotationSelection* , const QList<Annotation*>&, const QList<Annotation*>&)));
+    }
 
 }
 
@@ -299,7 +364,7 @@ void BioStruct3DGLWidget::connectExternalSignals()
 void BioStruct3DGLWidget::setBioStruct3DColorScheme( BioStruct3DColorScheme* clScheme )
 {
 	Q_ASSERT(clScheme != NULL);
-	renderer->updateColorScheme(clScheme);
+	biostructRenderer->setColorScheme(clScheme);
     colorScheme.reset(clScheme);
    
 }
@@ -307,18 +372,18 @@ void BioStruct3DGLWidget::setBioStruct3DColorScheme( BioStruct3DColorScheme* clS
 void BioStruct3DGLWidget::setBioStruct3DRenderer( BioStruct3DGLRenderer* r )
 {
 	Q_ASSERT(r != NULL);
-	renderer.reset(r);
+	biostructRenderer.reset(r);
 
 }
 
 void BioStruct3DGLWidget::contextMenuEvent( QContextMenuEvent *event )
 {
 	QMenu menu;
-    menu.addMenu(selectRendererMenu);
-    menu.addMenu(selectColorSchemeMenu);
-    menu.addAction(spinAction);
-    menu.addAction(setBackgroundColorAction);
-    menu.addAction(exportImageAction);
+    
+    foreach(QAction* action, getDisplayMenu()->actions()) {
+        menu.addAction(action);
+    }
+
     menu.addAction(closeAction);
 
 	menu.exec(event->globalPos());
@@ -330,6 +395,7 @@ void BioStruct3DGLWidget::sl_selectColorScheme(QAction* action)
 	QString schemeName = action->text();
     colorScheme = createCustomColorScheme(schemeName);
 	Q_ASSERT(colorScheme != NULL);
+        colorScheme->setSelectionColor(selectionColor);
 	setBioStruct3DColorScheme(colorScheme);
     currentColorSchemeName = schemeName;
  	updateGL();
@@ -343,22 +409,22 @@ void BioStruct3DGLWidget::sl_updateRenderSettings(const QStringList& list )
 }
 
 
-QMap<QString, QColor> BioStruct3DGLWidget::getSecStructAnnotationColors() const
+const QMap<QString, QColor> BioStruct3DGLWidget::getSecStructAnnotationColors() const
 {
     QMap<QString, QColor> colors;
     AnnotationSettingsRegistry* asr = AppContext::getAnnotationsSettingsRegistry();
 
     foreach (GObject* obj, biostrucDoc->getObjects() ) {
         if (obj->getGObjectType() == GObjectTypes::ANNOTATION_TABLE) {
-                AnnotationTableObject* ao = qobject_cast<AnnotationTableObject*>(obj);
-                foreach(Annotation* a, ao->getAnnotations()) {
-                        QString name = a->getAnnotationName();
-                        if ( (name == BioStruct3D::AlphaHelixAnnotationTag) || (name == BioStruct3D::BetaStrandAnnotationTag)
-                                || (name == BioStruct3D::TurnAnnotationTag) ) {
-                const AnnotationSettings* as = asr->getSettings(name);
-                colors.insert(name, as->color);
-				}
-			}
+            AnnotationTableObject* ao = qobject_cast<AnnotationTableObject*>(obj);
+            foreach(Annotation* a, ao->getAnnotations()) {
+                QString name = a->getAnnotationName();
+                if ( name == BioStruct3D::SecStructAnnotationTag) {
+                    QString ssName = a->getQualifiers().first().getQualifierValue();
+                    AnnotationSettings* as = asr->getAnnotationSettings(ssName);
+                    colors.insert(ssName, as->color);
+                }
+            }
         }
     }
 
@@ -378,10 +444,10 @@ const QMap<int, QColor> BioStruct3DGLWidget::getChainColors() const
  			foreach(Annotation* a, ao->getAnnotations()) {
  				QString name = a->getAnnotationName();
  				if (name.startsWith(BioStruct3D::MoleculeAnnotationTag)) {
-					int index = getQualifierValueByName(a, BioStruct3D::ChainIdQualifierName).toInt();
-					Q_ASSERT(index != 0);
-                const AnnotationSettings* as = asr->getSettings(name);
-                colorMap.insert(index, as->color);				
+					int chainId = getQualifierValueByName(a, BioStruct3D::ChainIdQualifierName).toInt();
+                    Q_ASSERT(chainId != 0);
+                    AnnotationSettings* as = asr->getAnnotationSettings(name);
+                    colorMap.insert(chainId, as->color);				
  				}
  			}
 		}
@@ -440,22 +506,18 @@ void BioStruct3DGLWidget::sl_onSequenceSelectionChanged( LRegionsSelection* s, c
 
     DNASequenceSelection* selection = qobject_cast<DNASequenceSelection*>(s);
     const DNASequenceObject* seqObj = selection->getSequenceObject();
-    if (selection) {
-        const QString seqName = seqObj->getGObjectName();
-        if (!seqName.startsWith(currentModelID))
-			return;
-        
-        QList<GObject*> relAnns = GObjectUtils::findObjectsRelatedToObjectByRole(seqObj, GObjectTypes::ANNOTATION_TABLE, GObjectRelationRole::SEQUENCE, biostrucDoc->getObjects());
-        AnnotationTableObject* a = relAnns.isEmpty() ? NULL : qobject_cast<AnnotationTableObject*>(relAnns.first());
-        if (a == NULL) {
-            log.error("annotation object not found!");
-            return;
-        }
-        int chainId = getChainIdFromAnnotationObject(a);
+    
+    // TODO: better to use objects relations here
+    if (!seqObj->getGObjectName().startsWith(biostruc.pdbId)) {
+        return;
+    }
+    
+    
+    if (seqObj) {
+        int chainId = getSequenceChainId(seqObj);
         Q_ASSERT(biostruc.moleculeMap.contains(chainId));
         colorScheme->updateSelectionRegion(chainId, added, removed);
-        //TODO: unobvious! Refactor interface 
-        renderer->updateColorScheme(colorScheme.get());
+        biostructRenderer->updateColorScheme();
         update();
     }
 
@@ -463,19 +525,22 @@ void BioStruct3DGLWidget::sl_onSequenceSelectionChanged( LRegionsSelection* s, c
 }
 
 
-int BioStruct3DGLWidget::getChainIdFromAnnotationObject(const AnnotationTableObject* ao) 
+int BioStruct3DGLWidget::getChainIdForAnnotationObject(AnnotationTableObject* ao) 
 {
     if ( this->chainIdCache.contains(ao) ) {
         return chainIdCache.value(ao);
     } else {
-        foreach (Annotation* a, ao->getAnnotations()) {
-            if (a->getAnnotationName().startsWith(BioStruct3D::MoleculeAnnotationTag) ) {
-                int chainId = getQualifierValueByName(a,BioStruct3D::ChainIdQualifierName).toInt();
-                chainIdCache.insert(ao,chainId);
-                return chainId;
-            } 
-        }
-         return 0;
+        int chainID = -1;
+        QList<GObjectRelation> relations = ao->findRelatedObjectsByRole(GObjectRelationRole::SEQUENCE);
+        Q_ASSERT(relations.count() == 1);
+        GObjectRelation relation = relations.first();
+        GObject* obj = biostrucDoc->findGObjectByName(relation.ref.objName);
+        DNASequenceObject* seqObj = qobject_cast<DNASequenceObject*>(obj);
+        if (seqObj) {
+            chainID = getSequenceChainId(seqObj);
+            chainIdCache.insert(ao, chainID);
+        } 
+        return chainID;
     }
 }
 
@@ -486,30 +551,34 @@ void BioStruct3DGLWidget::sl_onAnnotationSelectionChanged( AnnotationSelection*
         return;
     
     QList<LRegion> empty;
+    QList<GObject*> seqObjects = dnaView->getSequenceGObjectsWithContexts();
 
 	foreach (Annotation* annotation, added) {
-		if (annotation->getLocation().isEmpty())
+        if (annotation->getLocation().isEmpty()) { //TODO: there must be no such annotations!
             continue;
-		AnnotationTableObject* ao = annotation->getGObject(); 
-		if (!ao->getGObjectName().startsWith(currentModelID))
-			continue;
-		int chainId  = getChainIdFromAnnotationObject(ao);
-		Q_ASSERT(biostruc.moleculeMap.contains(chainId));
-		colorScheme->updateSelectionRegion(chainId, annotation->getLocation(), empty );
+        }
+
+		AnnotationTableObject* ao = annotation->getGObject();
+        int chainId  = getChainIdForAnnotationObject(ao);
+        if (chainId!=-1) {
+		    Q_ASSERT(biostruc.moleculeMap.contains(chainId));
+		    colorScheme->updateSelectionRegion(chainId, annotation->getLocation(), empty );
+        }
     }
   
     foreach (Annotation* annotation, removed) {
-       if (annotation->getLocation().isEmpty() )
+        if (annotation->getLocation().isEmpty() ) { //TODO: there must be no such annotations!
            continue;
+        }
 	   AnnotationTableObject* ao = annotation->getGObject(); 
-	   if (!ao->getGObjectName().startsWith(currentModelID))
-		   continue;
-	   int chainId  = getChainIdFromAnnotationObject(ao);
-       Q_ASSERT(biostruc.moleculeMap.contains(chainId));
-       colorScheme->updateSelectionRegion(chainId, empty, annotation->getLocation());
+	   int chainId  = getChainIdForAnnotationObject(ao);
+       if (chainId!=-1) {
+           Q_ASSERT(biostruc.moleculeMap.contains(chainId));
+           colorScheme->updateSelectionRegion(chainId, empty, annotation->getLocation());
+       }
     }
     
-    renderer->updateColorScheme(colorScheme.get());
+    biostructRenderer->updateColorScheme();
     update();
 }
 
@@ -542,13 +611,11 @@ void BioStruct3DGLWidget::sl_updateAnnimation()
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
     glRotatef(spinAngle, 0, 1, 0 );
-    glMultMatrixf(rotMatrix.getData());
-    glGetFloatv( GL_MODELVIEW_MATRIX, rotMatrix.getData());
+    glMultMatrixf(glFrame->getRotationMatrix());
+    glGetFloatv( GL_MODELVIEW_MATRIX, glFrame->getRotationMatrix());
     updateGL();
 }
 
-#define ZOOM_FACTOR_ID "ZOOM_FACTOR"
-#define ROTATION_MATRIX_ID "ROTATION_MATRIX"
 #define PRODUCT_NAME "Unipro Ugene"
 #define PLUGIN_NAME "BioStruct3D Viewer Plugin"
 #define COLOR_SCHEME_NAME "ColorScheme"
@@ -559,8 +626,7 @@ void BioStruct3DGLWidget::sl_updateAnnimation()
 QVariantMap BioStruct3DGLWidget::getState()
 {
     QVariantMap state;
-    state[ZOOM_FACTOR_ID] = QVariant::fromValue(zoomFactor);
-    state[ROTATION_MATRIX_ID] = rotMatrix.store();
+    glFrame->writeStateToMap(state);
     state[COLOR_SCHEME_NAME] = QVariant::fromValue(currentColorSchemeName);
     state[RENDERER_NAME] = QVariant::fromValue(currentGLRendererName);
     state[OBJECT_ID_NAME] = QVariant::fromValue(getBioStruct3DObjectName());
@@ -575,11 +641,7 @@ void BioStruct3DGLWidget::setState( const QVariantMap& state )
     if (state.isEmpty()) {
         return;
     }
-    zoomFactor = state.value(ZOOM_FACTOR_ID, DEFAULT_ZOOM).value<float>();
-    QVariantList rotML = state.value(ROTATION_MATRIX_ID).value<QVariantList>();
-    if (!rotML.isEmpty()) {
-        rotMatrix.load(rotML);
-    }
+    glFrame->setState(state);
     
     currentColorSchemeName = state.value(COLOR_SCHEME_NAME, BioStruct3DColorSchemeFactory::defaultFactoryName()).value<QString>();
     currentGLRendererName = state.value(RENDERER_NAME, BioStruct3DGLRendererFactory::defaultFactoryName()).value<QString>();
@@ -588,6 +650,7 @@ void BioStruct3DGLWidget::setState( const QVariantMap& state )
     setBioStruct3DRenderer(createCustomRenderer(currentGLRendererName));
     resizeGL(width(), height());
     updateGL();
+
 }
 
 const QString BioStruct3DGLWidget::getBioStruct3DObjectName() const
@@ -600,12 +663,20 @@ const QString BioStruct3DGLWidget::getBioStruct3DObjectName() const
     return ""; 
 }
 
-void BioStruct3DGLWidget::sl_setBackgroundColor()
+void BioStruct3DGLWidget::sl_settings()
 {
-    backgroundColor = QColorDialog::getColor();
-    qglClearColor(backgroundColor);
-    updateGL();
-
+    SettingsDialog dialog;
+    dialog.setBackgroundColor(backgroundColor);
+    dialog.setSelectionColor(selectionColor);
+    if(QDialog::Accepted == dialog.exec())
+    {
+        backgroundColor=dialog.getBackgroundColor();
+        selectionColor=dialog.getSelectionColor();
+        colorScheme->setSelectionColor(selectionColor);
+        this->makeCurrent();
+        qglClearColor(backgroundColor);
+        updateGL();
+    }
 }
 
 void BioStruct3DGLWidget::sl_closeWidget()
@@ -620,123 +691,257 @@ void BioStruct3DGLWidget::sl_exportImage()
     ExportImageDialog dialog(this);
     dialog.exec();
 
-//     QPixmap image = renderPixmap();
-//         
-//      QList<QByteArray> suppotedFormats = QImageWriter::supportedImageFormats();
-//     
-//     QString fileName = QFileDialog::getSaveFileName(this, tr("Export Image File"),
-//         "", tr("PNG images (*.png );; JPEG images (*.jpg) "));
-// 
-//     QByteArray fileFormat("PNG");
-//     image.save(fileName, fileFormat.constData());
-
 }
 
 void BioStruct3DGLWidget::zoom( float delta )
 {
-    const float maxZoom = 150.0;
-    const float minZoom = 2.0;
-    zoomFactor += delta;
-    if (zoomFactor < minZoom) {
-        zoomFactor = minZoom;
-        return;
-    }
-
-    if (zoomFactor > maxZoom) {
-        zoomFactor = maxZoom;
-        return;
-    }
-
-
-    resizeGL(width(), height());
-    updateGL();
-
-
+    bool syncLock = isSyncModeOn();
+    QList<GLFrame*> frames = frameManager->getActiveGLFrameList(glFrame.get(), syncLock);
+    foreach( GLFrame* frame, frames) {
+        frame->makeCurrent();
+        frame->performZoom(delta);
+        frame->updateViewPort();
+        frame->updateGL();
+    } 
 }
 
-void BioStruct3DGLWidget::saveDefaultsSettings()
+void BioStruct3DGLWidget::saveDefaultSettings()
 {
-    defaultsSettings[ZOOM_FACTOR_ID] = QVariant::fromValue(zoomFactor);
-    defaultsSettings[ROTATION_MATRIX_ID] = rotMatrix.store();
+    glFrame->writeStateToMap(defaultsSettings);
     defaultsSettings[COLOR_SCHEME_NAME] = QVariant::fromValue(currentColorSchemeName);
     defaultsSettings[RENDERER_NAME] = QVariant::fromValue(currentGLRendererName);
 
 }
 
 void BioStruct3DGLWidget::restoreDefaultSettigns()
-{
-    setState(defaultsSettings);
+{   
+    Q_ASSERT(!defaultsSettings.isEmpty());
+    bool syncLock = isSyncModeOn();
+    QList<GLFrame*> frames = frameManager->getActiveGLFrameList(glFrame.get(), syncLock);
+    foreach( GLFrame* frame, frames) {
+        frame->makeCurrent();
+        frame->setState(defaultsSettings);
+        frame->updateViewPort();
+        frame->updateGL();
+    } 
 }
 
 
 QMenu* BioStruct3DGLWidget::getDisplayMenu()
 {
-    if (displayMenu == NULL) {
-        displayMenu = new QMenu(this);
-        displayMenu->addMenu(selectRendererMenu);
-        displayMenu->addMenu(selectColorSchemeMenu);
-        displayMenu->addAction(spinAction);
-    }
-    
+    Q_ASSERT(displayMenu != NULL);
     return displayMenu;
 }
 
 
-///////////////////////////////////////////////////////////////////////////////////////////
-/// Matrix4x4
 
-Matrix4x4::Matrix4x4()
+void BioStruct3DGLWidget::createActiveModelIndexList()
 {
-	m[0] = 0; m[1] = 0; m[2] = 0; m[3] = 0;
-	m[4] = 0; m[5] = 0; m[6] = 0; m[7] = 0;
-	m[8] = 0; m[9] = 0; m[10] = 0; m[11] = 0;
-	m[12] = 0; m[13] = 0; m[14] = 0; m[15] = 0;
-}
-
-void Matrix4x4::loadIdentity()
-{
-	m[0] = 1; m[1] = 0; m[2] = 0; m[3] = 0;
-	m[4] = 0; m[5] = 1; m[6] = 0; m[7] = 0;
-	m[8] = 0; m[9] = 0; m[10] = 1; m[11] = 0;
-	m[12] = 0; m[13] = 0; m[14] = 0; m[15] = 1;
-}
-
-float& Matrix4x4::operator[]( unsigned int i )
-{
-	static float err = 0.0;
-	if (i < 16)
-		return m[i];
-	else  {
-		qCritical("Vector operator [] access out of range : %d", i);
-		return err;
-	}
-}
-
-Matrix4x4& Matrix4x4::operator=( const Matrix4x4& matrix )
-{
-    if (this == &matrix) { 
-        return *this;
+    int numModels = biostruc.modelMap.count();
+    if (numModels > 1) {
+        multipleModels = true;
+        // set all models active
+        for (int i = 0; i < numModels; ++i) {
+            activeModelIndexList.append(i);
+        }
+    } else {
+        multipleModels = false;
+        activeModelIndexList.append(0);
     }
-    memcpy((void*) m, (void*) matrix.m, sizeof(matrix.m) ); 
-    return *this;
+    
+}
+
+
+void BioStruct3DGLWidget::switchActiveModel(bool forward)
+{
+    if (activeModelIndexList.count() == 1) {
+        int index = activeModelIndexList.takeFirst();
+        int numModels = biostruc.modelMap.size();
+        if (forward) {
+            ++index;
+        } else {
+            --index;
+            if (index == -1 ) {
+                index =numModels - 1;
+            }
+        }
+        activeModelIndexList.append(index);
+    } else {
+        // if there are more then one model active, unselect all, set default index = 0
+        activeModelIndexList.clear();
+        activeModelIndexList.append(0);
+        setAllModelsActiveAction->setChecked(false);
+    }
+}
+
+
+void BioStruct3DGLWidget::sl_setNextModelAcitve()
+{
+    switchActiveModel(true);
+    updateGL();
+}
+
+
+void BioStruct3DGLWidget::sl_setPrevModelAcitve()
+{
+    switchActiveModel(false);
+    updateGL();
+}
+
+void BioStruct3DGLWidget::sl_setAllModelsActive()
+{
+    int numModels = biostruc.modelMap.count();
+    activeModelIndexList.clear();
+    for (int i = 0; i < numModels; ++i) {
+        activeModelIndexList.append(i);
+    }
+    updateGL();
 
 }
 
-void Matrix4x4::load( QVariantList values )
+
+void BioStruct3DGLWidget::writeImage2DToFile( int format, int options, int nbcol, const char *fileName )
 {
-    Q_ASSERT(values.size() == 16);
-    for (int i = 0; i < 16; ++i) {
-        m[i] = values.at(i).value<float>(); 
+    FILE *fp = NULL;
+    const char* FOPEN_ARGS = "wb";
+    const QByteArray title(fileName);
+    int state = GL2PS_OVERFLOW, buffsize = 0;
+    GLint viewport[4];
+    int sort = GL2PS_SIMPLE_SORT;
+        
+    fp = fopen(fileName, FOPEN_ARGS);
+
+    if(!fp){
+        QMessageBox::warning(this, tr("Error"),tr("Unable to open file %1 for writing").arg(fileName));
+        return;
+    }
+    
+    glGetIntegerv(GL_VIEWPORT,viewport);
+    
+    if (format == GL2PS_EPS) {
+        // hack -> make widget aspect ratio 1:1
+        if (width() > height()) {
+            int size = height();
+            resize(size, size);
+        }
     }
 
+    while(state == GL2PS_OVERFLOW){
+        buffsize += 2048*2048;
+        gl2psBeginPage(title.constData(), "Unipro UGENE BioStruct3D Viewer plugin", viewport, format, sort, options,
+            GL_RGBA, 0, NULL, nbcol, nbcol, nbcol, buffsize, fp, fileName);
+        paintGL();
+        state = gl2psEndPage();
+    }
+
+    fclose(fp);
+    
+    if (format == GL2PS_EPS) {
+        // restore sizes 
+        updateGeometry();
+    }
+  
 }
 
-QVariantList Matrix4x4::store()
+const QString BioStruct3DGLWidget::getPDBId() const
 {
-    QVariantList values;
-    for (int i = 0; i < 16; ++i ) {
-        values.append(QVariant::fromValue(m[i]));
-    }
-    return values;
+    return biostruc.pdbId;
 }
+
+int BioStruct3DGLWidget::getSequenceChainId( const DNASequenceObject* seqObj )
+{
+    const QVariantMap info =  seqObj->getDNASequence().info;
+    if (info.contains(DNAInfo::CHAIN_ID)) {
+        return info.value(DNAInfo::CHAIN_ID).toInt();
+    }
+
+    return -1;
+}
+
+void BioStruct3DGLWidget::loadColorSchemes()
+{
+    currentColorSchemeName =  BioStruct3DColorSchemeFactory::defaultFactoryName();
+
+    QList<QAction*>::iterator iter;
+    QList<QAction*> schemeActions = colorSchemeActions->actions();
+    for (iter = schemeActions.begin(); iter != schemeActions.end(); ++iter) {
+        if ((*iter)->text() == currentColorSchemeName) {
+            (*iter)->setChecked(true);
+            break;
+        }
+    }
+    Q_ASSERT(iter != schemeActions.end());
+    colorScheme.reset(colorSchemeFactoryMap.value(currentColorSchemeName)->createInstance(this));
+    colorScheme->setSelectionColor(selectionColor);
+}
+
+void BioStruct3DGLWidget::loadGLRenderers()
+{
+    currentGLRendererName =  BioStruct3DGLRendererFactory::defaultFactoryName();
+    QList<QAction*>::iterator iter;
+    QList<QAction*> renderActions = rendererActions->actions();
+    for (iter = renderActions.begin(); iter != renderActions.end(); ++iter) {
+        if ((*iter)->text() == currentGLRendererName) {
+            (*iter)->setChecked(true);
+            break;
+        }
+    }
+    Q_ASSERT(iter != renderActions.end());
+    const BioStruct3DColorScheme* scheme = colorScheme.get();  
+    biostructRenderer.reset(rendererFactoryMap.value(currentGLRendererName)->createInstance(biostruc,scheme));
+    
+    QString surfaceRendererName = ConvexMapRenderer::ID;
+    surfaceRenderer.reset(surfaceRendererFactoryMap.value(surfaceRendererName)->createInstance());
+    
+}
+
+bool BioStruct3DGLWidget::isSyncModeOn()
+{
+    Qt::KeyboardModifiers km = QApplication::keyboardModifiers();
+    bool synchronizationMode = km.testFlag(Qt::ShiftModifier) || frameManager->getSyncLock();
+    synchronizationMode &=  frameManager->getGLFrames().count() > 1 ;
+    return synchronizationMode;
+}
+
+void BioStruct3DGLWidget::sl_showSurface()
+{
+    {
+        // Calculate surface (only one model! too much computation for all models)
+        {
+            MolecularSurfaceFactory *factory=
+                    AppContext::getMolecularSurfaceFactoryRegistry()->getSurfaceFactory(qobject_cast<QAction *>(sender())->text());
+            molSurface.reset(factory->createInstance());
+        }
+        QList<SharedAtom> atoms;
+        int index = getActiveModelIndexList().first();
+        foreach (const SharedMolecule mol, biostruc.moleculeMap) {
+            const Molecule3DModel& model = mol->models.at(index);
+            atoms += model.atoms;
+        }
+        molSurface->calculate(atoms);
+    }
+    
+    makeCurrent();
+    updateGL();
+}
+
+void BioStruct3DGLWidget::sl_hideSurface()
+{
+    molSurface.reset();
+
+    makeCurrent();
+    updateGL();
+}
+
+void BioStruct3DGLWidget::sl_selectSurfaceRenderer(QAction*  action)
+{
+    {
+        MolecularSurfaceRendererFactory* f = surfaceRendererFactoryMap.value(action->text());
+        surfaceRenderer.reset( f->createInstance() );
+    }
+
+    makeCurrent();
+    updateGL();
+}
+
 } //namespace
