@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -18,22 +18,30 @@
 #include "BioStruct3DGLRender.h"
 
 
-namespace GB2 { 
+namespace GB2 {
 
-    class TubeGLRenderer : public BioStruct3DGLRenderer {
-        
-        QVector< SharedAtom > tubeAtoms; 
-        
-        void drawTubes(const BioStruct3DColorScheme* colorScheme); 
-    public:
-        
-        TubeGLRenderer(const BioStruct3D& struc, const BioStruct3DColorScheme* s);
-        
-        void drawBioStruct3D();
 
-        RENDERER_FACTORY(TubeGLRenderer)
+class TubeGLRenderer : public BioStruct3DGLRenderer {
+        
+    typedef QVector< SharedAtom > AtomsVector;
+    
+    struct Tube {
+        QMap<int, AtomsVector> modelsMap;
+    }; 
+    
 
-    };
+    QMap<int, Tube> tubeMap;
+       
+    void drawTubes(const BioStruct3DColorScheme* colorScheme); 
+public:
+        
+    TubeGLRenderer(const BioStruct3D& struc, const BioStruct3DColorScheme* s);
+        
+    void drawBioStruct3D();
+
+    RENDERER_FACTORY(TubeGLRenderer)
+
+};
 
 } //namespace
 
