@@ -1,68 +1,30 @@
 <?xml version="1.0" encoding="utf-8"?>
-<!DOCTYPE TS><TS version="1.1" language="en_US">
-<defaultcodec></defaultcodec>
-<context>
-    <name>GB2::GTest</name>
-    <message>
-        <location filename="../src/ORFMarkerTests.cpp" line="142"/>
-        <source>value not set %1</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/ORFMarkerTests.cpp" line="65"/>
-        <source>wrong value for %1</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/ORFMarkerTests.cpp" line="81"/>
-        <source>value not correct %1</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/ORFMarkerTests.cpp" line="147"/>
-        <source>Unable to convert. Value wrong %1</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/ORFMarkerTests.cpp" line="156"/>
-        <source>error can&apos;t cast to sequence from GObject</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/ORFMarkerTests.cpp" line="174"/>
-        <source>Expected and Actual lists of regions are different: %1 %2</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/ORFMarkerTests.cpp" line="179"/>
-        <source>One of the expected regions not found in results</source>
-        <translation></translation>
-    </message>
-</context>
+<!DOCTYPE TS>
+<TS version="2.0" language="en_US">
 <context>
     <name>GB2::LocalWorkflow::ORFPrompter</name>
     <message>
-        <location filename="../src/ORFWorker.cpp" line="108"/>
+        <location filename="../src/ORFWorker.cpp" line="109"/>
         <source> from %1</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/ORFWorker.cpp" line="119"/>
+        <location filename="../src/ORFWorker.cpp" line="120"/>
         <source>, allow ORFs &lt;u&gt;starting with any codon&lt;/u&gt; other than terminator</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/ORFWorker.cpp" line="121"/>
+        <location filename="../src/ORFWorker.cpp" line="122"/>
         <source>, take into account &lt;u&gt;alternative start codons&lt;/u&gt;</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/ORFWorker.cpp" line="124"/>
+        <location filename="../src/ORFWorker.cpp" line="125"/>
         <source>, &lt;u&gt;ignore non-terminated&lt;/u&gt; ORFs</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/ORFWorker.cpp" line="144"/>
+        <location filename="../src/ORFWorker.cpp" line="142"/>
         <source>For each nucleotide sequence&lt;u&gt;%1&lt;/u&gt;, find ORFs in &lt;u&gt;%2&lt;/u&gt; using the &lt;u&gt;%3&lt;/u&gt;.&lt;br&gt;Detect only ORFs &lt;u&gt;not shorter than %4 bps&lt;/u&gt;%5.&lt;br&gt;Output the list of found regions annotated as &lt;u&gt;%6&lt;/u&gt;.</source>
         <translation></translation>
     </message>
@@ -70,7 +32,7 @@
 <context>
     <name>GB2::LocalWorkflow::ORFWorker</name>
     <message>
-        <location filename="../src/ORFWorker.cpp" line="205"/>
+        <location filename="../src/ORFWorker.cpp" line="216"/>
         <source>Found %1 ORFs</source>
         <translation></translation>
     </message>
@@ -84,11 +46,6 @@
         <source>ORF annotations</source>
         <translation></translation>
     </message>
-    <message>
-        <location filename="../src/ORFWorker.cpp" line="50"/>
-        <source>Result annotation</source>
-        <translation></translation>
-    </message>
     <message>
         <location filename="../src/ORFWorker.cpp" line="51"/>
         <source>Search in</source>
@@ -99,11 +56,6 @@
         <source>Genetic code</source>
         <translation></translation>
     </message>
-    <message>
-        <location filename="../src/ORFWorker.cpp" line="53"/>
-        <source>Min length, bp</source>
-        <translation></translation>
-    </message>
     <message>
         <location filename="../src/ORFWorker.cpp" line="54"/>
         <source>Require stop codon</source>
@@ -125,17 +77,20 @@
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/ORFWorker.cpp" line="129"/>
+        <location filename="../src/ORFWorker.cpp" line="83"/>
+        <location filename="../src/ORFWorker.cpp" line="130"/>
         <source>both strands</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/ORFWorker.cpp" line="130"/>
+        <location filename="../src/ORFWorker.cpp" line="84"/>
+        <location filename="../src/ORFWorker.cpp" line="131"/>
         <source>direct strand</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/ORFWorker.cpp" line="131"/>
+        <location filename="../src/ORFWorker.cpp" line="85"/>
+        <location filename="../src/ORFWorker.cpp" line="132"/>
         <source>complement strand</source>
         <translation></translation>
     </message>
@@ -165,65 +120,85 @@
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/ORFWorker.cpp" line="74"/>
+        <location filename="../src/ORFWorker.cpp" line="68"/>
         <source>Finds Open Reading Frames (ORFs) in each supplied nucleotide sequence, stores found regions as annotations.&lt;p&gt;Protein sequences are skipped if any supplied to input.&lt;p&gt;&lt;dfn&gt;ORFs are DNA sequence regions that could potentially encode a protein, and usually give a good indication of the presence of a gene in the surrounding sequence.&lt;/dfn&gt;&lt;/p&gt;&lt;p&gt;In the sequence, ORFs are located between a start-code sequence (initiation codon) and a stop-code sequence (termination codon), defined by the selected genetic code.&lt;/p&gt;</source>
         <translation></translation>
     </message>
+    <message>
+        <location filename="../src/ORFWorker.cpp" line="50"/>
+        <source>Annotate as</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/ORFWorker.cpp" line="53"/>
+        <source>Min length</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/ORFWorker.cpp" line="79"/>
+        <source> bp</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/ORFWorker.cpp" line="193"/>
+        <source>Bad sequence supplied to ORFWorker: %1</source>
+        <translation></translation>
+    </message>
 </context>
 <context>
     <name>GB2::ORFDialog</name>
     <message>
-        <location filename="../src/ORFDialog.cpp" line="174"/>
+        <location filename="../src/ORFDialog.cpp" line="177"/>
         <source>cancel_button</source>
         <translation>Stop search</translation>
     </message>
     <message>
-        <location filename="../src/ORFDialog.cpp" line="174"/>
+        <location filename="../src/ORFDialog.cpp" line="177"/>
         <source>close_button</source>
         <translation>Close</translation>
     </message>
     <message>
-        <location filename="../src/ORFDialog.cpp" line="192"/>
+        <location filename="../src/ORFDialog.cpp" line="195"/>
         <source>progress_%1%</source>
         <translation>Progress %1%. </translation>
     </message>
     <message>
-        <location filename="../src/ORFDialog.cpp" line="194"/>
+        <location filename="../src/ORFDialog.cpp" line="197"/>
         <source>%1_results_found.</source>
         <translation>%1 results found.</translation>
     </message>
     <message>
-        <location filename="../src/ORFDialog.cpp" line="244"/>
+        <location filename="../src/ORFDialog.cpp" line="247"/>
         <source>warning</source>
         <translation>Warning!</translation>
     </message>
     <message>
-        <location filename="../src/ORFDialog.cpp" line="244"/>
+        <location filename="../src/ORFDialog.cpp" line="247"/>
         <source>results_list_not_empty_ask_clear</source>
         <translation>Results list contains results from the previous search. Clear?</translation>
     </message>
     <message>
-        <location filename="../src/ORFDialog.cpp" line="135"/>
+        <location filename="../src/ORFDialog.cpp" line="138"/>
         <source>start_codons</source>
         <translation>Start codons:</translation>
     </message>
     <message>
-        <location filename="../src/ORFDialog.cpp" line="136"/>
+        <location filename="../src/ORFDialog.cpp" line="139"/>
         <source>alt_start_codons</source>
         <translation>Alternative start codons:</translation>
     </message>
     <message>
-        <location filename="../src/ORFDialog.cpp" line="137"/>
+        <location filename="../src/ORFDialog.cpp" line="140"/>
         <source>stop_codons</source>
         <translation>Stop codons:</translation>
     </message>
     <message>
-        <location filename="../src/ORFDialog.cpp" line="372"/>
+        <location filename="../src/ORFDialog.cpp" line="376"/>
         <source>complem_strand</source>
         <translation>complement</translation>
     </message>
     <message>
-        <location filename="../src/ORFDialog.cpp" line="372"/>
+        <location filename="../src/ORFDialog.cpp" line="376"/>
         <source>direct_strand</source>
         <translation>direct</translation>
     </message>
@@ -231,12 +206,12 @@
 <context>
     <name>GB2::ORFMarkerPlugin</name>
     <message>
-        <location filename="../src/ORFMarkerPlugin.cpp" line="33"/>
+        <location filename="../src/ORFMarkerPlugin.cpp" line="34"/>
         <source>orfname</source>
         <translation>ORF Marker</translation>
     </message>
     <message>
-        <location filename="../src/ORFMarkerPlugin.cpp" line="33"/>
+        <location filename="../src/ORFMarkerPlugin.cpp" line="34"/>
         <source>orfdesc</source>
         <translation>Finds open reading frames (ORF) in DNA sequence</translation>
     </message>
@@ -244,9 +219,9 @@
 <context>
     <name>GB2::ORFViewContext</name>
     <message>
-        <location filename="../src/ORFMarkerPlugin.cpp" line="69"/>
-        <source>find_orfs</source>
-        <translation>Find ORFs</translation>
+        <location filename="../src/ORFMarkerPlugin.cpp" line="66"/>
+        <source>Find ORFs</source>
+        <translation></translation>
     </message>
 </context>
 <context>
@@ -272,6 +247,7 @@
         <translation>Both</translation>
     </message>
     <message>
+        <location filename="../src/ui/ORFDialogUI.ui" line="81"/>
         <location filename="../src/ui/ORFDialogUI.ui" line="84"/>
         <source>strand_direct_wit</source>
         <translation>Search frames in direct strand only</translation>
@@ -282,6 +258,7 @@
         <translation>Direct</translation>
     </message>
     <message>
+        <location filename="../src/ui/ORFDialogUI.ui" line="100"/>
         <location filename="../src/ui/ORFDialogUI.ui" line="103"/>
         <source>strand_complement_wit</source>
         <translation>Search frames in complement strand only</translation>
@@ -362,58 +339,55 @@
         <translation>Range end:</translation>
     </message>
     <message>
-        <location filename="../src/ui/ORFDialogUI.ui" line="476"/>
-        <source>sel_button</source>
-        <translation>Selection range</translation>
-    </message>
-    <message>
-        <location filename="../src/ui/ORFDialogUI.ui" line="489"/>
-        <source>pv_button</source>
-        <translation>PanView range</translation>
-    </message>
-    <message>
-        <location filename="../src/ui/ORFDialogUI.ui" line="502"/>
-        <source>wh_seq_button</source>
-        <translation>Sequence range</translation>
-    </message>
-    <message>
+        <location filename="../src/ORFWorker.cpp" line="53"/>
+        <location filename="../src/ui/ORFDialogUI.ui" line="133"/>
         <location filename="../src/ui/ORFDialogUI.ui" line="136"/>
         <source>min_len_tip</source>
         <translation>Ignore ORFs shorter than the specified length</translation>
     </message>
     <message>
+        <location filename="../src/ORFWorker.cpp" line="54"/>
+        <location filename="../src/ui/ORFDialogUI.ui" line="182"/>
         <location filename="../src/ui/ORFDialogUI.ui" line="185"/>
         <source>must_fit_tip</source>
         <translation>Ignore boundary ORFs which last beyound the search region
 (i.e. have no stop codon within the range).</translation>
     </message>
     <message>
+        <location filename="../src/ORFWorker.cpp" line="55"/>
+        <location filename="../src/ui/ORFDialogUI.ui" line="195"/>
         <location filename="../src/ui/ORFDialogUI.ui" line="198"/>
         <source>must_init_tip</source>
         <translation>Allow or not ORFs starting with any codon other than terminator</translation>
     </message>
     <message>
+        <location filename="../src/ORFWorker.cpp" line="56"/>
+        <location filename="../src/ui/ORFDialogUI.ui" line="214"/>
         <location filename="../src/ui/ORFDialogUI.ui" line="217"/>
         <source>allow_alt_tip</source>
         <translation>Allow ORFs starting with alternative initiation codons, 
 accordingly to the current translation table.</translation>
     </message>
     <message>
+        <location filename="../src/ui/ORFDialogUI.ui" line="65"/>
         <location filename="../src/ui/ORFDialogUI.ui" line="68"/>
         <source>strand_both_wit</source>
         <translation>Search in both strands</translation>
     </message>
     <message>
+        <location filename="../src/ui/ORFDialogUI.ui" line="470"/>
         <location filename="../src/ui/ORFDialogUI.ui" line="473"/>
         <source>sel_bt_tip</source>
         <translation>Set the range to current sequence selection</translation>
     </message>
     <message>
+        <location filename="../src/ui/ORFDialogUI.ui" line="483"/>
         <location filename="../src/ui/ORFDialogUI.ui" line="486"/>
         <source>pv_bt_tip</source>
         <translation>Set the range to Panoramic View range</translation>
     </message>
     <message>
+        <location filename="../src/ui/ORFDialogUI.ui" line="496"/>
         <location filename="../src/ui/ORFDialogUI.ui" line="499"/>
         <source>seq_bt_tip</source>
         <translation>Set the range to the whole sequence</translation>
@@ -438,5 +412,20 @@ accordingly to the current translation table.</translation>
         <source>len</source>
         <translation>Length</translation>
     </message>
+    <message>
+        <location filename="../src/ui/ORFDialogUI.ui" line="476"/>
+        <source>Selection range</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/ui/ORFDialogUI.ui" line="489"/>
+        <source>Overview range</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/ui/ORFDialogUI.ui" line="502"/>
+        <source>Sequence range</source>
+        <translation></translation>
+    </message>
 </context>
 </TS>
