@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -22,7 +22,7 @@ namespace GB2 {
 class GTest_RemoteRequest : public GTest {
     Q_OBJECT
 public:
-    SIMPLE_XML_TEST_BODY_WITH_FACTORY_AND_EXT(GTest_RemoteRequest, "plugin_remote-request", false);
+    SIMPLE_XML_TEST_BODY_WITH_FACTORY(GTest_RemoteRequest, "plugin_remote-request");
     ~GTest_RemoteRequest(){};
 
     void prepare();
@@ -30,7 +30,7 @@ public:
     Task::ReportResult report();
 private:
     AnnotationTableObject *ao;
-    RemoteRequestTask *task;
+    RemoteRequestToAnnotationsTask *task;
     int minLength;
     int maxLength;
     SendSelectionStrand strand;
