@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -10,7 +10,7 @@
 
 #include "TestViewController.h"
 #include "TestRunnerPlugin.h"
-#include "TestRunnerTask.h"
+#include <test_framework/TestRunnerTask.h>
 #include "TestViewReporter.h"
 
 #include <core_api/AppContext.h>
@@ -37,10 +37,11 @@ TestViewReporter::TestViewReporter(TestViewController* parent,QTreeWidget* tree,
     setupUi(this);
 
     save = new QAction(tr("save"), this);
+    save->setObjectName("action_save");
     connect(save, SIGNAL(triggered()), SLOT(sl_save()));
 
     curReportText=prepareHTMLText(tree,runTime);
-    this->reportText->setText(curReportText);
+    this->reportText->setHtml(curReportText);
     //saveAs("c:/1.html", curReportText);//test line
 
 }
@@ -53,19 +54,22 @@ const QString TestViewReporter::prepareHTMLText(QTreeWidget* tree,int runTime){
         int tpassed = 0;
         int tfailed = 0;
         int tnone = 0;
+        int texcluded = 0;
         for (int i=0, n = tree->topLevelItemCount(); i<n; i++) {
             TVItem* item = (TVItem*)tree->topLevelItem(i);     
             assert(item->isSuite());
             TVTSItem* tItem  = (TVTSItem*)item;
 
             int passed=0,failed=0,none=0;
+            int excluded = 0;
             int procent=100;
             bool norun=false;
 
-            tItem->getTestsState(&passed,&failed,&none);
+            tItem->getTestsState(&passed,&failed,&none, &excluded);
             tpassed=tpassed+passed;
             tfailed=tfailed+failed;
             tnone=tnone+none;
+            texcluded+=excluded; //TODO include to report
 
             if(failed!=0 && passed!=0){
                 procent=(passed*100)/(failed+passed);
@@ -83,6 +87,7 @@ const QString TestViewReporter::prepareHTMLText(QTreeWidget* tree,int runTime){
             tempRezult+=getHTMLStaticInfo(passed+failed+none);
             tempRezult+=getHTMLStaticInfo(passed);
             tempRezult+=getHTMLStaticInfo(failed);
+            tempRezult+=getHTMLStaticInfo(excluded);
             tempRezult+=getHTMLStatusBar(procent,norun);
             tempRezult+="</tr>\n";
         }
@@ -108,6 +113,7 @@ const QString TestViewReporter::prepareHTMLText(QTreeWidget* tree,int runTime){
         rezult+=getHTMLStaticInfo(tpassed+tfailed+tnone);
         rezult+=getHTMLStaticInfo(tpassed);
         rezult+=getHTMLStaticInfo(tfailed);
+        rezult+=getHTMLStaticInfo(texcluded);
         rezult+=getHTMLStatusBar(tprocent);
 
         rezult+=tempRezult;
@@ -164,7 +170,7 @@ const QString TestViewReporter::getHTMLRuntime(int data){
     rezult+=QString::number(data);
     rezult+=" s";
 
-    rezult+=("</th>\n<th>Number of tests</th>\n<th>Pass</th>\n<th>Fail</th>\n<th colspan=\"2\">Success Rate</th>\n</tr>\n\n<tr>\n");
+    rezult+=("</th>\n<th>Number of tests</th>\n<th>Pass</th>\n<th>Fail</th>\n<th>Excluded</th>\n<th colspan=\"2\">Success Rate</th>\n</tr>\n\n<tr>\n");
     return rezult;
 }
 
@@ -421,7 +427,7 @@ bool TestViewReporter::saveAs(const QString url,const QString data){
     std::auto_ptr<IOAdapter> io(iof->createIOAdapter());
     if (!io->open(url, IOAdapterMode_Write)) {
         return false;
-        //ti.error = IOAdapter::tr("error_opening_url_for_write '%1'").arg(url);
+        //ti.setError(  IOAdapter::tr("error_opening_url_for_write '%1'").arg(url) );
     }
     io->writeBlock(data.toUtf8());
     return true;
