@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -20,9 +20,6 @@ class QEvent;
 
 namespace GB2 {
 
-class WorkflowDocFormat;
-class WorkflowViewFactory;
-
 class WorkflowDesignerPlugin : public Plugin {
 	Q_OBJECT
 public:
@@ -33,24 +30,22 @@ public:
 class WorkflowDesignerService : public Service {
     Q_OBJECT
 public:
-    WorkflowDesignerService(Plugin* p);
+    WorkflowDesignerService();
     bool closeViews();
 protected:
-    //virtual Task* createServiceEnablingTask();
+    virtual Task* createServiceEnablingTask();
 
     virtual Task* createServiceDisablingTask();
 
     virtual void serviceStateChangedCallback(ServiceState oldState, bool enabledStateChanged);
 
 private slots:
-        void sl_showDesignerWindow();
-        void sl_showManagerWindow();
+    void sl_showDesignerWindow();
+    void sl_showManagerWindow();
 
 private:
     QAction*        designerAction;
     QAction*        managerAction;
-    WorkflowDocFormat* df;
-    WorkflowViewFactory* vf;
 };
 
 
