@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -13,13 +13,14 @@
 #include "WorkflowViewItems.h"
 #include "WorkflowDocument.h"
 #include "SceneSerializer.h"
-#include "WorkflowIOTasks.h"
+#include "WorkflowSceneTasks.h"
 #include "WorkflowPalette.h"
 #include "WorkflowEditor.h"
-#include "WorkflowRunTask.h"
 #include "WorkflowMetaDialog.h"
 #include "ChooseItemDialog.h"
 #include "SchemaConfigurationDialog.h"
+#include "WorkflowSamples.h"
+#include "GlassView.h"
 
 #include <workflow/WorkflowModel.h>
 #include <workflow/WorkflowRegistry.h>
@@ -28,6 +29,8 @@
 #include <workflow/WorkflowManager.h>
 #include <workflow_support/TypeMapEditor.h>
 #include <workflow_support/WorkflowUtils.h>
+#include <workflow_support/WorkflowRunTask.h>
+#include <workflow_support/WorkflowSettings.h>
 
 #include <core_api/AppContext.h>
 #include <core_api/MainWindow.h>
@@ -54,6 +57,7 @@
 #include <QtGui/QMessageBox>
 #include <QtGui/QCloseEvent>
 #include <QtGui/QToolBar>
+#include <QtGui/QClipboard>
 
 #include <QtXml/qdom.h>
 
@@ -65,22 +69,24 @@ namespace GB2 {
 
 static LogCategory log(ULOG_CAT_WD);
 
-#define SETTINGS QString("workflowview/")
 #define LAST_DIR SETTINGS + "lastdir"
 #define SPLITTER_STATE SETTINGS + "splitter"
 #define EDITOR_STATE SETTINGS + "editor"
 #define PALETTE_STATE SETTINGS + "palette"
+#define TABS_STATE SETTINGS + "tabs"
+
+enum {ElementsTab,SamplesTab};
 
 #define WS 1000
 WorkflowView::WorkflowView(WorkflowGObject* go) : MWMDIWindow(tr("Workflow Designer")), go(go),
-         currentProc(NULL), clipbrd(NULL)
+         currentProc(NULL), pasteCount(0)
 {
     scene = new WorkflowScene(this);
     scene->setSceneRect(QRectF(-WS, -WS, WS, WS));
     connect(scene, SIGNAL(processItemAdded()), SLOT(sl_procItemAdded()));
     connect(scene, SIGNAL(processDblClicked()), SLOT(sl_toggleStyle()));
 
-    palette = new WorkflowPalette(WorkflowEnv::getProtoRegistry(), this);
+    palette = new WorkflowPalette(WorkflowEnv::getProtoRegistry()/*, this*/);
     palette->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Ignored));
     connect(palette, SIGNAL(processSelected(Workflow::ActorPrototype*)), 
         SLOT(sl_selectProcess(Workflow::ActorPrototype*)));
@@ -88,9 +94,23 @@ WorkflowView::WorkflowView(WorkflowGObject* go) : MWMDIWindow(tr("Workflow Desig
     infoList = new QListWidget(this);
     connect(infoList, SIGNAL(itemDoubleClicked(QListWidgetItem*)), SLOT(sl_pickInfo(QListWidgetItem*)));
     splitter = new QSplitter(this);
-    splitter->addWidget(palette);
 
-    QGraphicsView* sceneView = new QGraphicsView(scene);
+    tabs = new QTabWidget(this);
+    tabs->insertTab(ElementsTab, palette, tr("Objects"));
+    SamplesWidget* samples = new SamplesWidget();
+    
+    tabs->insertTab(SamplesTab, samples, tr("Samples"));
+    splitter->addWidget(tabs);
+
+    QGraphicsView* sceneView = new GlassView(scene);
+    connect(samples, SIGNAL(setupGlass(GlassPane*)), sceneView, SLOT(setGlass(GlassPane*)));
+    connect(samples, SIGNAL(sampleSelected(QString)), this, SLOT(sl_pasteSample(QString)));
+    connect(tabs, SIGNAL(currentChanged(int)), samples, SLOT(cancelItem()));
+    connect(tabs, SIGNAL(currentChanged(int)), palette, SLOT(resetSelection()));
+    connect(tabs, SIGNAL(currentChanged(int)), scene, SLOT(setHint(int)));
+
+    //sceneView->setResizeAnchor(QGraphicsView::AnchorViewCenter);
+    sceneView->setAlignment(Qt::AlignCenter);
     infoSplitter = new QSplitter(Qt::Vertical, splitter);
     infoSplitter->addWidget(sceneView);
     {
@@ -100,6 +120,7 @@ WorkflowView::WorkflowView(WorkflowGObject* go) : MWMDIWindow(tr("Workflow Desig
         QVBoxLayout* vl = new QVBoxLayout(w);
         vl->setSpacing(0);
         vl->setMargin(0);
+        vl->setContentsMargins(0,0,0,0);
         vl->addWidget(infoList);
         w->hide();
         infoSplitter->addWidget(w);
@@ -122,19 +143,20 @@ WorkflowView::WorkflowView(WorkflowGObject* go) : MWMDIWindow(tr("Workflow Desig
     if (settings->contains(PALETTE_STATE)) {
         palette->restoreState(settings->getValue(PALETTE_STATE));
     }
-    splitter->setChildrenCollapsible(false);
+    tabs->setCurrentIndex(settings->getValue(TABS_STATE, SamplesTab).toInt());
+
     scene->views().at(0)->setDragMode(QGraphicsView::RubberBandDrag);
-    scene->views().at(0)->setRubberBandSelectionMode(Qt::ContainsItemShape);
+    //scene->views().at(0)->setRubberBandSelectionMode(Qt::ContainsItemShape);
     
     QHBoxLayout *layout = new QHBoxLayout;
     layout->addWidget(splitter);
+    layout->setSpacing(0);
+    layout->setMargin(0);
+    layout->setContentsMargins(0, 0, 0, 0);
     setLayout(layout);
 
     createActions();
     
-    connect(scene, SIGNAL(configurationChanged()), SLOT(sl_refreshActorDocs()));
-    connect(propertyEditor, SIGNAL(iterationSelected()), SLOT(sl_refreshActorDocs()));
-
     if (go) {
         //FIXME
         go->setView(this);
@@ -146,18 +168,25 @@ WorkflowView::WorkflowView(WorkflowGObject* go) : MWMDIWindow(tr("Workflow Desig
         } else {
             QList<Iteration> lst;
             SchemaSerializer::readIterations(lst, go->getXML().documentElement(), remapping);
-            scene->setIterations(lst);
+            //if (!lst.isEmpty()) 
+                scene->setIterations(lst);
             SchemaSerializer::readMeta(&meta, go->getXML().documentElement());
-            meta.url = go->getDocument()->getURL();
+            if (go->getDocument())
+                meta.url = go->getDocument()->getURL();
+            sl_updateTitle();
+            propertyEditor->resetIterations();
             scene->setModified(false);
         }
     } else {
         sl_newScene();
     }
+    connect(scene, SIGNAL(configurationChanged()), SLOT(sl_refreshActorDocs()));
+    connect(propertyEditor, SIGNAL(iterationSelected()), SLOT(sl_refreshActorDocs()));
+    connect(WorkflowSettings::watcher, SIGNAL(changed()), scene, SLOT(update()));
 }
 
 WorkflowView::~WorkflowView() {
-	delete clipbrd;
+    log.trace("~WorkflowView");
 }
 
 static QString percentStr = WorkflowView::tr("%");
@@ -270,6 +299,40 @@ void WorkflowView::createActions() {
     extStyle->setData(QVariant(ItemStyles::EXTENDED));
     connect(extStyle, SIGNAL(triggered()), SLOT(sl_setStyle()));
     styleActions << simpleStyle << extStyle;
+
+    unlockAction = new QAction(tr("Unlock Scene"), this);
+    unlockAction->setCheckable(true);
+    unlockAction->setChecked(true);
+    connect(unlockAction, SIGNAL(toggled(bool)), SLOT(sl_toggleLock(bool)));
+}
+
+void WorkflowView::sl_toggleLock(bool b) {
+    if (sender() != unlockAction) {
+        unlockAction->setChecked(b);
+        return;
+    }
+
+    if (b) {
+        scene->setRunner(NULL);
+    }
+
+    deleteAction->setEnabled(b);
+    selectAction->setEnabled(b);
+    copyAction->setEnabled(b);
+    pasteAction->setEnabled(b);
+    cutAction->setEnabled(b);
+    loadAction->setEnabled(b);
+    //newAction->setEnabled(b);
+
+    configureAction->setEnabled(b);
+    runAction->setEnabled(b);
+    validateAction->setEnabled(b);
+
+    propertyEditor->setEnabled(b);
+    palette->setEnabled(b);
+
+    scene->setLocked(!b);
+    scene->update();
 }
 
 void WorkflowView::sl_setStyle() {
@@ -368,10 +431,23 @@ void WorkflowView::setupViewMenu(QMenu* m) {
         ttMenu->addAction(a);
     }
     m->addMenu(ttMenu);
+
+    if (!unlockAction->isChecked()) {
+        m->addSeparator();
+        m->addAction(unlockAction);
+    }
 }
 
-void WorkflowView::setupContextMenu(QMenu* m, bool itemSelected) {
-    if (itemSelected) {
+void WorkflowView::setupContextMenu(QMenu* m) {
+    if (!unlockAction->isChecked()) {
+        m->addAction(unlockAction);
+        return;
+    }
+    if (!QApplication::clipboard()->text().isEmpty()) {
+        m->addAction(pasteAction);
+    }
+    QList<QGraphicsItem*> sel = scene->selectedItems();
+    if (!sel.isEmpty()) {
 		m->addAction(copyAction);
 		m->addAction(cutAction);
         m->addAction(deleteAction);
@@ -379,23 +455,26 @@ void WorkflowView::setupContextMenu(QMenu* m, bool itemSelected) {
         m->addAction(bringToFrontAction);
         m->addAction(sendToBackAction);
         m->addSeparator();
+        if (sel.size() == 1 && sel.first()->type() == WorkflowProcessItemType) {
+            QMenu* itMenu = new QMenu(tr("Item properties"));
+            WorkflowProcessItem* wit = qgraphicsitem_cast<WorkflowProcessItem*>(sel.first());
+            foreach(QAction* a, wit->getContextMenuActions()) {
+                itMenu->addAction(a);
+            }
+            m->addMenu(itMenu);
+        }
+        
         QMenu* ttMenu = new QMenu(tr("Item style"));
         foreach(QAction* a, styleActions) {
             ttMenu->addAction(a);
         }
         m->addMenu(ttMenu);
+        m->addSeparator();
     }
-	if (clipbrd) {
-		m->addAction(pasteAction);
-	}
     m->addAction(selectAction);
     m->addMenu(palette->createMenu(tr("Add item")));
 }
 
-#define ACTOR_REF Qt::UserRole
-#define PORT_REF Qt::UserRole + 1
-#define ITERATION_REF Qt::UserRole + 2
-
 void WorkflowView::sl_pickInfo(QListWidgetItem* info) {
     ActorId id = info->data(ACTOR_REF).value<ActorId>();
     foreach(QGraphicsItem* it, scene->items()) {
@@ -423,48 +502,13 @@ void WorkflowView::sl_pickInfo(QListWidgetItem* info) {
 bool WorkflowView::sl_validate(bool notify) {
     propertyEditor->commit();
     infoList->clear();
-    Schema schema = scene->getSchema();
-    bool good = true;
-    foreach (Actor* a, schema.procs) {
-        foreach(Port* p, a->getPorts()) {
-            QStringList l;
-            bool ag = p->validate(l);
-            good &= ag;
-            if (!l.isEmpty()) {
-                foreach(QString s, l) {
-                    QListWidgetItem* item = new QListWidgetItem(a->getProto()->icon, 
-                        QString("%1 : %2").arg(a->getLabel()).arg(s));
-                    item->setData(PORT_REF, p->getId());
-                    item->setData(ACTOR_REF, a->getId());
-                    infoList->addItem(item);
-                }
-            }
+    QList<QListWidgetItem*> lst;
+    bool good = DesignerUtils::validate(scene->getSchema(), &lst);
+
+    if (lst.count() != 0) {
+        foreach(QListWidgetItem* wi, lst) {
+            infoList->addItem(wi);
         }
-    }
-
-    foreach (Iteration it, scene->getIterations()) {
-        Schema sh;
-        QMap<ActorId, ActorId> map = SchemaSerializer::deepCopy(schema, &sh);
-        sh.applyConfiguration(it, map);
-
-        foreach (Actor* a, sh.procs) {
-            QStringList l;
-            bool ag = a->validate(l);
-            good &= ag;
-            if (!l.isEmpty()) {
-                foreach(QString s, l) {
-                    QListWidgetItem* item = new QListWidgetItem(a->getProto()->icon, 
-                        tr("Iteration '%3', %1 : %2").arg(a->getLabel()).arg(s).arg(it.name));
-                    item->setData(ACTOR_REF, map.key(a->getId()));
-                    item->setData(ITERATION_REF, it.id);
-                    //item->setToolTip(a->getDocumentation());
-                    infoList->addItem(item);
-                }
-            }
-        }
-    }
-
-    if (infoList->count() != 0) {
         infoList->parentWidget()->show();
         QList<int> s = infoSplitter->sizes();
         if (s.last() == 0) {
@@ -496,8 +540,14 @@ void WorkflowView::sl_launch() {
         //|TODO user choice
         sh.domain = WorkflowEnv::getDomainRegistry()->getAllIds().value(0);
     }
-    Task* t = new WorkflowRunTask(sh, scene->getIterations());
+    WorkflowRunTask* t = new WorkflowRunTask(sh, scene->getIterations());
     t->setReportingEnabled(true);
+    if (WorkflowSettings::monitorRun()) {
+        unlockAction->setChecked(false);
+        scene->setRunner(t);
+        connect(t, SIGNAL(si_ticked()), scene, SLOT(update()));
+        connect(new TaskSignalMapper(t), SIGNAL(si_taskFinished(Task*)), SLOT(sl_toggleLock()));
+    }
     AppContext::getTaskScheduler()->registerTopLevelTask(t);
 }
 
@@ -530,12 +580,29 @@ void WorkflowView::sl_selectProcess(Workflow::ActorPrototype* p) {
 void WorkflowView::sl_copyItems() {
 	QList<QGraphicsItem*> items = scene->selectedItems();
 	if (items.isEmpty()) return;
-	delete clipbrd;
-	clipbrd = new QDomDocument();
-	QDomElement root = clipbrd->createElement("root");
-	clipbrd->appendChild(root);
+	QDomDocument xml(SchemaSerializer::WORKFLOW_DOC);
+	QDomElement root = xml.createElement("root");
+	xml.appendChild(root);
 	SceneSerializer::saveItems(items, root);
-    //FIXME copy iteration data
+    QList<Iteration> lst;
+    foreach(const Iteration& it, scene->getIterations()) {
+        Iteration copy(it.name);
+        copy.id = it.id;
+        foreach(QGraphicsItem* item, items) {
+            WorkflowProcessItem* proc = qgraphicsitem_cast<WorkflowProcessItem*>(item);
+            if (proc && it.cfg.contains(proc->getProcess()->getId())) {
+                copy.cfg.insert(proc->getProcess()->getId(), it.cfg.value(proc->getProcess()->getId()));
+            }
+        }
+        if (!copy.cfg.isEmpty()) {
+            lst.append(copy);
+        }
+    }
+    if (!lst.isEmpty()) {
+        SchemaSerializer::saveIterations(lst, root);
+    }
+    lastPaste = xml.toString();
+    QApplication::clipboard()->setText(lastPaste);
     pasteCount = 0;
 }
 
@@ -544,18 +611,79 @@ void WorkflowView::sl_cutItems() {
 	scene->sl_deleteItem();
 }
 
-void WorkflowView::sl_pasteItems() {
-	if (!clipbrd) return;
-	scene->clearSelection();
-    //FIXME copy iteration data
+void WorkflowView::sl_pasteSample(const QString& s) {
+    tabs->setCurrentIndex(ElementsTab);
+    QDomDocument xml;
+    xml.setContent(s);
+   if (scene->items().isEmpty()) {
+        sl_pasteItems(s);
+        SchemaSerializer::readMeta(&meta, xml.documentElement());
+        sl_updateTitle();
+    } else {
+        saveState();
+        WorkflowGObject* smpl = new WorkflowGObject("Sample", xml);
+        WorkflowView* view = new WorkflowView(smpl);
+        AppContext::getMainWindow()->getMDIManager()->addMDIWindow(view);
+        AppContext::getMainWindow()->getMDIManager()->activateWindow(view);
+    }
+}
 
-    QMap<ActorId, ActorId> stub;
-    QString msg = SceneSerializer::xml2scene(clipbrd->documentElement(), scene, stub, true, true);
+void WorkflowView::sl_pasteItems(const QString& s) {
+    QString tmp = s.isNull() ? QApplication::clipboard()->text() : s;
+    if (tmp == lastPaste) {
+        ++pasteCount;
+    } else {
+        pasteCount = 0;
+        lastPaste = tmp;
+    }
+    QDomDocument xml;
+    if (!xml.setContent(lastPaste) || xml.doctype().name() != SchemaSerializer::WORKFLOW_DOC) {
+        return;
+    }
+    scene->clearSelection();
+    QMap<ActorId, ActorId> remapping;
+    QString msg = SceneSerializer::xml2scene(xml.documentElement(), scene, remapping, true, true);
     if (!msg.isEmpty()) {
         log.trace("Paste issues: " + msg);
     }
+    // merge iteration data
+    QList<Iteration> iterations;
+    SchemaSerializer::readIterations(iterations, xml.documentElement(), remapping);
+    if (!iterations.isEmpty()) {
+        QList<Iteration> current = scene->getIterations();
+        if (iterations.size() == 1 || current.size() == 1) {
+            // one-to-many mapping
+            Iteration& it = iterations.size() == 1 ? iterations[0] : current[0];
+            QList<Iteration>& target = iterations.size() == 1 ? current : iterations;
+            for (int i = 0; i < target.size(); i++)
+            {
+                target[i].cfg.unite(it.cfg); //unite is safe as no actor ids can coincide
+            }
+            scene->setIterations(target);
+        } else {
+            // many-to-many
+            foreach(const Iteration& pasted, iterations) {
+                bool gotIt = false;
+                for (int i = 0; i < current.size(); i++)
+                {
+                    Iteration& it = current[i];
+                    if (it.name == pasted.name && it.id == pasted.id) {
+                        it.cfg.unite(pasted.cfg);
+                        gotIt = true;
+                        break;
+                    } 
+                }
+                if (!gotIt) {
+                    current.append(pasted);
+                }
+            }
+            scene->setIterations(current);
+        }
+        propertyEditor->resetIterations();
+    }
+
 	QGraphicsItemGroup* g = scene->createItemGroup(scene->selectedItems());
-    int shift = 10*(++pasteCount);
+    int shift = GRID_STEP*(pasteCount);
 	g->moveBy(shift, shift);
 	scene->destroyItemGroup(g);
 }
@@ -661,7 +789,7 @@ void WorkflowView::sl_saveScene() {
         sl_updateTitle();
     }
     propertyEditor->commit();
-    Task* t = new SaveWorkflowSchemaTask(scene, meta); 
+    Task* t = new SaveWorkflowSceneTask(scene, meta); 
     AppContext::getTaskScheduler()->registerTopLevelTask(t);
 }
 
@@ -673,7 +801,7 @@ void WorkflowView::sl_saveSceneAs() {
     }
     propertyEditor->commit();
     meta = md.meta;
-    Task* t = new SaveWorkflowSchemaTask(scene, meta);
+    Task* t = new SaveWorkflowSceneTask(scene, meta);
     AppContext::getTaskScheduler()->registerTopLevelTask(t);
     sl_updateTitle();
 }
@@ -688,10 +816,11 @@ void WorkflowView::sl_loadScene() {
     QString url = QFileDialog::getOpenFileName(0, tr("Open workflow schema file"), dir, filter);
     if (!url.isEmpty()) {
         AppContext::getSettings()->setValue(LAST_DIR, QFileInfo(url).absoluteDir().absolutePath());
-        Task* t = new LoadWorkflowSchemaTask(scene, &meta, url); //FIXME unsynchronized meta usage
+        Task* t = new LoadWorkflowSceneTask(scene, &meta, url); //FIXME unsynchronized meta usage
         TaskSignalMapper* m = new TaskSignalMapper(t);
-        connect(m, SIGNAL(si_taskFinished()), SLOT(sl_updateTitle()));
-        connect(m, SIGNAL(si_taskFinished()), propertyEditor, SLOT(resetIterations()));
+        connect(m, SIGNAL(si_taskFinished(Task*)), SLOT(sl_updateTitle()));
+        connect(m, SIGNAL(si_taskFinished(Task*)), scene, SLOT(centerView()));
+        connect(m, SIGNAL(si_taskFinished(Task*)), propertyEditor, SLOT(resetIterations()));
         AppContext::getTaskScheduler()->registerTopLevelTask(t);
     }
 }
@@ -713,10 +842,15 @@ void WorkflowView::sl_updateTitle() {
     setWindowName(tr("Workflow Designer - %1").arg(meta.name));    
 }
 
-bool WorkflowView::onCloseEvent() {
+void WorkflowView::saveState() {
     AppContext::getSettings()->setValue(SPLITTER_STATE, splitter->saveState());
     AppContext::getSettings()->setValue(EDITOR_STATE, propertyEditor->saveState());
     AppContext::getSettings()->setValue(PALETTE_STATE, palette->saveState());
+    AppContext::getSettings()->setValue(TABS_STATE, tabs->currentIndex());
+}
+
+bool WorkflowView::onCloseEvent() {
+    saveState();
     if (!confirmModified()) {
         return false;
     }
@@ -728,7 +862,7 @@ bool WorkflowView::onCloseEvent() {
 
 bool WorkflowView::confirmModified() {
     propertyEditor->commit();
-    if (scene->isModified()) {
+    if (scene->isModified() && !scene->items().isEmpty()) {
         AppContext::getMainWindow()->getMDIManager()->activateWindow(this);
         int ret = QMessageBox::question(this, tr("Workflow Designer"),
             tr("The schema has been modified.\n"
@@ -749,7 +883,7 @@ bool WorkflowView::confirmModified() {
 
 
 WorkflowScene::WorkflowScene(WorkflowView *parent) 
-: QGraphicsScene(parent), controller(parent), modified(false) {
+: QGraphicsScene(parent), controller(parent), modified(false), locked(false), runner(NULL) {
 }
 
 Schema WorkflowScene::getSchema() const {
@@ -764,21 +898,20 @@ Schema WorkflowScene::getSchema() const {
             schema.flows << ((WBusItem*)it)->getBus();
         }
     }
+    schema.iterations = iterations;
 
     return schema;
 }
 
 void WorkflowScene::sl_deleteItem() {
-    //FIXME cleanup iterations as well
-    QList<QGraphicsItem*> items;
+    assert(!locked);
     QList<ActorId> ids;
+    QList<QGraphicsItem*> items;
     foreach(QGraphicsItem* it, selectedItems()) {
-        if (it->type() == WorkflowPortItemType) {
-            continue;
-        }
-        items << it;
-        if (it->type() == WorkflowProcessItemType) {
-            ids << qgraphicsitem_cast<WorkflowProcessItem*>(it)->getProcess()->getId();
+        switch (it->type()) {
+            case WorkflowProcessItemType:
+                ids << qgraphicsitem_cast<WorkflowProcessItem*>(it)->getProcess()->getId();
+                items << it;
         }
     }
     modified |= !items.isEmpty();
@@ -786,12 +919,21 @@ void WorkflowScene::sl_deleteItem() {
         removeItem(it);
         delete it;
     }
+    foreach(QGraphicsItem* it, selectedItems()) {
+        switch (it->type()) {
+            case WorkflowBusItemType:
+                removeItem(it);
+                delete it;
+                modified = true;
+                break;
+        }
+    }
     bool cfgModified = false;
     for (int i = 0; i<iterations.size(); i++) {
         Iteration& it = iterations[i];
-        foreach(IterationCfgKey key, it.cfg.uniqueKeys()) {
-            if (ids.contains(key.first)) {
-                it.cfg.remove(key);
+        foreach(const ActorId& id, ids) {
+            if (it.cfg.contains(id)) {
+                it.cfg.remove(id);
                 cfgModified = true;
             }
         }
@@ -815,7 +957,7 @@ QList<Actor*> WorkflowScene::getSelectedProcItems() const {
 
 void WorkflowScene::contextMenuEvent(QGraphicsSceneContextMenuEvent * e) {
     QMenu menu;
-    controller->setupContextMenu(&menu, !selectedItems().isEmpty());
+    controller->setupContextMenu(&menu);
     e->accept();
     menu.exec(e->screenPos());
 }
@@ -849,29 +991,25 @@ static bool canDrop(const QMimeData* m, QList<ActorPrototype*>& lst) {
 
 void WorkflowScene::dragEnterEvent(QGraphicsSceneDragDropEvent * event) {
     QList<ActorPrototype*> lst;
-    if (canDrop(event->mimeData(), lst)) {
+    if (!locked && canDrop(event->mimeData(), lst)) {
         event->acceptProposedAction();
-    } 
-    else
-    {
+    } else {
         QGraphicsScene::dragEnterEvent(event);
     }
 }
 
 void WorkflowScene::dragMoveEvent(QGraphicsSceneDragDropEvent * event) {
     QList<ActorPrototype*> lst;
-        if (canDrop(event->mimeData(), lst)) {
+    if (!locked && canDrop(event->mimeData(), lst)) {
         event->acceptProposedAction();
-    } 
-    else 
-    {
+    } else {
         QGraphicsScene::dragMoveEvent(event);
     }
 }
 
 void WorkflowScene::dropEvent(QGraphicsSceneDragDropEvent * event) {
     QList<ActorPrototype*> lst;
-    if (canDrop(event->mimeData(), lst))
+    if (!locked && canDrop(event->mimeData(), lst))
     {
         QList<QGraphicsItem *> targets = items(event->scenePos());
         bool done = false;
@@ -910,17 +1048,21 @@ void WorkflowScene::dropEvent(QGraphicsSceneDragDropEvent * event) {
 static int count = 0;
 void WorkflowScene::addProcess(Actor* proc, const QPointF& pos) {
     WorkflowProcessItem* it = new WorkflowProcessItem(proc);
-    addItem(it);
     it->setPos(pos);
-    it->createPorts();
+    bool first = items().isEmpty();
+    addItem(it);
     Actor* a = it->getProcess();
     a->setLabel(proc->getProto()->getDisplayName() + QString(" %1").arg(++count));
     modified = true;
     emit processItemAdded();
+    if (first) {
+        // erase drop hint
+        update();
+    }
 }
 
 void WorkflowScene::mousePressEvent(QGraphicsSceneMouseEvent *mouseEvent) {
-    if (!mouseEvent->isAccepted() && controller->selectedProto() && (mouseEvent->button() == Qt::LeftButton)) {
+    if (!locked && !mouseEvent->isAccepted() && controller->selectedProto() && (mouseEvent->button() == Qt::LeftButton)) {
         addProcess(controller->selectedProto()->createInstance(), mouseEvent->scenePos());
     }
     QGraphicsScene::mousePressEvent(mouseEvent);
@@ -938,29 +1080,34 @@ void WorkflowScene::sl_bringToFront()
     if (selectedItems().isEmpty())
         return;
 
-    QGraphicsItem *selectedItem = selectedItems().first();
-    qreal zValue = 0;
-    foreach (QGraphicsItem *item, selectedItem->collidingItems()) {
-        if (item->zValue() >= zValue /*&& item->type() == WorkflowProcessItemType*/)
-            zValue = item->zValue() + 1;
+    foreach(QGraphicsItem *selectedItem, selectedItems()) {
+        qreal zValue = selectedItem->zValue();
+        foreach (QGraphicsItem *item, selectedItem->collidingItems()) {
+            if (item->zValue() >= zValue /*&& item->type() == WorkflowProcessItemType*/)
+                zValue = item->zValue() + 1;
+        }
+        if (selectedItem->zValue() != zValue) {
+            selectedItem->setZValue(zValue);
+            modified = true;
+        }
     }
-    selectedItem->setZValue(zValue);
-    modified = true;
 }
 
 void WorkflowScene::sl_sendToBack()
 {
     if (selectedItems().isEmpty())
         return;
-
-    QGraphicsItem *selectedItem = selectedItems().first();
-    qreal zValue = 0;
-    foreach (QGraphicsItem *item, selectedItem->collidingItems()) {
-        if (item->zValue() <= zValue /*&& item->type() == WorkflowProcessItemType*/)
-            zValue = item->zValue() - 1;
+    foreach(QGraphicsItem *selectedItem, selectedItems()) {
+        qreal zValue = selectedItem->zValue();
+        foreach (QGraphicsItem *item, selectedItem->collidingItems()) {
+            if (item->zValue() <= zValue /*&& item->type() == WorkflowProcessItemType*/)
+                zValue = item->zValue() - 1;
+        }
+        if (selectedItem->zValue() != zValue) {
+            selectedItem->setZValue(zValue);
+            modified = true;
+        }
     }
-    selectedItem->setZValue(zValue);
-    modified = true;
 }
 
 void WorkflowScene::sl_selectAll() {
@@ -977,7 +1124,7 @@ void WorkflowScene::sl_reset() {
     foreach(QGraphicsItem* it, items()) {
         if (it->type() == WorkflowProcessItemType) {
             list << it;
-        } //else if (it->type() == WorkflowBusItemType)
+        }
     }
     modified = false;
     foreach(QGraphicsItem* it, list) {
@@ -991,4 +1138,73 @@ WorkflowScene::~WorkflowScene()
 {
     sl_reset();
 }
+
+void WorkflowScene::drawBackground(QPainter * painter, const QRectF & rect)
+{
+    if (WorkflowSettings::showGrid()) {
+        int step = GRID_STEP;
+        painter->setPen(QPen(QColor(200, 200, 255, 125)));
+        // draw horizontal grid
+        qreal start = round(rect.top(), step);
+        if (start > rect.top()) {
+            start -= step;
+        }
+        for (qreal y = start - step; y < rect.bottom(); ) {
+            y += step;
+            painter->drawLine(rect.left(), y, rect.right(), y);
+        }
+        // now draw vertical grid
+        start = round(rect.left(), step);
+        if (start > rect.left()) {
+            start -= step;
+        }
+        for (qreal x = start - step; x < rect.right(); ) {
+            x += step;
+            painter->drawLine(x, rect.top(), x, rect.bottom());
+        }
+    }
+
+    if (items().size() == 0) {
+        // draw a hint on empty scene
+        painter->setPen(Qt::darkGray);
+        QFont f = painter->font();
+        if (hint == SamplesTab) {
+        } else {
+            QTransform trans = painter->combinedTransform();
+            f.setFamily("Courier New");
+            f.setPointSizeF(f.pointSizeF()* 2./trans.m11());
+            painter->setFont(f);
+            QRectF res;
+            painter->drawText(sceneRect(), Qt::AlignCenter, tr("Drop here an object from palette"), &res);
+            QPixmap pix(":workflow_designer/images/leftarrow.png");
+            QPointF pos(res.left(), res.center().y());
+            pos.rx() -= pix.width() + GRID_STEP;
+            pos.ry() -= pix.height()/2;
+            painter->drawPixmap(pos, pix);
+        }
+    }
+}
+
+void WorkflowScene::setIterations( const QList<Iteration>& lst )
+{
+    assert(!locked);
+    iterations = lst; 
+    modified = true; 
+    emit configurationChanged();
+}
+
+void WorkflowScene::centerView() {
+    QRectF childRect;
+    foreach (QGraphicsItem *child, items()) {
+        QPointF childPos = child->pos();
+        QTransform matrix = child->transform() * QTransform().translate(childPos.x(), childPos.y());
+        childRect |= matrix.mapRect(child->boundingRect() | child->childrenBoundingRect());
+    }
+    QPointF zero = childRect.center();
+    //log.info(QString("center [%1 %2]").arg(zero.x()).arg(zero.y()));
+    //FIXME does not work
+    //views().first()->centerOn(zero);
+    update();
+}
+
 }//namespace
