@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -10,10 +10,13 @@
 
 #include "WorkflowViewItems.h"
 #include "ItemViewStyle.h"
+#include "WorkflowViewController.h"
 
 #include <workflow/WorkflowRegistry.h>
 #include <workflow/WorkflowModel.h>
 #include <workflow/IntegralBusModel.h>
+#include <workflow_support/WorkflowRunTask.h>
+#include <workflow_support/WorkflowSettings.h>
 
 #include <core_api/Log.h>
 #include <QBitmap>
@@ -27,6 +30,9 @@
 #include <QtGui/QTextDocument>
 #include <qmath.h>
 
+#include <workflow_support/QVariantUtils.h>
+#include <QtXml/qdom.h>
+
 namespace GB2 {
 
 static LogCategory log(ULOG_CAT_WD);
@@ -36,8 +42,18 @@ WorkflowProcessItem::WorkflowProcessItem(Actor* prc) : process(prc) {
     setToolTip(process->getProto()->getDocumentation());
     setFlag(QGraphicsItem::ItemIsMovable, true);
     setFlag(QGraphicsItem::ItemIsSelectable, true);
-    styles[ItemStyles::SIMPLE] = currentStyle = new SimpleProcStyle(this);
+    setAcceptHoverEvents(true);
+
+    styles[ItemStyles::SIMPLE] = new SimpleProcStyle(this);
     styles[ItemStyles::EXTENDED] = new ExtendedProcStyle(this);
+    StyleId sid = WorkflowSettings::defaultStyle();
+    if (!styles.contains(sid)) {
+        sid = ItemStyles::EXTENDED;
+    }
+    currentStyle = styles.value(sid);
+
+    currentStyle->setActive(true);
+    createPorts();
 }
 
 WorkflowProcessItem::~WorkflowProcessItem()
@@ -63,24 +79,25 @@ void WorkflowProcessItem::createPorts() {
     int num = process->getInputPorts().size() + 1;
     qreal pie = 180/num;
     int i = 1;
+    QGraphicsScene* sc = scene();
     foreach(Port* port, process->getInputPorts()) {
         WorkflowPortItem* pit = new WorkflowPortItem(this, port);
-        scene()->addItem(pit);
         ports << pit;
-        pit->setPos(pos());
-        pit->setZValue(zValue());
         pit->setOrientation(90 + pie*i++);
+        if (sc) {
+            sc->addItem(pit);
+        }
     }
     num = process->getOutputPorts().size() + 1;
     pie = 180/num;
     i = 1;
     foreach(Port* port, process->getOutputPorts()) {
         WorkflowPortItem* pit = new WorkflowPortItem(this, port);
-        scene()->addItem(pit);
         ports << pit;
-        pit->setPos(pos());
-        pit->setZValue(zValue());
         pit->setOrientation(270 + pie*i++);
+        if (sc) {
+            sc->addItem(pit);
+        }
     }
 }
 
@@ -95,7 +112,9 @@ void WorkflowProcessItem::sl_update() {
 
 void WorkflowProcessItem::setStyle(StyleId s) {
     prepareGeometryChange();
+    currentStyle->setActive(false);
     currentStyle = styles.value(s);
+    currentStyle->setActive(true);
     currentStyle->refresh();
     foreach(WorkflowPortItem* pit, ports) {
         pit->setStyle(s);
@@ -105,7 +124,11 @@ void WorkflowProcessItem::setStyle(StyleId s) {
 }
 
 QRectF WorkflowProcessItem::boundingRect(void) const {
-    return currentStyle->boundingRect();
+    QRectF brect = currentStyle->boundingRect();
+    if (/*getWorkflowScene()->getRunner()*/ true) {
+        brect.setTop(brect.top() - QFontMetrics(QFont()).height()*2 - 2);
+    }
+    return brect;
 }
 
 QPainterPath WorkflowProcessItem::shape () const {
@@ -117,6 +140,103 @@ void WorkflowProcessItem::paint(QPainter *painter,
                           QWidget *widget)
 {
     currentStyle->paint(painter,option,widget);
+    WorkflowRunTask* rt = getWorkflowScene()->getRunner();
+    if (rt) {
+        //{WorkerWaiting, WorkerReady, WorkerRunning, WorkerDone};
+        static QColor rsColors[4] = {QColor(234,143,7),"#04AA04", "#AA0404", "#0404AA"};
+            //{QColor(234,143,7),QColor(Qt::red),QColor(Qt::green),QColor(0,0,255)};
+        static QString rsNames[4] = {("Waiting"),("Ready"),("Running"),("Done")};
+
+        const QList<WorkerState> rsList = rt->getState(this->process);
+        WorkerState state = WorkerDone;
+        if (rsList.contains(WorkerRunning)) {
+            state = WorkerRunning;
+        } else if (rsList.contains(WorkerReady)) {
+            state = WorkerReady;
+        } else if (rsList.contains(WorkerWaiting)) {
+            state = WorkerWaiting;
+        }
+        QString stateName = rsNames[state];
+        QColor scolor = rsColors[state];
+        painter->setPen(scolor);
+        QRectF brect = boundingRect();
+        qreal fh = QFontMetrics(QFont()).height();
+
+        if (rsList.size() == 1) {
+            brect.setTop(brect.top()+2+fh);
+        }
+
+        painter->drawRoundedRect(brect, 5, 5);
+        painter->drawText(brect, Qt::AlignHCenter, stateName);
+        if (rsList.size() == 1) {
+            
+        } else {
+            uint vals[4] = {0,0,0,0};
+            for (int i = rsList.size();i>0;)
+            {
+                vals[rsList.at(--i)]++;
+            }
+
+            // draw progress bar
+            if (state != WorkerDone) {
+                
+                QRectF textRect(brect.topLeft()+QPointF(0,fh-1),QSizeF(brect.width(), 3));
+                textRect.setLeft(brect.left()+1);
+                textRect.setRight(brect.right()-1);
+                QColor fc = /*rsColors[WorkerDone];//*/QColor(0,80,222); 
+                fc.setAlpha(90);
+                painter->setPen(fc);
+                painter->drawRect(textRect);
+                qreal done = (qreal)vals[WorkerDone]/rsList.size();
+                QBrush brush(fc);            
+//                 QLinearGradient lg(textRect.topLeft(),textRect.topRight());
+//                 lg.setColorAt(0, fc);
+//                 lg.setColorAt(done, fc.lighter(128));
+//                 lg.setColorAt(done + 1./rsList.size(), QColor(Qt::white));
+                //lg.setColorAt(1, QColor(Qt::white));
+                //brush = QBrush(lg);
+                textRect.setRight(textRect.left() + textRect.width()*done);
+                painter->fillRect(textRect, brush);
+            }            
+            
+            // draw extended text
+            painter->save();
+            QTextDocument d;
+            d.setHtml("<center><font size='-1'>" 
+                + QString("<font color='%1'>%2/</font>"
+                " <!--font color='%3'>%4/</font-->"
+                " <font color='%5'>%6/</font>"
+                " <font color='%7'>%8/</font>"
+                " <font color='black'>%9</font>")
+                .arg(rsColors[WorkerWaiting].name()).arg(vals[WorkerWaiting])
+                .arg(rsColors[WorkerReady].name()).arg(vals[WorkerReady])
+                .arg(rsColors[WorkerRunning].name()).arg(vals[WorkerRunning])
+                .arg(rsColors[WorkerDone].name()).arg(vals[WorkerDone])
+                .arg(rsList.size())
+                + "</font></center>");
+            //d.setTextWidth(brect.width());
+            painter->translate(brect.center().x() - d.idealWidth()/2, brect.top() + fh);
+            d.drawContents(painter/*, brect*/);
+            painter->restore();
+
+
+//             qreal unit = brect.width()/rsList.size();
+//             qreal step = 3;
+//             qreal w = 10;
+//             QPointF base(brect.topLeft());
+//             base.ry() += 5;
+
+//             for (int i = 3; i>=0; i--) {
+//  //             base.ry() -= step;
+//  //             painter->setPen(rsColors[i]); painter->drawLine(base, base + QPointF(unit*vals[i],0));
+//                 QRectF rt(base, base + QPointF(unit*vals[i],w)); base = rt.topRight();
+//                 painter->setPen(rsColors[i]); 
+//                 QColor fc = rsColors[i]; fc.setAlpha(128);
+//                 painter->fillRect(rt, QBrush(fc));
+//                 painter->drawText(rt, Qt::AlignJustify|Qt::AlignVCenter, rsNames[i]);
+//             }
+        }
+    }
 }
 
 QVariant WorkflowProcessItem::itemChange ( GraphicsItemChange change, const QVariant & value ) 
@@ -130,14 +250,19 @@ QVariant WorkflowProcessItem::itemChange ( GraphicsItemChange change, const QVar
         case ItemZValueHasChanged:
 		{
             qreal z = qVariantValue<qreal>(value);
-            //currentStyle->setZValue(z);
-            //currentStyle->update();
 			foreach(WorkflowPortItem* pit, ports) {
 				pit->setZValue(z);
 			}
 		}
 		break;
     case ItemPositionChange:
+        if (WorkflowSettings::snap2Grid()) {
+            // value is the new position.
+            QPointF newPos = value.toPointF();
+            newPos.setX(round(newPos.x(), GRID_STEP));
+            newPos.setY(round(newPos.y(), GRID_STEP));
+            return newPos;
+        }
         /*foreach(WorkflowPortItem* pit, ports) {
             foreach(WBusItem*bit, pit->getDataFlows()) {
                 bit->prepareGeometryChange();
@@ -152,6 +277,13 @@ QVariant WorkflowProcessItem::itemChange ( GraphicsItemChange change, const QVar
             }
         }
 		break;
+    case ItemSceneHasChanged:
+        if (scene()) {
+            foreach(WorkflowPortItem* pit, ports) {
+                scene()->addItem(pit);
+            }
+        }
+        break;
 	case ItemSceneChange: 
 		if (qVariantValue<QGraphicsScene*>(value) == NULL) {
 			foreach(WorkflowPortItem* pit, ports) {
@@ -165,6 +297,48 @@ QVariant WorkflowProcessItem::itemChange ( GraphicsItemChange change, const QVar
     return QGraphicsItem::itemChange(change, value);
 }
 
+bool WorkflowProcessItem::sceneEvent( QEvent * event )
+{
+    if (currentStyle->sceneEventFilter(this, event)) {
+        return true;
+    }
+    return QGraphicsItem::sceneEvent(event);
+}
+
+QList<QAction*> WorkflowProcessItem::getContextMenuActions() const
+{
+    return currentStyle->getContextMenuActions();
+}
+
+void WorkflowProcessItem::saveState(QDomElement& el) const {
+    el.setAttribute("pos", QVariantUtils::var2String(pos()));
+    el.setAttribute("style", styles.key(currentStyle));
+    foreach(ItemViewStyle* style, styles) {
+        QDomElement stel = el.ownerDocument().createElement(style->getId());
+        style->saveState(stel);
+        if (stel.hasAttributes() || stel.hasChildNodes()) {
+            el.appendChild(stel);
+        }
+    }
+}
+
+void WorkflowProcessItem::loadState(QDomElement& el) {
+    const QString posS = el.attribute("pos");
+    const QPointF pos = QVariantUtils::String2Var(posS).toPointF();
+    assert(!pos.isNull());
+    setPos(pos);
+
+    foreach(ItemViewStyle* style, styles) {
+        QDomElement stel = el.elementsByTagName(style->getId()).item(0).toElement();
+        if (stel.isNull()) continue;
+        style->loadState(stel);
+    }
+    QString key = el.attribute("style");
+    if (styles.contains(key)) {
+        setStyle(key);
+    }
+}
+
 ///////////// PIO /////////////
 
 WorkflowPortItem* WorkflowPortItem::findNearbyBindingCandidate(const QPointF& pos) const {
@@ -195,7 +369,7 @@ static const int bl = (int) A/4;
 
 
 WorkflowPortItem::WorkflowPortItem(WorkflowProcessItem* owner, Port* p) 
-: /*StyledItem(owner), */ currentStyle(ItemStyles::SIMPLE),port(p),owner(owner),orientation(0), dragging(false), rotating(false),
+: /*StyledItem(owner), */ currentStyle(owner->getStyle()),port(p),owner(owner),orientation(0), dragging(false), rotating(false),
 sticky(false), highlight(false)
 {
     setFlag(QGraphicsItem::ItemIsSelectable, true);
@@ -205,6 +379,9 @@ sticky(false), highlight(false)
     tt += ").\nDrag it to connect to other process/port."
         "\nHold Alt key while dragging to change port orientation";
     setToolTip(tt);
+
+    setPos(owner->pos());
+    setZValue(owner->zValue());
 }
 
 WorkflowPortItem::~WorkflowPortItem()
@@ -225,19 +402,26 @@ void WorkflowPortItem::adaptOwnerShape() {
 
 void WorkflowPortItem::setOrientation(qreal angle) {
     orientation = angle;
-    /*if (ItemStyles::SIMPLE == currentStyle) {
+    bool snap2grid = WorkflowSettings::snap2Grid();
+    if (ItemStyles::SIMPLE == currentStyle) {
+        // common algorithm works fine as well,
+        // but gives visible distortions when snapping to grid
+        if (snap2grid) {
+            angle = round(angle, ANGLE_STEP);
+        }
+        angle = -angle;
         qreal x = R*qCos(angle*2*PI/360);
         qreal y = R*qSin(angle*2*PI/360);
         
         resetTransform();
         translate(x, y);
         rotate(angle);    
-    } else*/ 
-    {
+    } else {
+        
         resetTransform();
         QRectF rec = owner->boundingRect();
         QPolygonF pol(owner->shape().toFillPolygon());
-        qreal radius = qMax(rec.width(), rec.height());
+        qreal radius = qMax(rec.width(), rec.height()) * 2;
         //log.debug(QString("radius=%1 pol=%2").arg(radius).arg(pol.count()));
         QLineF centerLine(0,0,radius, 0);
         assert(pol.containsPoint(centerLine.p1(), Qt::WindingFill));
@@ -255,6 +439,21 @@ void WorkflowPortItem::setOrientation(qreal angle) {
             }
             p1 = p2;
         }
+        if (snap2grid && polyLine.p1().x() == polyLine.p2().x()) {
+            //vertical line, snap intersectPoint to grid
+            qreal v = round(intersectPoint.y(), GRID_STEP);
+            qreal min = qMin(polyLine.p1().y(), polyLine.p2().y());
+            qreal max = qMax(polyLine.p1().y(), polyLine.p2().y());
+            intersectPoint.setY(qBound(min, v, max));
+        } 
+        if (snap2grid && polyLine.p1().y() == polyLine.p2().y()) {
+            //horizontal line, do the same
+            qreal v = round(intersectPoint.x(), GRID_STEP);
+            qreal min = qMin(polyLine.p1().x(), polyLine.p2().x());
+            qreal max = qMax(polyLine.p1().x(), polyLine.p2().x());
+            intersectPoint.setX(qBound(min, v, max));
+        }
+
         translate(intersectPoint.x(), intersectPoint.y());
         //qreal polyAngle = polyLine.angle();
         qreal norm = polyLine.normalVector().angle();
@@ -262,7 +461,9 @@ void WorkflowPortItem::setOrientation(qreal angle) {
         if (df > 90 && df < 270) {
             norm += 180;
         }
-        //log.debug(QString("a=%1 pa=%2 pn=%3 n=%4 df=%5").arg(angle).arg(polyAngle).arg(polyLine.normalVector().angle()).arg(norm).arg(df));
+        //log.info(QString("centerLine=[%1,%2]->[%3,%4]").arg(centerLine.p1().x()).arg(centerLine.p1().y()).arg(centerLine.p2().x()).arg(centerLine.p2().y()));
+        //log.info(QString("polyLine=[%1,%2]->[%3,%4]").arg(polyLine.p1().x()).arg(polyLine.p1().y()).arg(polyLine.p2().x()).arg(polyLine.p2().y()));
+        //log.info(QString("a=%1 pa=%2 pn=%3 n=%4 df=%5").arg(angle).arg(polyAngle).arg(polyLine.normalVector().angle()).arg(norm).arg(df));
         rotate(-norm);
     }
 }
@@ -434,6 +635,27 @@ void WorkflowPortItem::paint(QPainter *painter,
 		    painter->drawEllipse(p1, A/2, A/2);
         }
     }
+    if (0 && port->getWidth() == 0) {
+        // draw a hint
+        painter->save();
+        QPen pen;
+        //pen.setWidthF(2);
+        pen.setStyle(Qt::DashLine);
+        painter->setPen(pen);
+        QPointF hc(R, 0);
+        drawArrow(painter, pen, hc, p1);
+        
+        painter->translate(R, 0);
+        painter->rotate(orientation);
+        QRectF approx(-10, -10, 200, 100);
+        QRectF htb = painter->boundingRect(approx, Qt::AlignCenter, port->getDisplayName());
+        //painter->drawRoundedRect(htb, 30, 30, Qt::RelativeSize);
+        
+        painter->drawRoundedRect(htb, 30, 30, Qt::RelativeSize);
+        painter->drawText(approx, Qt::AlignCenter, port->getDisplayName());
+        painter->setPen(Qt::red);painter->drawPoint(0,0);
+        painter->restore();
+    }
     if (dragging) {
 		QPen pen;
 		//pen.setWidthF(3);
@@ -503,7 +725,7 @@ void WorkflowPortItem::mouseMoveEvent( QGraphicsSceneMouseEvent * event ) {
 
 void WorkflowPortItem::mousePressEvent ( QGraphicsSceneMouseEvent * event ) {
     dragPoint = QPointF();
-    if ((event->buttons() & Qt::LeftButton)) {
+    if ((event->buttons() & Qt::LeftButton) && !getWorkflowScene()->isLocked()) {
         dragPoint = event->pos();
         if (event->modifiers() & portRotationModifier) {
             rotating = true;
@@ -554,13 +776,16 @@ void WorkflowPortItem::mouseReleaseEvent ( QGraphicsSceneMouseEvent * event ) {
 }
 
 void WorkflowPortItem::hoverEnterEvent ( QGraphicsSceneHoverEvent * event ) {
+    if (getWorkflowScene()->isLocked()) {
+        return;
+    }
     setCursor((event->modifiers() & portRotationModifier) ? 
 //portRotationCursor 
 QCursor(QPixmap(":workflow_designer/images/rot_cur.png"))
 : QCursor(Qt::OpenHandCursor));
 }
-void WorkflowPortItem::hoverLeaveEvent ( QGraphicsSceneHoverEvent * event ) {
-    Q_UNUSED(event);
+
+void WorkflowPortItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *) {
     unsetCursor();
 }
 
@@ -604,15 +829,7 @@ WBusItem::WBusItem(WorkflowPortItem* p1, WorkflowPortItem* p2)
     setFlag(QGraphicsItem::ItemIsSelectable, true);
     setZValue(-1000);
 
-    QGraphicsTextItem* text = new QGraphicsTextItem(src->getPort()->getDisplayName(), this);
-    text->setTextWidth(qMin(3*R, text->document()->idealWidth()));
-    QRectF tb = text->boundingRect();
-    text->setPos(-tb.width()/2, -tb.height()- 3);
-    text->setDefaultTextColor(QColor(Qt::gray).darker());
-    QFont f = text->font();
-    f.setWeight(QFont::Light);
-    text->setFont(f);
-    this->text = text;
+    this->text = new HintItem(src->getPort()->getDisplayName(), this);
     connect(dst->getPort(), SIGNAL(bindingChanged()), this, SLOT(sl_update()));
 }
 
@@ -654,9 +871,12 @@ void WBusItem::hoverLeaveEvent ( QGraphicsSceneHoverEvent * event ) {
 }
 
 QRectF WBusItem::boundingRect(void) const {
-    return (mapFromItem(dst, dst->boundingRect()).boundingRect() 
-        | mapFromItem(src, src->boundingRect()).boundingRect() 
-        | text->boundingRect().translated(text->pos()));
+    QRectF brect(mapFromItem(dst, dst->boundingRect()).boundingRect() | mapFromItem(src, src->boundingRect()).boundingRect());
+    QRectF trect(text->boundingRect().translated(text->pos()));
+    if (/*getWorkflowScene()->getRunner()*/ true) {
+        trect.setTop(trect.top() - trect.height());
+    }
+    return brect | trect;
 }
 
 QPainterPath WBusItem::shape () const {
@@ -706,6 +926,38 @@ void WBusItem::paint ( QPainter * painter, const QStyleOptionGraphicsItem * opti
     QRectF textRec = text->boundingRect().translated(text->pos());
     painter->fillRect(textRec, QBrush(yc));
     painter->drawRect(textRec);
+
+    WorkflowRunTask* rt = getWorkflowScene()->getRunner();
+    if (rt) {
+        int num = rt->getMsgNum(this->bus);
+        QString rts = QString("%1 in queue").arg(num);
+        QRectF rtb = textRec.translated(0, -QFontMetricsF(QFont()).height());
+        painter->drawText(rtb, Qt::AlignHCenter, rts);
+        if (num == 0) {
+            return;
+        }
+        qreal dx = (p2.x() - p1.x())/num;
+        qreal dy = (p2.y() - p1.y())/num;
+        QPointF dp(dx,dy);
+        QColor c1("#AA0404");
+        painter->setPen(c1);
+        c1.setAlphaF(0.8);
+        QColor c2(Qt::white);
+        c2.setAlphaF(0.8);
+        while (num-- > 0) {
+            QPainterPath p;
+            p.addEllipse(p1, 3,3);
+            QRadialGradient rg(p1+QPointF(1.,-1.), 3);
+
+            rg.setColorAt(1, c1);
+            rg.setColorAt(0, c2);
+            QBrush br(rg);
+
+            painter->fillPath(p,br);
+            //painter->drawEllipse(p1, 3,3);
+            p1 += dp;
+        }
+    }
 }
 
 void WBusItem::sl_update() {
@@ -717,4 +969,20 @@ bool WBusItem::validate() {
     return dst->getPort()->validate(lst);
 }
 
+void WBusItem::saveState(QDomElement& el) const {
+    el.setAttribute("hint-pos", QVariantUtils::var2String(text->pos()));
+}
+void WBusItem::loadState(QDomElement& el) {
+    if (el.hasAttribute("hint-pos")) {
+        QPointF pos = QVariantUtils::String2Var(el.attribute("hint-pos")).toPointF();
+        if (!pos.isNull()) {
+            text->setPos(pos);
+        }
+    }
+}
+
+WorkflowScene* StyledItem::getWorkflowScene() const
+{
+    return qobject_cast<WorkflowScene*>(scene());
+}
 }//namespace
