@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -486,7 +486,7 @@ int SiteconAlgorithm::calculateWeights(const MAlignment& ma, QVector<PositionSta
 
 bool SiteconModel::checkState(bool doAssert) const {
     //1 check  settings
-    assert(!doAssert || settings.windowSize > 0);
+    assert(!doAssert || settings.windowSize > 0); Q_UNUSED(doAssert);
     if (settings.windowSize <=0) {
         return false;
     }
@@ -532,7 +532,7 @@ bool SiteconModel::checkState(bool doAssert) const {
         } else {
             //assert(!doAssert || nWeights <= MAX_WEIGHTS_ALG2);
             if (nWeights > MAX_WEIGHTS_ALG2) {
-                log.trace(QString("Number of Algorithm 2 weights %1, pos %2, model file %3").arg(nWeights).arg(pos).arg(modelFileName));
+                log.trace(QString("Number of Algorithm 2 weights %1, pos %2, model name %3").arg(nWeights).arg(pos).arg(modelName));
                 //return false;
             }
         }
