@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -44,14 +44,14 @@ void GTest_CalculateACGTContent::init(XMLTestFormat *tf, const QDomElement& el)
 
     docName = el.attribute(DOC_ATTR);
     if (docName.isEmpty()) {
-        stateInfo.error = GTest::tr("value not set %1").arg(DOC_ATTR);
+        failMissingValue(DOC_ATTR);
         return;
     } 
 
     QString expected = el.attribute(EXPECTED_RESULTS_ATTR);
     QStringList expectedList = expected.split(QRegExp("\\,")); //may be QRegExp("\\,")
     if (expectedList.size() != 4) {
-        stateInfo.error = GTest::tr("here must be 4 items in %1").arg(EXPECTED_RESULTS_ATTR);
+        stateInfo.setError(  QString("here must be 4 items in %1").arg(EXPECTED_RESULTS_ATTR) );
         return;
     } 
     int i = 0, sum = 0;
@@ -59,14 +59,14 @@ void GTest_CalculateACGTContent::init(XMLTestFormat *tf, const QDomElement& el)
         bool isOk;
         int m = str.toInt(&isOk);
         if(!isOk) {
-            stateInfo.error = GTest::tr("Wrong conversion to the integer for one of the %1").arg(EXPECTED_RESULTS_ATTR);
+            stateInfo.setError(  QString("Wrong conversion to the integer for one of the %1").arg(EXPECTED_RESULTS_ATTR) );
             return;
         }
         expectedACGT[i++] = m;
         sum += m;
     }
     if(sum < 100 || sum > 102) {
-        stateInfo.error = GTest::tr("Wrong %1 values").arg(EXPECTED_RESULTS_ATTR);
+        stateInfo.setError(  QString("Wrong %1 values").arg(EXPECTED_RESULTS_ATTR) );
         return;
     }
 }
@@ -74,22 +74,22 @@ void GTest_CalculateACGTContent::init(XMLTestFormat *tf, const QDomElement& el)
 void GTest_CalculateACGTContent::prepare(){
     Document* doc = getContext<Document>(this, docName);
     if (doc == NULL) {
-        stateInfo.error = GTest::tr("context not found %1").arg(docName);
+        stateInfo.setError(  QString("context not found %1").arg(docName) );
         return;
     }
     QList<GObject*> list = doc->findGObjectByType(GObjectTypes::MULTIPLE_ALIGNMENT);
     if (list.size() == 0) {
-        stateInfo.error = GTest::tr("container of object with type \"%1\" is empty").arg(GObjectTypes::MULTIPLE_ALIGNMENT);
+        stateInfo.setError(  QString("container of object with type \"%1\" is empty").arg(GObjectTypes::MULTIPLE_ALIGNMENT) );
         return;
     }
     GObject *obj = list.first();
     if(obj==NULL){
-        stateInfo.error = GTest::tr("object with type \"%1\" not found").arg(GObjectTypes::MULTIPLE_ALIGNMENT);
+        stateInfo.setError(  QString("object with type \"%1\" not found").arg(GObjectTypes::MULTIPLE_ALIGNMENT) );
         return;
     }
     MAlignmentObject *mao = qobject_cast<MAlignmentObject*>(obj);
     if(mao==NULL){
-        stateInfo.error = GTest::tr("error can't cast to MAlignmentObject from GObject");
+        stateInfo.setError(  QString("error can't cast to MAlignmentObject from GObject") );
         return;
     }
     ma = mao->getMAlignment();
@@ -102,7 +102,7 @@ void GTest_CalculateACGTContent::run() {
 Task::ReportResult GTest_CalculateACGTContent::report(){
     for(int i = 0; i < 4; i++) {
         if (expectedACGT[i] != s.acgtContent[i]) {
-            stateInfo.error = GTest::tr("Actual results not equal with expected");
+            stateInfo.setError(  QString("Actual results not equal with expected") );
             return ReportResult_Finished;
         }
     }
@@ -121,13 +121,13 @@ void GTest_CalculateDispersionAndAverage::init(XMLTestFormat *tf, const QDomElem
         bool isOk;
         int pos = posStr.toInt(&isOk);
         if(!isOk) {
-            stateInfo.error = GTest::tr("Wrong conversion to the integer for one of the %1").arg(DINUCLEOTIDE_POSITIONS);
+            stateInfo.setError(  QString("Wrong conversion to the integer for one of the %1").arg(DINUCLEOTIDE_POSITIONS) );
             return;
         }
         foreach(QString propStr, propsList) {
             int propIndex = propStr.toInt(&isOk);
             if(!isOk) {
-                stateInfo.error = GTest::tr("Wrong conversion to the integer for one of the %1").arg(PROPERTIES_INDEXES);
+                stateInfo.setError(  QString("Wrong conversion to the integer for one of the %1").arg(PROPERTIES_INDEXES) );
                 return;
             }
             ResultVector r;
@@ -135,24 +135,24 @@ void GTest_CalculateDispersionAndAverage::init(XMLTestFormat *tf, const QDomElem
             r.push_back(propIndex);
             int exp = qRound((*expResIt).toFloat(&isOk) * 10000);
             if(!isOk) {
-                stateInfo.error = GTest::tr("Wrong conversion to the integer for one of the %1").arg(EXPECTED_RESULTS_ATTR);
+                stateInfo.setError(  QString("Wrong conversion to the integer for one of the %1").arg(EXPECTED_RESULTS_ATTR) );
                 return;
             }
             r.push_back(exp);
             if(expResIt == expectedStrList.end()){
-                stateInfo.error = GTest::tr("Too less items in %1").arg(EXPECTED_RESULTS_ATTR);
+                stateInfo.setError(  QString("Too less items in %1").arg(EXPECTED_RESULTS_ATTR) );
                 return;
             }
             expResIt++;
             exp = qRound((*expResIt).toFloat(&isOk) * 10000);
             if(!isOk) {
-                stateInfo.error = GTest::tr("Wrong conversion to the integer for one of the %1").arg(EXPECTED_RESULTS_ATTR);
+                stateInfo.setError(  QString("Wrong conversion to the integer for one of the %1").arg(EXPECTED_RESULTS_ATTR) );
                 return;
             }
             r.push_back(exp);
             expectedResults.push_back(ResultVector(r));
             if(expResIt == expectedStrList.end()){
-                stateInfo.error = GTest::tr("Too less items in %1").arg(EXPECTED_RESULTS_ATTR);
+                stateInfo.setError(  QString("Too less items in %1").arg(EXPECTED_RESULTS_ATTR) );
                 return;
             }
             expResIt++;
@@ -161,7 +161,7 @@ void GTest_CalculateDispersionAndAverage::init(XMLTestFormat *tf, const QDomElem
     
     docName = el.attribute(DOC_ATTR);
     if (docName.isEmpty()) {
-        stateInfo.error = GTest::tr("value not set %1").arg(DOC_ATTR);
+        failMissingValue(DOC_ATTR);
         return;
     }
 }
@@ -169,22 +169,22 @@ void GTest_CalculateDispersionAndAverage::init(XMLTestFormat *tf, const QDomElem
 void GTest_CalculateDispersionAndAverage::prepare() {
     Document* doc = getContext<Document>(this, docName);
     if (doc == NULL) {
-        stateInfo.error = GTest::tr("context not found %1").arg(docName);
+        stateInfo.setError(  QString("context not found %1").arg(docName) );
         return;
     }
     QList<GObject*> list = doc->findGObjectByType(GObjectTypes::MULTIPLE_ALIGNMENT);
     if (list.size() == 0) {
-        stateInfo.error = GTest::tr("container of object with type \"%1\" is empty").arg(GObjectTypes::MULTIPLE_ALIGNMENT);
+        stateInfo.setError(  QString("container of object with type \"%1\" is empty").arg(GObjectTypes::MULTIPLE_ALIGNMENT) );
         return;
     }
     GObject *obj = list.first();
     if(obj==NULL){
-        stateInfo.error = GTest::tr("object with type \"%1\" not found").arg(GObjectTypes::MULTIPLE_ALIGNMENT);
+        stateInfo.setError(  QString("object with type \"%1\" not found").arg(GObjectTypes::MULTIPLE_ALIGNMENT) );
         return;
     }
     MAlignmentObject *mao = qobject_cast<MAlignmentObject*>(obj);
     if(mao==NULL){
-        stateInfo.error = GTest::tr("error can't cast to MAlignmentObject from GObject");
+        stateInfo.setError(  QString("error can't cast to MAlignmentObject from GObject") );
         return;
     }
     ma = mao->getMAlignment();
@@ -210,11 +210,11 @@ Task::ReportResult GTest_CalculateDispersionAndAverage::report() {
             expAve = rv[2],
             expSdev = rv[3];
         if (sdev != expSdev) {
-            stateInfo.error = GTest::tr("Expected and Actual 'SDev' values are different: %1 %2").arg(expSdev/10000).arg(sdev/10000);
+            stateInfo.setError(  QString("Expected and Actual 'SDev' values are different: %1 %2").arg(expSdev/10000).arg(sdev/10000) );
             return ReportResult_Finished;
         }
         if (average != expAve) {
-            stateInfo.error = GTest::tr("Expected and Actual 'Average' values are different: %1 %2").arg(expAve/10000).arg(average/10000);
+            stateInfo.setError(  QString("Expected and Actual 'Average' values are different: %1 %2").arg(expAve/10000).arg(average/10000) );
             return ReportResult_Finished;
         }
 
@@ -227,19 +227,19 @@ void GTest_CalculateFirstTypeError::init(XMLTestFormat *tf, const QDomElement& e
 
     docName = el.attribute(DOC_ATTR);
     if (docName.isEmpty()) {
-        stateInfo.error = GTest::tr("value not set %1").arg(DOC_ATTR);
+        failMissingValue(DOC_ATTR);
         return;
     }
 
     QString windowSizeStr = el.attribute(OFFSET_ATTR);
     if (docName.isEmpty()) {
-        stateInfo.error = GTest::tr("value not set %1").arg(OFFSET_ATTR);
+        failMissingValue(OFFSET_ATTR);
         return;
     }
     bool isOk;
     offset = windowSizeStr.toInt(&isOk);
     if (!isOk) {
-        stateInfo.error = GTest::tr("Wrong conversion to the integer for one of the %1").arg(OFFSET_ATTR);
+        stateInfo.setError(  QString("Wrong conversion to the integer for one of the %1").arg(OFFSET_ATTR) );
         return;
     }
 
@@ -247,7 +247,7 @@ void GTest_CalculateFirstTypeError::init(XMLTestFormat *tf, const QDomElement& e
     foreach(QString str, expectedStrList){
         int exp = qRound(str.toFloat(&isOk) * 10000);
         if(!isOk) {
-            stateInfo.error = GTest::tr("Wrong conversion to the integer for one of the %1").arg(EXPECTED_RESULTS_ATTR);
+            stateInfo.setError(  QString("Wrong conversion to the integer for one of the %1").arg(EXPECTED_RESULTS_ATTR) );
             return;
         }
         expectedResult.push_back(exp);
@@ -257,22 +257,22 @@ void GTest_CalculateFirstTypeError::init(XMLTestFormat *tf, const QDomElement& e
 void GTest_CalculateFirstTypeError::prepare() {
     Document* doc = getContext<Document>(this, docName);
     if (doc == NULL) {
-        stateInfo.error = GTest::tr("context not found %1").arg(docName);
+        stateInfo.setError(  QString("context not found %1").arg(docName) );
         return;
     }
     QList<GObject*> list = doc->findGObjectByType(GObjectTypes::MULTIPLE_ALIGNMENT);
     if (list.size() == 0) {
-        stateInfo.error = GTest::tr("container of object with type \"%1\" is empty").arg(GObjectTypes::MULTIPLE_ALIGNMENT);
+        stateInfo.setError(  QString("container of object with type \"%1\" is empty").arg(GObjectTypes::MULTIPLE_ALIGNMENT) );
         return;
     }
     GObject *obj = list.first();
     if(obj==NULL){
-        stateInfo.error = GTest::tr("object with type \"%1\" not found").arg(GObjectTypes::MULTIPLE_ALIGNMENT);
+        stateInfo.setError(  QString("object with type \"%1\" not found").arg(GObjectTypes::MULTIPLE_ALIGNMENT) );
         return;
     }
     MAlignmentObject *mao = qobject_cast<MAlignmentObject*>(obj);
     if(mao==NULL){
-        stateInfo.error = GTest::tr("error can't cast to MAlignmentObject from GObject");
+        stateInfo.setError(  QString("error can't cast to MAlignmentObject from GObject") );
         return;
     }
     ma = mao->getMAlignment();
@@ -295,7 +295,7 @@ Task::ReportResult GTest_CalculateFirstTypeError::report() {
         //printf("Expected: %i", exp);
         //printf(" Actual: %i \r\n", act);
         if(act != exp){
-            stateInfo.error = GTest::tr("Expected and Actual values are different: %1 %2").arg(s).arg(result.size());
+            stateInfo.setError(  QString("Expected and Actual values are different: %1 %2").arg(s).arg(result.size()) );
             return ReportResult_Finished;
         }
         i++;
@@ -308,19 +308,19 @@ void GTest_CalculateSecondTypeError::init(XMLTestFormat *tf, const QDomElement&
 
     docName = el.attribute(DOC_ATTR);
     if (docName.isEmpty()) {
-        stateInfo.error = GTest::tr("value not set %1").arg(DOC_ATTR);
+        failMissingValue(DOC_ATTR);
         return;
     }
 
     QString windowSizeStr = el.attribute(OFFSET_ATTR);
     if (docName.isEmpty()) {
-        stateInfo.error = GTest::tr("value not set %1").arg(OFFSET_ATTR);
+        failMissingValue(OFFSET_ATTR);
         return;
     }
     bool isOk;
     offset = windowSizeStr.toInt(&isOk);
     if (!isOk) {
-        stateInfo.error = GTest::tr("Wrong conversion to the integer for one of the %1").arg(OFFSET_ATTR);
+        stateInfo.setError(  QString("Wrong conversion to the integer for one of the %1").arg(OFFSET_ATTR) );
         return;
     }
 
@@ -328,7 +328,7 @@ void GTest_CalculateSecondTypeError::init(XMLTestFormat *tf, const QDomElement&
     foreach(QString str, expectedStrList){
         int exp = str.toInt(&isOk);
         if(!isOk) {
-            stateInfo.error = GTest::tr("Wrong conversion to the integer for one of the %1").arg(EXPECTED_RESULTS_ATTR);
+            stateInfo.setError(  QString("Wrong conversion to the integer for one of the %1").arg(EXPECTED_RESULTS_ATTR) );
             return;
         }
         expectedResult.push_back(exp);
@@ -338,22 +338,22 @@ void GTest_CalculateSecondTypeError::init(XMLTestFormat *tf, const QDomElement&
 void GTest_CalculateSecondTypeError::prepare() {
     Document* doc = getContext<Document>(this, docName);
     if (doc == NULL) {
-        stateInfo.error = GTest::tr("context not found %1").arg(docName);
+        stateInfo.setError(  QString("context not found %1").arg(docName) );
         return;
     }
     QList<GObject*> list = doc->findGObjectByType(GObjectTypes::MULTIPLE_ALIGNMENT);
     if (list.size() == 0) {
-        stateInfo.error = GTest::tr("container of object with type \"%1\" is empty").arg(GObjectTypes::MULTIPLE_ALIGNMENT);
+        stateInfo.setError(  QString("container of object with type \"%1\" is empty").arg(GObjectTypes::MULTIPLE_ALIGNMENT) );
         return;
     }
     GObject *obj = list.first();
     if(obj==NULL){
-        stateInfo.error = GTest::tr("object with type \"%1\" not found").arg(GObjectTypes::MULTIPLE_ALIGNMENT);
+        stateInfo.setError(  QString("object with type \"%1\" not found").arg(GObjectTypes::MULTIPLE_ALIGNMENT) );
         return;
     }
     MAlignmentObject *mao = qobject_cast<MAlignmentObject*>(obj);
     if(mao==NULL){
-        stateInfo.error = GTest::tr("error can't cast to MAlignmentObject from GObject");
+        stateInfo.setError(  QString("error can't cast to MAlignmentObject from GObject") );
         return;
     }
     ma = mao->getMAlignment();
@@ -366,7 +366,8 @@ void GTest_CalculateSecondTypeError::run() {
     s.numSequencesInAlignment = ma.getNumSequences();
     s.windowSize = ma.getLength();
     SiteconModel m;
-    m.modelFileName = (getContext<Document>(this, docName))->getURL();
+    m.aliURL = (getContext<Document>(this, docName))->getURL();
+    m.modelName = QFileInfo(m.aliURL).baseName();
     m.settings = s;
     m.matrix = SiteconAlgorithm::calculateDispersionAndAverage(ma, s, stateInfo);
     SiteconAlgorithm::calculateWeights(ma, m.matrix, m.settings, false, stateInfo);
@@ -383,7 +384,7 @@ Task::ReportResult GTest_CalculateSecondTypeError::report() {
         printf("Expected: %i", exp);
         printf(" Actual: %i \r\n", act);
         if(act != exp){
-            stateInfo.error = GTest::tr("Expected and Actual values are different: %1 %2").arg(s).arg(result.size());
+            stateInfo.setError(  QString("Expected and Actual values are different: %1 %2").arg(s).arg(result.size()) );
             return ReportResult_Finished;
         }
         i++;
@@ -396,7 +397,7 @@ void GTest_SiteconSearchTask::init(XMLTestFormat *tf, const QDomElement& el) {
 
     seqName = el.attribute(SEQNAME_ATTR);
     if (seqName.isEmpty()) {
-        stateInfo.error = GTest::tr("value not set %1").arg(SEQNAME_ATTR);
+        failMissingValue(SEQNAME_ATTR);
         return;
     }
 
@@ -407,7 +408,7 @@ void GTest_SiteconSearchTask::init(XMLTestFormat *tf, const QDomElement& el) {
     
     QString strandStr = el.attribute(STRAND_ATTR);
     if (strandStr.isEmpty()) {
-        stateInfo.error = GTest::tr("value not set %1").arg(STRAND_ATTR);
+        failMissingValue(STRAND_ATTR);
         return;
     }
     if (strandStr == "direct") {
@@ -420,19 +421,19 @@ void GTest_SiteconSearchTask::init(XMLTestFormat *tf, const QDomElement& el) {
         complOnly = false;
         isNeedCompliment = true;
     }else {
-        stateInfo.error = GTest::tr("%1 has incorrect value").arg(STRAND_ATTR);
+        stateInfo.setError(  QString("%1 has incorrect value").arg(STRAND_ATTR) );
         return;
     }
 
     bool isOk;
     QString tre = el.attribute(TRESH_ATTR);
     if (tre.isEmpty()) {
-        stateInfo.error = GTest::tr("value not set %1").arg(TRESH_ATTR);
+        failMissingValue(TRESH_ATTR);
         return;
     }
     tresh = tre.toInt(&isOk);
     if(!isOk){
-        stateInfo.error = GTest::tr("unable to convert %1 to integer").arg(TRESH_ATTR);
+        stateInfo.setError(  QString("unable to convert %1 to integer").arg(TRESH_ATTR) );
         return;
     }
 
@@ -444,7 +445,7 @@ void GTest_SiteconSearchTask::init(XMLTestFormat *tf, const QDomElement& el) {
             QString middleStr = props[0], scoreStr = props[1], strStr = props[2];
             int middle = middleStr.toInt(&isOk);
             if(!isOk){
-                stateInfo.error = GTest::tr("unable to convert %1 to integer").arg(EXPECTED_RESULTS_ATTR);
+                stateInfo.setError(  QString("unable to convert %1 to integer").arg(EXPECTED_RESULTS_ATTR) );
                 return;
             }
             bool compl_;
@@ -462,12 +463,12 @@ void GTest_SiteconSearchTask::init(XMLTestFormat *tf, const QDomElement& el) {
                 }
                 */
             } else {
-                stateInfo.error = GTest::tr("%1 has incorrect value").arg(STRAND_ATTR);
+                stateInfo.setError(  QString("%1 has incorrect value").arg(STRAND_ATTR) );
                 return;
             }
             float psum = scoreStr.toFloat(&isOk);      
             if(!isOk){
-                stateInfo.error = GTest::tr("unable to convert %1 to float").arg(EXPECTED_RESULTS_ATTR);
+                stateInfo.setError(  QString("unable to convert %1 to float").arg(EXPECTED_RESULTS_ATTR) );
                 return;
             }
             SiteconSearchResult ssr;
@@ -482,16 +483,17 @@ void GTest_SiteconSearchTask::init(XMLTestFormat *tf, const QDomElement& el) {
 void GTest_SiteconSearchTask::prepare() {
     DNASequenceObject * mySequence = getContext<DNASequenceObject>(this, seqName);
     if(mySequence==NULL){
-        stateInfo.error = GTest::tr("error can't cast to sequence from GObject");
+        stateInfo.setError(  QString("error can't cast to sequence from GObject") );
         return;
     }
     seqData = mySequence->getSequence();
+    SiteconSearchCfg cfg;
+    cfg.complOnly = complOnly;
+    cfg.minPSUM = tresh;
     if(isNeedCompliment){
-        DNATranslation* complTT = GObjectUtils::findComplementTT(mySequence);
-        task = new SiteconSearchTask(model, seqData.constData(), mySequence->getSequenceLen(), complTT, complOnly, tresh);    
-    }else{
-        task = new SiteconSearchTask(model, seqData.constData(), mySequence->getSequenceLen(), NULL, complOnly, tresh);    
+        cfg.complTT = GObjectUtils::findComplementTT(mySequence);
     }
+    task = new SiteconSearchTask(model, seqData.constData(), mySequence->getSequenceLen(), cfg, 0);    
     addSubTask(task);
 }
 
@@ -519,7 +521,7 @@ Task::ReportResult GTest_SiteconSearchTask::report() {
     }
     */
     if(results.size() != expectedResults.size()){
-        stateInfo.error = GTest::tr("expected and equal result lists not equal by size, expected: %1, actual: %2").arg(expectedResults.size()).arg(results.size());
+        stateInfo.setError(  QString("expected and equal result lists not equal by size, expected: %1, actual: %2").arg(expectedResults.size()).arg(results.size()) );
         return ReportResult_Finished;
     }
     /**/
@@ -532,7 +534,7 @@ Task::ReportResult GTest_SiteconSearchTask::report() {
         }
     }
     if(matchesCount != expectedResults.size()){
-        stateInfo.error = GTest::tr("expected and equal result lists not equal");
+        stateInfo.setError(  QString("expected and equal result lists not equal") );
         return ReportResult_Finished;
     }
     return ReportResult_Finished;
