@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -13,6 +13,7 @@
 #include "SiteconMath.h"
 #include "SiteconPlugin.h"
 
+#include <core_api/AppContext.h>
 #include <core_api/IOAdapter.h>
 #include <util_text/TextUtils.h>
 #include <util_gui/DialogUtils.h>
@@ -46,6 +47,8 @@ QString SiteconIO::getFileFilter(bool includeAll) {
 
 SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, TaskStateInfo& si) {
     SiteconModel model;
+    model.modelName = QFileInfo(url).baseName();
+
     QList<DiProperty*> props = SiteconPlugin::getDinucleotiteProperties();
     model.settings.props = props;
 
@@ -55,7 +58,7 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
 
     std::auto_ptr<IOAdapter> io(iof->createIOAdapter());
     if (!io->open(url, IOAdapterMode_Read)) {
-        si.error = Translations::errorOpeningFileRead(url);
+        si.setError(  Translations::errorOpeningFileRead(url) );
         return model;
     }
     QByteArray text;
@@ -69,7 +72,7 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
     while ((blockLen = io->readBlock(block.data(), BUFF_SIZE)) > 0) {
         text.append(QByteArray::fromRawData(block.data(), blockLen));
         if (text.size() > 1000*1000) {
-            si.error = IOAdapter::tr("file_is_too_large");
+            si.setError(Translations::errorFileTooLarge(url));
             break;
         }
     }
@@ -83,7 +86,7 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
     STATE state = FILE_S;
     QString line = reader.readLine();
     if (line!=FILE_HEADER) {
-        si.error = tr("not_sitecon_model_file");
+        si.setError(  tr("not_sitecon_model_file") );
         return model;
     }
     QSet<int> passedPropsAve;
@@ -122,18 +125,18 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
                 {
                     int i = line.indexOf(' ');
                     if (i == -1) {
-                        si.error = tr("error_parsing_settings_line_%1").arg(line);
+                        si.setError(  tr("error_parsing_settings_line_%1").arg(line) );
                         break;
                     }
                     QString name = line.left(i);
                     QString val = line.mid(i+1);
                     bool ok;
                     if (name == "ORIGIN") {
-                        model.modelFileName = val;
+                        model.aliURL = val;
                     } else if (name == "W") {
                         model.settings.windowSize = val.toInt(&ok);
                         if (!ok) {
-                            si.error = tr("error_parsing_settings_window_size_%1").arg(line);
+                            si.setError(  tr("error_parsing_settings_window_size_%1").arg(line) );
                             break;
                         } 
                         model.matrix.resize(model.settings.windowSize-1);
@@ -146,25 +149,25 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
                     } else if (name == "CLEN") {
                         model.settings.secondTypeErrorCalibrationLen = val.toInt(&ok);
                         if (!ok) {
-                            si.error = tr("error_parsing_calibration_len_%1").arg(line);
+                            si.setError(  tr("error_parsing_calibration_len_%1").arg(line) );
                             break;
                         } 
                     } else if (name == "RSEED") {
                         model.settings.randomSeed = val.toInt(&ok);
                         if (!ok) {
-                            si.error = tr("error_parsing_rseed_%1").arg(line);
+                            si.setError(  tr("error_parsing_rseed_%1").arg(line) );
                             break;
                         } 
                     } else if (name == "NSEQ") {
                         model.settings.numSequencesInAlignment = val.toInt(&ok);
                         if (!ok) {
-                            si.error = tr("error_parsing_nsequence_in_ali_%1").arg(line);
+                            si.setError(  tr("error_parsing_nsequence_in_ali_%1").arg(line) );
                             break;
                         } 
                     } else if (name == "WALG") {
                         int alg = val.toInt(&ok);
                         if (!ok) {
-                            si.error = tr("error_parsing_nsequence_in_ali_%1").arg(line);
+                            si.setError(  tr("error_parsing_nsequence_in_ali_%1").arg(line) );
                             break;
                         } 
                         if (alg == 0) {
@@ -172,7 +175,7 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
                         } else if (alg == 2) {
                             model.settings.weightAlg = SiteconWeightAlg_Alg2;
                         } else {
-                            si.error = tr("illegal_weight_alg_%1").arg(line);
+                            si.setError(  tr("illegal_weight_alg_%1").arg(line) );
                             break;
                         }
                     }
@@ -184,7 +187,7 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
                 {
                     QStringList l = line.split(MATRIX_VAL_SEPARATOR);
                     if (l.size()-2 !=  model.settings.windowSize - 1) { //num values == size-2, modelLen = w-1
-                        si.error = tr("model_size_not_matched_%1_expected_%2").arg(l.size()-2).arg(model.settings.windowSize-1);
+                        si.setError(  tr("model_size_not_matched_%1_expected_%2").arg(l.size()-2).arg(model.settings.windowSize-1) );
                         break;
                     }
                     QString propNum = l.first();
@@ -207,7 +210,7 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
                                 passedPropsWeight.insert(idx);
                             }
                             if (duplicate) {
-                                si.error = tr("duplicate_prop_%1").arg(propNum);
+                                si.setError(  tr("duplicate_prop_%1").arg(propNum) );
                                 break;
                             }
                             break;
@@ -217,7 +220,7 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
                         break;
                     }
                     if (idx == -1) {
-                        si.error = tr("property_not_recognized_%1").arg(line);
+                        si.setError(  tr("property_not_recognized_%1").arg(line) );
                         break;
                     }
                     //setup position specific value for property
@@ -233,7 +236,7 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
                             fval = (float)valStr.trimmed().toDouble(&ok);
                         }
                         if (!ok) {
-                            si.error = tr("error_parsing_matrix_val_%1_in_line_%2").arg(valStr).arg(line);
+                            si.setError(  tr("error_parsing_matrix_val_%1_in_line_%2").arg(valStr).arg(line) );
                             break;
                         }
                         if (state == AVE_S) {
@@ -251,7 +254,7 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
                 {
                     QStringList l = line.split(' ');
                     if (l.size() != 2) {
-                        si.error = tr("error_parsing_errors_line_%1").arg(line);
+                        si.setError(  tr("error_parsing_errors_line_%1").arg(line) );
                         break;
                     }
                     bool ok = true;
@@ -260,16 +263,16 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
                     percentStr.left(percentStr.length()-1);
                     int p = percentStr.toInt(&ok);
                     if (!ok) {
-                        si.error = tr("error_parsing_error_val_%1").arg(line);
+                        si.setError(  tr("error_parsing_error_val_%1").arg(line) );
                         break;
                     }
                     float e = (float)errStr.toDouble(&ok);
                     if (!ok) {
-                        si.error = tr("error_parsing_error_val_%1").arg(line);
+                        si.setError(  tr("error_parsing_error_val_%1").arg(line) );
                         break;
                     }
                     if (p < 0 || p >= 100) {
-                        si.error = tr("illegal_err_val_%1").arg(line);
+                        si.setError(  tr("illegal_err_val_%1").arg(line) );
                         break;
                     }
                     if (state == ERR1_S) {
@@ -279,7 +282,7 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
                     }
                 }
                 break;
-            default: si.error = tr("error_parsing_file_line_%1").arg(line);
+            default: si.setError(  tr("error_parsing_file_line_%1").arg(line) );
         }
     }
     
@@ -288,17 +291,17 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
     }
 
     if (passedPropsAve.size() != passedPropsSDev.size()) {
-        si.error = tr("ave_props_in_file_%1_not_matched_sdev_props_%2").arg(passedPropsAve.size()).arg(passedPropsSDev.size());
+        si.setError(  tr("ave_props_in_file_%1_not_matched_sdev_props_%2").arg(passedPropsAve.size()).arg(passedPropsSDev.size()) );
         return model;
     }
     
     if (passedPropsAve.size() != passedPropsWeight.size()) {
-        si.error = tr("ave_props_in_file_%1_not_matched_weight_props_%2").arg(passedPropsAve.size()).arg(passedPropsWeight.size());
+        si.setError(  tr("ave_props_in_file_%1_not_matched_weight_props_%2").arg(passedPropsAve.size()).arg(passedPropsWeight.size()) );
         return model;
     }
     
     if (passedPropsAve.size() != props.size()) {
-        si.error = tr("props_in_file_%1_not_matched_actual_props_%2").arg(passedPropsAve.size()).arg(props.size());
+        si.setError(  tr("props_in_file_%1_not_matched_actual_props_%2").arg(passedPropsAve.size()).arg(props.size()) );
         return model;
     }
     
@@ -326,14 +329,14 @@ SiteconModel SiteconIO::readModel(IOAdapterFactory* iof, const QString& url, Tas
     }
 
     if (model.err1.contains(defaultVal) || model.err2.contains(defaultVal)) {
-        si.error = tr("error_info_not_complete");
+        si.setError(  tr("error_info_not_complete") );
         return model;
     }
     model.deviationThresh = (float)critchi(model.settings.chisquare, model.settings.numSequencesInAlignment - 1) / model.settings.numSequencesInAlignment;
 
     bool ok = model.checkState(false);
     if(!ok) {
-        si.error = tr("model_verification_error");
+        si.setError(  tr("model_verification_error") );
     } 
     return model;
 }
@@ -388,7 +391,7 @@ void SiteconIO::writeModel(IOAdapterFactory* iof, const QString& url, TaskStateI
     }
     
     res.append('\n').append(SETTINGS_HEADER).append("\n");
-    res.append("ORIGIN ").append(model.modelFileName).append('\n');
+    res.append("ORIGIN ").append(model.aliURL).append('\n');
     res.append("W ").append(QByteArray::number(model.settings.windowSize)).append('\n');
     res.append("SEED ").append(QByteArray::number(model.settings.randomSeed)).append('\n');
     res.append("CLEN ").append(QByteArray::number(model.settings.secondTypeErrorCalibrationLen)).append('\n');
@@ -433,12 +436,12 @@ void SiteconIO::writeModel(IOAdapterFactory* iof, const QString& url, TaskStateI
 
     std::auto_ptr<IOAdapter> io(iof->createIOAdapter());
     if (!io->open(url, IOAdapterMode_Write)) {
-        si.error = Translations::errorOpeningFileWrite(url);
+        si.setError(  Translations::errorOpeningFileWrite(url) );
         return;
     }
     int len = io->writeBlock(res);
     if (len != res.size()) {
-        si.error = Translations::errorWritingFile(url);
+        si.setError(  Translations::errorWritingFile(url) );
         return;
     }
     io->close();
@@ -447,14 +450,13 @@ void SiteconIO::writeModel(IOAdapterFactory* iof, const QString& url, TaskStateI
 void SiteconReadTask::run() {
     IOAdapterFactory* iof = AppContext::getIOAdapterRegistry()->getIOAdapterFactoryById(BaseIOAdapters::url2io(url));
     model = SiteconIO::readModel(iof, url, stateInfo);
-    if (model.modelFileName.isEmpty()) {
-        QFileInfo fi(url);
-        model.modelFileName = fi.baseName();
-    }
 }
 
 void SiteconWriteTask::run() {
     IOAdapterFactory* iof = AppContext::getIOAdapterRegistry()->getIOAdapterFactoryById(BaseIOAdapters::url2io(url));
+    if (fileMode & SaveDoc_Roll && !DialogUtils::rollFile(url, stateInfo)) {
+        return;
+    }
     SiteconIO::writeModel(iof, url, stateInfo, model);
 }
 
