@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -14,7 +14,6 @@
 #include "DIProperties.h"
 
 #include <core_api/PluginModel.h>
-#include <core_api/AppContext.h>
 #include <core_api/ObjectViewModel.h>
 
 #include <QtGui/QMenu>
@@ -52,7 +51,6 @@ protected slots:
 
 protected:
     virtual void initViewContext(GObjectView* view);
-    virtual void buildMenu(GObjectView* v, QMenu* m);
 };
 
 class SiteconAlgorithmTests {
