@@ -1,121 +1,6 @@
 <?xml version="1.0" encoding="utf-8"?>
-<!DOCTYPE TS><TS version="1.1" language="en_US">
-<context>
-    <name>GB2::GTest</name>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="420"/>
-        <source>value not set %1</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="44"/>
-        <source>here must be 4 items in %1</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="321"/>
-        <source>Wrong conversion to the integer for one of the %1</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="59"/>
-        <source>Wrong %1 values</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="331"/>
-        <source>context not found %1</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="336"/>
-        <source>container of object with type &quot;%1&quot; is empty</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="341"/>
-        <source>object with type &quot;%1&quot; not found</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="346"/>
-        <source>error can&apos;t cast to MAlignmentObject from GObject</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="95"/>
-        <source>Actual results not equal with expected</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="145"/>
-        <source>Too less items in %1</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="203"/>
-        <source>Expected and Actual &apos;SDev&apos; values are different: %1 %2</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="207"/>
-        <source>Expected and Actual &apos;Average&apos; values are different: %1 %2</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="376"/>
-        <source>Expected and Actual values are different: %1 %2</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="455"/>
-        <source>%1 has incorrect value</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="437"/>
-        <source>unable to convert %1 to integer</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="460"/>
-        <source>unable to convert %1 to float</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="475"/>
-        <source>error can&apos;t cast to sequence from GObject</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="512"/>
-        <source>expected and equal result lists not equal by size, expected: %1, actual: %2</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconAlgorithmTests.cpp" line="525"/>
-        <source>expected and equal result lists not equal</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/DIPropertiesTests.cpp" line="46"/>
-        <source>Wrong conversion to the integer for %1</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/DIPropertiesTests.cpp" line="67"/>
-        <source>Given key &apos;%1&apos; don&apos;t present in property map</source>
-        <translation></translation>
-    </message>
-</context>
-<context>
-    <name>GB2::IOAdapter</name>
-    <message>
-        <location filename="../src/SiteconIO.cpp" line="62"/>
-        <source>file_is_too_large</source>
-        <translation>File is too large</translation>
-    </message>
-</context>
+<!DOCTYPE TS>
+<TS version="2.0" language="en_US">
 <context>
     <name>GB2::LocalWorkflow::SiteconBuildPrompter</name>
     <message>
@@ -124,7 +9,7 @@
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconBuildWorker.cpp" line="102"/>
+        <location filename="../src/SiteconBuildWorker.cpp" line="101"/>
         <source>%1 build SITECON model.</source>
         <translation></translation>
     </message>
@@ -210,7 +95,7 @@
 <context>
     <name>GB2::LocalWorkflow::SiteconReadPrompter</name>
     <message>
-        <location filename="../src/SiteconIOWorkers.cpp" line="142"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="151"/>
         <source>Read model(s) from &lt;u&gt;%1&lt;/u&gt;</source>
         <translation></translation>
     </message>
@@ -218,7 +103,7 @@
 <context>
     <name>GB2::LocalWorkflow::SiteconReader</name>
     <message>
-        <location filename="../src/SiteconIOWorkers.cpp" line="183"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="192"/>
         <source>Loaded SITECON model from %1</source>
         <translation></translation>
     </message>
@@ -226,17 +111,17 @@
 <context>
     <name>GB2::LocalWorkflow::SiteconSearchPrompter</name>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="85"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="103"/>
         <source>For each sequence from &lt;u&gt;%1&lt;/u&gt;,</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="100"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="115"/>
         <source>%1 search transcription factor binding sites (TFBS) %2.&lt;br&gt;Recognize sites with &lt;u&gt;similarity %3%&lt;/u&gt;, process &lt;u&gt;%4&lt;/u&gt;.&lt;br&gt;Output the list of found regions annotated as &lt;u&gt;%5&lt;/u&gt;.</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="86"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="104"/>
         <source>with all profiles provided by &lt;u&gt;%1&lt;/u&gt;,</source>
         <translation></translation>
     </message>
@@ -244,100 +129,128 @@
 <context>
     <name>GB2::LocalWorkflow::SiteconSearchWorker</name>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="92"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="88"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="110"/>
         <source>both strands</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="93"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="89"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="111"/>
         <source>direct strand</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="94"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="90"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="112"/>
         <source>complement strand</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="41"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="45"/>
         <source>Sitecon Model</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="42"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="46"/>
         <source>Sequence</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="43"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="47"/>
         <source>SITECON annotations</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="49"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="53"/>
         <source>Result annotation</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="50"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="54"/>
         <source>Search in</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="51"/>
-        <source>Minimal threshold, %</source>
-        <translation></translation>
-    </message>
-    <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="147"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="168"/>
         <source>Search TFBS in %1</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="167"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="172"/>
+        <source>Bad sequence supplied to SiteconSearch: %1</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/SiteconSearchWorker.cpp" line="196"/>
         <source>Found %1 TFBS</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="41"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="45"/>
         <source>Profile data to search with.</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="42"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="46"/>
         <source>Input nucleotide sequence to search in.</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="43"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="47"/>
         <source>Annotations marking found TFBS sites.</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="49"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="53"/>
         <source>Annotation name for marking found regions</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="50"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="54"/>
         <source>Which strands should be searched: direct, complement or both.</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="57"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="67"/>
         <source>Search TFBS</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchWorker.cpp" line="60"/>
+        <location filename="../src/SiteconSearchWorker.cpp" line="68"/>
         <source>Searches each input sequence for transcription factor binding sites significantly similar to specified SITECON profiles. In case several profiles were supplied, searches with all profiles one by one and outputs merged set of annotations for each sequence.</source>
         <translation></translation>
     </message>
+    <message>
+        <location filename="../src/SiteconSearchWorker.cpp" line="55"/>
+        <source>Min score</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/SiteconSearchWorker.cpp" line="56"/>
+        <source>Min Err1</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/SiteconSearchWorker.cpp" line="56"/>
+        <source>Alternative setting for filtering results, minimal value of Error type I.&lt;br&gt;Note that all thresholds (by score, by err1 and by err2) are applied when filtering results.</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/SiteconSearchWorker.cpp" line="58"/>
+        <source>Max Err2</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/SiteconSearchWorker.cpp" line="58"/>
+        <source>Alternative setting for filtering results, max value of Error type II.&lt;br&gt;Note that all thresholds (by score, by err1 and by err2) are applied when filtering results.</source>
+        <translation></translation>
+    </message>
 </context>
 <context>
     <name>GB2::LocalWorkflow::SiteconWritePrompter</name>
     <message>
-        <location filename="../src/SiteconIOWorkers.cpp" line="153"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="161"/>
         <source>Save the profile(s) from &lt;u&gt;%1&lt;/u&gt; to %2.</source>
         <translation></translation>
     </message>
@@ -345,7 +258,12 @@
 <context>
     <name>GB2::LocalWorkflow::SiteconWriter</name>
     <message>
-        <location filename="../src/SiteconIOWorkers.cpp" line="209"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="210"/>
+        <source>Unspecified URL for writing Sitecon</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/SiteconIOWorkers.cpp" line="225"/>
         <source>Writing SITECON model to %1</source>
         <translation></translation>
     </message>
@@ -353,7 +271,7 @@
 <context>
     <name>GB2::SiteconADVContext</name>
     <message>
-        <location filename="../src/SiteconPlugin.cpp" line="98"/>
+        <location filename="../src/SiteconPlugin.cpp" line="99"/>
         <source>run_sitecon</source>
         <translation>Search TFBS with SITECON</translation>
     </message>
@@ -505,162 +423,167 @@
 <context>
     <name>GB2::SiteconIO</name>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="76"/>
+        <location filename="../src/SiteconIO.cpp" line="79"/>
         <source>not_sitecon_model_file</source>
         <translation>Not a sitecon model</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="115"/>
+        <location filename="../src/SiteconIO.cpp" line="118"/>
         <source>error_parsing_settings_line_%1</source>
         <translation>Error parsing settings, line %1</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="126"/>
+        <location filename="../src/SiteconIO.cpp" line="129"/>
         <source>error_parsing_settings_window_size_%1</source>
         <translation>Error parsing window size: %1</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="139"/>
+        <location filename="../src/SiteconIO.cpp" line="142"/>
         <source>error_parsing_calibration_len_%1</source>
         <translation>Error parsing calibration len: %1</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="145"/>
+        <location filename="../src/SiteconIO.cpp" line="148"/>
         <source>error_parsing_rseed_%1</source>
         <translation>Error parsing RSEED: %1</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="157"/>
+        <location filename="../src/SiteconIO.cpp" line="154"/>
+        <location filename="../src/SiteconIO.cpp" line="160"/>
         <source>error_parsing_nsequence_in_ali_%1</source>
         <translation>Error parsing number of sequence in original alignment: %1</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="165"/>
+        <location filename="../src/SiteconIO.cpp" line="168"/>
         <source>illegal_weight_alg_%1</source>
         <translation>Illegal weight algorithm: %1</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="177"/>
+        <location filename="../src/SiteconIO.cpp" line="180"/>
         <source>model_size_not_matched_%1_expected_%2</source>
         <translation>Model size not matched: %1, expected: %2</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="210"/>
+        <location filename="../src/SiteconIO.cpp" line="213"/>
         <source>property_not_recognized_%1</source>
         <translation>Property not recognized: %1</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="226"/>
+        <location filename="../src/SiteconIO.cpp" line="229"/>
         <source>error_parsing_matrix_val_%1_in_line_%2</source>
         <translation>Error parsing matrix value: %1, line %2</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="244"/>
+        <location filename="../src/SiteconIO.cpp" line="247"/>
         <source>error_parsing_errors_line_%1</source>
         <translation>Error parsing &apos;Errors&apos; line %1</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="258"/>
+        <location filename="../src/SiteconIO.cpp" line="256"/>
+        <location filename="../src/SiteconIO.cpp" line="261"/>
         <source>error_parsing_error_val_%1</source>
         <translation>Error parsing value: %1</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="262"/>
+        <location filename="../src/SiteconIO.cpp" line="265"/>
         <source>illegal_err_val_%1</source>
         <translation>Illegal value: %1</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="272"/>
+        <location filename="../src/SiteconIO.cpp" line="275"/>
         <source>error_parsing_file_line_%1</source>
         <translation>Error parsing line: %1</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="281"/>
+        <location filename="../src/SiteconIO.cpp" line="284"/>
         <source>ave_props_in_file_%1_not_matched_sdev_props_%2</source>
         <translation>Number of &apos;average&apos; and  &apos;sdev&apos; properties not matches</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="286"/>
+        <location filename="../src/SiteconIO.cpp" line="289"/>
         <source>ave_props_in_file_%1_not_matched_weight_props_%2</source>
         <translation>Number of &apos;average&apos; and &apos;weight&apos; properties not matches</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="291"/>
+        <location filename="../src/SiteconIO.cpp" line="294"/>
         <source>props_in_file_%1_not_matched_actual_props_%2</source>
         <translation>Property in file %1 is not the same as built-in: %2</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="319"/>
+        <location filename="../src/SiteconIO.cpp" line="322"/>
         <source>error_info_not_complete</source>
         <translation>Error info in file is not complete</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="326"/>
+        <location filename="../src/SiteconIO.cpp" line="329"/>
         <source>model_verification_error</source>
         <translation>Model verification error</translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="200"/>
+        <location filename="../src/SiteconIO.cpp" line="203"/>
         <source>duplicate_prop_%1</source>
         <translation>Duplicate property: %1</translation>
     </message>
     <message>
+        <location filename="../src/SiteconIOWorkers.cpp" line="39"/>
         <location filename="../src/SiteconIOWorkers.cpp" line="105"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="114"/>
         <source>Sitecon model</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconIOWorkers.cpp" line="90"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="53"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="99"/>
         <source>Location</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconIOWorkers.cpp" line="107"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="116"/>
         <source>Read Sitecon model</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconIOWorkers.cpp" line="98"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="107"/>
         <source>Write Sitecon model</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconIOWorkers.cpp" line="46"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="45"/>
         <source>SITECON</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconIO.cpp" line="21"/>
+        <location filename="../src/SiteconIO.cpp" line="22"/>
         <source>Sitecon models</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconIOWorkers.cpp" line="54"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="53"/>
         <source>Location of the data file(s).</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconIOWorkers.cpp" line="90"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="99"/>
         <source>Location hint for the target file.</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconIOWorkers.cpp" line="96"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="105"/>
         <source>Input Sitecon model</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconIOWorkers.cpp" line="98"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="107"/>
         <source>Saves all input SITECON profiles to specified location.</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconIOWorkers.cpp" line="105"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="114"/>
         <source>Loaded SITECON profile data.</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/SiteconIOWorkers.cpp" line="107"/>
+        <location filename="../src/SiteconIOWorkers.cpp" line="116"/>
         <source>Reads SITECON profiles from file(s). The files can be local or Internet URLs.</source>
         <translation></translation>
     </message>
@@ -668,22 +591,22 @@
 <context>
     <name>GB2::SiteconPlugin</name>
     <message>
-        <location filename="../src/SiteconPlugin.cpp" line="37"/>
+        <location filename="../src/SiteconPlugin.cpp" line="38"/>
         <source>sitecon_plugin</source>
         <translation>SITECON</translation>
     </message>
     <message>
-        <location filename="../src/SiteconPlugin.cpp" line="37"/>
+        <location filename="../src/SiteconPlugin.cpp" line="38"/>
         <source>sitecon_plugin_desc</source>
-        <translation>SITECON - is a program pakage for revealing and analysis of conservative conformational and physicochemical properties in transcription factor binding sites sets.</translation>
+        <translation>SITECON - is a program package for revealing and analysis of conservative conformational and physicochemical properties in transcription factor binding sites sets.</translation>
     </message>
     <message>
-        <location filename="../src/SiteconPlugin.cpp" line="42"/>
+        <location filename="../src/SiteconPlugin.cpp" line="44"/>
         <source>sitecon_build</source>
         <translation>Build new SITECON model from alignment</translation>
     </message>
     <message>
-        <location filename="../src/SiteconPlugin.cpp" line="48"/>
+        <location filename="../src/SiteconPlugin.cpp" line="50"/>
         <source>sitecon_menu</source>
         <translation>SITECON</translation>
     </message>
@@ -724,37 +647,40 @@
         <translation>Select file with SITECON model</translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchDialogController.cpp" line="290"/>
+        <location filename="../src/SiteconSearchDialogController.cpp" line="164"/>
+        <location filename="../src/SiteconSearchDialogController.cpp" line="262"/>
+        <location filename="../src/SiteconSearchDialogController.cpp" line="273"/>
+        <location filename="../src/SiteconSearchDialogController.cpp" line="286"/>
         <source>error</source>
         <translation>Error</translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchDialogController.cpp" line="203"/>
-        <source>%1%<byte value="x9"/>err1=%2<byte value="x9"/>err2=%3</source>
+        <location filename="../src/SiteconSearchDialogController.cpp" line="199"/>
+        <source>%1%	err1=%2	err2=%3</source>
         <translation>%1%,  first type error %2,  second type error %3</translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchDialogController.cpp" line="267"/>
+        <location filename="../src/SiteconSearchDialogController.cpp" line="262"/>
         <source>model not selected</source>
         <translation>No model selected</translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchDialogController.cpp" line="278"/>
+        <location filename="../src/SiteconSearchDialogController.cpp" line="273"/>
         <source>range_is_too_small</source>
         <translation>Range is too small</translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchDialogController.cpp" line="290"/>
+        <location filename="../src/SiteconSearchDialogController.cpp" line="286"/>
         <source>error_parsing_min_score</source>
         <translation>Error parsing minumum score</translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchDialogController.cpp" line="355"/>
+        <location filename="../src/SiteconSearchDialogController.cpp" line="351"/>
         <source>complement_strand</source>
         <translation>Complement strand</translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchDialogController.cpp" line="355"/>
+        <location filename="../src/SiteconSearchDialogController.cpp" line="351"/>
         <source>direct_strand</source>
         <translation>Direct strand</translation>
     </message>
@@ -767,12 +693,12 @@
         <translation>SITECON search</translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchTask.cpp" line="22"/>
+        <location filename="../src/SiteconSearchTask.cpp" line="19"/>
         <source>sitecon_search_parallel</source>
         <translation>SITECON search parallel subtask</translation>
     </message>
     <message>
-        <location filename="../src/SiteconSearchTask.cpp" line="41"/>
+        <location filename="../src/SiteconSearchTask.cpp" line="39"/>
         <source>internal_error_invalid_psum:%1</source>
         <translation>Internal error: invalid PSUM: %1</translation>
     </message>
@@ -798,6 +724,7 @@
         <translation>Iinput alignment (nucleic)</translation>
     </message>
     <message>
+        <location filename="../src/ui/SiteconBuildDialog.ui" line="31"/>
         <location filename="../src/ui/SiteconBuildDialog.ui" line="48"/>
         <source>...</source>
         <translation></translation>
@@ -883,21 +810,29 @@
         <translation></translation>
     </message>
     <message>
+        <location filename="../src/SiteconBuildWorker.cpp" line="51"/>
+        <location filename="../src/ui/SiteconBuildDialog.ui" line="63"/>
         <location filename="../src/ui/SiteconBuildDialog.ui" line="92"/>
         <source>win_tip</source>
         <translation>Window size must be not less than the consensus sequence for this particular site and should not exceed TFBS alignment length. Optimal is to choose this parameter a bit less than the alignment length, but not more than 50.</translation>
     </message>
     <message>
+        <location filename="../src/SiteconBuildWorker.cpp" line="52"/>
+        <location filename="../src/ui/SiteconBuildDialog.ui" line="165"/>
         <location filename="../src/ui/SiteconBuildDialog.ui" line="188"/>
         <source>seq_len_tip</source>
         <translation>Length of random synthetic sequences used to calibrate the profile. Should not be less than window size.</translation>
     </message>
     <message>
+        <location filename="../src/SiteconBuildWorker.cpp" line="53"/>
+        <location filename="../src/ui/SiteconBuildDialog.ui" line="114"/>
         <location filename="../src/ui/SiteconBuildDialog.ui" line="143"/>
         <source>seed_tip</source>
         <translation>The random seed, where &lt;n&gt; is a positive integer. You can use this option to generate reproducible results for different runs on the same data.</translation>
     </message>
     <message>
+        <location filename="../src/SiteconBuildWorker.cpp" line="54"/>
+        <location filename="../src/ui/SiteconBuildDialog.ui" line="224"/>
         <location filename="../src/ui/SiteconBuildDialog.ui" line="247"/>
         <source>algo_tip</source>
         <translation>Optional feature, in most cases applying no weight will fit. In some cases choosing algorithm 2 will increase the recognition quality.</translation>
@@ -956,6 +891,7 @@
         <translation>Complement strand</translation>
     </message>
     <message>
+        <location filename="../src/ui/SiteconSearchDialog.ui" line="129"/>
         <location filename="../src/ui/SiteconSearchDialog.ui" line="264"/>
         <source>range</source>
         <translation>Range</translation>
@@ -1026,6 +962,8 @@
         <translation>Cancel</translation>
     </message>
     <message>
+        <location filename="../src/SiteconSearchWorker.cpp" line="55"/>
+        <location filename="../src/ui/SiteconSearchDialog.ui" line="42"/>
         <location filename="../src/ui/SiteconSearchDialog.ui" line="58"/>
         <source>min_err_tip</source>
         <translation>Recognition quality threshold, should be less than 100%. Choosing too low threshold will lead to recognition of too many TFBS recognised with too low trustworthiness. Choosing too high threshold may result in no TFBS recognised.</translation>
