@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -16,27 +16,27 @@
 
 #define	TRACE	0
 
-static void LogPP(const ProfPos &PP)
-	{
-    char *g_LetterToChar = getMuscleContext()->alpha.g_LetterToChar;
-
-	Log("ResidueGroup   %u\n", PP.m_uResidueGroup);
-	Log("AllGaps      %d\n", PP.m_bAllGaps);
-	Log("Occ          %.3g\n", PP.m_fOcc);
-	Log("LL=%.3g LG=%.3g GL=%.3g GG=%.3g\n", PP.m_LL, PP.m_LG, PP.m_GL, PP.m_GG);
-	Log("Freqs        ");
-	for (unsigned i = 0; i < 20; ++i)
-		if (PP.m_fcCounts[i] > 0)
-			Log("%c=%.3g ", LetterToChar(i), PP.m_fcCounts[i]);
-	Log("\n");
-	}
+//static void LogPP(const ProfPos &PP)
+//	{
+//    char *g_LetterToChar = getMuscleContext()->alpha.g_LetterToChar;
+//
+//	Log("ResidueGroup   %u\n", PP.m_uResidueGroup);
+//	Log("AllGaps      %d\n", PP.m_bAllGaps);
+//	Log("Occ          %.3g\n", PP.m_fOcc);
+//	Log("LL=%.3g LG=%.3g GL=%.3g GG=%.3g\n", PP.m_LL, PP.m_LG, PP.m_GL, PP.m_GG);
+//	Log("Freqs        ");
+//	for (unsigned i = 0; i < 20; ++i)
+//		if (PP.m_fcCounts[i] > 0)
+//			Log("%c=%.3g ", LetterToChar(i), PP.m_fcCounts[i]);
+//	Log("\n");
+//	}
 
 static void AssertProfPosEq(const ProfPos *PA, const ProfPos *PB, unsigned i)
 	{
 	const ProfPos &PPA = PA[i];
 	const ProfPos &PPB = PB[i];
-#define	eq(x)	if (PPA.m_##x != PPB.m_##x) { LogPP(PPA); LogPP(PPB); Quit("AssertProfPosEq." #x); }
-#define be(x)	if (!BTEq(PPA.m_##x, PPB.m_##x)) { LogPP(PPA); LogPP(PPB); Quit("AssertProfPosEq." #x); }
+#define	eq(x)	if (PPA.m_##x != PPB.m_##x) { /*LogPP(PPA); LogPP(PPB);*/ Quit("AssertProfPosEq." #x); }
+#define be(x)	if (!BTEq(PPA.m_##x, PPB.m_##x)) { /*LogPP(PPA); LogPP(PPB);*/ Quit("AssertProfPosEq." #x); }
 	eq(bAllGaps)
 	eq(uResidueGroup)
 
