@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -373,15 +373,15 @@ Done:
 		Log("TB success\n");
 	return Eq;
 	}
-
-static const char *LocalScoreToStr(SCORE s)
-	{
-	static char str[16];
-	if (s < -100000)
-		return "     *";
-	sprintf(str, "%6.1f", s);
-	return str;
-	}
+//warning: unsafe
+//static const char *LocalScoreToStr(SCORE s)
+//	{
+//	static char str[16];
+//	if (s < -100000)
+//		return "     *";
+//	sprintf(str, "%6.1f", s);
+//	return str;
+//	}
 
 static void LogDP(const SCORE *DPM_, const ProfPos *PA, const ProfPos *PB,
   unsigned uPrefixCountA, unsigned uPrefixCountB)
@@ -525,18 +525,18 @@ static void ListTB(char *TBM_, const ProfPos *PA, const ProfPos *PB,
 		Log("\n");
 		}
 	}
-
-static const char *BitsToStr(char Bits)
-	{
-	static char Str[32];
-
-	sprintf(Str, "%cM %cD %cE %cI %cJ",
-	  Get_M_Char(Bits),
-	  Get_D_Char(Bits),
-	  Get_E_Char(Bits),
-	  Get_I_Char(Bits),
-	  Get_J_Char(Bits));
-	}
+//warning: unsafe
+//static const char *BitsToStr(char Bits)
+//	{
+//	static char Str[32];
+//
+//	sprintf(Str, "%cM %cD %cE %cI %cJ",
+//	  Get_M_Char(Bits),
+//	  Get_D_Char(Bits),
+//	  Get_E_Char(Bits),
+//	  Get_I_Char(Bits),
+//	  Get_J_Char(Bits));
+//	}
 #endif	// TRACE
 
 static inline void SetBitTBM(char **TB, unsigned i, unsigned j, char c)
