@@ -1,122 +1,6 @@
 <?xml version="1.0" encoding="utf-8"?>
-<!DOCTYPE TS><TS version="1.1" language="ru_RU">
-<defaultcodec></defaultcodec>
-<context>
-    <name>GB2::GTest</name>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="190"/>
-        <source>value not set %1</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="59"/>
-        <source>context not found %1</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="65"/>
-        <source>container of object with type &quot;%1&quot; is empty</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="71"/>
-        <source>object with type &quot;%1&quot; not found</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="77"/>
-        <source>error can&apos;t cast to multiple alignment from GObject</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="132"/>
-        <source>document not found %1</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="140"/>
-        <source>MAlignmentObjects count not matched %1, expected %2</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="151"/>
-        <source>MAlignmentObjects name not matched &quot;%1&quot;, expected &quot;%2&quot;</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="170"/>
-        <source>aligned sequence not found &quot;%1&quot;</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="160"/>
-        <source>Aligned sequences &quot;%1&quot; length not matched &quot;%2&quot;, expected &quot;%3&quot;</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="164"/>
-        <source>Aligned sequences &quot;%1&quot; not matched &quot;%2&quot;, expected &quot;%3&quot;</source>
-        <translation type="unfinished"></translation>
-    </message>
-</context>
-<context>
-    <name>GB2::GTest_uMuscleAddUnalignedSequenceToProfile</name>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="206"/>
-        <source>error parsing gap value &apos;%1&apos;, line %2</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="217"/>
-        <source>error result-ali-len &apos;%1&apos;</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="228"/>
-        <source>alignment document not found in context: %1</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="233"/>
-        <source>sequence document not found in context: %1</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="238"/>
-        <source>no alignment obejct found in doc: %1</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="246"/>
-        <source>no sequence objects found in doc: %1</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="257"/>
-        <source>number of sequences not matches number of gaps in test: %1 sequences and %2 gap lines</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="275"/>
-        <source>result alignment length notmatches: %1, expected: %2</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="280"/>
-        <source>unexpected number of sequences in result: %1, expected: %2</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="292"/>
-        <source>illegal gap found! pos: %1, sequence: %2</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <location filename="../src/umuscle_tests/umuscleTests.cpp" line="301"/>
-        <source>gap not found! pos: %1, sequence: %2</source>
-        <translation type="unfinished"></translation>
-    </message>
-</context>
+<!DOCTYPE TS>
+<TS version="2.0" language="ru_RU">
 <context>
     <name>GB2::LocalWorkflow::MusclePrompter</name>
     <message>
@@ -125,7 +9,7 @@
         <translation> из %1</translation>
     </message>
     <message>
-        <location filename="../src/MuscleWorker.cpp" line="79"/>
+        <location filename="../src/MuscleWorker.cpp" line="78"/>
         <source>For each MSA&lt;u&gt;%1&lt;/u&gt;, build the alignment using &lt;u&gt;&quot;%2&quot; preset&lt;/u&gt; and send it to output.</source>
         <translation>Для каждого выравнивания&lt;u&gt;%1&lt;/u&gt;, запустить MUSCLE в конфигурации &lt;u&gt;&quot;%2&quot;&lt;/u&gt; и выдать результат на  выход.</translation>
     </message>
@@ -168,12 +52,12 @@
         <translation>Результат выравнивания.</translation>
     </message>
     <message>
-        <location filename="../src/MuscleWorker.cpp" line="45"/>
+        <location filename="../src/MuscleWorker.cpp" line="44"/>
         <source>MUSCLE is public domain multiple alignment software for protein and nucleotide sequences.&lt;p&gt;&lt;dfn&gt;MUSCLE stands for MUltiple Sequence Comparison by Log-Expectation.&lt;/dfn&gt;&lt;/p&gt;</source>
         <translation>Пакет MUSCLE предназначен для выравнивания множественных протеиновых и нуклеотидных последовательностей.</translation>
     </message>
     <message>
-        <location filename="../src/MuscleWorker.cpp" line="115"/>
+        <location filename="../src/MuscleWorker.cpp" line="116"/>
         <source>Aligned %1 with MUSCLE</source>
         <translation>Выровнял %1 с MUSCLE</translation>
     </message>
@@ -183,7 +67,7 @@
         <translation>Выбор между режимами максимальной точности, скорости либо компромиса между ними. По умолчанию оптимизируется точность выравнивания.</translation>
     </message>
     <message>
-        <location filename="../src/MuscleWorker.cpp" line="39"/>
+        <location filename="../src/MuscleWorker.cpp" line="38"/>
         <source>Do not rearrange aligned sequences (-stable switch of MUSCLE). &lt;p&gt;Otherwise, MUSCLE re-arranges sequences so that similar sequences are adjacent in the output file. This makes the alignment easier to evaluate by eye. </source>
         <translation>Сохранять начальный порядок последовательностей в выравнивании. &lt;p&gt;Иначе, MUSCLE группирует похожие последовательности в выходном выравнивании, для удобства визуального сравнения.</translation>
     </message>
@@ -191,85 +75,94 @@
 <context>
     <name>GB2::MuscleAdapter</name>
     <message>
-        <location filename="../src/MuscleAdapter.cpp" line="199"/>
+        <location filename="../src/MuscleAdapter.cpp" line="50"/>
         <source>No sequences in input file</source>
         <translation>Выравнивание не содержит данных</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAdapter.cpp" line="226"/>
+        <location filename="../src/MuscleAdapter.cpp" line="77"/>
         <source>alignment_is_empty</source>
         <translation>Выравнивание не содержит данных</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAdapter.cpp" line="596"/>
+        <location filename="../src/MuscleAdapter.cpp" line="28"/>
+        <location filename="../src/MuscleAdapter.cpp" line="176"/>
+        <location filename="../src/MuscleAdapter.cpp" line="263"/>
+        <location filename="../src/MuscleAdapter.cpp" line="452"/>
         <source>Internal MUSCLE error: %1</source>
         <translation>Внутренняя ошибка MUSCLE: %1</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAdapter.cpp" line="400"/>
+        <location filename="../src/MuscleAdapter.cpp" line="256"/>
         <source>Invalid input alignment</source>
         <translation>Некорректное входное выравнивание</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAdapter.cpp" line="607"/>
+        <location filename="../src/MuscleAdapter.cpp" line="272"/>
+        <location filename="../src/MuscleAdapter.cpp" line="463"/>
         <source>Incompatible alphabets</source>
         <translation>Несовместимые алфавиты</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAdapter.cpp" line="445"/>
+        <location filename="../src/MuscleAdapter.cpp" line="301"/>
         <source>Aligning profiles</source>
         <translation>Выравнивание</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAdapter.cpp" line="448"/>
+        <location filename="../src/MuscleAdapter.cpp" line="304"/>
         <source>Building output</source>
         <translation>Подготовка результата</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAdapter.cpp" line="589"/>
+        <location filename="../src/MuscleAdapter.cpp" line="445"/>
         <source>Profile is not aligned</source>
         <translation>Выравнивание не нормализовано</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAdapter.cpp" line="636"/>
+        <location filename="../src/MuscleAdapter.cpp" line="492"/>
         <source>Aligning sequence %1 of %2</source>
         <translation>Выравнивается последовательность %1 из %2</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAdapter.cpp" line="668"/>
+        <location filename="../src/MuscleAdapter.cpp" line="524"/>
         <source>Merging results: %1 of %2</source>
         <translation>Объединяются результаты: %1 из %2</translation>
     </message>
+    <message>
+        <location filename="../src/MuscleUtils.cpp" line="69"/>
+        <source>Unsupported alphabet: %1</source>
+        <translation>Неподходящий алфавит: %1 </translation>
+    </message>
 </context>
 <context>
     <name>GB2::MuscleAddSequencesToProfileTask</name>
     <message>
-        <location filename="../src/MuscleTask.cpp" line="123"/>
+        <location filename="../src/MuscleTask.cpp" line="196"/>
         <source>MUSCLE align profiles &apos;%1&apos; vs &apos;%2&apos;</source>
         <translation>MUSCLE выравнивает &apos;%1&apos; к &apos;%2&apos;</translation>
     </message>
     <message>
-        <location filename="../src/MuscleTask.cpp" line="125"/>
+        <location filename="../src/MuscleTask.cpp" line="198"/>
         <source>MUSCLE align &apos;%2&apos; by profile &apos;%1&apos;</source>
         <translation>MUSCLE добавляет &apos;%2&apos; к &apos;%1&apos;</translation>
     </message>
     <message>
-        <location filename="../src/MuscleTask.cpp" line="141"/>
+        <location filename="../src/MuscleTask.cpp" line="214"/>
         <source>input_format_error</source>
         <translation>Не удалось определить формат файла</translation>
     </message>
     <message>
-        <location filename="../src/MuscleTask.cpp" line="178"/>
+        <location filename="../src/MuscleTask.cpp" line="251"/>
         <source>Sequences in file have different alphabets %1</source>
         <translation>Последовательности в файле имеют разные алфавиты: %1</translation>
     </message>
     <message>
-        <location filename="../src/MuscleTask.cpp" line="196"/>
+        <location filename="../src/MuscleTask.cpp" line="269"/>
         <source>No sequences found in file %1</source>
         <translation>Файл не содержит последовательностей: %1</translation>
     </message>
     <message>
-        <location filename="../src/MuscleTask.cpp" line="198"/>
+        <location filename="../src/MuscleTask.cpp" line="271"/>
         <source>No alignment found in file %1</source>
         <translation>Файл не содержит выравниваний: %1</translation>
     </message>
@@ -287,47 +180,47 @@
         <translation>Неправильный регион</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAlignDialogController.cpp" line="60"/>
+        <location filename="../src/MuscleAlignDialogController.cpp" line="68"/>
         <source>MUSCLE default</source>
         <translation>По умолчанию</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAlignDialogController.cpp" line="61"/>
+        <location filename="../src/MuscleAlignDialogController.cpp" line="69"/>
         <source>&lt;p&gt;The default settings are designed to give the best accuracy</source>
         <translation>&lt;p&gt;Наилучшая точность выравнивания</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAlignDialogController.cpp" line="62"/>
+        <location filename="../src/MuscleAlignDialogController.cpp" line="70"/>
         <source>&lt;p&gt;&lt;b&gt;Command line:&lt;/b&gt; muscle &lt;no-parameters&gt;</source>
         <translation>&lt;p&gt;&lt;b&gt;Командная строка:&lt;/b&gt; muscle &lt;no-parameters&gt;</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAlignDialogController.cpp" line="66"/>
+        <location filename="../src/MuscleAlignDialogController.cpp" line="74"/>
         <source>Large alignment</source>
         <translation>Большие выравнивания</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAlignDialogController.cpp" line="67"/>
+        <location filename="../src/MuscleAlignDialogController.cpp" line="75"/>
         <source>&lt;p&gt;If you have a large number of sequences (a few thousand), or they are very long, then the default settings may be too slow for practical use. A good compromise between speed and accuracy is to run just the first two iterations of the algorithm</source>
         <translation>&lt;p&gt;При наличии тысяч последовательностей либо их большой длине, конфигурация по умолчанию может оказаться неприемлемо медленной. Хороший компромисс между скоростью и точностью обеспечивается при прогоне только первых 2-х итераций алгоритма. </translation>
     </message>
     <message>
-        <location filename="../src/MuscleAlignDialogController.cpp" line="68"/>
+        <location filename="../src/MuscleAlignDialogController.cpp" line="76"/>
         <source>&lt;p&gt;&lt;b&gt;Command line:&lt;/b&gt; muscle &lt;i&gt;-maxiters 2&lt;/i&gt;</source>
         <translation>&lt;p&gt;&lt;b&gt;Командная строка:&lt;/b&gt; muscle &lt;i&gt;-maxiters 2&lt;/i&gt;</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAlignDialogController.cpp" line="72"/>
+        <location filename="../src/MuscleAlignDialogController.cpp" line="80"/>
         <source>Refine only</source>
         <translation>Только улучшить</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAlignDialogController.cpp" line="73"/>
+        <location filename="../src/MuscleAlignDialogController.cpp" line="81"/>
         <source>&lt;p&gt;Improves existing alignment without complete realignment</source>
         <translation>&lt;p&gt;Улучшение существующего выравнивания</translation>
     </message>
     <message>
-        <location filename="../src/MuscleAlignDialogController.cpp" line="74"/>
+        <location filename="../src/MuscleAlignDialogController.cpp" line="82"/>
         <source>&lt;p&gt;&lt;b&gt;Command line:&lt;/b&gt; muscle &lt;i&gt;-refine&lt;/i&gt;</source>
         <translation>&lt;p&gt;&lt;b&gt;Командная строка:&lt;/b&gt; muscle &lt;i&gt;-refine&lt;/i&gt;</translation>
     </message>
@@ -335,32 +228,33 @@
 <context>
     <name>GB2::MuscleGObjectTask</name>
     <message>
-        <location filename="../src/MuscleTask.cpp" line="224"/>
+        <location filename="../src/MuscleTask.cpp" line="297"/>
         <source>MUSCLE align &apos;%1&apos;</source>
         <translation>MUSCLE выравнивает &apos;%1&apos;</translation>
     </message>
     <message>
-        <location filename="../src/MuscleTask.cpp" line="227"/>
+        <location filename="../src/MuscleTask.cpp" line="300"/>
         <source>MUSCLE refine &apos;%1&apos;</source>
         <translation>MUSCLE улучшает &apos;%1&apos;</translation>
     </message>
     <message>
-        <location filename="../src/MuscleTask.cpp" line="230"/>
+        <location filename="../src/MuscleTask.cpp" line="303"/>
         <source>MUSCLE add to profile &apos;%1&apos;</source>
         <translation>MUSCLE добавляет в выравнивание &apos;%1&apos;</translation>
     </message>
     <message>
-        <location filename="../src/MuscleTask.cpp" line="233"/>
+        <location filename="../src/MuscleTask.cpp" line="306"/>
         <source>MUSCLE align profiles</source>
         <translation>MUSCLE выравнивает пару выравниваний</translation>
     </message>
     <message>
-        <location filename="../src/MuscleTask.cpp" line="248"/>
+        <location filename="../src/MuscleTask.cpp" line="321"/>
         <source>object_removed</source>
         <translation>Объект был удалён</translation>
     </message>
     <message>
-        <location filename="../src/MuscleTask.cpp" line="275"/>
+        <location filename="../src/MuscleTask.cpp" line="325"/>
+        <location filename="../src/MuscleTask.cpp" line="348"/>
         <source>object_is_state_locked</source>
         <translation>Объект заблокирован</translation>
     </message>
@@ -368,59 +262,141 @@
 <context>
     <name>GB2::MuscleMSAEditorContext</name>
     <message>
-        <location filename="../src/MusclePlugin.cpp" line="87"/>
+        <location filename="../src/MusclePlugin.cpp" line="97"/>
         <source>Align sequences to profile with MUSCLE</source>
         <translation>Выровнять с дополнительными последовательностями ...</translation>
     </message>
     <message>
-        <location filename="../src/MusclePlugin.cpp" line="94"/>
+        <location filename="../src/MusclePlugin.cpp" line="104"/>
         <source>Align profile to profile with MUSCLE</source>
         <translation>Выровнять с другим выравниванием ...</translation>
     </message>
     <message>
-        <location filename="../src/MusclePlugin.cpp" line="138"/>
+        <location filename="../src/MusclePlugin.cpp" line="148"/>
         <source>Select file with sequences</source>
         <translation>Выбор файла последовательностей</translation>
     </message>
     <message>
-        <location filename="../src/MusclePlugin.cpp" line="153"/>
+        <location filename="../src/MusclePlugin.cpp" line="163"/>
         <source>Select file with alignment</source>
         <translation>Выбор файла выравнивания</translation>
     </message>
     <message>
-        <location filename="../src/MusclePlugin.cpp" line="80"/>
+        <location filename="../src/MusclePlugin.cpp" line="90"/>
         <source>Align with MUSCLE</source>
         <translation>Выровнять ...</translation>
     </message>
 </context>
+<context>
+    <name>GB2::MuscleParallelTask</name>
+    <message>
+        <location filename="../src/MuscleParallel.cpp" line="30"/>
+        <source>MuscleParallelTask</source>
+        <translation></translation>
+    </message>
+</context>
 <context>
     <name>GB2::MusclePlugin</name>
     <message>
-        <location filename="../src/MusclePlugin.cpp" line="35"/>
+        <location filename="../src/MusclePlugin.cpp" line="41"/>
         <source>MUSCLE</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/MusclePlugin.cpp" line="36"/>
-        <source>A port of MUSCLE package for multiple sequence alignment. Check http://www.drive5.com/muscle/ for original</source>
-        <translation>Порт пакета MUSCLE для выравнивания множественных последовательностей. 
-Оригинальный сайт http://www.drive5.com/muscle/</translation>
+        <location filename="../src/MusclePlugin.cpp" line="42"/>
+        <source>A port of MUSCLE package for multiple sequence alignment. Check http://www.drive5.com/muscle/ for the original version</source>
+        <translation>Порт пакета MUSCLE для выравнивания множественных последовательностей. ￼Сайт оригинального пакета http://www.drive5.com/muscle/</translation>
+    </message>
+</context>
+<context>
+    <name>GB2::MusclePrepareTask</name>
+    <message>
+        <location filename="../src/MuscleParallel.cpp" line="74"/>
+        <source>Internal parallel MUSCLE error: %1</source>
+        <translation>Внутренняя ошибка: %1</translation>
+    </message>
+    <message>
+        <location filename="../src/MuscleParallel.cpp" line="114"/>
+        <source>No sequences in input file</source>
+        <translation>Выравнивание не содержит данных</translation>
+    </message>
+    <message>
+        <location filename="../src/MuscleParallel.cpp" line="141"/>
+        <source>alignment_is_empty</source>
+        <translation>Выравнивание не содержит данных</translation>
     </message>
 </context>
 <context>
     <name>GB2::MuscleTask</name>
     <message>
-        <location filename="../src/MuscleTask.cpp" line="32"/>
+        <location filename="../src/MuscleTask.cpp" line="38"/>
         <source>MUSCLE alignment</source>
         <translation>Выравнивание MUSCLE</translation>
     </message>
 </context>
 <context>
-    <name>MuscleAdapter</name>
+    <name>GB2::ProgressiveAlignTask</name>
     <message>
-        <location filename="../src/MuscleAdapter.cpp" line="88"/>
-        <source>unsupported_alphabet_%1</source>
-        <translation>Алфавит не поддерживается: &quot;%1&quot;</translation>
+        <location filename="../src/MuscleParallel.cpp" line="255"/>
+        <source>ProgressiveAlignTask</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/MuscleParallel.cpp" line="277"/>
+        <source>Internal parallel MUSCLE error: %1</source>
+        <translation>Внутренняя ошибка: %1</translation>
+    </message>
+    <message>
+        <location filename="../src/MuscleParallel.cpp" line="282"/>
+        <source>alignment &quot;%1&quot; Parallel MUSCLE Iter 1 accomplished. Time elapsed %2 ms</source>
+        <translation></translation>
+    </message>
+</context>
+<context>
+    <name>GB2::ProgressiveAlignWorker</name>
+    <message>
+        <location filename="../src/MuscleParallel.cpp" line="330"/>
+        <source>ProgressiveAlignWorker</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/MuscleParallel.cpp" line="344"/>
+        <source>Internal parallel MUSCLE error: %1</source>
+        <translation>Внутренняя ошибка: %1</translation>
+    </message>
+</context>
+<context>
+    <name>GB2::RefineTask</name>
+    <message>
+        <location filename="../src/MuscleParallel.cpp" line="504"/>
+        <source>RefineTask</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/MuscleParallel.cpp" line="527"/>
+        <source>Internal parallel MUSCLE error: %1</source>
+        <translation>Внутренняя ошибка: %1</translation>
+    </message>
+</context>
+<context>
+    <name>GB2::RefineTreeTask</name>
+    <message>
+        <location filename="../src/MuscleParallel.cpp" line="459"/>
+        <source>RefineTreeTask</source>
+        <translation></translation>
+    </message>
+    <message>
+        <location filename="../src/MuscleParallel.cpp" line="471"/>
+        <source>Internal parallel MUSCLE error: %1</source>
+        <translation>Внутренняя ошибка: %1</translation>
+    </message>
+</context>
+<context>
+    <name>GB2::RefineWorker</name>
+    <message>
+        <location filename="../src/MuscleParallel.cpp" line="581"/>
+        <source>Internal parallel MUSCLE error: %1</source>
+        <translation type="unfinished">Внутренняя ошибка: %1</translation>
     </message>
 </context>
 <context>
@@ -447,31 +423,45 @@
     </message>
     <message>
         <location filename="../src/ui/MuscleAlignDialog.ui" line="72"/>
+        <source>Max iterations</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location filename="../src/ui/MuscleAlignDialog.ui" line="111"/>
+        <source>Max time (minutes)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location filename="../src/ui/MuscleAlignDialog.ui" line="156"/>
         <source>Region to align</source>
         <translation>Выравнивать:</translation>
     </message>
     <message>
-        <location filename="../src/ui/MuscleAlignDialog.ui" line="101"/>
         <source>Custom range</source>
-        <translation>Указанный регион</translation>
+        <translation type="obsolete">Указанный регион</translation>
     </message>
     <message>
-        <location filename="../src/ui/MuscleAlignDialog.ui" line="152"/>
+        <location filename="../src/ui/MuscleAlignDialog.ui" line="236"/>
         <source>-</source>
         <translation></translation>
     </message>
     <message>
-        <location filename="../src/ui/MuscleAlignDialog.ui" line="78"/>
+        <location filename="../src/ui/MuscleAlignDialog.ui" line="175"/>
         <source>Whole alignment</source>
         <translation>Целые последовательности</translation>
     </message>
     <message>
-        <location filename="../src/ui/MuscleAlignDialog.ui" line="198"/>
+        <location filename="../src/ui/MuscleAlignDialog.ui" line="185"/>
+        <source>Column range</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <location filename="../src/ui/MuscleAlignDialog.ui" line="277"/>
         <source>Align</source>
         <translation>Выровнять</translation>
     </message>
     <message>
-        <location filename="../src/ui/MuscleAlignDialog.ui" line="208"/>
+        <location filename="../src/ui/MuscleAlignDialog.ui" line="287"/>
         <source>Cancel</source>
         <translation>Отменить</translation>
     </message>
