@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -14,6 +14,8 @@
 #include "windows.h"
 #endif
 
+#include <stdio.h>
+
 #define MAX_CACHE_SIZE 5000
 
 namespace GB2 {
@@ -34,15 +36,15 @@ LogCache::~LogCache() {
 void LogCache::sl_onMessage(const LogMessage& msg) {
     messages.append(new LogMessage(msg.category, msg.level, msg.text));
 
-#ifdef _DEBUG
-    QByteArray ba = msg.text.toLocal8Bit();
-    char* buf = ba.data();
-#ifdef Q_OS_WIN32
-    // a bit of magic to workaround Windows console encoding issues
-    CharToOemA(buf,buf);
-#endif
-    printf("%s\n", buf);
-#endif
+    if (print_to_console == true){
+        QByteArray ba = msg.text.toLocal8Bit();
+        char* buf = ba.data();
+    #ifdef Q_OS_WIN32
+        // a bit of magic to workaround Windows console encoding issues
+        CharToOemA(buf,buf);
+    #endif
+        printf("%s\n", buf);
+    }
     updateSize();
 }
 
