@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -10,7 +10,7 @@
 
 #include "ProjectViewImpl.h"
 
-#include "../AppContextImpl.h"
+#include <core_services/AppContextImpl.h>
 
 #include <core_api/ProjectModel.h>
 #include <core_api/Settings.h>
@@ -20,16 +20,20 @@
 #include <core_api/GObject.h>
 
 #include <gobjects/GObjectTypes.h>
+#include <gobjects/GObjectUtils.h>
 
 #include <util_tasks/SaveDocumentTask.h>
 #include <util_tasks/LoadDocumentTask.h>
 #include <util_tasks/MultiTask.h>
 #include <util_tasks/CopyDataTask.h>
 #include <util_tasks/AddDocumentTask.h>
+#include <util_tasks/SaveDocumentTask.h>
 
 #include <util_gui/AddNewDocumentDialogImpl.h>
 #include <util_gui/AddExistingDocumentDialogImpl.h>
 #include <util_gui/GUIUtils.h>
+#include <util_gui/DialogUtils.h>
+#include <util_gui/CopyDocumentDialogController.h>
 #include <selection/SelectionUtils.h>
 
 #include <QtGui/QtGui>
@@ -43,20 +47,36 @@ namespace GB2 {
 
 static LogCategory log(ULOG_CAT_CORE_SERVICES);
 
-
 #define SETTINGS_ROOT QString("projecview/")
 
 ProjectViewWidget::ProjectViewWidget() {
     setupUi(this);
+    groupModeMenu = new QMenu(tr("Group mode"), this);
+    groupModeMenu->setObjectName("group_Mode_Menu");
+    groupModeButton->setMenu(groupModeMenu);
+    groupModeButton->setPopupMode(QToolButton::InstantPopup);
+    groupModeButton->setIcon(QIcon(":ugene/images/project_filter.png"));
+    
     setObjectName(DOCK_PROJECT_VIEW);
     setWindowTitle(tr("Project"));
     setWindowIcon(QIcon(":ugene/images/project.png"));
 }
 
+static ProjectTreeGroupMode getLastGroupMode() {
+    int n = AppContext::getSettings()->getValue(SETTINGS_ROOT + "groupMode", ProjectTreeGroupMode_ByDocument).toInt();
+    n = qBound((int)ProjectTreeGroupMode_Min, n, (int)ProjectTreeGroupMode_Max);
+    return (ProjectTreeGroupMode)n;
+}
+
+static void saveGroupMode(ProjectTreeGroupMode m) {
+    AppContext::getSettings()->setValue(SETTINGS_ROOT + "groupMode", (int)m);
+}
+
+
 //////////////////////////////////////////////////////////////////////////
 // ProjectViewImpl
-ProjectViewImpl::ProjectViewImpl(Plugin* p)
-: ProjectView(p, tr("projectview_sname"), tr("projectview_sdesc"))
+ProjectViewImpl::ProjectViewImpl()
+: ProjectView(tr("projectview_sname"), tr("projectview_sdesc"))
 {
     w = NULL;
     projectTreeController = NULL;
@@ -64,6 +84,8 @@ ProjectViewImpl::ProjectViewImpl(Plugin* p)
     addExistingDocumentAction = NULL;
 //    addNewDocumentAction = NULL;
     saveSelectedDocsAction = NULL;
+    relocateDocumentAction = NULL;
+    addExistingDocumentAction = NULL;
 
     //todo: move it somewhere else -> object views could be openend without project view service active
     registerBuiltInObjectViews();
@@ -101,13 +123,22 @@ void ProjectViewImpl::enable() {
     addExistingDocumentAction = new QAction(QIcon(":ugene/images/advanced_open.png"), tr("open_as"), w);
     addExistingDocumentAction->setShortcut(QKeySequence(Qt::CTRL | Qt::SHIFT | Qt::Key_O));
     addExistingDocumentAction->setShortcutContext(Qt::ApplicationShortcut);
+    connect(addExistingDocumentAction, SIGNAL(triggered()), SLOT(sl_onAddExistingDocument()));
 
     //addNewDocumentAction = new QAction(QIcon(":ugene/images/add_new_document.png"), tr("add_new_document_action"), w);
-    saveSelectedDocsAction = new QAction(QIcon(":ugene/images/save_selected_documents.png"), tr("save_selected_modified_docs_action"), w);
-    connect(addExistingDocumentAction, SIGNAL(triggered()), SLOT(sl_onAddExistingDocument()));
     //connect(addNewDocumentAction, SIGNAL(triggered()), SLOT(sl_onAddNewDocument()));
+
+    saveSelectedDocsAction = new QAction(QIcon(":ugene/images/save_selected_documents.png"), tr("save_selected_modified_docs_action"), w);
     connect(saveSelectedDocsAction, SIGNAL(triggered()), SLOT(sl_onSaveSelectedDocs()));
 
+    relocateDocumentAction = new QAction(tr("Relocate.."), w);
+    relocateDocumentAction->setIcon(QIcon(":ugene/images/relocate.png"));
+    connect(relocateDocumentAction, SIGNAL(triggered()), SLOT(sl_relocate()));
+
+    saveCopyAction = new QAction(tr("Save a copy.."), w);
+    saveCopyAction->setIcon(QIcon(":ugene/images/save_copy.png"));
+    connect(saveCopyAction, SIGNAL(triggered()), SLOT(sl_saveCopy()));
+
     initView();
 
     MainWindow* mw = AppContext::getMainWindow();
@@ -144,6 +175,7 @@ void ProjectViewImpl::disable() {
     MainWindow* mw = AppContext::getMainWindow();
     if (w!=NULL) {
         saveWidgetState(w);
+        saveGroupMode(projectTreeController->getModeSettings().groupMode);
     }	
     delete w;
     w = NULL;
@@ -181,16 +213,25 @@ void ProjectViewImpl::restoreWidgetState(ProjectViewWidget* w) {
 }
 
 void ProjectViewImpl::initView() {
-    w->documentLabel->setText(tr("documents_label"));
-    w->viewLabel->setText(tr("bookmarks_label"));
+    //w->objectBox->setTitle(tr("documents_label"));
+    //w->bookmarkBox->setTitle(tr("bookmarks_label"));
     
     assert(projectTreeController == NULL);
     ProjectTreeControllerModeSettings s;
+    s.groupMode = getLastGroupMode();
     s.loadTaskProvider = this;
     projectTreeController = new ProjectTreeController(w, w->documentTreeWidget, s);
+    projectTreeController->setObjectName("document_Filter_Tree_Controller");
     connect(projectTreeController, SIGNAL(si_onPopupMenuRequested(QMenu&)), SLOT(sl_onDocTreePopupMenuRequested(QMenu&)));
     connect(projectTreeController, SIGNAL(si_doubleClicked(GObject*)), SLOT(sl_onDoubleClicked(GObject*)));
 
+    w->groupModeMenu->addAction(projectTreeController->getGroupByDocumentAction());
+    w->groupModeMenu->addAction(projectTreeController->getGroupByTypeAction());
+    w->groupModeMenu->addAction(projectTreeController->getGroupFlatAction());
+
+    connect(w->nameFilterEdit, SIGNAL(textChanged(const QString&)), SLOT(sl_filterTextChanged(const QString&)));
+    w->nameFilterEdit->installEventFilter(this);
+
     assert(objectViewController == NULL);
     objectViewController = new ObjectViewTreeController(w->viewTreeWidget);
 
@@ -204,6 +245,13 @@ bool ProjectViewImpl::eventFilter(QObject *obj, QEvent *event) {
         if (ov->isPersistent()) {
             saveViewState(ov, GObjectViewState::APP_CLOSING_STATE_NAME);
         }
+    } else if (w!=NULL && w->nameFilterEdit == obj) {
+        if (event->type() == QEvent::KeyPress) {
+            QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
+            if (keyEvent->modifiers() == Qt::NoModifier && keyEvent->key() == Qt::Key_Escape) {
+                w->nameFilterEdit->clear();
+            }
+        }
     }
     return QObject::eventFilter(obj, event);
 }
@@ -318,15 +366,17 @@ void ProjectViewImpl::sl_onAddExistingDocument() {
         QString url_w = m.dwnldPath;
         tasks.push_back( new CopyDataTask(iof, m.url, iow, url_w) );
         multiTaskName += "download_";
-        newdoc = new Document( format, iow, url_w, m.formatSettings );
+        newdoc = new Document(format, iow, url_w, QList<UnloadedObjectInfo>(), m.formatSettings);
     } else {
-        newdoc = new Document( format, iof, m.url, m.formatSettings );
+        newdoc = new Document(format, iof, m.url, QList<UnloadedObjectInfo>(), m.formatSettings);
     }
     if (m.readOnly) {
         newdoc->setUserModLock(true);
     }
-    tasks.push_back( new AddDocumentTask(newdoc) );
-    tasks.push_back( new LoadUnloadedDocumentAndOpenViewTask(newdoc) );
+    Task * adt = new AddDocumentTask(newdoc);
+    adt->setSubtaskProgressWeight(0);
+    tasks.append(adt);
+    tasks.append(new LoadUnloadedDocumentAndOpenViewTask(newdoc));
     multiTaskName += "add_load_document_task";
     AppContext::getTaskScheduler()->registerTopLevelTask( new MultiTask(multiTaskName, tasks) );
 
@@ -380,6 +430,9 @@ void ProjectViewImpl::sl_onDoubleClicked(GObject* o) {
         openActions<<tmp;
     }
     if (openActions.isEmpty()) { 
+        if (o->isUnloaded()) {
+            AppContext::getTaskScheduler()->registerTopLevelTask(new LoadUnloadedDocumentTask(o->getDocument()));
+        }
         return;
     }
     if (openActions.size() == 1 ) {
@@ -496,6 +549,38 @@ void ProjectViewImpl::buildAddToViewMenu(const MultiGSelection& ms, QMenu* m) {
     m->addAction(action);
 }
 
+void ProjectViewImpl::buildRelocateMenu(QMenu* m) {
+    const DocumentSelection* docSelection = getDocumentSelection();
+    const GObjectSelection* objSelection = getGObjectSelection();
+    if (!objSelection->isEmpty() || docSelection->getSelectedDocuments().size()!=1) {
+        return;
+    }
+    Document* doc = docSelection->getSelectedDocuments().first();
+    if (doc->isLoaded()) {
+        DocumentFormatRegistry *dfr =  AppContext::getDocumentFormatRegistry();
+        QList<DocumentFormatId> ids = dfr->getRegisteredFormats();
+        QList<DocumentFormat*> allWritableFormats;
+        foreach(DocumentFormatId id, ids){
+            DocumentFormat *format = dfr->getFormatById(id);
+            DocumentFormatConstraints c;
+            c.mustSupportWrite = true;
+            if(format->checkConstraints(c)){
+                allWritableFormats.append(format);
+            }
+        }
+        foreach(DocumentFormat *f ,allWritableFormats){
+            foreach(GObject *gobj, doc->getObjects()){
+                if(f->isObjectOpSupported(doc, DocumentFormat::DocObjectOp_Add, gobj->getGObjectType())){
+                     m->addAction(saveCopyAction);
+                     return;
+                }
+            }
+        }
+    } else {
+        m->addAction(relocateDocumentAction);
+    }
+}
+
 void ProjectViewImpl::buildViewMenu(QMenu& m) {
     QMenu* openViewMenu = new QMenu(tr("open_view_menu"), &m);
     QMenu* addToViewMenu= new QMenu(tr("add_to_view_menu"), &m);
@@ -518,7 +603,6 @@ void ProjectViewImpl::buildViewMenu(QMenu& m) {
     openViewMenu->setDisabled(openViewMenu->isEmpty());
     m.insertMenu(m.actions().first(), openViewMenu);
 
-
     bool hasModifiedDocs = false;
     foreach(Document* doc, docSelection->getSelectedDocuments()) {
         if (doc->isTreeItemModified()) {
@@ -526,6 +610,8 @@ void ProjectViewImpl::buildViewMenu(QMenu& m) {
             break;
         }
     }
+    buildRelocateMenu(&m);
+
     saveSelectedDocsAction->setEnabled(hasModifiedDocs);
     m.addAction(saveSelectedDocsAction);
 }
@@ -592,13 +678,94 @@ void ProjectViewImpl::sl_onDocumentLoadedStateChanged() {
     //do nothing..
 }
 
+void ProjectViewImpl::sl_filterTextChanged(const QString& t) {
+    assert(projectTreeController!=NULL);
+    ProjectTreeControllerModeSettings settings = projectTreeController->getModeSettings();
+    settings.tokensToShow = t.split(' ', QString::SkipEmptyParts);
+    projectTreeController->updateSettings(settings);
+}
+
+void ProjectViewImpl::sl_relocate() {
+    const DocumentSelection* ds = getDocumentSelection();
+    Document* d = ds->isEmpty() ? NULL : ds->getSelectedDocuments().first();
+    if (d == NULL) {
+        return;
+    }
+    if (d->isLoaded()) { //TODO: support loaded docs relocation?
+        return;
+    }
+    LastOpenDirHelper h;
+    h.url = QFileDialog::getOpenFileName(w, tr("Select new file location"), h.dir);
+    if (h.url.isEmpty()) {
+        return;
+    }
+    AppContext::getTaskScheduler()->registerTopLevelTask(new RelocateDocumentTask(d->getURL(), h.url));    
+}
+
+void ProjectViewImpl::sl_saveCopy() {
+    const DocumentSelection* ds = getDocumentSelection();
+    Document* d = ds->isEmpty() ? NULL : ds->getSelectedDocuments().first();
+    if (d == NULL) {
+        return;
+    }
+    if (!d->isLoaded()) {
+        return;
+    }
+    QString filter = DialogUtils::prepareDocumentsFileFilter(d->getDocumentFormatId(), false);
+    LastOpenDirHelper h;
+    CopyDocumentDialogController dialog(filter, d);
+    int result = dialog.exec();
+    h.url = dialog.getFilename();
+    if (result == QDialog::Accepted){
+        if (h.url.isEmpty()) {
+            return;
+        }
+        if (AppContext::getProject()->findDocumentByURL(h.url)) {
+            QMessageBox::critical(w, tr("Error"), tr("Document with the same URL is added to the project. \n Remove it from the project first."));
+            return;
+        }
+        bool addToProject = dialog.getAddToProjectFlag();
+        IOAdapterFactory* iof = AppContext::getIOAdapterRegistry()->getIOAdapterFactoryById(BaseIOAdapters::url2io(h.url));
+        DocumentFormatRegistry *dfr =  AppContext::getDocumentFormatRegistry();
+        DocumentFormat *df = dfr->getFormatById(dialog.getDocumentFormatId());
+        if (iof == NULL) {
+            return;
+        }
+        QList<GObject*> GObjList = d->getObjects();
+        Document *dd;
+        dd = df->createNewDocument(iof, h.url, d->getGHintsMap());
+        foreach(GObject* go, GObjList){
+            if(df->isObjectOpSupported(dd, DocumentFormat::DocObjectOp_Add, go->getGObjectType())){
+                GObject *cl = go->clone();
+                dd->addObject(cl);
+            }
+        }
+        foreach(GObject* o, dd->getObjects()) {
+            GObjectUtils::updateRelationsURL(o, d->getURL(), h.url);
+        }
+        Task* t = NULL;
+        if (addToProject) {
+            Project *p = AppContext::getProject();
+            dd->setModified(true);
+            p->addDocument(dd);
+        } else {
+            t = new SaveDocumentTask(dd, iof, h.url);
+            AppContext::getTaskScheduler()->registerTopLevelTask(t);
+        }
+    }
+}
+
+void ProjectViewImpl::highlightItem(Document* doc){
+    assert(doc);
+    projectTreeController->highlightItem(doc);
+}
 //////////////////////////////////////////////////////////////////////////
 // Tasks
 
 //EnableProjectViewTask
 
 EnableProjectViewTask::EnableProjectViewTask(ProjectViewImpl* _pvi) 
-: Task(tr("enable_project_view"), TaskFlags_NR_DWF), pvi(_pvi)
+: Task(tr("enable_project_view"), TaskFlag_NoRun), pvi(_pvi)
 {
 }
 
@@ -610,21 +777,11 @@ Task::ReportResult EnableProjectViewTask::report() {
 
 
 DisableProjectViewTask::DisableProjectViewTask(ProjectViewImpl* _pvi) 
-: Task(tr("disable_project_view"), TaskFlags_NR_DWF_SSSOR), pvi(_pvi)
+: Task(tr("disable_project_view"), TaskFlags_NR_FOSCOE), pvi(_pvi)
 {
 }
 
-void DisableProjectViewTask::prepare() {
-    if (AppContext::getProject()->isTreeItemModified()) {
-        addSubTask(AppContext::getProjectService()->saveProjectTask(SaveProjectTaskKind_SaveProjectAndDocumentsAskEach));
-    }
-}
-
-
 Task::ReportResult DisableProjectViewTask::report() {
-    if (propagateSubtaskError()) {
-        return ReportResult_Finished;
-    }
     pvi->disable();
     return ReportResult_Finished;
 }
