@@ -1,6 +1,6 @@
 /*****************************************************************
 * Unipro UGENE - Integrated Bioinformatics Suite
-* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
+* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
@@ -31,7 +31,7 @@ public slots:
 class ScriptManagerDialog : public QDialog {
     Q_OBJECT
 public:
-    ScriptManagerDialog( ScriptRegistryService * sr );
+    ScriptManagerDialog( ScriptRegistryService * sr, QWidget *p = NULL );
 private:
     ScriptManagerWidgetImpl * w;
 };
